/**
 * This class is responsible for managing and notifying listeners.
 */
 
 
/**
 * Get the single instance of this class.
 */
function getEventServer() {
	if (!top.EventServer) {
		// create the event server
	
	
		/**
		 * Creates the event server instance
		 */ 
		top.EventServer = new Object();
		
		
		/**
		 * The table containing all the listeners.
		 */
		listeners = new Array();
		
		
		// services
		
		
		/**
		 * Add a listener.
		 * @param listener The listener to add. 
		 * @throws Error if the listener is undefined or null.
		 */
		top.EventServer.addListener = function(listener) {
			assertDefined(listener, "The listener to add is undefined or null: " + listener);
			
			listeners[listeners.length] = listener;
		}
		
		
		/**
		 * Remove a listener. 
		 * @param listener The listener to remove.
		 * @throws Error if the listener is undefined or null.
		 */
		top.EventServer.removeListener = function(listener) {
			assertDefined(listener, "The listener to remove is undefined or null: " + listener);
			
			var length = listeners.length;
			for (var i = 0; i < length; i++) {
				if (listener == listeners[i]) {
					listeners[i] = null;
					listeners = removeNullEntries(listeners);
				}
			}
		} 
		
		
		/**
		 * Notify the listeners that a event has happened. 
		 * @param event The event to notify the listeners about.
		 * @throws Error if event is undefined or null.
		 */
		top.EventServer.notifyListeners = function(event) {
			assertDefined(event, "The event is undefined or null: " + event);
			
			var length = listeners.length;
			for (var i = 0; i < length; i++)  {
				if (exists(listeners[i]) && listeners[i].listeningToEvent(event)) {
					listeners[i].update(event);
				}
			}
		}
		
		
		/**
		 * Checks if a given listener is among the registered listeners. 
		 * @param listener The listener to check.
		 * @return True if listener is among the registered listeners false otherwise.
		 * @throws Error if listener is undefined or null.
		 */
		top.EventServer.isListener = function(listener) {
			assertExists(listener, "The listener to check is undefined or null");
			
			var length = listeners.length;
			for (var i = 0; i < length; i++)  {
				if (exists(listeners[i]) && listeners[i] == listener) {
					return true;
				}
			}
			return false;
		}
		 
	}
	
	return top.EventServer;
}
