
/**
 * Stop anybody else from seeing this event.
 * @param event The event to stop. 
 */
function stopPropagation(event) {
	if(event == null) {
		return;
	}
	if (event.stopPropagation) {
			event.stopPropagation();
	}
	else {
		// IE
		event.cancelBubble = true;
	}	
}


/**
 * Prevent the default actions of a event.
 * @param event The event to stop the default actions for.
 */
function stopDefaultAction(event) {
	if (event == null) {
		return;
	}
	
	if (event.preventDefault) {
		event.preventDefault();
	}
	else {
		// IE
		event.returnValue = false;
	}
}


/**
 * Register a event handler.
 * @param event The event to register a handler for.
 * @param handler The handler for that event.
 * @param element The element to register the event handler on, if not set the event 
 * is registered at the document. 
 */
function registerEventHandler(event, handler, element) {
	assertDefined(event, "The event is not defined or null");
	assertDefined(handler, "The handler is not defined or null");
	
	var registerAtElement = exists(element) ? element : document;
	if (registerAtElement.addEventListener) {
		registerAtElement.addEventListener(event, handler, true);
	}
	
	else if(registerAtElement.attachEvent()) {
		// IE 5+1
		registerAtElement.attachEvent("on" + event, handler);
		
	}
}


/**
 * Removes a event handler.
 * @param event the event to remove the handler for.
 * @param handler The handler to remove for that event. 
 * @param element The element to remove the event handler from, if not set the event
 * is removed from the document. 
 */
function removeEventHandler(event, handler, element) {
	assertDefined(event, "The event is not defined or null");
	assertDefined(handler, "The handler is not defined or null");
	
	var registerAtElement = exists(element) ? element : document;
	if (registerAtElement.removeEventListener) {
		registerAtElement.removeEventListener(event, handler, true);
	}
	else if(registerAtElement.detachEvent) {
		// IE 5+
		registerAtElement.detachEvent("on" + event, handler);
	}
}


/**
 * Returns the target element of a event.
 * @param event The event to get the target element for.
 */
function getTargetElement(event) {
	if (!event) {
		// IE
		event = window.event;
	}
	
	if (event.target) {
		return event.target;
	}
	else {
		// IE
		return event.srcElement;
	}
}


/**
 * Gets the related target of a mouseover event. For mouseover events the related target is the element the 
 * mouse left when it moved over the target. For mouseout events, it is the element the mouse entered when
 * leaving the target. 
 * @param event The event to get the related target for. 
 * @return The related target.
 */
function relatedTarget(event) {
	event = exists(event) ? event : window.event;
	if (exists(event.relatedTarget)) {
		return event.relatedTarget;
	}
	
	// IE does not have a relatedTarget property
	if (event.type == "mouseover") {
		return event.fromElement;
	}
	else if (event.type == "mouseout") {
		return event.toElement;
	}
}