// disable right-click for everything (except elements which explisitly have the .oncontextmenu set)
document.oncontextmenu = onGlobalRightClick;
document.onkeydown = onKeyDown;
document.onkeyup = onKeyUp;

// keep track of the last clicked table entry
var lastTableObject = null;
var lastTdNode = null;
var lastEntryFieldName = null;
var lastKeyEvent = null;


/**
 * The ids of the right click menus created by this.
 */
var createdRightClickMenus = new Array();


/**
 * Handles the right-click event for the (global) document.
 * @param event The event object.
 */
function onGlobalRightClick(event) {
	var e = event || window.event;

	// prevent the default context menu to pop up in Firefox (Linux)	
	if (isFirefox()) {
	 	e.preventDefault();
	 	e.stopPropagation(); 	
	}
	
	return false;
}


/**
 * Handles the onkeydown event.
 */
function onKeyDown(event) {
	var e = event || window.event;
	lastKeyEvent = e;
}


/**
 * Handles the onkeyup event.
 */
function onKeyUp(event) {
	lastKeyEvent = null;
}


/**
 * Shows the right-click menu for entries in the protocol table.
 */
function showProtocolTableMenu(event) {
	return showMenu(event, document.getElementById("protocoltableentrymenu"));
}


/**
 * Shows the right-click menu for entries in the host table.
 */
function showHostTableMenu(event) {
	return showMenu(event, document.getElementById("hosttableentrymenu"));	
}


/**
 * Shows the right-click menu for entries in the detail tables (protocol and nlhost detail table). 
 */
function showDetailTableMenu(event) {
	return showMenu(event, document.getElementById("protocoldetailtableentrymenu"));
}


/**
 * Shows the right-click menu for entries in the conversation table.
 */
function showConversationTableMenu(event) {
	return showMenu(event, document.getElementById("conversationtableentrymenu"));
}


/**
 * Shows the right-click menu for entries in a table.
 */
function showDefaultMenu(event) {
	return showMenu(event, document.getElementById("tableentrymenu"));
}

/**
 * Shows the right-click menu for entries in the "Defined alarms" window.
 */
function showDefinedAlarmsMenu(event) {
	return showMenu(event, document.getElementById("definedalarmstableentrymenu"));
}


/**
 * Shows the right-click menu for entries in the "Triggered alarms" window.
 */
function showTriggeredAlarmsMenu(event) {
	return showMenu(event, document.getElementById("triggeredalarmstableentrymenu"));
}


/**
 * Shows the right-click menu for entries in the "Triggered alarms" window.
 */
function showTriggeredMasterAlarmsMenu(event) {
	return showMenu(event, document.getElementById("triggeredmasteralarmstableentrymenu"));
}


/**
 * Shows the right-click menu for entries in the "Add/edit alarm action" window.
 */
function showAlarmActionMenu(event) {
	return showMenu(event, document.getElementById("alarmactionsmenu"));
}


/**
 * Shows the right-click menu for entries in the alarm action library table.
 */
function showAlarmActionLibraryMenu(event) {
	return showMenu(event, document.getElementById("alarmactionslibrarymenu"));
}


/**
 * Shows the right-click menu for entries in the top networks table. 
 */
function showTopNetworksTableMenu(event) {
	return showMenu(event, document.getElementById("topNetworkstableentrymenu"));
}


/**
 * Shows the defined alarms window, based on the window that was clicked.
 */
function showDefinedAlarmsWindow() {
	// get the current active window
	var currentActiveWindow = WindowManager.instance().getActiveWindow();
	if (currentActiveWindow.id.indexOf("triggeredalarmswindow") > -1) {
		var startIndex = currentActiveWindow.id.indexOf("triggeredalarmswindow");
		var tableName = currentActiveWindow.id.substring(0, startIndex);
		WindowManager.instance().activateWindow(tableName + "definedalarmswindow");
	}
}

/**
 * Shows the triggered alarms window, based on the window that was clicked.
 */
function showTriggeredAlarmsWindow() {
	// get the current active window
	var currentActiveWindow = WindowManager.instance().getActiveWindow();
	if (currentActiveWindow.id.indexOf("definedalarmswindow") > -1) {
		var startIndex = currentActiveWindow.id.indexOf("definedalarmswindow");
		var tableName = currentActiveWindow.id.substring(0, startIndex);
		WindowManager.instance().activateWindow(tableName + "triggeredalarmswindow");
	}
}


/**
 * Shows the menu for the specified element id.
 */
function showMenu(event, elementId) {
	hideMenus();
	var tableentrymenu = elementId;
	var e = event || window.event;	
	// check for CTRL+left-click
	if (e.type == "mousedown" && lastKeyEvent != null && lastKeyEvent.ctrlKey) {
		stopPropagation(e);
	}
	else if (e.type == "mousedown" && (lastKeyEvent == null || !lastKeyEvent.ctrlKey)) {
		return;
	}
	var srcElement = isIE() || isOpera() ? e.srcElement : e.target;
	
	// keep track of the last clicked table entry
	lastTableObject = tableObject(srcElement);
	lastTdNode = findTdNode(srcElement);
	lastEntryFieldName = entryFieldName(srcElement);
	var rightedge = document.body.clientWidth - e.clientX;
	var bottomedge = document.body.clientHeight - e.clientY;
	var topMenu = getElementWithAttribute(document.body, "topMenu", "id", "div");
	if (rightedge < tableentrymenu.offsetWidth) {
		tableentrymenu.style.left = document.body.scrollLeft + e.clientX - tableentrymenu.offsetWidth + "px";
	}
	else {
		tableentrymenu.style.left = document.body.scrollLeft + e.clientX + "px";
	}

	if (e.clientY > (getWindowHeight(window) - tableentrymenu.offsetHeight)) {
		tableentrymenu.style.top = document.body.scrollTop + e.clientY - tableentrymenu.offsetHeight + "px";
	}
	else {
		tableentrymenu.style.top = document.body.scrollTop + (e.clientY - topMenu.offsetHeight) + "px";
	}
	tableentrymenu.style.visibility = "visible";
	getEventServer().notifyListeners(new Event("rightClickMenuShown", exists(lastTableObject) ? lastTableObject.id : null));
	return false;
}



/**
 * Returns the table object which was right-clicked.
 */
function tableObject(srcElement) {
	if (!exists(srcElement)) {
		return undefined;
	}
	var tableNode = getTableFromTd(srcElement.parentNode);
	// tableNode might not exists if anything but a table was clicked
	return exists(tableNode) && !isEmpty(tableNode.id) ? TableManager.instance().getTable(tableNode.id) : null;
}


/**
 * Gets the table for the given td element. The table must have the id attribute set.
 * @param td The td element from which to get the table.
 * @return The table object.
 */
function getTableFromTd(td) {
	var parentNode = td.parentNode;
	if (!isDefined(parentNode) || parentNode == null) {
		return null;
	}

	var maxChildren = 10;
	var i = 0;
	// loop through all parent nodes until we find a table having the id attribute set
	while (true && i < maxChildren) {
		if (!exists(parentNode)) {
			break;
		}
		if (parentNode.tagName == "TABLE" && isDefined(parentNode.id)) {
			return parentNode;
		}
		parentNode = parentNode.parentNode;
		i++;
	}
}


/**
 * Returns the name of the field in the entry which was right-clicked.
 */
function entryFieldName(srcElement) {
	var tdNode = findTdNode(srcElement);
	// tdNode might not exists, if anything else than a table was clicked
	return exists(tdNode) ? tdNode.getAttribute("name") : null;
}


/**
 * Hides all the right-click menus.
 */
function hideMenus() {
	hideMenu(document.getElementById("conversationtableentrymenu"));
	hideMenu(document.getElementById("hosttableentrymenu"));
	hideMenu(document.getElementById("protocoltableentrymenu"));
	hideMenu(document.getElementById("tableentrymenu"));
	hideMenu(document.getElementById("definedalarmstableentrymenu"));
	hideMenu(document.getElementById("triggeredalarmstableentrymenu"));
	hideMenu(document.getElementById("triggeredmasteralarmstableentrymenu"));
	hideMenu(document.getElementById("alarmactionsmenu"));
	hideMenu(document.getElementById("alarmactionslibrarymenu"));
	hideMenu(document.getElementById("protocoldetailtableentrymenu"));
	hideMenu(document.getElementById("topNetworkstableentrymenu"));
	
	
	for (var i = 0; i < createdRightClickMenus.length; i++) {
		hideMenu(document.getElementById(createdRightClickMenus[i]));
	}
	
	if (exists(lastTableObject)) {
		getEventServer().notifyListeners(new Event("rightClickMenuHidden", lastTableObject.id));
	}
	
	// make sure there are no references to the clicked table entry until next time it is clicked
	lastTableObject = null;
	lastTdNode = null;
	lastEntryFieldName = null;
}


/**
 * Hides a menu.
 * @param menu The html code of the menu to hide. 
 */
function hideMenu(menu) {
	if (exists(menu)) {
		menu.style.visibility = "hidden";
		menu.style.top = "0px";
		menu.style.left = "0px";
	}
}


/**
 * Hightlights an entry in the right-click menu when mouse is over.
 */
function highLight(event) {
	var e = event || window.event;
	var srcElement = isIE() || isOpera() ? e.srcElement : e.target;
	if (getClassName(srcElement) == "menuitems" || getClassName(srcElement) == "menuItemHighLight") {
		setClass(srcElement, "menuItemHighLight");
	}
}

/**
 * Lowlights an entry in the right-click menu when mouse is out.
 */
function lowLight(event) {
	var e = event || window.event;
	var srcElement = isIE() || isOpera() ? e.srcElement : e.target;
	if (srcElement.nodeType != 1) {
		srcElement = firstElementParentNode(srcElement);
	}
	if (srcElement.nodeType == 1 && (getClassName(srcElement) == "menuitems" || getClassName(srcElement) == "menuItemHighLight")) {
		setClass(srcElement, "menuitems");
		window.status = "";
   	}
}


/**
 * Returns the first parent element with a node element nodetype.
 * @param node The node to get the element parent node of.
 * @return The fist element parent node of node, node if node has no parents.
 */
function firstElementParentNode(node) {
	if (!exists(node.parentNode)) {
		// the node has no parents
		return node;
	}
	
	while(true) {
		if (!exists(node.parentNode)) {
			return node;
		}
		node = node.parentNode;
		if (node.nodeType == 1) {
			return node;
		}		
	}
}
 


/**
 * Shows the help when right-clicking inside a table.
 * @param url The help url to show.
 */
function showHelp(url) {
	top.tabSection.activateTab("Help");	
	top.frames["Help"].location.href = url;
}


/**
 * Creates a right click menu container. To this container right click menu items can be added later. 
 * @param id The id to set for the container. 
 * @param parentNode The node to which the right click menu is to be appended. 
 * @throws Error if id is undefined null or empty.
 * @throws Error if parentNode is undefined or null.
 */
function createRightClickContainer(id, parentNode) {
	assertNotEmpty(id, "The id for the right click container to set is undefined, null or empty");
	assertExists(parentNode, "The node to append the right click container to is undefined or null");
	var rightClickMenu = document.createElement("div");
	rightClickMenu.setAttribute("id", id);
	setClass(rightClickMenu, "rightclickmenu");
	rightClickMenu.onmouseover = highLight;
	rightClickMenu.onmouseout = lowLight;
	rightClickMenu.style.visibility = "hidden";
	parentNode.appendChild(rightClickMenu);
	createdRightClickMenus[createdRightClickMenus.length] = id;
	rightClickMenu.style.top = "0px";
	rightClickMenu.style.left = "0px";
	return rightClickMenu;
}


/**
 * Add a right click menu item to a given right click menu.
 * @param id The id of the right click menu container to add the right click element to.
 * @param itemId The id of the right click menu item to add.
 * @param itemText The text to show for the item.
 * @param functionToCall The function to call when the item has been clicked.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if itemId is undefined, null or empty.
 * @throws Error if itemText is undefined null or empty.
 * @throws Error if functionToCall is undefined or null. 
 */
function addRightClickMenuItem(id, itemId, itemText, functionToCall) {
	assertNotEmpty(id, "The id of the right click menu to add a item to is null or undefined");
	assertNotEmpty(itemId, "The id of the item to add is undefined, null or empty");
	assertNotEmpty(itemText, "The text to set for the item is undefined null or empty");
	assertExists(functionToCall, "The function to call when the right click menu item was clicked is undefined or null");
	var rightClickMenu = document.getElementById(id);
	if (!exists(rightClickMenu)) {
		throw new Error("Right click menu with id: " + id + " does not exist");
	}
	var rightClickItem = document.createElement("div");
	rightClickItem.setAttribute("id", itemId);
	setClass(rightClickItem, "menuitems");
	rightClickItem.onmousedown = function(event) {
		functionToCall(event),
		lowLight(event);
		hideMenus();
	};
	rightClickMenu.appendChild(rightClickItem);
	rightClickItem.appendChild(document.createTextNode(itemText));
}



/**
 * Shows a given right click menu.
 * @param id The id of the right click menu to show. 
 * @param event The event that triggered the showing of the right click menu.
 * @throws Error if id is undefined null or empty.
 * @throws Error if event is undefined null or empty.
 */
function showRightClickMenu(id, event) {
	assertNotEmpty(id, "The id of the right click menu to show is undefined, null or empty");
	assertExists(event, "The event is undefined null or empty");
	var rightClickMenu = document.getElementById(id);
	if (!exists(rightClickMenu)) {
		return;
	}
	showMenu(event, rightClickMenu);
}


/**
 * Shows a right click menu item.
 * @param id The id of the right click menu to show the item in. 
 * @param itemId The id of the item to show.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if itemId is undefined, null or empty.
 */
function showRightClickItem(id, itemId) {
	assertNotEmpty(id, "The id of the right click menu to show a item in is undefined, null or empty");
	assertNotEmpty(itemId, "The id of the item to show is undefined, null or empty");
	var rightClickMenu = document.getElementById(id);
	if (!exists(rightClickMenu)) {
		return;
	}
	var item = getElementWithAttribute(rightClickMenu, itemId, "id", "div");
	if (!exists(item)) {
		return;
	}
	setClass(item, "menuitems");
}
 
 
/** 
 * Hides a right click menu item.
 * @param id The id of the right click menu to hide the item in. 
 * @param itemId The id of the item to hide.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if itemId is undefined, null or empty.
 */
function hideRightClickItem(id, itemId) {
	assertNotEmpty(id, "The id of the right click menu to show a item in is undefined, null or empty");
	assertNotEmpty(itemId, "The id of the item to show is undefined, null or empty");
	var rightClickMenu = document.getElementById(id);
	if (!exists(rightClickMenu)) {
		return;
	}
	var item = getElementWithAttribute(rightClickMenu, itemId, "id", "div");
	if (!exists(item)) {
		return;
	}
	setClass(item, "inactive");
}