/**
 * This class shows the program actions.  
 */


// constants


/**
 * The base url used to load email alarm actions. The session id and the alarm id need to be
 * appended to this url.
 */
var LOAD_PROGRAM_ACTION_BASE_URL = "/?resource=AlarmManager&mode=get&command=programAlarmAction&";


/**
 * The base url used to load library program alarm actions. The session id needs to be appended to this url.
 */
var LOAD_LIBRARY_PROGRAM_ACTION_BASE_URL = "/?resource=AlarmManager&mode=get&command=libraryProgramAction&";



// construction


/**
 * Factory method to create new program alarm action table objects.
 * @param id The id of the table to update.
 * @param url The url to use to update the table.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * If this is not specified the default value is used.
 * @returns A new alarm action table object.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createProgramActionTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);
	
	var subInstance = new ProgramActionsTable();
	var superInstance = createActionTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	instance["supercreateDataTable"] = superInstance["supercreateDataTable"];
	instance["superupdateTableRow"] = superInstance["superupdateTableRow"];
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);
	instance.eventListener.addEvent(WINDOW_SHOWN_EVENT, processAlarmActionWindowShownEvent);

	return instance;
}
 

/**
 * Subclasses actiontabls.js to add behaviour needed in the email alarmactions table.
 */
function ProgramActionsTable() {
	
}


// implementation


/**
 * Inserts a field value into a given html node. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
ProgramActionsTable.prototype.insertFieldValue = function(field, node, cell) {
	if (!this.isHeader(field) && field.getName().indexOf("programName") > -1) {
		node.appendChild(document.createTextNode(URLDecode(field.getValue())));
	}
	else if (!this.isHeader(field) && field.getName().indexOf("parameters") > -1) {
		node.appendChild(document.createTextNode(URLDecode(field.getValue())));
	}
	else if (!this.isHeader(field) && field.getName().indexOf("description") > -1) {
		node.appendChild(document.createTextNode(URLDecode(field.getValue())));
	}
	else {
		node.appendChild(document.createTextNode(field.getValue()));
	}
}