

/**
 * Used for triggering the alarm when the value is above the trigger value.
 */
var UPPER = 0;


/**
 * Used for triggering the alarm when the value is below the lower trigger value.
 */
var LOWER = 1;

	
/**
 * The padding used between the sections (Table, Entry...) of the add alarm window.
 */
var SECTION_PADDING = 10;


/**
 * The default value for the description field.
 */
var DEFAULT_DESCRIPTION = "[Enter a description]";
 
 
/**
 * The entry fields used for the tables.
 */
var entryFieldNames = new Array();
entryFieldNames["protocol"] = ["portTypeContainer", "portInputContainer"];
entryFieldNames["nlhost"] = ["ipInputContainer"];
entryFieldNames["nlmatrix"] = ["srcInputContainer", "dstInputContainer"];
entryFieldNames["alhost"] = ["portTypeContainer", "portInputContainer", "ipInputContainer"];
entryFieldNames["almatrix"] = ["portTypeContainer", "portInputContainer", "srcInputContainer", "dstInputContainer"];
entryFieldNames["host"] = ["macInputContainer"];
entryFieldNames["matrix"] = ["srcMacInputContainer", "dstMacInputContainer"];
entryFieldNames["network"] = [];
entryFieldNames["summary"] = [];


/**
 * The functions used to create the enty part of the add alarm url. This functions 
 * do not return the whole url to add a alarm, but jus the entry part (field, and ip, port...).
 */
var entryUrlCreators = new Array();
entryUrlCreators["protocol"] = protocolEntryUrl;
entryUrlCreators["alhost"] = protocolPerHostEntryUrl;
entryUrlCreators["nlhost"] = hostEntryUrl;
entryUrlCreators["nlmatrix"] = conversationsUrl;
entryUrlCreators["almatrix"] = protocolsPerConversationUrl;
entryUrlCreators["host"] = networkCardsUrl;
entryUrlCreators["matrix"] = networkCardConversationUrl;
entryUrlCreators["network"] = networkUrl;
entryUrlCreators["summary"] = summaryUrl;


/**
 * Mapping between the table names as defined on client side and the table names on server side.
 * (topNProtocolsTable -> protocols, summaryTable -> summary)
 */
var tableNames = new Array();
tableNames["summaryTable"] = "summary";
tableNames["topNProtocolsTable"] = "protocol";
tableNames["myProtocolsTable"] = "protocol";
tableNames["topNHostsTable"] = "nlhost";
tableNames["myHostsTable"] = "nlhost";
tableNames["topNHostListenersTable"] = "nlhost";
tableNames["topNHostTalkersTable"] = "nlhost";
tableNames["topNConversationsTable"] = "nlmatrix";
tableNames["myConversationsTable"] = "nlmatrix";
tableNames["myProtocolsPerHostTable"] = "alhost";
tableNames["protocolDetailTable"] = "protocol";
tableNames["protocolTopHostsTable"] = "alhost";
tableNames["protocolTopConversationsTable"] = "almatrix";
tableNames["nlhostDetailTable"] = "nlhost";
tableNames["nlhostTopProtocolsTable"] = "alhost";
tableNames["nlhostTopConversationsTable"] = "nlmatrix";


/**
 * The default value for the unit select tags, indexed by the select tags name. 
 */
var UNIT_SELECT_DEFAULT_SELECTED = {octetsUnitChooser: "MB", octetsUnitChooserDelta: "Mbit/s", 
		packetsUnitChooserDelta: "Packets", packetsUnitChooser: "Packets", 
		entriesUnitChooser: "Entries", entryUnitChooser: "Entries"};  


/**
 * The base url used to set a alarm.
 */
var SET_ALARM_BASE_URL = "/?resource=AlarmManager&mode=set&command=add&format=xml&sid=" + getSid();


/**
 * The base url used to edit a alarm.
 */
var EDIT_ALARM_BASE_URL = "/?resource=AlarmManager&mode=set&command=edit&format=xml&sid=" + getSid();


/**
 * The edit alarm window title.
 */
var EDIT_ALARM_WINDOW_TITLE = "Edit alarm";


/**
 * The copy alarm window title.
 */
var COPY_ALARM_WINDOW_TITLE = "Copy alarm";


/**
 * The add alarm window title.
 */
var ADD_ALARM_WINDOW_TITLE = "Add alarm";


/**
 * The event fired when a alarm has been set. 
 */
var ALARM_SET_EVENT = "alarmSet";


/**
 * The event fired when a alarm has been edited. 
 */
var ALARM_EDITED_EVENT = "alarmEdited";


/**
 * The field validator used to validate the lower trigger value. 
 */
var lowerTriggerValidator;


// construction


/**
 * Factory method to create the alarm detail window.
 * @param id The id of the window.
 * @throws Error if id is undefined, null or empty.
 */
function createAlarmDetailWindow(id) {
	assertNotEmpty(id, "The id to set is undefined, null or empty");
	
	var superInstance = new Window(id);
	var subInstance = new AlarmDetailWindow();
	subInstance = createSubClass(superInstance, subInstance);
	
	WindowManager.instance().removeWindow(superInstance);
	WindowManager.instance().registerNewWindow(subInstance);
	
	// the windows drag helper is initialized in the window constructor, we need to change the drag helpers
	// window reference to be this window not the super instance of Window
	subInstance.dragHelper.window = subInstance;
	
	initValidators();
	return subInstance;
}


/**
 * The alarm detail window.
 */
function AlarmDetailWindow() {
	
}


// overridden functions from window


/**
 * Returns this window as xml. This is overridden to set a fixed height 
 * for the alarm detail window.
 */
AlarmDetailWindow.prototype.toXml = function() {
	// because of messages and entries shown the height of the alarm detail window
	// can vary when a layout is saved, but when the layout is loaded again the messages
	// and entries are not visible any more and the height becomes incorrect, therefore
	// we set a fixed height and width for the alarm detail window
	var xmlString = 
		"<window>" +
			"<windowId>" + this.id + "</windowId>" + 
			"<xPosition>" + this.getXPosition() + "</xPosition>" +
			"<yPosition>" + this.getYPosition() + "</yPosition>" +
			"<width>498</width>" +
			"<height>417</height>";
	xmlString += this.statusToXml();
	xmlString += this.colapsedContentToXml();
	xmlString += "</window>";
	return xmlString;
}


// implementation


/**
 * Deletes a triggered alarm, when it is clicked from the right-click menu.
 */
function deleteAlarm() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var tableName = lastTableObject.id;
		var tableObj = TableManager.instance().getTable(tableName);
		
		// a triggered alarm table was not found
		if (tableObj == null || tableName == null || (tableName.indexOf("Triggered") < 0 && tableName.indexOf("Defined") < 0)) {
			return;
		}
		
		if (row != null) {
			// get the id and delete the entry
			var id = getFieldFromRow("id", row);
			if (id != null) {
				tableObj.deleteEntry("id=" + id.value);
			}
		}
	}
}


/**
 * Shows the trigger values and times for a given triggered alarm in a table.
 */
function showTriggerValues() {
	WindowManager.instance().activateWindow('triggervalueswindow'); 
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var triggerValuesTable = TableManager.instance().getTable("triggervaluestable");
		
		if (row != null) {
			triggerValuesTable.clear();
			triggerValuesTable.setData(row);
			triggerValuesTable.updateHtmlTable();
		}
	}
}


/**
 * Opens up a defined alarm for editing, when it is clicked from the right-click menu in the table.
 */
function editDefinedAlarm() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var tableName = lastTableObject.id;
		var tableObj = TableManager.instance().getTable(tableName);
		
		// an alarm table was not found
		if (tableObj == null || tableName == null || tableName.indexOf("Alarm") < 0) {
			return;
		}

		if (row != null) {
			tableObj.editAlarm("id=" + getFieldFromRow("id", row).value);
		}
	}
}


/**
 * Opens up a defined alarm for copying, when it is clicked from the right-click menu in the table. 
 */
function copyDefinedAlarm() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var tableName = lastTableObject.id;
		var tableObj = TableManager.instance().getTable(tableName);
		
		// an alarm table was not found
		if (tableObj == null || tableName == null || tableName.indexOf("Alarm") < 0) {
			return;
		}

		if (row != null) {
			tableObj.copyAlarm("id=" + getFieldFromRow("id", row).value);
		}
	}
}


/**
 * Shows the edit alarm window.
 */
function showEditAlarmWindow() {
	disableEntryFields();
	var alarmWindow = WindowManager.instance().getWindow('alarmDetailWindow');
	alarmWindow.setWindowTitle(EDIT_ALARM_WINDOW_TITLE);
	WindowManager.instance().activateWindow(alarmDetailWindow().getId());
	clearAlarmMessage();
	hideSetAlarmButton();
	showEditAlarmButton();
	showEditAlarmActionsButton();
	hideSetAlarmActionButton();
}


/**
 * Shows the copy alarm window. 
 */
function showCopyAlarmWindow() {
	enableEntryFields();
	var alarmWindow = WindowManager.instance().getWindow('alarmDetailWindow');
	alarmWindow.setWindowTitle(COPY_ALARM_WINDOW_TITLE);
	WindowManager.instance().activateWindow(alarmDetailWindow().getId());
	clearAlarmMessage();
	hideEditAlarmButton();
	showSetAlarmButton();
	hideEditAlarmActionsButton();
	showSetAlarmActionsButton();
	getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActions", "name", "input").disabled = true;
}


/**
 * Shows the add alarm window.
 */
function showAddAlarmWindow() {
	enableEntryFields();
	var alarmWindow = WindowManager.instance().getWindow('alarmDetailWindow');
	alarmWindow.setWindowTitle(ADD_ALARM_WINDOW_TITLE);
	WindowManager.instance().activateWindow(alarmWindow.getId());
	alarmWindow.resized();
	clearAlarmWindow();
	hideEditAlarmButton();
	showSetAlarmButton();
	hideEditAlarmActionsButton();
	showSetAlarmActionsButton();
	getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActions", "name", "input").disabled = true;
	
	// a source element exists, thus fill out known fields in the form
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var tableName = lastTableObject.id;
		
		// set the table name
		setTableName(tableName);
		tableChosen();

		// set the field type, if available
		var fieldSelection = fieldChooser(tableName);
		for (var i = 0; i < fieldSelection.length; i++) {
			var fieldName = counterIdNames[tableNames[tableName]][fieldSelection[i].value];
			if (isDefined(fieldNameMapping[lastEntryFieldName]) && fieldNameMapping[lastEntryFieldName] == fieldName) {
				fieldSelection.selectedIndex = parseInt(fieldSelection.options[i].value) + 1;
				fieldChosen();
			}
		}
		
		if (row != null) {
			// set the port number, if available
			var port = getFieldFromRow("protocolId", row);
			if (port != null) {
				setPortFieldValue(port.getValue());
				
				// set the port type (TCP, UDP)
				setPortTypeFieldValue(portType(port.getValue()));
			}
			
			// set the ip address, if available
			var ipAddress = getFieldFromRow("address", row);
			if (ipAddress != null) {	
				setIpFieldValue(ipAddress.getValue());
			}
			ipAddress = getFieldFromRow("src", row);
			if (ipAddress != null) {	
				setIpFieldValue(ipAddress.getValue());
			}
			
			// set the src ip address, if available
			var srcAddress = getFieldFromRow("srcAddress", row);
			if (srcAddress != null) {
				setSrcIpFieldValue(srcAddress.getValue());
			}
			
			// set the dst ip address, if available
			var dstAddress = getFieldFromRow("dstAddress", row);
			if (dstAddress != null) {
				setDstIpFieldValue(dstAddress.getValue());
			}
		}
	}
}


/**
 * Shows a defined alarm table.
 */
function showDefinedAlarms() {
	// a source element exists, thus fill out known fields in the form
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var tableName = lastTableObject.id;
		switch (tableName) {
			case "summaryTable": 
				WindowManager.instance().activateWindow('summarydefinedalarmswindow');		
				break;
			case "topNProtocolsTable":
			case "myProtocolsTable":
			case "protocolDetailTable":
			case "myProtocolsPerHostTable":
				WindowManager.instance().activateWindow('protocoldefinedalarmswindow');
				break;
			case "topNHostsTable":
			case "nlhostDetailTable":
			case "topNHostsMatrix":
			case "topNHostTalkersTable":
			case "topNHostListenersTable":
			case "myHostsTable":
				WindowManager.instance().activateWindow('hostsdefinedalarmswindow');
				break;
			case "topNConversationsTable":
			case "myConversationsTable":
				WindowManager.instance().activateWindow('conversationsdefinedalarmswindow');
				break;
			case "protocolTopHostsTable":
			case "protocolTopConversationsTable":
			case "nlhostTopConversationsTable":
			case "nlhostTopProtocolsTable":
				WindowManager.instance().activateWindow('alldefinedalarmswindow');
				break;
			default:
				alert("No such table: " + tableName);
		}
	}
}


/**
 * Shows a triggered alarm table.
 */
function showTriggeredAlarms() {
	// a source element exists, thus fill out known fields in the form
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var tableName = lastTableObject.id;
		switch (tableName) {
			case "summaryTable": 
				WindowManager.instance().activateWindow('summarytriggeredalarmswindow');		
				break;
			case "topNProtocolsTable":
			case "myProtocolsTable":
			case "protocolDetailTable":
			case "myProtocolsPerHostTable":
				WindowManager.instance().activateWindow('protocoltriggeredalarmswindow');
				break;
			case "topNHostsTable":
			case "nlhostDetailTable":
			case "topNHostsMatrix":
			case "topNHostTalkersTable":
			case "topNHostListenersTable":
			case "myHostsTable":
				WindowManager.instance().activateWindow('hoststriggeredalarmswindow');
				break;
			case "topNConversationsTable":
			case "myConversationsTable":
				WindowManager.instance().activateWindow('conversationstriggeredalarmswindow');
				break;
			case "protocolTopHostsTable":
			case "nlhostTopConversationsTable":
			case "nlhostTopProtocolsTable":
			case "protocolTopConversationsTable":
				WindowManager.instance().activateWindow('alltriggeredalarmswindow');
				break;
			default:
				alert("No such table: " + tableName);
		}
	}
}


/**
 * Extracts the port type (TCP, UDP, Ethernet).
 * @param ianaPortNumber The port number specified on with a dotted notation: 1.2048.x.x.
 * @return The string representation of the port type: "ethernet", "ip", "tcp", "udp". 
 *  	   If neither of these were found, return "tcp" as default.
 */
function portType(ianaPortNumber) {
	// TCP
	if (ianaPortNumber.indexOf("1.2048.6.") > -1) {
		return "tcp";
	}
	
	// UDP
	if (ianaPortNumber.indexOf("1.2048.17.") > -1) {
		return "udp";
	}

	// Ethernet
	if (ianaPortNumber.indexOf("1.2048.") > -1) {
		return "ethernet";
	}
	
	// IP
	if (ianaPortNumber.indexOf("1.") > -1) {
		return "ip";
	}
	
	// return TCP as default, if neither Ethernet, IP, TCP or UDP were found
	return "tcp";
}


/**
 * Extracts the port number.
 * @param ianaPortNumber The port number specified with a dotted notation: 1.2048.x.x.
 */
function portNumber(ianaPortNumber) {
	var offset = ianaPortNumber.lastIndexOf(".");
	return ianaPortNumber.substring(parseInt(offset) + 1, ianaPortNumber.length);
}


/**
 * Looks up the the field chooser selection list for the specified table.
 * @param tableName The name of the table (summaryTable, topNProtocolsTable,...)
 */
function fieldChooser(tableName) {
	return getElementWithAttribute(alarmDetailWindow().getHtmlCode(), (tableNames[tableName] + "FieldChooser"), "name", "select");
}


/**
 * Clears the alarm detail window.
 */ 
function clearAlarmWindow() {
	var alarmWindow = WindowManager.instance().getWindow('alarmDetailWindow');
	var tableChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "tableChooser", "name", "select");
	selectInSelector(tableChooser, "");
	deactivateEntryFields();
	deactivateFieldChoosers();
	showFieldChooser("");
	
	setAlarmType("true");
	resetPortField();
	resetPortSelect();
	resetIpField();
	resetSrcIpField();
	resetDstIpField();
	resetMacField();
	resetSrcMacField();
	resetDstMacField();
	setTriggerType("upper");
	setUpperTriggerValue("");
	setLowerTriggerValue("");
	setVisibilityFieldValue("0");
	setSeverityFieldValue("0");
	setDescriptionFieldValue("[Enter a description]");
	setSamplingTime("");
	clearAlarmMessage();
}


/**
 * Enter has been pressed in one of the input fields. Set or edit the alarm.
 */
function enterPressed() {
	if (alarmDetailWindow().getWindowTitle() == EDIT_ALARM_WINDOW_TITLE) {
		editAlarm();		
	}
	else {
		setAlarm();
	}
}


/**
 * Sets a alarm.
 */
function setAlarm() {
	clearAlarmMessage();
	var url = createSetAlarmUrl();
	if (!isNull(url)) {
		loadXML(url, alarmSet);
	}
}


/**
 * Edits a alarm.
 */
function editAlarm() {
	var url = createEditAlarmUrl();
	if (!isNull(url)) {
		loadXML(url, alarmEdited);
	}
}


/**
 * A alarm has been edited.
 * @param response The xml response from the server.
 */
function alarmEdited(response) {
	if (containsNoAccessMessage(response)) {
		setMessage(undefined, "No access");
	}
	else if (containsErrorMessage(response)) {
		setMessage(undefined, getErrorMessage(response));
	}	
	else {
		setMessage("Successfully edited alarm", undefined);
		fireAlarmEditedEvent();
	}
}


/**
 * A alarm has been set. 
 * @param response The xml response.
 */
function alarmSet(response) {
	if (containsNoAccessMessage(response)) {
		setMessage(undefined, "No access");
	}
	else if (containsErrorMessage(response)) {
		setMessage(undefined, getErrorMessage(response));
	}	
	else {
		getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActions", "name", "input").disabled = false;
		setMessage("Successfully set alarm", undefined);
		fireAlarmSetEvent();
	}
	WindowManager.instance().getWindow("alarmactionswindow").setAlarmActionId(response);
}


/**
 * Returns the url to edit a alarm based on the values set in the alarm detail window, 
 * null if the url could not be created.
 */
function createEditAlarmUrl() {
	try {
		return EDIT_ALARM_BASE_URL + createAlarmParameterUrl() + "&id=" + getIdFieldValue();
	}
	catch (e) {
		setMessage(undefined, e.message);
		return null;
	}
}


/**
 * Returns the url to add a alarm based on the values set in the alarm detail window, 
 * null if the url could not be created.
 */
function createSetAlarmUrl() {
	try {
		return SET_ALARM_BASE_URL + createAlarmParameterUrl();
	}
	catch (e) {
		setMessage(undefined, e.message);
		return null;
	}
}


/**
 * Returns the part of a url with the alarm paramters set, based on the values set
 * in the alarm detail window.
 */
function createAlarmParameterUrl() {
	if (isEmpty(selectedTableName())) {
		throw new Error("No table selected");
		return null;
	}
	
	var url = "&table=" + selectedTableName();	
	if (isEmpty(selectedFieldValue())) {
		throw new Error("No field selected");
	}
	url += "&counterId=" + selectedFieldValue();
	
	url += entryUrlCreators[selectedTableName()]();
	url += alarmTypeUrl();
	url += triggerUrl();
	
	url += "&visibility=" + visibilityFieldValue();
	url += "&severity=" + severityFieldValue();
	url += descriptionUrl();
	return url;
}


/**
 * Returns the entry part of a add alarm url for the summary table.
 */
function summaryUrl() {
	return "&src=" + ipFieldValue();
}


/**
 * Returns the entry part of a add alarm url for the network table.
 */
function networkUrl() {
	return "&src=" + getNetworkEntryChooserValue();
}


/**
 * Returns the entry part of a add alarm url for the network card conversations table.
 */
function networkCardConversationUrl() {
	assertValidSrcMacSet();
	assertValidDstMacSet();	
	return "&src=" + srcMacFieldValue() + "&dst=" + dstMacFieldValue();
}


/**
 * Returns the entry part of a add alarm url for the network card table.
 */
function networkCardsUrl() {
	assertValidMacSet();
	return "&src=" + macFieldValue();
}


/**
 * Returns the entry part of a add alarm url for the protocols per conversatino table.
 */
function protocolsPerConversationUrl() {
	assertValidPortSet();
	assertValidSrcIpSet();
	assertValidDstIpSet();
	
	return "&pid=" + portFieldValue() + "&src=" + srcIpFieldValue() + "&dst=" + dstIpFieldValue();
}


/**
 * Returns the entry part of a add alarm url for the conversations table.
 */
function conversationsUrl() {
	assertValidSrcIpSet();
	assertValidDstIpSet();
	return "&src=" + srcIpFieldValue() + "&dst=" + dstIpFieldValue();
}


/**
 * Returns the entry part of a add alarm url for the host table.
 */
function hostEntryUrl() {
	assertValidIpSet();
	return "&src=" + ipFieldValue();
}


/**
 * Returns the entry part of a add alarm url, for the protocols per conversation table.
 */
function protocolPerHostEntryUrl() {
	assertValidPortSet();
	assertValidIpSet();
	return "&pid=" + portFieldValue() + "&src=" + ipFieldValue();
}


/**
 * Returns the entry part of a add alarm url, for the protocol table.
 */
function protocolEntryUrl() {
	assertValidPortSet();
	return "&pid=" + portFieldValue();
}


/**
 * Returns the alarm type part of a add alarm url (Delta alarm, absolute alarm).
 */
function alarmTypeUrl() {
	var url = "";
	var absoluteAlarm = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "absoluteAlarm", "value", "input");
	if (absoluteAlarm.checked) {
		url += "&absoluteAlarm=true";
	}
	else if (!absoluteAlarm.checked && !isValidSamplingTime()) {
		throw new Error("Invalid sampling time set");
	}
	else {
		var samplingTime = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTime", "name", "input");
		url += "&samplingTime=" + samplingTime.value;
	}	
	return url;
}


/**
 * Returns the trigger part of a add alarm url.
 */
function triggerUrl() {
	var url = "";
	if (!isUpperTriggerSet() && !isLowerTriggerSet()) {
		throw new Error("No trigger value set");
	}
	if (isTriggerTypeUpper() && !isUpperTriggerSet()) {
		throw new Error("Upper trigger not set");
	}
	if (isTriggerTypeLower() && !isLowerTriggerSet()) {
		throw new Error("Lower trigger not set");
	}
	if (isUpperTriggerSet() && isLowerTriggerSet()) {
		assertUpperTriggerLargerThanLower();
	}
	
	url += "&triggerType=" + triggerType();
	if (isUpperTriggerSet()) {
		assertUpperTriggerValid();
		url += "&upperTriggerValue=" + upperTriggerValue();
	}
	
	if (isLowerTriggerSet()) {
		assertLowerTriggerValid();
		url += "&lowerTriggerValue=" + lowerTriggerValue();
	}
	
	url += "&upperUnit=" + selectedUpperTriggerUnit();
	url += "&lowerUnit=" + selectedLowerTriggerUnit();
	return url;
}


/**
 * Returns the description part of a add/edit alarm url.
 */
function descriptionUrl() {
	var url = "";
	if (!isValidDescription()) {
		throw new Error("; is not allowed in the description");
	}
	if (DEFAULT_DESCRIPTION == descriptionFieldValue()) {
		url += "&description=";
	}
	else {
		url += "&description=" + encodeURIComponent(descriptionFieldValue());
	}
	return url;
}


/**
 * Throws a Error if the lower trigger value is larger than the upper.
 */
function assertUpperTriggerLargerThanLower() {
	if(!isDelta()) {
		// if absolute alarm, only the upper trigger value is used
		return;
	}
	
	var upperUnit = AlarmUnit.getInstance(selectedUpperTriggerUnit());
	var lowerUnit = AlarmUnit.getInstance(selectedLowerTriggerUnit());
	if (upperUnit.isByteType() && !lowerUnit.isByteType()) {
		throw new Error("Invalid lower trigger unit set");
	}
	if (upperUnit.isPerSecond() && !lowerUnit.isPerSecond()) {
		throw new Error("Invalid lower trigger unit set");
	}
	if (!upperUnit.isPerSecond() && lowerUnit.isPerSecond()) {
		throw new Error("Invalid lower trigger unit set");
	}
	if (!isNumber(upperTriggerValue())) {
		throw new Error("Upper trigger value set is not a number");
	}
	if (!isNumber(lowerTriggerValue())) {
		throw new Error("Lower trigger value set is not a number");
	}
	if (upperUnit.isByteType() && lowerUnit.isByteType() && (upperUnit.toBytes(upperTriggerValue()) < lowerUnit.toBytes(lowerTriggerValue()))) {
		throw new Error("Lower trigger larger than upper trigger");
	}
	else if (upperUnit.isByteType() && lowerUnit.isByteType() && (upperUnit.toBytes(upperTriggerValue()) == lowerUnit.toBytes(lowerTriggerValue()))) {
		throw new Error("Upper and lower trigger values are equal");
	}
	else if (!upperUnit.isByteType() && !lowerUnit.isByteType() && upperTriggerValue() < lowerTriggerValue()) {
		throw new Error("Lower trigger larger than upper trigger");
	}
	else if (!upperUnit.isByteType() && !lowerUnit.isByteType() && upperTriggerValue() == lowerTriggerValue()) {
		throw new Error("Upper and lower trigger values are equal");
	}
}


/**
 * Gets the trigger type currently selected in the alarmdetail window (LOWER, UPPER).
 */
function triggerType() {
	var upperTriggerCheckBox = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerType", "value", "input");	
	if (upperTriggerCheckBox.checked == true) {
		return UPPER;
	}
	else {
		return LOWER;	
	}
}


/**
 * Throws a error if the source ip field is empty or contains a invalid ip address.
 */ 
function assertValidSrcIpSet() {
	if (isEmpty(srcIpFieldValue())) {
		throw new Error("No source ip address set");
	}
	if (!validIpNumbers(srcIpFieldValue())) {
		throw new Error("Invalid number in source ip address. Valid is 0-255");
	}
	if (!isValidIpAddress(srcIpFieldValue())) {
		throw new Error("Invalid source ip address set. Valid format is xxx.xxx.xxx.xxx");
	}	
}


/**
 * Throws a error if the destination ip field is empty or contains a invalid ip address.
 */
function assertValidDstIpSet() {
	if (isEmpty(dstIpFieldValue())) {
		throw new Error("No destination ip address set");
	}
	if (!validIpNumbers(dstIpFieldValue())) {
		throw new Error("Invalid number in destination ip address. Valid is 0-255");
	}
	if (!isValidIpAddress(dstIpFieldValue())) {
		throw new Error("Invalid destination ip address set. Valid format is xxx.xxx.xxx.xxx");
	}
}


/**
 * Throws a error if the ip field is empty or contains invalid ip address.
 */
function assertValidIpSet() {
	if (isEmpty(ipFieldValue())) {
		throw new Error("No ip address set");
	}
	if (!validIpNumbers(ipFieldValue())) {
		throw new Error("Invalid number in ip address. Valid is 0-255");
	}
	if (!isValidIpAddress(ipFieldValue())) {
		throw new Error("Invalid ip address set. Valid format is xxx.xxx.xxx.xxx");
	}	
}


/**
 * Throws a error if the port field is empty or invalid.
 */
function assertValidPortSet() {
	if (isEmpty(portFieldValue())) {
		throw new Error("No port set");
	}
	if (!isEmpty(portFieldValue()) && !validPortNumbers(portFieldValue())) {
		throw new Error("Invalid port number in port. Valid is 0-65535");
	}
	if (!isValidPortIdString(portFieldValue())) {
		throw new Error("Invalid port set");
	}
}


/**
 * Throws a error if the mac address field is empty or invalid. 
 */
function assertValidMacSet() {
	if (isEmpty(macFieldValue())) {
		throw new Error("No mac address set");
	}
	if (!validMacNumbers(macFieldValue())) {
		throw new Error("Invalid number in mac address. Valid is 0-99 and a-f");
	}
	if (!isValidMac(macFieldValue())) {
		throw new Error("Invalid mac address set. Valid format is xx:xx:xx:xx:xx:xx");
	}
}	


/**
 * Throws a error if the source mac address field is empty or invalid. 
 */
function assertValidSrcMacSet() {
	if (isEmpty(srcMacFieldValue())) {
		throw new Error("No source mac address set");
	}
	if (!validMacNumbers(srcMacFieldValue())) {
		throw new Error("Invalid number in source mac address. Valid is 0-99 and a-f");
	}
	if (!isValidMac(srcMacFieldValue())) {
		throw new Error("Invalid source mac address set. Valid format is xx:xx:xx:xx:xx:xx");
	}
}


/**
 * Throws a error if the destination mac address field is empty or invalid. 
 */
function assertValidDstMacSet() {
	if (isEmpty(dstMacFieldValue())) {
		throw new Error("No destination mac address set");
	}
	if (!validMacNumbers(dstMacFieldValue())) {
		throw new Error("Invalid number in destination mac address. Valid is 0-99 and a-f");
	}
	if (!isValidMac(dstMacFieldValue())) {
		throw new Error("Invalid destination mac address set. Valid format is xx:xx:xx:xx:xx:xx");
	}
}


/**
 * Show the edit alarm button.
 */
function showEditAlarmButton() {
	var editAlarmButtonContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmButtonContainer", "name", "div");
	setClass(editAlarmButtonContainer, "active");
	var editAlarmButton = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmButton", "name", "input");
	setClass(editAlarmButton, "active");
}


/**
 * Hide the edit alarm button.
 */ 
function hideEditAlarmButton() {
	var editAlarmButtonContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmButtonContainer", "name", "div");
	setClass(editAlarmButtonContainer, "inactive");
	var editAlarmButton = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmButton", "name", "input");
	setClass(editAlarmButton, "inactive");
}


/**
 * Show the set alarm button.
 */
function showSetAlarmButton() {
	var setAlarmButtonContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmButtonContainer", "name", "div");
	setClass(setAlarmButtonContainer, "active");
	var setAlarmButton = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmButton", "name", "input");
	setClass(setAlarmButton, "active");
}


/**
 * Hides the set alarm button.
 */
function hideSetAlarmButton() {
	var setAlarmButtonContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmButtonContainer", "name", "div");
	setClass(setAlarmButtonContainer, "inactive");
	var setAlarmButton = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmButton", "name", "input");
	setClass(setAlarmButton, "inactive");
}


/**
 * Shows the set alarm action button.
 */
function showSetAlarmActionsButton() {
	var setAlarmActionsContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActionsContainer", "name", "div");
	setClass(setAlarmActionsContainer, "active");
	var setAlarmActions = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActions", "name", "input");
	setClass(setAlarmActions, "active");
}


/**
 * Hide the set alarm action button.
 */
function hideSetAlarmActionButton() {
	var setAlarmActionsContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActionsContainer", "name", "div");
	setClass(setAlarmActionsContainer, "inactive");
	var setAlarmActions = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "setAlarmActions", "name", "input");
	setClass(setAlarmActions, "inactive");
}


/**
 * Shows the edit alarm action button.
 */
function showEditAlarmActionsButton() {
	var editAlarmActionsContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmActionsContainer", "name", "div");
	setClass(editAlarmActionsContainer, "active");
	var editAlarmActions = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmActions", "name", "input");
	setClass(editAlarmActions, "active");
}


/**
 * Hides the edit alarm action button.
 */
function hideEditAlarmActionsButton() {
	var editAlarmActionsContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmActionsContainer", "name", "div");
	setClass(editAlarmActionsContainer, "inactive");
	var editAlarmActions = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "editAlarmActions", "name", "input");
	setClass(editAlarmActions, "inactive");
}


/**
 * Gets the selected lower trigger unit value.
 */
function selectedLowerTriggerUnit() {
	var unitName = unitChooserName();
	if (isEmpty(unitName)) {
		return "Entries";
	}
	var lowerTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTrigger", "name", "tr");
	var unitChooser = getElementWithAttribute(lowerTrigger, unitName, "name", "select");
	return unitChooser.options[unitChooser.selectedIndex].value;
}


/**
 * Gets the selected upper trigger unit value.
 */
function selectedUpperTriggerUnit() {
	var unitName = unitChooserName();
	if (isEmpty(unitName)) {
		return "Entries";
	}
	var upperTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTrigger", "name", "tr");
	var unitChooser = getElementWithAttribute(upperTrigger, unitName, "name", "select");
	return unitChooser.options[unitChooser.selectedIndex].value;
}


/**
 * Sets the sampling time field value.
 * @param time The sampling time to set.
 * @throws Error if time is undefined or null.
 */
function setSamplingTime(time) {
	assertExists(time, "Sampling time to set is undefined or null");
	var samplingTime = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTime", "name", "input");
	samplingTime.value = time;
}


/**
 * Returns true if the sampling time value set is valid, false otherwise.
 */
function isValidSamplingTime() {
	var samplingTime = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTime", "name", "input");
	if (isIntString(samplingTime.value) && !isNegative(parseInt(samplingTime.value))) {
		return true;
	}
	return false;
}
 

/**
 * Sets a alarm type.
 * @param absolute True to set a absolute alarm, false to set a delta alarm.
 */
function setAlarmType(absolute) {
	if ("true".equalsIgnoreCase(absolute)) {
		var absoluteAlarmRadioButton = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "absoluteAlarm", "value", "input");
		absoluteAlarmRadioButton.checked = true;
		absoluteAlarm();
		return;
	}
	else if ("false".equalsIgnoreCase(absolute)) {
		var deltaAlarmRadioButton = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "deltaAlarm", "value", "input");
		deltaAlarmRadioButton.checked = true;
		deltaAlarm();
	}
}


/**
 * Sets the id field value. 
 * @param value The value to set for the id field.
 * @throws Error if value is undefined or null.
 */
function setIdFieldValue(value) {
	assertExists(value, "Value to set for the alarm id field does not exist");
	var alarmId = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmId", "name", "input");
	alarmId.value = value;
}


/**
 * Get the value of the alarm id field.
 */
function getIdFieldValue() {
	var alarmId = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmId", "name", "input");
	return alarmId.value;
}

 
/**
 * Resets the dst mac field value.
 */
function resetDstMacField() {
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");
	dstMac.maxLength = "23";
	dstMac.size = "23";
	dstMac.value = "[Enter dst mac address]";
}


/**
 * Sets the value of the dst mac field value.
 * @param value The value to set. 
 * @throws Error if the value is undefined or null.
 */
function setDstMacFieldValue(value) {
	assertExists(value, "The value to set for the destination mac address field is undefined or null");
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");
	dstMac.value = value;
}


/**
 * Returns the value of the destination mac address field.
 */
function dstMacFieldValue() {
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");
	return dstMac.value;
}


/**
 * The dst mac field has been focused, remove the default text. 
 */
function dstMacFieldFocused() {
	if(dstMacFieldValue() == '[Enter dst mac address]') {
		setDstMacFieldValue('');
	} 
	this.maxLength = '17';
}


/**
 * Reset the src mac field.
 */
function resetSrcMacField() {
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	srcMac.maxLength = "23";
	srcMac.size = "23";
	srcMac.value = "[Enter src mac address]";
}


/**
 * Sets the value of the src mac address field
 * @param value The value to set. 
 * @throws Error if the value is undefined or null.
 */
function setSrcMacFieldValue(value) {
	assertExists(value, "The value to set for the source mac address field is undefined or null");
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	srcMac.value = value;
}


/**
 * Returns the value of the source mac address field.
 */
function srcMacFieldValue() {
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	return srcMac.value;
}


/**
 * The source mac field has been focused, remove the default text.
 */
function srcMacFieldFocused() {
	if (srcMacFieldValue() == '[Enter src mac address]') { 
		setSrcMacFieldValue('');
	} 
	this.maxLength = '17';
}


/**
 * Resets the mac field.
 */
function resetMacField() {
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	mac.maxLength = "19";
	mac.size = "19";
	mac.value = "[Enter mac address]";
}


/**
 * Sets the value of the mac address field.
 * @param value The value to set. 
 * @throws Error if value is undefined or null.
 */
function setMacFieldValue(value) {
	assertExists(value, "The value to set for the mac address field is undefined or null");
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	mac.value = value;
}


/**
 * Returns the value of the mac address field.
 */
function macFieldValue() {
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	return mac.value;
}


/**
 * The mac field has been focused, remove the default text. 
 */
function macFieldFocused() {
	if (macFieldValue() == '[Enter mac address]') {
		setMacFieldValue('');
	} 
	this.maxLength = '17';
}


/**
 * Resets the src ip field.
 */
function resetSrcIpField() {
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	src.maxLength = "22";
	src.size = "22";
	src.value = "[Enter src ip address]";
}


/**
 * Set the value of the src ip address field.
 * @param value The value to set.
 * @throws Error if value is undefined or null.
 */
function setSrcIpFieldValue(value) {
	assertExists(value, "The value to set for the source ip field is undefined or null");
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	src.value = value;
}


/**
 * Returns the value if the ip field.
 */
function srcIpFieldValue() {
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	return src.value;
}


/**
 * The source ip field has been focused, remove the default text. 
 */
function srcIpFieldFocused() {
	if (srcIpFieldValue() == '[Enter src ip address]') { 
		setSrcIpFieldValue('');
	} 
	this.maxLength = '15';
}


/**
 * Resets the dst ip address field.
 */
function resetDstIpField() {
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	dst.maxLength = "22";
	dst.size = "22";
	dst.value = "[Enter dst ip address]";
}


/**
 * Sets the value of the dst ip address field.
 * @param value The value to set.
 * @throws Error if value is undefined or null.
 */
function setDstIpFieldValue(value) {
	assertExists(value, "The value to set for the destination ip field is undefined or null");
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	dst.value = value;
}


/**
 * Returns the value of the destination ip address field.
 */
function dstIpFieldValue() {
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	return dst.value;	
}


/**
 * The destination ip field has been focused, remove the default text. 
 */
function dstIpFieldFocused() {
	if (dstIpFieldValue() == '[Enter dst ip address]') {
		setDstIpFieldValue('');
	} 
	this.maxLength = '15';
}


/**
 * Resets the ip field value.
 */
function resetIpField() {
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	ip.maxLength = "18";
	ip.size = "18";
	ip.value = "[Enter ip address]";
}


/**
 * Sets the value of the ip field.
 * @param value The ip address to set.
 * @throws Error if value is undefined or null.
 */
function setIpFieldValue(value) {
	assertExists(value, "The value to set for the ip field is undefined or null");
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	ip.value = value;
}


/**
 * Gets the value of the ip field.
 */
function ipFieldValue() {
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	return ip.value;
}


/**
 * The ip field has been focused, remove the default text.
 */
function ipFieldFocused() {
	if (ipFieldValue() == '[Enter ip address]') {
		setIpFieldValue('');
	} 
	this.maxLength = '15';
}


/**
 * Set the value of the visibility field.
 * @param visibility The value of the visibility option to set.
 * @throws Errror if visibility is null or undefined. 
 */
function setVisibilityFieldValue(visibility) {
	assertExists(visibility, "The visibility to set is undefined or null");
	var visibilityField = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "visibility", "name", "select");
	visibilityField.value = visibility;
}


/**
 * Get the value of the visibility field.
 */
function visibilityFieldValue() {
	var visibility = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "visibility", "name", "select");
	return visibility.options[visibility.selectedIndex].value;
}


/**
 * Set the severity field value.
 * @param severity The value of the severity option to set.
 * @throws Error if severity is undefined or null.
 */
function setSeverityFieldValue(severity) {
	assertExists(severity, "The severity to set is undefined or null");
	var severitySelect = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "severity", "name", "select");
	severitySelect.value = severity;
}


/**
 * Get the value of the severity field.
 */
function severityFieldValue() {
	var severity = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "severity", "name", "select");
	return severity.options[severity.selectedIndex].value;
}


/**
 * Returns true if the description is valid, false otherwise. 
 */
function isValidDescription() {
	if (descriptionFieldValue().indexOf(";") > -1) {
		return false;
	}
	return true;
}


/**
 * The description field has been focused, remove the default text.
 */
function descriptionFieldFocused() {
	if (descriptionFieldValue() == '[Enter a description]') {
		setDescriptionFieldValue("");
	}
}


/**
 * Set the description field value.
 * @param description The description to set. 
 * @throws Error if description is undefined or null.
 */
function setDescriptionFieldValue(description) {
	assertExists(description, "The description to set is undefined or null");
	var descriptionField = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "description", "name", "input");
	descriptionField.value = description;
}


/**
 * Get the value of the description field.
 */
function descriptionFieldValue() {
	var description = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "description", "name", "input");
	return URLDecode(description.value);
}


/**
 * Resets the port field.
 */
function resetPortField() {
	setPortFieldValue("[Enter port]");
}


/**
 * Resets the port select tag.
 */
function resetPortSelect() {
	setPortTypeFieldValue("tcp");
}


/**
 * Sets the value of the port field.
 * @param value The value to set. 
 * @throws Error if value is undefined or null.
 */
function setPortFieldValue(value) {
	assertExists(value, "The value to set for the port field is undefined or null");
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	port.value = value;
}


/**
 * The port input field has been focused, remove the default input.
 */
function alarmPortfieldFocused() {
	if (portFieldValue() == '[Enter port]') {
		setSelectedPortNumber();
	} 
	tt_Hide();
}


/**
 * Reset the network entry chooser.
 */
function resetNetworkEntryChooser() {
	// select the All packets choice
	setNetworkEntryChooserValue("1");	
}


/**
 * Set the value of the network entry chooser. 
 * @param value The value to set. 
 * @throws Error if the value to set is undefined or null.
 */
function setNetworkEntryChooserValue(value) {
	assertExists(value, "The value to set for the port type field is undefined or null");
	var networkEntryChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "networkEntryChooser", "name", "select");
	selectInSelector(networkEntryChooser, value);
}


/**
 * Get the value of the network entry chooser. 
 */
function getNetworkEntryChooserValue() {
	var networkEntryChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "networkEntryChooser", "name", "select");
	return networkEntryChooser.value;
}


/**
 * Sets the value of the port type field (TCP, UDP).
 * @param value The value to set.
 */
function setPortTypeFieldValue(value) {
	assertExists(value, "The value to set for the port type field is undefined or null");
	var portTypeSelection = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolTypeFieldChooser", "name", "select");
	selectInSelector(portTypeSelection, value);
}


/**
 * Get the value of the port field.
 */
function portFieldValue() {
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	return port.value;
}

 
/**
 * Returns the html code of the table chooser.
 */
function tableChooser() {
	return getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "tableChooser", "name", "select");
}


/**
 * Set the mesage field.
 * @param successMessage The success message to set. 
 * @param errorMessage The error message to set. 
 */
function setMessage(sucessMessage, errorMessage) {
	if (isDefined(sucessMessage) && !isEmpty(sucessMessage)) {
		setClass(getAlarmMessageContainer(), "ok");
		getAlarmMessageContainer().innerHTML = "<nobr>" + sucessMessage + "</nobr>";
	}
	else if (isDefined(errorMessage) && !isEmpty(errorMessage)) {
		setClass(getAlarmMessageContainer(), "error");
		getAlarmMessageContainer().innerHTML = "<nobr>" + errorMessage + "</nobr>";
	}
}



/**
 * Clears the error message.
 */
function clearAlarmMessage() {
	// decrease the window and hide the message
	var height = getAlarmMessageContainer().offsetHeight;
	getAlarmMessageContainer().innerHTML = "";
}


/**
 * Returns the html code of the container for alarm messages in the 
 * add alarm window.
 */
function getAlarmMessageContainer() {
	return getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmMessage", "name", "div");
}


/**
 * A table has been chosen, show the appropriate entry fields.
 */
function tableChosen() {
	clearAlarmMessage();
	var selectedTable = selectedTableName();
	deactivateEntryFields();
	deactivateFieldChoosers();
	hideTriggerUnitChoosers();
	showFieldChooser(selectedTable);
	if (selectedTable == "") {
		return;
	}
	
	if (selectedTable == "protocol") {
		showProtocolEntryFields();
		resetPortField();
		resetPortSelect();
	}
	else if (selectedTable == "nlhost") {
		showNlHostEntryFields();
		resetIpField()
	}
	else if (selectedTable == "nlmatrix") {
		showConversationEntryFields();
		resetSrcIpField();
		resetDstIpField();
	}	
	else if (selectedTable == "alhost") {
		showProtocolsPerHostEntryFields();
		resetPortField();
		resetIpField();
		resetPortSelect();
	}
	else if (selectedTable == "almatrix") {
		showProtocolsPerConversationEntryFields();
		resetPortField();
		resetSrcIpField();
		resetDstIpField();
		resetPortSelect();
	}
	else if (selectedTable == "host") {
		showHostEntryFields();
		resetMacField();
	}
	else if (selectedTable == "matrix") {
		showNetworkCardConversationFields();
		resetSrcMacField();
		resetDstMacField();
	}
	else if (selectedTable == "network") {
		showNetworkFields();	
		resetNetworkEntryChooser();
	}
	else if (selectedTable == "summary") {
		showSummaryFields();
	}
}


/**
 * Shows the network entry fields. 
 */
function showNetworkFields() {
	var networkEntryChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "networkEntryChooser", "name", "select");
	setClass(networkEntryChooser, "alarmEntrySelect");
}


/**
 * Shows the summary entry fields.
 */
function showSummaryFields() {
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	setClass(ip, "alarmEntryElement");
	ip.value = getServerAddress();
}


/**
 * Shows the network card conversation entry fields.
 */
function showNetworkCardConversationFields() {
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");
	setClass(srcMac, "alarmEntryElement");
	setClass(dstMac, "alarmEntryElement");
}


/**
 * Shows the host entry fields.
 */
function showHostEntryFields() {
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	setClass(mac, "alarmEntryElement");
}


/**
 * Shows the protocol per conversation entry fields.
 */
function showProtocolsPerConversationEntryFields() {
	showProtocolEntryFields();
	showConversationEntryFields();
}


/**
 * Shows the protocol per host entry fields.
 */
function showProtocolsPerHostEntryFields() {
	showProtocolEntryFields();
	showNlHostEntryFields();
}


/**
 * Shows the conversations entry fields.
 */
function showConversationEntryFields() {
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	setClass(src, "alarmEntryElement");
	setClass(dst, "alarmEntryElement");
}


/**
 * Shows the nlhost entry fields.
 */
function showNlHostEntryFields() {
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	setClass(ip, "alarmEntryElement");
}


/**
 * Shows the protocol entry fields.
 */
function showProtocolEntryFields() {
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	var protocolTypeFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolTypeFieldChooser", "name", "select");
	setClass(port, "alarmEntryElement");
	var protocolTypeClass = isIE() ? "alarmEntrySelectPadding" : "alarmEntrySelect";
	setClass(protocolTypeFieldChooser, protocolTypeClass);
}


/**
 * Deactivates all the entry fields.
 */
function deactivateEntryFields() {
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	var protocolTypeFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolTypeFieldChooser", "name", "select");
	setClass(port, "inactive");
	setClass(protocolTypeFieldChooser, "inactive");
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	setClass(ip, "inactive");
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	setClass(src, "inactive");
	setClass(dst, "inactive");
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	setClass(mac, "inactive");
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");
	setClass(srcMac, "inactive");
	setClass(dstMac, "inactive");
	var networkEntryChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "networkEntryChooser", "name", "select");
	setClass(networkEntryChooser, "inactive");
}


/**
 * Sets the selected port number into the port input field.
 */
function setSelectedPortNumber() {
	var protocolTypeFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolTypeFieldChooser", "name", "select");
	setPortNumber(protocolTypeFieldChooser.value);
}


/**
 * Show the field chooser for a table.
 * @param tableName The name of the table to show the field chooser for.
 */
function showFieldChooser(tableName) {
	var fieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), tableName + "FieldChooser", "name", "select");
	if (!exists(fieldChooser)) {
		// show the default field chooser
		fieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "defaultFieldChooser", "name", "select");
	}
	setClass(fieldChooser, isIE() ? "fieldChooserIE" : "fieldChooser");
}


/**
 * Gets the name of the selected table, a empty string if no table has been selected.
 */
function selectedTableName() {
	return tableChooser().options[tableChooser().selectedIndex].value;
}


/**
 * Deactivates all the field choosers.
 */
function deactivateFieldChoosers() {
	for (table in entryFieldNames) {
		var fieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), table + "FieldChooser", "name", "select");
		selectInSelector(fieldChooser, "");
		setClass(fieldChooser, "inactive");
	}
	// deactivate the default field chooser
	var defaultFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "defaultFieldChooser", "name", "select");
	setClass(defaultFieldChooser, "inactive");
	
	if (isIE()) {
		// we need to reset the select to show or hide for the window
		alarmDetailWindow().initSelects();
	}
}


/**
 * A field has been chosen.
 */
function fieldChosen() {
	showTriggerUnitChooser();
}


/**
 * Returns the selected field, null if no table is selected.
 */
function selectedFieldValue() {
	var tableName = selectedTableName();
	if (!exists(tableName) || isEmpty(tableName)) {
		// no table selected
		return null;
	}	
	var fieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), tableName + "FieldChooser", "name", "select");
	return fieldChooser.options[fieldChooser.selectedIndex].value;
}


/**
 * Returns the selected fields text, null if no table is selected.
 */
function selectedFieldText() {
	var tableName = selectedTableName();
	if (!exists(tableName) || isEmpty(tableName)) {
		// no table selected
		return null;
	}	
	var fieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), tableName + "FieldChooser", "name", "select");
	return fieldChooser.options[fieldChooser.selectedIndex].text;
}


/**
 * Shows the trigger unit chooser for the selected field.
 */
function showTriggerUnitChooser() {
	hideTriggerUnitChoosers();
	var unitName = unitChooserName();
	if (isEmpty(unitName)) {
		return;
	}
	
	var upperTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTrigger", "name", "tr");
	var unitChooser = getElementWithAttribute(upperTrigger, unitName, "name", "select");
	setClass(unitChooser, "active");
	resetUnitChooser(unitChooser);
	
	if (isDelta()) {
		var lowerTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTrigger", "name", "tr");
		unitChooser = getElementWithAttribute(lowerTrigger, unitName, "name", "select");
		setClass(unitChooser, "active");
		resetUnitChooser(unitChooser);
	}
}


/**
 * Resets a unit choosers selected index. 
 * @param unitSelect The select tag to reset the index in. 
 */
function resetUnitChooser(unitSelect) {
	selectInSelector(unitSelect, UNIT_SELECT_DEFAULT_SELECTED[unitSelect.getAttribute("name")]);
}


/**
 * Gets the unit chooser select tag to use, based on the field and delta value set.
 */
function unitChooserName() {
	var unitName = "";
	var field = selectedFieldText();
	if (!exists(field)) {
		// the user might not have chosen a field yet
		return "";
	}
	if ((field.indexOf("Bytes") > -1) & !isDelta()) {
		unitName = "octetsUnitChooser"
	}
	else if ((field.indexOf("Bytes") > -1) && isDelta()) {
		unitName = "octetsUnitChooserDelta"
	}
	else if ((field.indexOf("Multicast bytes") > -1) && !isDelta()) {
		unitName = "octetsUnitChooser"
	}
	else if ((field.indexOf("Multicast bytes") > -1) && isDelta()) {
		unitName = "octetsUnitChooserDelta"
	}
	else if ((field.indexOf("Broadcast bytes") > -1) && !isDelta()) {
		unitName = "octetsUnitChooser"
	}
	else if ((field.indexOf("Broadcast bytes") > -1) && isDelta()) {
		unitName = "octetsUnitChooserDelta"
	}
	else if ((field.indexOf("Packets") > -1) && isDelta()) {
		unitName = "packetsUnitChooserDelta"
	}
	else if ((field.indexOf("Packets") > -1) && !isDelta()) {
		unitName = "packetsUnitChooser";
	}
	else if ((field.indexOf("Broadcast packets") > -1) && isDelta()) {
		unitName = "packetsUnitChooserDelta"
	}
	else if ((field.indexOf("Broadcast packets") > -1) && !isDelta()) {
		unitName = "packetsUnitChooser";
	}
	else if ((field.indexOf("Multicast packets") > -1) && isDelta()) {
		unitName = "packetsUnitChooserDelta"
	}
	else if ((field.indexOf("Multicast packets") > -1) && !isDelta()) {
		unitName = "packetsUnitChooser"
	}
	else if ((field.indexOf("Mcast/bcast") > -1) && !isDelta()) {
		unitName = "packetsUnitChooser";
	}
	else if ((field.indexOf("Mcast/bcast") > -1) && isDelta()) {
		unitName = "packetsUnitChooserDelta";
	}
	else if (isDelta()) {
		unitName = "entriesUnitChooser";
	}
	else {
		unitName = "entryUnitChooser";
	}
	return unitName;
}


/**
 * Hides all the trigger unit choosers in the add/edit alarm window. 
 */
function hideTriggerUnitChoosers() {
	hideUpperTriggerUnitChoosers();
	hideLowerTriggerUnitChoosers();
}


/**
 * Hides all the lower trigger unit choosers in the add/edit alarm window. 
 */
function hideLowerTriggerUnitChoosers() {
	var triggers = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTrigger", "name", "tr");
	var selects = triggers.getElementsByTagName("select");
	for (var i = 0; i < selects.length; i++) {
		setClass(selects[i], "inactive");
	}
}


/**
 * Hides all the upper trigger unit choosers in the add/edit alarm window. 
 */
function hideUpperTriggerUnitChoosers() {
	var triggers = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTrigger", "name", "tr");
	var selects = triggers.getElementsByTagName("select");
	for (var i = 0; i < selects.length; i++) {
		setClass(selects[i], "inactive");
	}
}



// alarm type


/**
 * Returns true if the alarm type is delta, false otherwise.
 */
function isDelta() {
	var delta = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "deltaAlarm", "value", "input");
	return delta.checked;
}


/**
 * The delta alarm checkbox has been clicked.
 */
function deltaAlarm() {
	// enable the sampling time
	var samplingTime = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTime", "name", "input");
	samplingTime.disabled = false;
	showTriggerUnitChooser();
	
	// enable the lower trigger fields
	var lowTriggerType = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowTriggerType", "value", "input");
	lowTriggerType.disabled = false;
	var lowerTriggerValue = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTriggerValue", "name", "input");
	lowerTriggerValue.disabled = false;
	var lowerTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTrigger", "name", "tr");
	setClass(lowerTrigger, "visible");
	var samplingTimeCell = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTimeCell", "name", "td");
	setClass(samplingTimeCell, "visible");	
}


/**
 * The absolute alarm checkbox has been clicked.
 */
function absoluteAlarm() {
	// disable the sampling time
	var samplingTime = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTime", "name", "input");
	samplingTime.disabled = true;
	showTriggerUnitChooser();
	
	// disable the lower trigger fields
	var lowTriggerType = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowTriggerType", "value", "input");
	lowTriggerType.disabled = true;
	var lowerTriggerValue = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTriggerValue", "name", "input");
	lowerTriggerValue.disabled = true;
	var lowerTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTrigger", "name", "tr");
	setClass(lowerTrigger, "invisible");
	var samplingTimeCell = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTimeCell", "name", "td");
	setClass(samplingTimeCell, "invisible");
}


// lower trigger


/**
 * Sets a trigger type (lower, upper).
 * @param triggerType The trigger type to set (lower, upper).
 */
function setTriggerType(triggerType) {
	if ("lower".equalsIgnoreCase(triggerType)) {
		var triggerTypeCheckBox = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowTriggerType", "value", "input");
		triggerTypeCheckBox.checked = true;
	}
	else if ("upper".equalsIgnoreCase(triggerType)) {
		var triggerTypeCheckBox = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerType", "value", "input");
		triggerTypeCheckBox.checked = true;
	}	
}


/**
 * Returns true if the trigger type set is lower, false otherwise.
 */
function isTriggerTypeLower() {
	var triggerTypeCheckBox = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowTriggerType", "value", "input");
	return triggerTypeCheckBox.checked;
}	


/**
 * Set the lower trigger unit.
 * @param unit The value of the lower trigger unit option to select.
 */
function setLowerTriggerUnit(unit) {
	var lowerTriggerContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTrigger", "name", "tr");
	if (!isEmpty(unitChooserName())) {
		var triggerUnitSelect = getElementWithAttribute(lowerTriggerContainer, unitChooserName(), "name", "select");
		selectInSelector(triggerUnitSelect, unit);
	}
}


/**
 * The lower trigger unit has been changed. 
 */
function lowerUnitSet() {
	// trigger a reavulation of the lower trigger value
	var event = new Object();
	event.keyCode = 0;
	lowerTriggerValidator.keyUpHandler();
}


/**
 * Set the lower trigger value.
 * @param value The lower trigger value to set.
 */
function setLowerTriggerValue(value) {
	var lowerTriggerValue = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTriggerValue", "name", "input");
	lowerTriggerValue.value = value;
}


/**
 * Returns the lower trigger set as a number.
 */
function lowerTriggerValue() {
	var lowerTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTriggerValue", "name", "input");
	if (!isEmpty(lowerTrigger.value)) {
		return toNumber(lowerTrigger.value);
	}
	return null;
}


/**
 * Returns true if tha lower trigger value is set. 
 */
function isLowerTriggerSet() {
	var lowerTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTriggerValue", "name", "input");
	if (isEmpty(lowerTrigger.value)) {
		return false;
	}

	return true;
}


/**
 * Throws a error if the lower trigger value set is not valid. 
 */
function assertLowerTriggerValid() {
	if (!isLowerTriggerValid()) {
		throw new Error("Invalid lower trigger value set");
	}
}
 

/**
 * Returns true if the lower trigger value set is valid, false otherwise.
 */
function isLowerTriggerValid() {
	if (!isNumber(lowerTriggerValue()) || isNegative(lowerTriggerValue())) {
		return false;
	}
	if (lowerTriggerValue() != null && 0 == lowerTriggerValue()) {
		return false;
	}
	return true;
}


/**
 * Validates the lower trigger value onkeyup.
 * @returns True if the lower trigger value is valid and false otherwise. 
 */
function validateLowerTriggerValue() {
	if (!isDelta()) {
		return true;
	}
	if (!isLowerTriggerValid()) {
		lowerTriggerValidator.setMessage("Invalid lower trigger value set");
		return false;
	}
	
	var upperUnit = AlarmUnit.getInstance(selectedUpperTriggerUnit());
	var lowerUnit = AlarmUnit.getInstance(selectedLowerTriggerUnit());
	if (upperUnit.isByteType() && !lowerUnit.isByteType()) {
		lowerTriggerValidator.setMessage("Invalid lower trigger unit set");
		return false;
	}
	if (upperUnit.isPerSecond() && !lowerUnit.isPerSecond()) {
		lowerTriggerValidator.setMessage("Invalid lower trigger unit set");
		return false;
	}
	if (!upperUnit.isPerSecond() && lowerUnit.isPerSecond()) {
		lowerTriggerValidator.setMessage("Invalid lower trigger unit set");
		return false;
	}
	if (upperUnit.isByteType() && lowerUnit.isByteType() && (upperUnit.toBytes(upperTriggerValue()) < lowerUnit.toBytes(lowerTriggerValue()))) {
		lowerTriggerValidator.setMessage("Lower trigger larger than upper trigger");
		return false;
	}
	else if (upperUnit.isByteType() && lowerUnit.isByteType() && (upperUnit.toBytes(upperTriggerValue()) == lowerUnit.toBytes(lowerTriggerValue()))) {
		lowerTriggerValidator.setMessage("Upper and lower trigger values are equal");
		return false;
	}
	else if (!upperUnit.isByteType() && !lowerUnit.isByteType() && upperTriggerValue() < lowerTriggerValue()) {
		lowerTriggerValidator.setMessage("Lower trigger larger than upper trigger");
		return false;
	}
	else if (!upperUnit.isByteType() && !lowerUnit.isByteType() && upperTriggerValue() == lowerTriggerValue()) {
		lowerTriggerValidator.setMessage("Upper and lower trigger values are equal");
		return false;
	}
	return true;
}


// upper trigger


/**
 * Returns true if the trigger type set is upper, false otherwise.
 */
function isTriggerTypeUpper() {
	var triggerTypeCheckBox = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerType", "value", "input");
	return triggerTypeCheckBox.checked;
}


/**
 * Set the upper trigger unit type.
 * @param unit The value of the upper trigger option to select.
 */
function setUpperTriggerUnit(unit) {
	var upperTriggerContainer = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTrigger", "name", "tr");
	if (!isEmpty(unitChooserName())) {
		var triggerUnitSelect = getElementWithAttribute(upperTriggerContainer, unitChooserName(), "name", "select");
		selectInSelector(triggerUnitSelect, unit);
	}
}


/**
 * Set the upper trigger value.
 * @param value The upper trigger value to set.  
 */
function setUpperTriggerValue(value) {
	var upperTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerValue", "name", "input");
	upperTrigger.value = value;
}


/**
 * Returns the upper trigger value set as a number, null if upper trigger value is empty.
 */
function upperTriggerValue() {
	var upperTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerValue", "name", "input");
	if (!isEmpty(upperTrigger.value)) {
		return toNumber(upperTrigger.value);
	}
	return null;
}


/**
 * Returns true if the upper trigger value is set, false otherwise.
 */ 
function isUpperTriggerSet() {
	var upperTrigger = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerValue", "name", "input");
	if (isEmpty(upperTrigger.value)) {
		return false;
	}
	return true;
}


/**
 * Throws a error if the upper trigger value set is not valid. 
 */
function assertUpperTriggerValid() {
	if (!isUpperTriggerValid()) {
		throw new Error("Invalid upper trigger value set");
	}
}


/**
 * Returns true if the upper trigger value set is valid, false otherwise. 
 */
function isUpperTriggerValid() {
	if (!exists(upperTriggerValue()) || !isNumber(upperTriggerValue()) || isNegative(upperTriggerValue())) {
		return false;
	}
	if (upperTriggerValue() != null && 0 == upperTriggerValue()) {
		return false;
	}
	return true;
}


/**
 * Returns the add alarm window.
 */
function alarmDetailWindow() {
	return WindowManager.instance().getWindow("alarmDetailWindow");
}


// utility functions


/**
 * Returns the index of the right-clicked table entry.
 * Returns -1 if the index could not be found.
 */
function entryIndex(srcElement) {
	var table = tableObj;
	var tdNode = tdN;
	var row = table.getRowFromTd(tdNode);
	if (row != null) {
		var index = getFieldFromRow("index", row);
		return index.getId();
	}
	else {
		return -1;
	}
}


/**
 * Fills in prefix of the IANA port number in the "port: " field, based on the selected protocol type.
 */
function setPortNumber(protocolType) {
	var portTypeSelector = document.getElementsByName("protocolTypeFieldChooser");
	var portInputField = document.getElementsByName("alarmPortField");
	if (portTypeSelector[0].value == "ethernet") {
		portInputField[0].value = "1.";
	}
	else if (portTypeSelector[0].value == "ip") {
		portInputField[0].value = "1.2048.";
	}
	else if (portTypeSelector[0].value == "tcp") {
		portInputField[0].value = "1.2048.6.";
	}
	else if (portTypeSelector[0].value == "udp") {
		portInputField[0].value = "1.2048.17.";
	}
}


/**
 * Set the prefix of a protocol in the protocol input field, based on the selected protocol type.
 */
function setProtocolPrefix() {
	var portInputField = document.getElementsByName("alarmPortField");
	var originalValue = portInputField[0].value;
	var portTypeSelector = document.getElementsByName("protocolTypeFieldChooser");
	
	if (!isEmpty(PROTOCOL_PREFIXES[portTypeSelector[0].value]) && originalValue.indexOf(".") < 0){
		portInputField[0].value = PROTOCOL_PREFIXES[portTypeSelector[0].value] + originalValue;
	}
}


/**
 * Initializes the field validators in the window. 
 */
function initValidators() {
	var messageField = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmMessage", "name", "div");
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	var alarmPortFieldValidator = createPortIdFieldValidator(port, messageField);
	alarmPortFieldValidator.setEnterFunction(enterPressed);
	alarmPortFieldValidator.setOnBlurFunction(setProtocolPrefix);
	alarmPortFieldValidator.setOnFocusFunction(alarmPortfieldFocused);
	
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	var ipFieldValidator = createIpFieldValidator(ip, messageField);
	ipFieldValidator.setEnterFunction(enterPressed);
	ipFieldValidator.setOnFocusFunction(ipFieldFocused);
	
	var srcIp = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	var srcIpFieldValidator = createIpFieldValidator(srcIp, messageField);
	srcIpFieldValidator.setEnterFunction(enterPressed);
	srcIpFieldValidator.setOnFocusFunction(srcIpFieldFocused);
	
	var dstIp = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	var dstIpFieldValidator = createIpFieldValidator(dstIp, messageField);
	dstIpFieldValidator.setEnterFunction(enterPressed);
	dstIpFieldValidator.setOnFocusFunction(dstIpFieldFocused);
	
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	var macFieldValidator = createMacFieldValidator(mac, messageField);
	macFieldValidator.setEnterFunction(enterPressed);
	macFieldValidator.setOnFocusFunction(macFieldFocused);
	
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	var srcMacFieldValidator = createMacFieldValidator(srcMac, messageField);
	srcMacFieldValidator.setEnterFunction(enterPressed);
	srcMacFieldValidator.setOnFocusFunction(srcMacFieldFocused);
	
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");
	var dstMacFieldValidator = createMacFieldValidator(dstMac, messageField);
	dstMacFieldValidator.setEnterFunction(enterPressed);
	dstMacFieldValidator.setOnFocusFunction(dstMacFieldFocused);
	
	var upperTriggerValue = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "upperTriggerValue", "name", "input");
	var upperTriggerValidator = new InputFieldValidator(upperTriggerValue, messageField);
	upperTriggerValidator.setEnterFunction(enterPressed);
	upperTriggerValidator.setValidationFunction(isUpperTriggerValid);
	upperTriggerValidator.setMessage("Invalid upper trigger value set");
	
	var lowerTriggerValue = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "lowerTriggerValue", "name", "input");
	lowerTriggerValidator = new InputFieldValidator(lowerTriggerValue, messageField);
	lowerTriggerValidator.setEnterFunction(enterPressed);
	lowerTriggerValidator.setValidationFunction(validateLowerTriggerValue);
	lowerTriggerValidator.setMessage("Invalid lower trigger value set");
	
	var samplingTime = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "samplingTime", "name", "input");
	var samplingTimeValidator = new InputFieldValidator(samplingTime, messageField);
	samplingTimeValidator.setEnterFunction(enterPressed);
	samplingTimeValidator.setValidationFunction(isValidSamplingTime);
	samplingTimeValidator.setMessage("Invalid sampling time set");
	
	var description = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "description", "name", "input");
	var descriptionValidator = new InputFieldValidator(description, messageField);
	descriptionValidator.setEnterFunction(enterPressed);
	descriptionValidator.setValidationFunction(isValidDescription);
	descriptionValidator.setOnFocusFunction(descriptionFieldFocused);
	descriptionValidator.setMessage("; is not allowed in the description");
}


/**
 * Disables all the entry fields. 
 */
function disableEntryFields() {
	var tableChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "tableChooser", "name", "select");
	var protocolFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolFieldChooser", "name", "select");
	var nlhostFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "nlhostFieldChooser", "name", "select");
	var nlmatrixFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "nlmatrixFieldChooser", "name", "select");	
	var alhostFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alhostFieldChooser", "name", "select");
	var almatrixFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "almatrixFieldChooser", "name", "select");
	var hostFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "hostFieldChooser", "name", "select");
	var matrixFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "matrixFieldChooser", "name", "select");
	var networkFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "networkFieldChooser", "name", "select");
	var summaryFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "summaryFieldChooser", "name", "select");
	var protocolTypeFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolTypeFieldChooser", "name", "select");
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");

	tableChooser.disabled = true;
	protocolFieldChooser.disabled = true;
	nlhostFieldChooser.disabled = true;
	nlmatrixFieldChooser.disabled = true;
	alhostFieldChooser.disabled = true;
	almatrixFieldChooser.disabled = true;
	hostFieldChooser.disabled = true;
	matrixFieldChooser.disabled = true;
	networkFieldChooser.disabled = true;
	summaryFieldChooser.disabled = true;
	protocolTypeFieldChooser.disabled = true;
	port.disabled = true;
	ip.disabled = true;
	src.disabled = true;
	dst.disabled = true;
	mac.disabled = true;
	srcMac.disabled = true;
	dstMac.disabled = true;
}


/**
 * Enables all the entry fields.
 */
function enableEntryFields() {
	var tableChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "tableChooser", "name", "select");
	var protocolFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolFieldChooser", "name", "select");
	var nlhostFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "nlhostFieldChooser", "name", "select");
	var nlmatrixFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "nlmatrixFieldChooser", "name", "select");	
	var alhostFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alhostFieldChooser", "name", "select");
	var almatrixFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "almatrixFieldChooser", "name", "select");
	var hostFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "hostFieldChooser", "name", "select");
	var matrixFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "matrixFieldChooser", "name", "select");
	var networkFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "networkFieldChooser", "name", "select");
	var summaryFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "summaryFieldChooser", "name", "select");
	var protocolTypeFieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "protocolTypeFieldChooser", "name", "select");
	var port = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "alarmPortField", "name", "input");
	var ip = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "ip", "name", "input");
	var src = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "src", "name", "input");
	var dst = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dst", "name", "input");
	var mac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "mac", "name", "input");
	var srcMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "srcMac", "name", "input");
	var dstMac = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "dstMac", "name", "input");

	tableChooser.disabled = false;
	protocolFieldChooser.disabled = false;
	nlhostFieldChooser.disabled = false;
	nlmatrixFieldChooser.disabled = false;
	alhostFieldChooser.disabled = false;
	almatrixFieldChooser.disabled = false;
	hostFieldChooser.disabled = false;
	matrixFieldChooser.disabled = false;
	networkFieldChooser.disabled = false;
	summaryFieldChooser.disabled = false;
	protocolTypeFieldChooser.disabled = false;
	port.disabled = false;
	ip.disabled = false;
	src.disabled = false;
	dst.disabled = false;
	mac.disabled = false;
	srcMac.disabled = false;
	dstMac.disabled = false;
}


/**
 * Fires a alarm set event.
 */
function fireAlarmSetEvent() {
	getEventServer().notifyListeners(new Event(ALARM_SET_EVENT, undefined));
}

/**
 * Fires a alarm edited event. 
 */
function fireAlarmEditedEvent() {
	getEventServer().notifyListeners(new Event(ALARM_EDITED_EVENT, undefined));
}