

/**
 * Alarms can be triggered multiple times. This is the separator for 
 * multiple alarm values. 
 */
var ALARM_VALUE_SEPARATOR = ",";

 
/**
 * Mapping between field names and counterIds for the tables.
 */
var counterIdNames = new Array();
counterIdNames["host"] = {0: "Packets in", 1: "Packets out", 2: "Bytes in", 3: "Bytes out", 4: "Broadcast packets", 5: "Broadcast bytes", 6: "Multicast packets", 7: "Multicast bytes"};
counterIdNames["matrix"] = {0: "Packets", 1: "Bytes"};
counterIdNames["nlhost"] = {0: "Packets in", 1: "Packets out", 2: "Bytes in", 3: "Bytes out", 4: "Out mac non Unicast Packets", 5: "Protocols", 6: "Bytes", 7: "Conversations", 8: "Packets"};
counterIdNames["nlmatrix"] = {0: "Packets", 1: "Bytes", 2: "Protocols"};
counterIdNames["alhost"] = {0: "Packets in", 1: "Packets out", 2: "Bytes in", 3: "Bytes out"};
counterIdNames["almatrix"] = {0: "Packets", 1: "Bytes"};
counterIdNames["protocol"] = {0: "Packets", 1: "Bytes", 2: "Hosts", 3: "Conversations"};
counterIdNames["network"] = {0: "Packets", 1: "Bytes"};
counterIdNames["summary"] = {0: "Protocols", 1: "Hosts", 2: "Conversations", 3: "Cards", 4: "Packets", 5: "Bytes"};


/**
 * Mapping between field names as defined in the "name" tag on each cell in the tables, and 
 * the field names defined in the counterIdNames map.
 */
var fieldNameMapping = new Array();
fieldNameMapping["octets"] = "Bytes";
fieldNameMapping["packets"] = "Packets";
fieldNameMapping["hosts"] = "Hosts";
fieldNameMapping["conversations"] = "Conversations";
fieldNameMapping["inPackets"] = "Packets in";
fieldNameMapping["outPackets"] = "Packets out";
fieldNameMapping["inOctets"] = "Bytes in";
fieldNameMapping["outOctets"] = "Bytes out";
fieldNameMapping["cards"] = "Cards";
fieldNameMapping["protocols"] = "Protocols";
fieldNameMapping["protocolCount"] = "Protocols";
fieldNameMapping["packetCount"] = "Packets";
fieldNameMapping["octetCount"] = "Bytes";
fieldNameMapping["hostCount"] = "Hosts";
fieldNameMapping["conversationCount"] = "Conversations";
fieldNameMapping["cardCount"] = "Cards";


/**
 * The severity names.
 */
var severityNames = new Array();
severityNames[0] = "Information";
severityNames[1] = "Warning";
severityNames[2] = "Serious";
severityNames[3] = "Critical";


/**
 * The table names shown in the table indexed by the table names loaded from the server.
 */
var tableLabels = new Array();
tableLabels["protocol"] = "Protocol";
tableLabels["nlhost"] = "Host";
tableLabels["nlmatrix"] = "Conversation";
tableLabels["alhost"] = "Host protocol";
tableLabels["almatrix"] = "Conversation protocol";
tableLabels["host"] = "Network card";
tableLabels["matrix"] = "NIC conversation";
tableLabels["network"] = "Ethernet";
tableLabels["summary"] = "Network summary";


/**
 * The severity names.
 */
var severities = new Array("Information", "Warning", "Serious", "Critical");


/**
 * The colors to use for the differend severities.
 */
var severityColors = {Information: "green", Warning: "#b7b903", Serious: "#f58909", Critical: "red"};


/**
 * The visibility names.
 */
var visibilityNames = new Array();
visibilityNames[0] = "All";
visibilityNames[1] = "Super";
visibilityNames[2] = "Admin";


/**
 * The base url used to delete a alarm.
 */
var DELETE_ALARM_BASE_URL = "/?resource=AlarmManager&mode=set&command=deleteDefined&sid=" + getSid();


/**
 * The width of the edit button to the right in the alarm table. 
 */
var EDIT_BUTTONS_WIDTH = 15;


/**
 * The width of the delete butotn to the right in the alarm table.
 */
var DELETE_BUTTONS_WIDTH = 15;

 
/**
 * Functions used to enter the put a alarm from a alarm table row into the alarm detail window.
 */
var entryCreators = new Array();
entryCreators["protocol"] = protocolEntry;
entryCreators["nlhost"] = hostEntry;
entryCreators["nlmatrix"] = conversationsEntry;
entryCreators["alhost"] = protocolsPerHostsEntry;
entryCreators["almatrix"] = protocolsPerConversationsEntry;
entryCreators["host"] = networkCardsEntry;
entryCreators["matrix"] = networkCardConversationsEntry;
entryCreators["summary"] = summaryEntry;
entryCreators["network"] = networkEntry;


/**
 * The names of the network entries keyed by their index. 
 */
var networkEntryNames = new Array();
networkEntryNames[1] = "All packets";
networkEntryNames[2] = "Unicast packets";
networkEntryNames[3] = "Broadcast packets";
networkEntryNames[4] = "Multicast packets";
networkEntryNames[5] = "Packets 64 bytes";
networkEntryNames[6] = "Packets 65 to 127 bytes";
networkEntryNames[7] = "Packets 128 to 255 bytes";
networkEntryNames[8] = "Packets 256 to 511 bytes";
networkEntryNames[9] = "Packets 512 to 1023 bytes";
networkEntryNames[10] = "Packets 1024 to 1517 bytes";
networkEntryNames[11] = "Packets 1518 bytes";
networkEntryNames[12] = "Oversized packets";

 
// construction


/**
 * Factory method to create new alarmTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createAlarmTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new AlarmTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	instance["superupdateTableRow"] = superInstance["superupdateTableRow"];
	instance["supercreateRowCode"] = superInstance["supercreateRowCode"];
	// in the mytables the add entry table is always right under the table containing the data
	// to do this the size of the data table container is set according to how many rows are shown
	// in the alarmtable this is not needed, override the methods implementing this functionality
	instance["isVerticalScrollBarVisible"] = superInstance["superisVerticalScrollBarVisible"];
	instance["superupdateHtmlTable"] = superInstance["superupdateHtmlTable"];	
	instance["createDataTable"] = superInstance["supercreateDataTable"];	
	instance["supersetData"] = superInstance["supersetData"];
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	instance.eventListener.addEvent("resized", processAlarmTableResizedEvent);
	
	instance.eventListener.addEvent(ALARM_SET_EVENT, processAlarmSetEvent);
	instance.eventListener.addEvent(ALARM_EDITED_EVENT, processAlarmEditedEvent);

	// overwrite the repeater to call the load function on the AlarmTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}


/**
 * Subclasses table.js to add specifc behaviour for alarm tables.
 */
function AlarmTable() {

	// Set to true if the edit link is to be shown for the alarms
	this.showEditLink = true;
}


// event processing


/**
 * Process a resize event.
 * @param event The resize event to process.
 */
function processAlarmTableResizedEvent(event) {
	if (event.getData()["id"] != this.windowName) {
		// the table only cares about events from its window, none of the others
		return;
	}
	if (this.noAccessShown) {
		return;
	}
	
	var resizedWidth = event.getData()["width"];
	var resizedHeight = event.getData()["height"];
	this.resizeTable(resizedHeight, resizedWidth);
}


/**
 * Processes a alarm set event.
 * @param event The event to process. 
 */
function processAlarmSetEvent(event) {
	if (this.noAccessShown) {
		return;
	}
	this.restartUpdate();
}


/**
 * Processes a alarm edited event.
 * @param event The event to process. 
 */
function processAlarmEditedEvent(event) {
	if (this.noAccessShown) {
		return;
	}
	this.restartUpdate();
}


/**
 * Toggles whether the edit link is shown. This does not trigger a
 * repaint of the table, call this before the table is painted.
 * @param showEditLink Set this to true if the edit link is to be shown, false otherwise.  
 * @throws IllegalArgumentException if showEditLink is undefined or null.
 */
AlarmTable.prototype.toggleEditLink = function(showEditLink) {
	assertExists(showEditLink, "Show edit link is undefined or null");
	this.showEditLink = showEditLink;
}


// implementation


/**
 * Edits a alarm.
 * @param value The value for the id field of the row to edit. 
 */
AlarmTable.prototype.editAlarm = function(value) {
	showEditAlarmWindow();
	this.insertAlarm(value);
}


/**
 * Copies a alarm.
 * @param value The value for the id field of the row to edit.
 */
AlarmTable.prototype.copyAlarm = function(value) {
	showCopyAlarmWindow();
	this.insertAlarm(value);
}


/**
 * Inserts a alarm into the add/edit alarm window. 
 * @param value The value for the id field of the alarm to insert. 
 */
AlarmTable.prototype.insertAlarm = function(value) {
	var id = getParameterValue(value, "id", "&");
	var row = this.getRowWithFieldValue("id", id);
	if (!exists(row)) {
		// no alarm with this id was found
		return;
	}
	
	this.setAlarmEntry(row);
	this.setAlarmType(row);
	setTriggerType(getFieldFromRow("triggerType", row).getValue());
	setUpperTriggerValue(getFieldFromRow("upperTriggerValue", row).getValue());
	setUpperTriggerUnit(getFieldFromRow("upperUnit", row).getValue());
	setLowerTriggerValue(getFieldFromRow("lowerTriggerValue", row).getValue());
	setLowerTriggerUnit(getFieldFromRow("lowerUnit", row).getValue());
	setVisibilityFieldValue(getFieldFromRow("visibility", row).getValue());
	setSeverityFieldValue(getFieldFromRow("severity", row).getValue());
	setDescriptionFieldValue(URLDecode(getFieldFromRow("description", row).getValue()));
	setIdFieldValue(getFieldFromRow("id", row).getValue());
}


/**
 * Sets the alarm type in the alarm detail window according to a given table
 * row containing a alarm.
 * @param alarm The table row representing the alarm.
 */
AlarmTable.prototype.setAlarmType = function(alarm) {
	var absoluteAlarm = getFieldFromRow("absoluteAlarm", alarm).getValue();
	setAlarmType(absoluteAlarm);
	if ("false".equalsIgnoreCase(absoluteAlarm)) {
		setSamplingTime(getFieldFromRow("samplingTime", alarm).getValue());
	}
}


/**
 * Sets the alarm entry in the alarm detail window according to the alarm
 * in a give table row.
 * @param alarm The table row representing the alarm. 
 */
AlarmTable.prototype.setAlarmEntry = function(alarm) {
	var table = getFieldFromRow("table", alarm);
	var tableChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), "tableChooser", "name", "select");
	selectInSelector(tableChooser, table.getValue());
	tableChosen();
	
	this.setField(alarm);	
	if (exists(entryCreators[table.getValue()])) {
		entryCreators[table.getValue()](alarm);
	}
}


/**
 * Inputs the network entry from a given alarm table row
 * into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function networkEntry(alarm) {
	setNetworkEntryChooserValue(getFieldFromRow("src", alarm).getValue());
}


/**
 * Inputs the summary entry form a given alarm table row
 * into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function summaryEntry(alarm) {
	setIpFieldValue(getFieldFromRow("src", alarm).getValue());
}
 
 
/**
 * Inputs the network card conversations from a given alarm table row
 * into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function networkCardConversationsEntry(alarm) {
	setSrcMacFieldValue(getFieldFromRow("src", alarm).getValue());
	setDstMacFieldValue(getFieldFromRow("dst", alarm).getValue());
}


/**
 * Inputs the network card entries from a given alarm table
 * row into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function networkCardsEntry(alarm) {
	setMacFieldValue(getFieldFromRow("src", alarm).getValue());
}


/**
 * Inputs the protocol per conversation entries from a given alarm table
 * row into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function protocolsPerConversationsEntry(alarm) {
	setPortFieldValue(getFieldFromRow("pid", alarm).getValue())
	setSrcIpFieldValue(getFieldFromRow("src", alarm).getValue());
	setDstIpFieldValue(getFieldFromRow("dst", alarm).getValue());
}


/**
 * Inputs the protocol per host entries from a given alarm table
 * row into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function protocolsPerHostsEntry(alarm) {
	setPortFieldValue(getFieldFromRow("pid", alarm).getValue())
	setIpFieldValue(getFieldFromRow("src", alarm).getValue());
}


/**
 * This function inputs the nlmatrix entries from a given alarm table
 * row into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function conversationsEntry(alarm) {
	setSrcIpFieldValue(getFieldFromRow("src", alarm).getValue());
	setDstIpFieldValue(getFieldFromRow("dst", alarm).getValue());
}


/**
 * This function inputs the nlhost entries from a given alarm talbe
 * row into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function hostEntry(alarm) {
	setIpFieldValue(getFieldFromRow("src", alarm).getValue());
}

 
/**
 * This function inputs the protocol entries from a given alarm talbe
 * row into the alarm detail window.
 * @param alarm The table row representing the alarm.
 */
function protocolEntry(alarm) {
	setPortFieldValue(getFieldFromRow("pid", alarm).getValue())
}


/**
 * Sets the table name of the alarm (selects it from the drop-down box).
 * @param tableName The name of the table (summaryTable, topNProtocolsTable,...)
 */
function setTableName(tableName) {
	var tableSelection = tableChooser();
	for (var i = 0; i < tableSelection.length; i++) {
		if (tableSelection.options[i].value == tableNames[tableName]) {
			tableSelection.selectedIndex = i;
			return;
		}
	}
}


/**
 * Sets the field in the alarm detail window according to a table row
 * containing a alarm.
 * @parameter alarm The table row representing the alarm.
 */
AlarmTable.prototype.setField = function(alarm) {
	var counterId = getFieldFromRow("counterId", alarm);
	var tableName = selectedTableName();
	var fieldChooser = getElementWithAttribute(alarmDetailWindow().getHtmlCode(), tableName + "FieldChooser", "name", "select");
	selectInSelector(fieldChooser, counterId.getValue());
	fieldChosen();
}


/**
 * Deletes a entry. This is overridden to not change the height of the table
 * as it is done in the supper class MyTable.
 * @param value The value for the id field of the row to delete. 
 */
AlarmTable.prototype.deleteEntry = function(value) {	
	var rowNumber = this.getIdRowNumber(value);
	if (rowNumber == -1) {
		return;
	}
	var deleteRow = this.rows[rowNumber];
	var deleteUrl = this.createDeleteEntryUrl(value);
	this.deleteRow(rowNumber);
	if (!isEmpty(deleteUrl)) {
		loadXML(deleteUrl);
	}
	
	// a row has been delete the scroll bar if any might have dissappeared
	this.adjustSize();
	
	// the data in the table has been changed fire a udpated event
	this.fireUpdatedEvent();
	
	// after deleting a row the row classes might need to set again, because
	// of alternating row classes might be wrong after a row has been delete
	for (var i = 0; i < this.getTableCode().rows.length; i++) {
		this.addRowClasses(this.getTableCode().rows[i], i);
	}
	
	this.entryDeleted(deleteRow);
}


/**
 * Creates the url needed to delete a alarm.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
AlarmTable.prototype.createDeleteEntryUrl = function(value) {
	var id = getParameterValue(value, "id", "&");
	return DELETE_ALARM_BASE_URL + "&id=" + id + "&";
}


/**
 * Set data into the table based on a xmlDocument. To 
 * show the data on screen call paint(). This is overridden to add the entry field.
 * @param xmlDoc The data in xml format.
 * @param tagName The name of the tag the data for each row is in.
 * @param throws Error if xmlDoc is undefined or null.
 * @param throws Error if tagName is undefined, null or empty.
 */
AlarmTable.prototype.setData = function (xmlDoc, tagName) {
	assertDefined(xmlDoc, "XmlDoc is undefined or null");
	assertNotEmpty(tagName, "TagName is undefined, null or empty");
	var rows = xmlDoc.getElementsByTagName(tagName);
	
	var len = rows.length;
	for (var i = 0; i < len; i++) {
		var row = this.parseXmlRow(rows[i]);
		row[row.length] = this.createEntryField(row)
		this.rows[this.rowCounter] = row;
		this.rowCounter++;
	}
	
	if (!this.sortable) {
		return;
	}
	
	// sort the table
	if (exists(this.columnToSort)) {
		this.sort(this.columnToSort);
	}
	if (this.reversed) {
		this.reverseColumn(this.columnToSort);
	}
}


/**
 * Updates the html table according to the data in memory.
 * This is overridden to make room for the delete column.
 */
AlarmTable.prototype.updateHtmlTable = function() {
	this.callSuper("updateHtmlTable");
	if (!this.cellWidthDecreased && this.calculateButtonColumnWidth() > 0) {
		// make room for the button column
		this.resizeAllColumns(this.calculateButtonColumnWidth() * (-1));
		this.cellWidthDecreased = true;
	}
}


/**
 * Create a table cell for each entry in a array.
 * This is overridden to add the delete column to the data rows. 
 * @param data The array to create data from.
 * @param row The row to insert the table cells created.
 * @throws Error if data is null or not defined.
 * @throws Error if row is null or not defined.
 */
AlarmTable.prototype.createRowCode = function(data, row) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = data;
	argumentsToSuper[1] = row;
	this.callSuper("createRowCode", argumentsToSuper);
	
	// add the delete column if the row is defined and not the header row
	if (!isDefined(row) || row.length == 0) {
		return;
	}

	if (!this.cellWidthDecreased) {
		// make room for the button column
		this.resizeAllColumns(this.buttonsCellWidth() * (-1));
		this.cellWidthDecreased = true;
	}
	this.addButtonColumn(data, row);
}


/**
 * Adds the mouseover class for a row.
 * @param row The html row to set the color for.
 */
AlarmTable.prototype.addRowMouseOverClass = function(row) {
	if (!isDefined(this.mouseOverClass)) {
		return;
	}
	if (row.counter > 0) {
		// the mouse is currently over the row
		return;
	}
	
	var mouseOverClass = this.mouseOverClass;
	var originalClass = row.getAttribute("class");
	row.counter = 0;
	var rowIndex = this.getHtmlRowIndex(row);
	var thisObj = this;
	row.onmouseover = function () {
		this.counter = this.counter + 1;
		setClass(this, mouseOverClass);
		if (thisObj.rightClickMenuShown == false) {
			Tip(thisObj.createAlarmTooltipText(thisObj.getRowFromTd(this.cells[0])), BGCOLOR, '#ffffe1', DELAY, 500);
		}
	}
	row.onmouseout = function () {
		this.counter = 0;
		thisObj.addRowClass(row, rowIndex);
	}
}


/**
 * Creates the text to show in a alarm tooltip.
 * @param alarm Array containing the alarm fields. 
 * @return The text to shown in the tooltip for the alarm.  
 */
AlarmTable.prototype.createAlarmTooltipText = function(alarm) {
	var label = "Upper trigger: " + getFieldFromRow("upperTriggerValue", alarm).getValue() + " " + getFieldFromRow("upperUnit", alarm).getValue();
	label += "<br>Lower trigger: " + getFieldFromRow("lowerTriggerValue", alarm).getValue() + " " + getFieldFromRow("lowerUnit", alarm).getValue();
	label += "<br>Visibility: " + visibilityNames[getFieldFromRow("visibility", alarm).getValue()];
	return label;
}


/**
 * Creates the entry field for a data row. The entry field combines
 * the src, dst and pid field.
 * @param dataRow The data row to create the entry field for.
 * @return The created entry field.
 */
AlarmTable.prototype.createEntryField = function(dataRow) {
	entryFieldValue = getFieldFromRow("src", dataRow).getValue();
	var table = getFieldFromRow("table", dataRow).getValue();
	if ("network".equalsIgnoreCase(table)) {
		return new Field("entry", networkEntryNames[entryFieldValue]);
	}
	var dstField = getFieldFromRow("dst", dataRow);
	if (!isEmpty(dstField.getValue())) {
		entryFieldValue += " -> " + dstField.getValue();
	}
	var protocolField = getFieldFromRow("pid", dataRow);
	if (!isEmpty(entryFieldValue) && !isEmpty(protocolField.getValue())) {
		entryFieldValue += " - ";
	}
	if (!isEmpty(protocolField.getValue())) {
		entryFieldValue += protocolField.getValue();
	}
	return new Field("entry", entryFieldValue);
}


/**
 * Adds the button cell to a given table row. 
 * @param data The array to create data from.
 * @param row The row to add the delete cell to.
 */
AlarmTable.prototype.addButtonColumn = function(data, row) {
	var cell = row.insertCell(row.cells.length);
	cell.setAttribute("name", "buttonCell");
	cell.style.padding = "2px";
	cell.setAttribute("valign", "middle");
	setClass(cell, "deleteCell");
	
	if (this.isHeader(data[0])) {
		cell.style.width = this.buttonsCellWidth() + "px";
		return;
	}
	cell.style.width = this.buttonsCellWidth() + "px";
	var nobr = document.createElement("nobr");
	cell.appendChild(nobr);
	this.addDeleteLink(nobr, data);
	if (this.showEditLink) {
		this.addEditLink(nobr, data, "editDefined editImage");
	}
}


/**
 * Adds the edit link to a table cell.
 * @param cell The table cell to append the edit link to.
 * @param data The array to create data from.
 * @param className The name of the class to set for hte edit link.
 */
AlarmTable.prototype.addEditLink = function(cell, data, className) {
	var editLink = document.createElement("a");
	cell.appendChild(editLink);
	editLink.setAttribute("name", "editLink");
	this.setEditLinkOnClick(editLink, data);
	
	var editImage = document.createElement("div");
	editImage.setAttribute("name", "editAlarmImage");
	editImage.style.cursor = "pointer";
	setClass(editImage, className);
	editImage.onmouseover = showAlarmButtonTooltip;
	editLink.appendChild(editImage);
}



/**
 * Appends the delete link to a table cell. This is overridden to set a differend
 * onmouseover on the delete image. 
 * @param cell The table cell to append the delete link to.
 * @param data The array to create data from.
 */
AlarmTable.prototype.addDeleteLink = function(cell, data) {
	var deleteLink = document.createElement("a");
	cell.appendChild(deleteLink);
	deleteLink.setAttribute("name", "deleteLink");
	this.setDeleteLinkOnClick(deleteLink, data);
	
	var deleteImage = document.createElement("div");
	deleteImage.setAttribute("name", "deleteEntryImage");
	deleteImage.style.cursor = "pointer";
	deleteImage.onmouseover = showAlarmButtonTooltip;
	setClass(deleteImage, "deleteAlarmImage");
	deleteLink.appendChild(deleteImage);
}


/**
 * Shows the tooltip for a alarm button. The propagation of the event
 * is stopped to prevent the alarm detail tooltip form being shown.
 */
function showAlarmButtonTooltip(event) {
	if (!event) {
		// IE
		event = window.event;
	}
	showTableTooltip(event);
	// stop the alarm detail tooltip from being shown
	stopPropagation(event);
}


/**
 * Set the onclick value for a edit link. 
 * @param deleteLink The link to set the onclick value for. 
 * @param data The array to create data from.
 */
AlarmTable.prototype.setEditLinkOnClick = function(editLink , data) {
	// set the value of this rows id field as the name of the link, this is needed
	// when the link is clicked to edit the correct row. 
	var rowIndex = 0;
	for (var i = 0; i < this.rows.length; i++) {
		if (this.rows[i] == data) {
			rowIndex = i;
		}
	}
	editLink.setAttribute("id", this.getRowId(this.rows[rowIndex]));
	var thisObj = this;
	editLink.onclick = function() {
		thisObj.editAlarm(this.getAttribute("id"));
	}
}


/**
 * Updates a html row of the table according to the data in the specified data row.
 * This is overridden to update the delete link as the order of the row changes,
 * this need to be done or the delete links my delete the wrong rows.
 * @param dataRow The array containing the data to insert into the html row.
 * @param htmlRow The html row that is to be update with the data in dataRow.
 */
AlarmTable.prototype.updateTableRow = function(dataRow, htmlRow) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = dataRow;
	argumentsToSuper[1] = htmlRow;
	this.callSuper("updateTableRow", argumentsToSuper);
	
	// update the rows delete link, so the link deletes the correct entry
	var deleteLink = getElementWithAttribute(htmlRow, "deleteLink", "name", "a");
	if (exists(deleteLink)) {
		this.setDeleteLinkOnClick(deleteLink, dataRow);
	}
	
	// update the rows edit link, so the link edits the correct entry
	var editLink = getElementWithAttribute(htmlRow, "editLink", "name", "a");
	if (exists(editLink)) {
		this.setEditLinkOnClick(editLink, dataRow);
	}
	
	// udpate the css class of the html row, when new triggered alarms are added
	// and the table is sorted by trigger time new alarms are shown at the top, need
	// to update rows css class to match the new alarms severity
	var index = -1;
	for (var i = 0; i < this.rows.length; i++) {
		if (dataRow == this.rows[i]) {
			index = i;
		}
	}
	this.addRowClasses(htmlRow, index);
}


/**
 * Inserts a field value into a given html node. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertFieldValue = function(field, node, cell) {
	if (!this.isHeader(field) && field.getName().indexOf("counterId") > -1) {
		this.insertFieldName(field, node , cell);
	}	
	else if (!this.isHeader(field) && field.getName().indexOf("severity") > -1) {
		this.insertSeverityName(field, node , cell);
	}
	else if (!this.isHeader(field) && field.getName().indexOf("visibility") > -1) {
		this.insertVisibilityName(field, node, cell);
	}
	else if (!this.isHeader(field) && field.getName().indexOf("triggerTime") > -1) {
		this.insertTriggerTime(field, node, cell);
	}
	else if (!this.isHeader(field) && field.getName().indexOf("absoluteAlarm") > -1) {
		this.insertAlarmType(field, node, cell);
	}
	else if (!this.isHeader(field) && field.getName().indexOf("triggerValue") > -1) {
		this.insertTriggerValue(field, node, cell);
	}
	else if (!this.isHeader(field) && field.getName().indexOf("description") > -1) {
		node.appendChild(document.createTextNode(URLDecode(field.getValue())));
	}
	else if (!this.isHeader(field) && field.getName().indexOf("table") > -1) {
		node.appendChild(document.createTextNode(tableLabels[field.getValue()]));
	}	
	else {
		node.appendChild(document.createTextNode(field.getValue()));
	}
}

	
/**
 * Inserts the trigger value for a given field into a html node. 
 * @param field The field to insert the value for. 
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertTriggerValue = function(field, node, cell) {
	// alarms triggered multiple times have multiple values for the trigger 
	// value, only show the newest trigger value
	var triggerValue = field.getValue();
	var index = triggerValue.lastIndexOf(ALARM_VALUE_SEPARATOR);
	if (index > -1) {
		triggerValue = triggerValue.substring(index + ALARM_VALUE_SEPARATOR.length);
	}
	node.appendChild(document.createTextNode(triggerValue));
}


/**
 * Inserts the formatted alarm type into a given html node.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertAlarmType = function(field, node, cell) {
	if ("true" == field.getValue()) {
		node.appendChild(document.createTextNode("Absolute"));
	}
	else {
		node.appendChild(document.createTextNode("Delta"));
	}
}


/**
 * Inserts the formatted trigger time into a given html node. 
  *@param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertTriggerTime = function(field, node, cell) {
	// alarms triggered multiple times have multiple values for the trigger 
	// time, only show the newest trigger time
	var time = field.getValue();
	var index = time.lastIndexOf(ALARM_VALUE_SEPARATOR);
	if (index > -1) {
		time = time.substring(index + ALARM_VALUE_SEPARATOR.length);
	}
	//date.toLocaleString()
	node.appendChild(document.createTextNode(this.formatTriggerTime(time)));
}


/**
 * Formats the trigger time. 
 * @param time The trigger time to format. 
 * @return The formatted trigger time.
 */
AlarmTable.prototype.formatTriggerTime = function(time) {
	var date = new Date(parseInt(time));
	var dateString = "";
	var day = date.getDate() < 10 ? "0" + date.getDate() : date.getDate();	
	// the months start at 0
	var month = (date.getMonth() + 1) < 10 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1);
	// the getYear methods returns the number of years since 1900
	var year = (date.getYear() + 1900) + "";
	year = year.substring(2);
	var minutes = date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes();
	var seconds = date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds();
	dateString = day + "." + month + "." + year + " " +  date.getHours() + ":" + minutes + ":" + seconds;
	return dateString;
}


/**
 * Inserts a visibility name into a given html node. The visibility number
 * is replaced with its name.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertVisibilityName = function(field, node, cell) {
	node.appendChild(document.createTextNode(visibilityNames[field.getValue()]));
}


/**
 * Inserts a severity name into a given html node. The severity number 
 * is replaced with its name.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertSeverityName = function(field, node, cell) {
	cell.style.color = severityColors[severityNames[field.getValue()]];
	node.appendChild(document.createTextNode(severityNames[field.getValue()]));
}


/**
 * Inserts a field name value into a given html node. The field id is 
 * replaced with its name.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
AlarmTable.prototype.insertFieldName = function(field, node, cell) {
	var row = this.getRowFromField(field);
	var tableField = getFieldFromRow("table", row);
	var fieldName = this.getFieldName(field.getValue(), tableField.getValue());
	node.appendChild(document.createTextNode(fieldName));
}


/**
 * Returns the field name for a given fieldId and table.
 * @param id The field id to get the field name for.
 * @param table The table name.
 */
AlarmTable.prototype.getFieldName = function(id, table) {
	return counterIdNames[table][id];
}


/**
 * Gets the width of the buttons cell to use in this table. 
 * @return The width to set for the cell containing the buttons (delete/edit). 
 */
AlarmTable.prototype.buttonsCellWidth = function() {
	if (!this.showEditLink) {
		return DELETE_BUTTONS_WIDTH;
	}
	return EDIT_BUTTONS_WIDTH + DELETE_BUTTONS_WIDTH;
}