/**
 * The chart manager administrates and manages the charts. 
 */
 
 
// constants


/**
 * The name of the chart right click menu container. 
 */
ChartManager.CHART_RIGHT_CLICK_MENU_NAME = "chartRightClick";


/**
 * The id of the set chart mode right click menu item.
 */
ChartManager.CHART_RIGHT_CLICK_MODE_ITEM_ID = "setChartMode";

 
/**
 * The id of the help right click menu item.
 */
ChartManager.CHART_RIGHT_CLICK_HELP_ITEM_ID = "helpItem";


/**
 * The base url used to set the normal chart mode.
 */
ChartManager.SET_NORMAL_MODE_BASE_URL = "/?resource=Statistics&mode=set&command=gaugeMode&action=set&gaugeMode=normal&"


/**
 * The base url used to set the max chart mode.
 */
ChartManager.SET_MAX_MODE_BASE_URL = "/?resource=Statistics&mode=set&command=gaugeMode&action=set&gaugeMode=max&"


/**
 * The url used to load the chart mode. 
 */
ChartManager.GET_CHART_MODE_URL = "/?resource=Statistics&mode=get&command=gaugeMode&";


/**
 * The url used to remove a gauge mode.
 */
ChartManager.REMOVE_CHART_MODE_URL = "/?resource=Statistics&mode=set&command=gaugeMode&action=remove&"


/**
 * Together with fields value the value of these fields are used to create a charts title.
 */ 
var chartTitleFields = new Array();
chartTitleFields["summaryTable"] = "probeName";
chartTitleFields["topNProtocolsTable"] = "protocolName";
chartTitleFields["myProtocolsTable"] = "protocolName";
chartTitleFields["topNHostsTable"] = "hostName";
chartTitleFields["myHostsTable"] = "hostName";
chartTitleFields["topNHostListenersTable"] = "hostName";
chartTitleFields["topNHostTalkersTable"] = "hostName";
var topNConverstationIds = new Array();
topNConverstationIds[0] = "srcHostName";
topNConverstationIds[1] = "dstHostName";
chartTitleFields["topNConversationsTable"] = topNConverstationIds;
chartTitleFields["myConversationsTable"] = topNConverstationIds;
var myProtocolsPerHosIds = new Array();
myProtocolsPerHosIds[0] = "protocolName";
myProtocolsPerHosIds[1] = "hostName";
chartTitleFields["myProtocolsPerHostTable"] = myProtocolsPerHosIds;
chartTitleFields["protocolDetailTable"] = "protocolName";
chartTitleFields["protocolTopHostsTable"] = "hostName";
var protocolTopConversationsTableIds = new Array();
protocolTopConversationsTableIds[0] = "srcHostName";
protocolTopConversationsTableIds[1] = "dstHostName";
chartTitleFields["protocolTopConversationsTable"] = protocolTopConversationsTableIds;
chartTitleFields["nlhostDetailTable"] = "hostName";
chartTitleFields["nlhostTopProtocolsTable"] = "protocolName";
chartTitleFields["nlhostTopConversationsTable"] = protocolTopConversationsTableIds;
chartTitleFields["topNNetworksTable"] = "probeName";


/**
 * Maps the specified column name for each table with the corresponding column index in the applet.
 * This because we need to know which column to get the interval data for. 
 */ 
var appletColumnMap = new Array();
appletColumnMap["summaryTable"] = {protocolCount: "0", hostCount: "1", conversationCount: "2", cardCount: "3", packetCount: "4", octetCount: "5"};   
appletColumnMap["topNProtocolsTable"] = {octets: "1", packets: "0", hosts: "2", conversations: "3"};   
appletColumnMap["myProtocolsTable"] = {octets: "1", packets: "0", hosts: "2", conversations: "3"};   
appletColumnMap["topNHostsTable"] = {octets: "6", packets: "8", conversations: "7", protocols: "5"};   
appletColumnMap["myHostsTable"] = {inPackets: "0", outPackets: "3", inOctets: "2", outOctets: "3", octets: "6", packets: "8", conversations: "7", protocols: "5"};   
appletColumnMap["topNHostListenersTable"] = {inOctets: "2", inPackets: "0", protocols: "5", conversations: "7"};   
appletColumnMap["topNHostTalkersTable"] = {outOctets: "3", outPackets: "1", protocols: "5", conversations: "7"};   
appletColumnMap["topNConversationsTable"] = {packets: "0", octets: "1", protocols: "2"};   
appletColumnMap["myConversationsTable"] = {packets: "0", octets: "1", protocols: "2"};   
appletColumnMap["myProtocolsPerHostTable"] = {inPackets: "0", outPackets: "1", inOctets: "2", outOctets: "3"};
appletColumnMap["protocolDetailTable"] = {octets: "1", packets: "0", hosts: "2", conversations: "3"};
appletColumnMap["protocolTopHostsTable"] = {inPackets: "0", outPackets: "1", inOctets: "2", outOctets: "3"};
appletColumnMap["protocolTopConversationsTable"] = {packets: "0", octets: "1"};
appletColumnMap["nlhostDetailTable"] = {octets: "6", packets: "8", conversations: "7", protocols: "5"};
appletColumnMap["nlhostTopProtocolsTable"] = {inPackets: "0", outPackets: "1", inOctets: "2", outOctets: "3"};
appletColumnMap["nlhostTopConversationsTable"] = {packets: "0", octets: "1", protocols: "2"}; 
appletColumnMap["nlhostTopConversationsTable"] = {packets: "0", octets: "1", protocols: "2"};
appletColumnMap["topNNetworksTable"] = {protocolCount: "0", hostCount: "1", conversationCount: "2", cardCount: "3", packetCount: "4", octetCount: "5"};


// construction


/**
 * The single instance of this class. 
 */
ChartManager.instanceObject = new ChartManager();


/**
 * Constructs a chartmanager.
 */
function ChartManager() {
	
	
	/**
	 * True if the chart manager has been inited yet.
	 */	
	this.inited = false;
	
	
	/**
	 * The slider used to let the user set the max value.
	 */
	this.slider;
	
	
	/**
	 * The select to select the max unit in the set chart mode window.
	 */
	this.maxUnit;
	
	
	/**
	 * The button used to set the max value in the set chart mode window.
	 */
	this.setMaxButton;
	
	
	/**
	 * The container used to hold messages in the set chart mode window.
	 */
	this.modeMessageContainer;
	
	
	/**
	 * The normal mode radio button in the chart mode window.
	 */
	this.normalRadio;
	
	
	/**
	 * The max mode radio button in the chart mode window.
	 */
	this.maxRadio;
	
	
	/**
	 * When the chart mode has been changed we restart updating of the chart to make the effects of the 
	 * mode change visible at once. This is the id of the chart the mode window is for.
	 */
	this.chartIdMode;
	
	
	/**
	 * The html code of the set chart mode.
	 */
	this.chartMode;
	
	
	/**
	 * The window the chart is in for which the chart mode has been shown
	 */
	this.chartModeWindow;
	 
	 
	/**
	 * The event listener used. 
	 */
	this.eventListener = new Listener(this);
	
	 
	// set the events to listen to, when any action is taken with a window the chart mode setup is removed
	this.eventListener.addEvent(WINDOW_COLLAPSED_EVENT, hideChartMode);
	this.eventListener.addEvent(WINDOW_EXPANDED_EVENT, hideChartMode);
	this.eventListener.addEvent(WINDOW_SHOWN_EVENT, hideChartMode);
	this.eventListener.addEvent(WINDOW_HIDDEN_EVENT, hideChartMode);
	this.eventListener.addEvent(WINDOW_ACTIVATED_EVENT, hideChartMode);
	this.eventListener.addEvent(WINDOW_DEACTIVATED_EVENT, hideChartMode);
	this.eventListener.addEvent(WINDOW_CLOSED, processWindowClosedChartManager);
	this.eventListener.addEvent(WINDOW_MINIMIZED, hideChartMode);
	this.eventListener.addEvent(WINDOW_MAXIMIZED, hideChartMode);
	this.eventListener.addEvent("chartDetached", hideChartMode);
	this.eventListener.addEvent("chartAttached", hideChartMode);
	this.eventListener.addEvent("chartHidden", hideChartMode);
	this.eventListener.addEvent("chartShown", hideChartMode);
	this.eventListener.addEvent("chartClosed", processChartClosedEventChartManager);
	this.eventListener.addEvent("resized", hideChartMode);
	this.eventListener.addEvent("chartExpanded", hideChartMode);
	this.eventListener.addEvent("chartColapsed", hideChartMode);
	// while the layout is set no charts are to be updated, to prevent multiple request
	this.eventListener.addEvent(START_SET_LAYOUT_EVENT, cancelChartLoading);
	this.eventListener.addEvent(LAYOUT_SET_EVENT, restartChartLoading);
	// while the current probe is switched no charts are to be updated, when the new probe set
	// could not be contacted and the chart updates before we detect this the charts could load
	// incorrect or no data
	this.eventListener.addEvent(START_SWITCH_PROBE_EVENT, cancelChartLoading);
	this.eventListener.addEvent(DONE_SWITCH_PROBE_EVENT, restartChartLoading);
	this.eventListener.addEvent(START_SET_WINDOW_LAYOUT, processStartSetWindowLayoutEvent);
	this.eventListener.addEvent(WINDOW_LAYOUT_SET, processWindowLayoutSetEvent);
	
	
	/**
	 * The array containing the registered charts. 
	 */
	this.charts = new Array();
	
	
	/**
	 * The chart loading status, if this is set to true chart should load, if this is set to false charts
	 * should not uodate. 
	 */
	this.chartLoadStatus = true;
	
	
	/**
	 * The charts in the windows with ids in this table are not to be updated. When window layouts are set
	 * we do not update the windows charts to prevent multiple requests to the server. Window layouts can 
	 * be set after the layout set event has been fired. Hidden windows for example do not get their layout 
	 * set until they are activated.  
	 */
	this.stopChartLoading = new Array();
}


/**
 * Gets the single instance of this class in use. 
 */
ChartManager.instance = function() {
	if (!this.instanceObject.inited) {
		this.instanceObject.init();
	}
	return this.instanceObject;
}
 
 
// services



/**
 * Set the loading status of the charts. When this is set to false the charts should not update, 
 * set to true and they will be loaded.
 * @param chartLoadStatus  The chart load status to set. 
 */
ChartManager.prototype.setLoadChartStatus = function(chartLoadStatus) {
	this.chartLoadStatus = chartLoadStatus;
}


/**
 * Get the chart loading status. 
 * @param windowId The id of the window the chart is in. 
 * @return The chart load status. 
 * @throws Error if windowId is undefined, null or empty.
 */
ChartManager.prototype.getLoadChartStatus = function(windowId) {
	assertExists(windowId, "WindowId is undefined null or empty");
	var settingWindowLayout = exists(this.stopChartLoading[windowId]) ? this.stopChartLoading[windowId] : false;
	if (this.chartLoadStatus == false || settingWindowLayout == true) {
		return false;
	}
	return true;
}


/**
 * Register a new chart.
 * @param id The id of the chart.
 * @param chart The chart object to register.
 * @throws Error if the id is null, not defined or empty.
 * @throws Error if the chart is not defined or null.
 */
ChartManager.prototype.registerNewChart = function(chart) {
	assertDefined(chart, "Chart is null or not defined");
	this.charts[this.charts.length] = chart;
}


/**
 * Removed a chart.
 * @param chart The chart instance to remove.
 */
ChartManager.prototype.removeChart = function(chart) {
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (chart == this.charts[i]) {
			this.charts[i] = null;
			this.charts = removeNullEntries(this.charts);
		}
	}
}


/**
 * Gets the charts currenlty registered. 
 * @return A array containing the current registered charts. 
 */
ChartManager.prototype.getCharts = function() {
	return this.charts;
}


/**
 * Gets a chart.
 * @param id The id of the chart to get.
 * @return The chart, null if an chart with this name not exists.
 * @throws Error if the id is null, not defined or empty.
 */
ChartManager.prototype.getChart = function(id) {
	assertNotEmpty(id, "Id is null, undefined or empty");
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (isDefined(this.charts[i].htmlCode) && id == this.charts[i].getId()) {
			return this.charts[i];
		}
	}
	return null;
}


/**
 * Gets a chart container. 
 * @param id the id of one of the charts in the chart container. 
 * @return the chart, null if a container containing a chart with id does not exist.
 * @throws Error if the id is null, not defined or empty.
 */
ChartManager.prototype.getChartContainer = function(id) {
	assertNotEmpty(id, "Id is null, undefined or empty");
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (Chart.CHART_CONTAINER != this.charts[i].getType() && Chart.LINE_CHART_CONTAINER != this.charts[i].getType()) {
			continue;
		}
		
		if (this.charts[i].containsChart(id)) {
			return this.charts[i];			
		}
	}
	return null;
}


// chart creation


/**
 * Used by the table to create chart urls. This function is called as a method of the 
 * table object by the table.
 * @param linkNode the link created by the table.
 * @param field The field the link is for.
 */
function createUrl(linkNode, field) {
	linkNode.onclick = createChart;
	addEmbeddedChartRequest(field, linkNode, this);
}


/**
 * Creates a chart based on a block of xml.
 * @param chartXml The block of xml to create the chart from.
 * @param window The window the chart is to be in.
 * @return The chart created.
 */
function createChartXml(chartXml, window) {
	var tableName = getTagNodeValue("table", chartXml);
	if (isEmpty(tableName)) {
		// missing or empty table name for the chart
		return;
	}
	if (Chart.INTERVAL_CHART == getChartTypeXml(chartXml)) {
		return createIntervalChartXml(chartXml, window);
	}
	else if (Chart.CHART_CONTAINER == getChartTypeXml(chartXml)) {
		return createChartContainerXml(chartXml, window);
	}
}


/**
 * Creates a interval based on block of xml.
 * @param intervalChartXml The xml data to create the interval chart from.
 * @window The window the chart is to be in. 
 * @return The created interval chart. 
 */
function createIntervalChartXml(intervalChartXml, window) {
	var tableId = getTagNodeValue("table", intervalChartXml);
	var chartTitle = URLDecode(getTagNodeValue("title", intervalChartXml));	
	var intervalChartCode = createLineChartCode(getTagNodeValue("chartId", intervalChartXml));

	var intervalChart = createLineChart(intervalChartCode, chartTitle, window, tableId);
	setEntryIdXml(intervalChartXml, intervalChart);
	return intervalChart;
}


/**
 * Creates a chart container from a block of xml.
 * @param chartContainerXml The xml data to create the chart container from. 
 * @param window The window the chart is to be appended to.  
 */
function createChartContainerXml(chartContainerXml, window) {
	var tableId = getTagNodeValue("table", chartContainerXml);
	var chartTitle = URLDecode(getTagNodeValue("title", chartContainerXml));	
	var chartContainerCode = createChartContainerCode(chartTitle, getTagNodeValue("chartId", chartContainerXml));
	setClass(chartContainerCode, "inactive");
	
	var chartContainer = createChartContainer(chartContainerCode, chartTitle, window, tableId);
	setEntryIdXml(chartContainerXml, chartContainer);
	return chartContainer;
}


/**
 * Called when a link has been clicked, creates the chart for the clicked link based on the embedded 
 * chart request. 
 */
function createChart() {
	var embeddedRequest = getTekstValue(getElementWithAttribute(this, "embeddedChartRequest", "name", "div"));
	var chart;
	if (getWindowRequest(embeddedRequest).isAttachedChartTitle(createChartTitleRequest(embeddedRequest))) {
		getWindowRequest(embeddedRequest).activateChartWithTitle(createChartTitleRequest(embeddedRequest));		
	}
	else if (Chart.INTERVAL_CHART == getChartTypeEmbeddedRequest(embeddedRequest)) {
		chart = createIntervalChartRequest(embeddedRequest);
	}
	else if (Chart.CHART_CONTAINER == getChartTypeEmbeddedRequest(embeddedRequest)) {
		chart = createChartContainerRequest(embeddedRequest);
	}
		
	if (exists(chart)) {
		setChartSize(chart, getWindowRequest(embeddedRequest));
		attachChartToWindow(chart, getWindowRequest(embeddedRequest));
	}	
}


/**
 * Creates a interval chart based on a embedded request. 
 * @param embeddedRequest The request to create the chart from. 
 * @return The created interval chart.
 */
function createIntervalChartRequest(embeddedRequest) {
	var tableId = getParameterValue(embeddedRequest, "tableId", "&");
	var chartTitle = createChartTitleRequest(embeddedRequest);
	var intervalChartCode = createLineChartCode(createChartIdRequest(embeddedRequest));

	var intervalChart = createLineChart(intervalChartCode, chartTitle, getWindowRequest(embeddedRequest), tableId);
	setEntryIdEmbeddedRequest(embeddedRequest, intervalChart);
	return intervalChart;
}


/**
 * Creates a chart container containing a line and gauge chart based on a embedded request. 
 * @param embeddedRequest The request to create the charts from.
 * @return The created chart container. 
 */
function createChartContainerRequest(embeddedRequest) {
	var tableId = getParameterValue(embeddedRequest, "tableId", "&");
	var chartTitle = createChartTitleRequest(embeddedRequest);
	var chartContainerCode = createChartContainerCode(chartTitle, createChartIdRequest(embeddedRequest));
	setClass(chartContainerCode, "inactive");
	
	var chartContainer = createChartContainer(chartContainerCode, chartTitle, getWindowRequest(embeddedRequest), tableId);
	setEntryIdEmbeddedRequest(embeddedRequest, chartContainer);
	return chartContainer;
}


/**
 * Sets the size of a new chart. The height is set to the default height, the
 * width according to the window.  
 * @param chart The chart to resize. 
 * @param window The window to set the size according to. 
 */
function setChartSize(chart, window) {
	var height = Chart.DEFAULT_HEIGHT;
	height = Chart.CHART_CONTAINER == chart.getType() ? Chart.CHART_CONTAINER_DEFAULT_HEIGHT : height;
	chart.setHeight(height);
	chart.setWidth(parseInt(window.getContainer().style.width));
}


/**
 * Get the field from a block of xml data. 
 * @param xml The xml to get the field from.
 * @return The field from the xml data, -1 if no field in the request.  
 */
function getFieldXml(xml) {
	var field = getTagNodeValue("field", xml);
	if (isEmpty(field)) {
		return "-1";
	}
	return field;
}


/**
 * Get the window from a embedded request. 
 * @param embeddedRequest The request to get the window from.
 * @return The window the chart is to be appended to.
 */
function getWindowRequest(embeddedRequest) {
	var windowId = getParameterValue(embeddedRequest, "windowId", "&");
	return WindowManager.instance().getWindow(windowId);
}

 
/**
 * Get the chart title from a embedded request.
 * @param embeddedRequest The embedded request to get the title from.
 * @return The title to set for the chart. 
 */
function createChartTitleRequest(embeddedRequest) {
	var chartTitle = getParameterValue(embeddedRequest, "chartTitle", "&");
	return URLDecode(chartTitle);	
}


/**
 * Get the chart id from a embedded request. 
 * @param embeddedRequest The embedded request to get the id from. 
 * @return The id to use for the chart. 
 */
function createChartIdRequest(embeddedRequest) {
	return createChartTitleRequest(embeddedRequest) + ":" + uniqueInteger(); 
}


/**
 * Get the chart type from a block of xml.
 * @param chartXml The xml to get the chart type from.
 * @return The chart type value in the xml data. 
 */
function getChartTypeXml(chartXml) {
	var type = getTagNodeValue("type", chartXml);
	if (!isNumberString(type)) {
		// invalid value for chart type
		return;
	}
	return parseInt(type);
}


/**
 * Gets the chart type of a chart to be created based on a embedded request. 
 * @param embeddedRequest The embedded request to create the chart from. 
 * @return The type of the chart to be created (Chart.CHART_CONTAINER, Chart.INTERVAL_CHART).
 */
function getChartTypeEmbeddedRequest(embeddedRequest) {
	var chartTitle = getParameterValue(embeddedRequest, "chartTitle", "&");
	var windowId = getParameterValue(embeddedRequest, "windowId", "&");
	if (((chartTitle.indexOf("Packets") > -1) || (chartTitle.indexOf("Bytes") > -1)) && windowId == "summarywindow") {
		return Chart.CHART_CONTAINER;
	}
	else if (((chartTitle.indexOf("Packets") > -1) || (chartTitle.indexOf("Bytes") > -1)) && windowId == "topnNetworksWindow") {
		return Chart.CHART_CONTAINER;
	}
	return Chart.INTERVAL_CHART;
}



/**
 * Set the entry id in the chart (src, dst, protocol, field) based on some xml data. 
 * @param xml The xml data to get the values to set from.
 * @param chart The chart to set the values in.  
 */
function setEntryIdXml(xml, chart) {
	chart.setSrcAddress(getTagNodeValue("src", xml));
	chart.setDstAddress(getTagNodeValue("dst", xml));
	chart.setProtocol(getTagNodeValue("protocol", xml));
	chart.setField(parseInt(getFieldXml(xml)));
}


/**
 * Set the entry id in the chart (src, dst, protocol, field) based on a embedded request. 
 * @param embeddedRequest The embedded request to get the values to set from.
 * @param chart The chart to set the values in.  
 */
function setEntryIdEmbeddedRequest(embeddedRequest, chart) {
	chart.setSrcAddress(getParameterValue(embeddedRequest, "src", "&"));
	chart.setDstAddress(getParameterValue(embeddedRequest, "dst", "&"));
	chart.setProtocol(getParameterValue(embeddedRequest, "protocol", "&"));
	chart.setField(parseInt(getParameterValue(embeddedRequest, "field", "&")));
}


/**
 * Creates the html code of a chart container and the images it contains. 
 * @param title The title to set for the chart. 
 * @param id The id to set for the chart. 
 * @return The created html code for the chart container. 
 */
function createChartContainerCode(title, id) {
	// create the container 	
	var chartContainer = createContainer(id, "chartContainer");
	// we set the size this way due to problems with early firefox versions
	chartContainer.setAttribute("style", "height: 0px; width: 0px");
	
	// create the line chart
	var lineChartContainer = createContainer("line", "lineChartContainer");
	chartContainer.appendChild(lineChartContainer);
	lineChartContainer.setAttribute("name", "lineChartContainer");
	// the ids mus be unique
	var lineChart = createImageTag("line " + title + uniqueInteger(), "");
	setClass(lineChart, "chart");
	lineChartContainer.appendChild(lineChart);

	// create the gauge chart
	var gaugeChartClass = isIE() ? "ieGaugeChartContainer" : "gaugeChartContainer";
	var gaugeChartContainer = createContainer("gauge", gaugeChartClass);
	gaugeChartContainer.oncontextmenu = showSetChartMode;
	gaugeChartContainer.setAttribute("name", "gaugeChartContainer");
	chartContainer.appendChild(gaugeChartContainer);
	// the ids mus be unique
	var gaugeChart = createImageTag("gauge " + title + uniqueInteger(), "");
	setClass(gaugeChart, "chart");
	gaugeChartContainer.appendChild(gaugeChart);
	return chartContainer;
}


/**
 * Creates and returns a container for a line or gauge chart.
 * @param name The name to set for the container.
 * @param className The name of the class to set for the container.
 */
function createContainer(name, className) {
	var container = document.createElement("div");
	container.setAttribute("name", name);
	setClass(container, className);
	return container;
}


/**
 * Creates the html code for a line chart.
 * @param id The id to set for the tag. 
 */
function createLineChartCode(id) {
	var container = createContainer(id, "chartContainer");
	container.setAttribute("name", id);
	var imageTag = createImageTag(id, "");
	setClass(imageTag, "chart");
	container.appendChild(imageTag);
	setClass(container, "inactive");
	return container;
}


/**
 * Creates and returns a image tag.
 * @param id The id to set for the image.
 * @param src The src to set for the image. 
 */
function createImageTag(id, src) {
	var imageTag = document.createElement("img");
	imageTag.setAttribute("id", id);
	imageTag.src = src;
	return imageTag;
}	


/**
 * Attaches a chart to a window.
 * @param chart The Chart to attach.
 * @param window The window to attach to.
 */
function attachChartToWindow(chart, window) {
	// don't attach a new chart if it already exists, we use the title to identify
	// the charts because the id is unique for each chart
	if (!window.isAttachedChartTitle(chart.getTitle())) {
		// chart does not exist in the window, attach and show the chart
		window.appendChart(chart);
		chart.showLoading();
		return;
	}
	
	window.activateChartWithTitle(chart.getTitle());
	if (Chart.TOPN_CHART != chart.getType()) {
		// if no topn chart remove it from the charts because this is a extra instance of the same
		// chart, there is only one instance of the topn chart so do not remove it
		ChartManager.instance().removeChart(chart);
	}
}


/**
 * Adds a embedded chart request to a link. We use embedded chart requests to avoid using javascript
 * closures which can cause memory leaking in IE. The embedded request contains everything needed
 * to create the chart when the link is clicked, and is in the format: key1=value1&key2=value2&
 * @param field The field to add the link request for.
 * @param linkTag The actual link tag to add the request to.
 * @param table The table the link is in.
 */	
function addEmbeddedChartRequest(field, linkTag, table) {
	var embeddedRequest = createEmbeddedChartRequest(field, table.getRowFromField(field), table, table.getWindow());
	if (table.getRowId) {
		// the table is one of the myTables, in myTables entries might have index -1
		// when the entry does not exist at the server side, once the entry gets some traffic
		// the index is changed, to update the index in the charts url we need to know which row 
		// the chart is for, we do this by getting the value for the MyTables id fields, the id 
		// fields in a MyTable are unique for each row.
		var row = table.getRowFromField(field);
		embeddedRequest += "&rowId=" + table.getRowId(row) + "&";
	}

	var divTag = document.createElement("div");
	divTag.setAttribute("name", "embeddedChartRequest");
	divTag.style.display = "none";
	divTag.appendChild(document.createTextNode(embeddedRequest));
	linkTag.appendChild(divTag);
}


/**
 * To avoid using javascript closures (which can cause memory leaks in IE) we embedd a hidden text string
 * in the chart links in the table. The string contains everything we need to generate the chart
 * when it is clicked. The format of the string is key1=value1&key2=value2&...
 * @return The created request. 
 */
function createEmbeddedChartRequest(field, row, table, window) {
	var tableAppletColumnMap = appletColumnMap[table.id];
	var embeddedRequest = getEntryString(field, row);
	embeddedRequest += "windowId=" + window.getId() + "&";
	embeddedRequest += "tableId=" + table.getId() + "&";
	embeddedRequest += "field=" + tableAppletColumnMap[field.getName()] + "&";
	embeddedRequest += "chartTitle=" + createChartTitle(field, row, table.getHeaderRow(), table.getId()) + "&";
	embeddedRequest += "fieldName=" + field.getName() + "&";
	return embeddedRequest;
}


/**
 * Creates the entry id string (src,dst,protocol) for a field. 
 * @param field The field to get the entry string for.
 * @param row The row the field is in.  
 * @return The entry id fields as a string (protocol,src,dst).
 */
function getEntryString(field, row) {
	var entryString = "";
	var protocolField = getFieldFromRow("protocolId", row);
	var protocolValue = "0";
	if (!isNull(protocolField)) {
		protocolValue = protocolField.getValue();
	}
	var entryString = "&protocol=" + protocolValue + "&";
	
	var srcField = getFieldFromRow("address", row);
	srcField = isNull(srcField) ? getFieldFromRow("srcAddress", row) : srcField;
	srcField = isNull(srcField) ? getFieldFromRow("src", row) : srcField;
	var srcValue = "0";
	if (!isNull(srcField)) {
		srcValue = srcField.getValue();
	}
	entryString += "src=" + srcValue + "&";
	
	var dstField = getFieldFromRow("dstAddress", row);
	var dstValue = "0";
	if (!isNull(dstField)) {
		dstValue = dstField.getValue();
	}
	entryString += "dst=" + dstValue + "&";
	return entryString;
}


/**
 * Creates the title for a given chart and a given field. 
 * @param field The field to create the link id for.
 * @param row The row to the field is in.
 * @param headerRow The header row for the table. 
 * @param The name of the table the field is in.
 */
function createChartTitle(field, row, headerRow, tableId) {
	var id = "";
	
	if (!isDefined(row) || isNull(row)) {
		row = this.getRowFromField(field);
	}
	if (isNull(row)) {
		return id;
	}
	var idFieldName = chartTitleFields[tableId];
	if (idFieldName instanceof Array) {
		var firstIdField = getFieldFromRow(idFieldName[0], row);
		var secondIdField = getFieldFromRow(idFieldName[1], row);
		id = firstIdField.getValue() + " - " + secondIdField.getValue();
	}
	else {
		var idField = getFieldFromRow(idFieldName, row);	
		if(isDefined(idField) && !isNull(idField)) {
			id = idField.getValue();
		}
	}
	
	var headerValue = getFieldFromRow(field.getName(), headerRow).getValue();
	return id + ", " + headerValue;
}


// event


/**
 * Processes a window layout set event. When the layout for the window is set the charts can be updated againg. 
 * @param even The event to process. 
 */
function processWindowLayoutSetEvent(event) {
	this.stopChartLoading[event.getData()] = false;
}


/**
 * Processes a start setting window layout event, while a window layout is beeing set the windows
 * charts are not updated. 
 * @param event The event to process.
 */
function processStartSetWindowLayoutEvent(event) {
	this.stopChartLoading[event.getData()] = true;
}


/**
 * Cancels the loading of all the charts.  
 */
function cancelChartLoading(event) {
	this.setLoadChartStatus(false);
	
	// stop the updating of the charts
	for (var i = 0; i < this.charts.length; i++) {
		this.charts[i].cancelUpdate();
	}
}


/**
 * Loads all the charts in shown and detached windows. 
 */
function restartChartLoading(event) {
	this.setLoadChartStatus(true);
	// reload all charts in shown and detached windows that are not minimized
	for (var i = 0; i < this.charts.length; i++) {
		var chart = this.charts[i];
		if (chart.window.isShown() && !chart.window.isMinimized() && chart.window.getActiveChart() == chart) {	
			chart.restartUpdate();
		}
	}
}


// gauge chart mode


/**
 * Shows the set chart mode tag.
 */
function showSetChartMode(event) {
	event = exists(event) ? event : window.event;
	var element = ChartManager.instance().chartMode;
	// the height and the width of the chart mode setup are known
	var yPos = isYpositionInDesktop(event.clientY + 177) ? event.clientY : calculateMaxYPos() - 177;
	var xPos = isXPositionInDesktop(event.clientX + 252) ? event.clientX : calculateMaxXPos() - 252;
	ChartManager.instance().chartMode.style.left = xPos + "px";
	ChartManager.instance().chartMode.style.top = yPos + "px";
	setClass(ChartManager.instance().chartMode, "setChartMode");
	
	// hide the byte unit chooser if the chart is not a byte chart
	if (this.parentNode.getAttribute("name").indexOf("Bytes") > -1) {
		setClass(ChartManager.instance().maxUnit, "active");
	}
	else {
		setClass(ChartManager.instance().maxUnit, "inactive");
	}
	
	// set the id of the chart which has been clicked, the element clicked is the div tag
	// containing the gauge chart, this divs tag parent element has the id of the chart
	ChartManager.instance().chartIdMode.value = this.parentNode.getAttribute("name");
	ChartManager.instance().loadChartMode();
	
	var chartWindow = ChartManager.instance().getChart(ChartManager.instance().chartIdMode.value).window;
	ChartManager.instance().chartModeWindow = chartWindow;
	if (!chartWindow.containsMoveElement(ChartManager.instance().chartMode)) {
		chartWindow.addMoveElement(ChartManager.instance().chartMode);
	}
}


/**
 * A window has been closed, remove the max value for the chart if the window
 * contains a detached byte or packet chart.
 * @param event The event to process.  
 */
function processWindowClosedChartManager(event) {
	var chart = event.getData().getActiveChart();
	var modeChart = chart.getTitle().indexOf("Bytes") > -1 || chart.getTitle().indexOf("Packets") > -1;
	var gaugeChart = chart.getType() == Chart.CHART_CONTAINER; 
	if (!modeChart || !gaugeChart) {
		return;
	}
	loadXML(ChartManager.instance().createRemoveModeUrl(chart.getId()));
}

 
/**
 * A chart has been closed, remove the max value for the chart.
 * @param event The event to process. 
 */
function processChartClosedEventChartManager(event) {
	hideChartMode(event);
	var chart = event.getData();
	var modeChart = chart.getTitle().indexOf("Bytes") > -1 || chart.getTitle().indexOf("Packets") > -1;
	var gaugeChart = chart.getType() == Chart.CHART_CONTAINER; 
	if (!modeChart || !gaugeChart) {
		return;
	}
	
	// remove the max mode that might be set for the chart
	loadXML(ChartManager.instance().createRemoveModeUrl(chart.getId()));
}


/**
 * The close button in the set chart mode tag has been clicked, close it.
 */
function hideChartMode(event) {
	if (!ChartManager.instance().chartModeSetupShown()) {
		return;
	}
	
	ChartManager.instance().hideChartMode();	
	var id = ChartManager.instance().chartIdMode.value;
	if (exists(ChartManager.instance().chartModeWindow)) {
		ChartManager.instance().chartModeWindow.removeMoveElement(ChartManager.instance().chartMode);
		ChartManager.instance().chartModeWindow = null;
	}
}


/**
 * Hides the set chart mode tag.
 */
ChartManager.prototype.hideChartMode = function() {
	setClass(this.chartMode, "inactive");
}


/**
 * Returns true if the chart mode setup is shown, false otherwise.
 */
ChartManager.prototype.chartModeSetupShown = function() {
	if ("inactive" == getClassName(this.chartMode)) {
		return false;
	}
	return true;
}


/**
 * Loads the chart mode. 
 */
ChartManager.prototype.loadChartMode = function() {
	var url = ChartManager.GET_CHART_MODE_URL + "&sid=" + getSid() + "&chartId=" + this.chartIdMode.value;
	var chart = isEmpty(this.chartIdMode.value) ? null : this.getChart(this.chartIdMode.value);
	loadXML(url, "modeLoaded", this);
}


/**
 * The chart mode has been loaded, parse the reply from the server and set the
 * mode in the chart mode value. 
 * @param response The response from the server. 
 */
ChartManager.prototype.modeLoaded = function(response) {
	var mode = getTagNodeValue("mode", response);
	if ("normal".equalsIgnoreCase(mode)) {
		this.normalRadio.checked = true;
		this.normalSelected(false);
	}
	else {
		this.maxRadio.checked = true;
		this.maxSelected();
		this.setMaxValue(getTagNodeValue("maxValue", response));
	}
}


/**
 * Sets the max value and the max unit according to a given amount of bits. 
 * @param value The max value to set. 
 */
ChartManager.prototype.setMaxValue = function(value) {
	if (isEmpty(value) || !isNumberString(value)) {
		return;
	}
	// Gbits
	if (value > (1000 * 1000 * 1000)) {
		value = value / (1000 * 1000 * 1000);
		this.maxUnit.selectedIndex = 3;
	} 
	// Mbits
	else if (value > (1000 * 1000)) {
		value = value / (1000 * 1000);
		this.maxUnit.selectedIndex = 2;
	}
	// Kbits
	else if (value > 1000) {
		value = value / 1000;
		this.maxUnit.selectedIndex = 1;
	}
	else {
		this.maxUnit.selectedIndex = 0;
	}
	this.slider.setValue(parseInt(value));
}


/**
 * The normal mode has been selected in the chart right click menu.
 * @send Set this to false to not send a request to the server with normal mode. 
 */
ChartManager.prototype.normalSelected = function(send) {
	this.normalRadio.checked = true;
	this.maxRadio.checked = false;
	this.slider.toggleDisabled(true, "disabledChartModeSlider");
	this.maxUnit.selectedInde = 0;
	this.maxUnit.disabled = true;
	this.setMaxButton.disabled = true;
	this.clearModeMessage();
	var url = this.createNormalModeUrl();
	if ((!exists(send) || send == true) && !isNull(url)) {
		loadXML(url, "modeSet", this);
	}
}


/**
 * The max mode has been selected in the chart right click menu.
 */
ChartManager.prototype.maxSelected = function() {
	this.maxRadio.checked = true;
	this.normalRadio.checked = false;
	this.slider.toggleDisabled(false);
	this.maxUnit.disabled = false;
	this.setMaxButton.disabled = false;
	this.clearModeMessage();
}


/**
 * Set the max mode. 
 */
ChartManager.prototype.setMaxMode = function() {
	var url = this.createMaxModeUrl();
	if (!isNull(url)) {
		loadXML(url, "modeSet", this);
	}
}


/**
 * Callback function for the setting of the chart mode. 
 * @respone The response from the server. 
 */
ChartManager.prototype.modeSet = function(response) {
	if (containsErrorMessage(response)) {
		this.setModeMessage(undefined, getErrorMessage(response));
		return;
	}	
	this.setModeMessage(getSuccessMessage(response), undefined);
	var chart = this.getChart(this.chartIdMode.value);
	if (exists(chart)) {
		chart.restartUpdate();
	}
}


/**
 * Sets a message in the set mode window. 
 * @param success The success message to show.
 * @param error The error message to show.
 */
ChartManager.prototype.setModeMessage = function(success, error) {
	if (isDefined(success) && !isEmpty(success)) {
		setClass(this.modeMessageContainer, "ok");
		this.modeMessageContainer.innerHTML = success;
	}
	else if (isDefined(error) && !isEmpty(error)) {
		setClass(this.modeMessageContainer, "error");
		this.modeMessageContainer.innerHTML = error;
	}
}


/**
 * Clears the mode message.
 */
ChartManager.prototype.clearModeMessage = function() {
	this.modeMessageContainer.innerHTML = "";
}


/**
 * Creates the normal mode url.
 */
ChartManager.prototype.createNormalModeUrl = function() {
	return ChartManager.SET_NORMAL_MODE_BASE_URL + "&sid=" + getSid() + "&chartId=" + this.chartIdMode.value;
}


/**
 * Creates the max mode url.
 */
ChartManager.prototype.createMaxModeUrl = function() {
	try {
		var url = ChartManager.SET_MAX_MODE_BASE_URL + "&sid=" + getSid() + "&" + this.maxUrl() + "chartId=" + this.chartIdMode.value + "&"; 
		var chart = isEmpty(this.chartIdMode.value) ? null : this.getChart(this.chartIdMode.value);
		return url;
	}
	catch (e) {
		this.setModeMessage(undefined, e.message);
		return null;
	}
}


/**
 * Creates the remove chart mode url.
 * @param id The id of the chart to remove the mode for. 
 * @return The remove chart mode url.
 */
ChartManager.prototype.createRemoveModeUrl = function(id) {
	return ChartManager.REMOVE_CHART_MODE_URL + "&sid=" + getSid() + "&" + "chartId=" + id;
}
 
 
/**
 * Creates the max value oart of a set max mode url.
 * @return The max part of a set max url.
 * @throws Error if no max value is set. 
 */
ChartManager.prototype.maxUrl = function() {
	var maxValue = this.slider.getValue();
	if (!isNumber(maxValue)) {
		throw Error("Max value set is not a number");
	}
	if (maxValue <= 0) {
		throw Error("Max value must be larger than 0");
	}
	var unitFactor = this.maxUnit.options[this.maxUnit.selectedIndex].value;
	return "maxValue=" + (parseInt(maxValue) * unitFactor) + "&";
}


/**
 * Creates the set chart mode setup screen.
 */
ChartManager.prototype.createSetChartModeSetup = function() {
	this.chartMode = document.createElement("div");
	setClass(this.chartMode, "setChartMode");
	this.chartMode.setAttribute("id", ChartManager.CHART_RIGHT_CLICK_MENU_NAME);
	Desktop.instance().getDesktopCode().appendChild(this.chartMode);
	
	var table = document.createElement("table");
	table.style.width = "100%";
	table.setAttribute("cellpadding", "2");
	table.setAttribute("celspacing", "0");
	var tableBody = document.createElement("TBODY");
	table.appendChild(tableBody);
	this.chartMode.appendChild(table);
	
	var headerRow = table.insertRow(0);
	headerRow.style.width = "100%";
	var headerCell = headerRow.insertCell(0);
	headerCell.colSpan = "2";
	headerCell.style.width = "100%";
	headerCell.appendChild(this.createChartModeHeader());
	
	var normalRow = table.insertRow(1);
	var normalCell = normalRow.insertCell(0);
	normalCell.setAttribute("align", "left");
	this.normalRadio = document.createElement("input");
	this.normalRadio.setAttribute("value", "Normal");
	this.normalRadio.setAttribute("type", "radio");
	this.normalRadio.setAttribute("name", "chartMode");
	this.normalRadio.style.padding = "0px"
	this.normalRadio.style.margin = "0px"
	this.normalRadio.checked = true;
	var thisObject = this;
	this.normalRadio.onclick = function() {
		thisObject.normalSelected();
	}
	var normalTable = document.createElement("table");
	normalTable.setAttribute("cellpadding", "0");
	normalTable.setAttribute("celspacing", "0");
	var row = normalTable.insertRow(0);
	var cell = row.insertCell(0);
	cell.setAttribute("align", "left");
	cell.appendChild(this.normalRadio);
	cell = row.insertCell(1);
	cell.appendChild(document.createTextNode("Normal"));
	normalCell.appendChild(normalTable);
	
	var maxRow = table.insertRow(2);
	var maxCell = maxRow.insertCell(0);
	this.maxRadio = document.createElement("input");
	this.maxRadio.setAttribute("type", "radio");
	this.maxRadio.setAttribute("value", "Max");
	this.maxRadio.setAttribute("name", "chartMode");
	this.maxRadio.style.padding = "0px"
	this.maxRadio.style.margin = "0px"
	this.maxRadio.onclick = function() {
		thisObject.maxSelected();
	}
	var maxTable = document.createElement("table");
	maxTable.setAttribute("cellpadding", "0");
	maxTable.setAttribute("celspacing", "0");
	var row = maxTable.insertRow(0);
	var cell = row.insertCell(0);
	cell.setAttribute("align", "left");
	cell.appendChild(this.maxRadio);
	cell = row.insertCell(1);
	cell.appendChild(document.createTextNode("Max"));
	maxCell.appendChild(maxTable);	
	
	var sliderRow = table.insertRow(3);
	var sliderCell = sliderRow.insertCell(0);
	this.slider = new Slider(1000);
	this.slider.setParent(sliderCell);
	this.slider.init("chartModeSlider");
	this.slider.toggleDisabled(true, "disabledChartModeSlider");
	
	var maxUnitCell = sliderRow.insertCell(1);
	this.maxUnit = document.createElement("select");
	this.maxUnit.setAttribute("name", "maxUnit");
	this.maxUnit.add(new Option("bit/s", 1), undefined);
	this.maxUnit.add(new Option("Kbit/s", 1000), undefined);
	this.maxUnit.add(new Option("Mbit/s", 1000000), undefined);
	this.maxUnit.add(new Option("Gbit/s", 1000000000), undefined);
	this.maxUnit.selectedIndex = 0;
	this.maxUnit.disabled = true;
	maxUnitCell.appendChild(this.maxUnit);
	
	var setMaxButtonRow = table.insertRow(4);
	var setMaxButtonCell = setMaxButtonRow.insertCell(0);
	setMaxButtonCell.colSpan = "2";
	setMaxButtonCell.setAttribute("align", "right");
	this.setMaxButton = document.createElement("input");
	this.setMaxButton.setAttribute("type", "button");
	this.setMaxButton.setAttribute("name", "setMaxValue");
	this.setMaxButton.setAttribute("value", "Set max value");
	this.setMaxButton.disabled = true;
	this.setMaxButton.style.marginBottom = "5px";
	this.setMaxButton.onclick = function() {
		thisObject.setMaxMode();
		thisObject.clearModeMessage();
	}
	setMaxButtonCell.appendChild(this.setMaxButton);
	
	var messageRow = table.insertRow(5);
	messageRow.setAttribute("id", "chartModeMessageRow");
	var messageCell = messageRow.insertCell(0);
	messageCell.setAttribute("id", "chartModeMessageCell");
	messageCell.colSpan = "2";
	this.modeMessageContainer = document.createElement("div");
	this.modeMessageContainer.style.height = "15px";
	messageCell.appendChild(this.modeMessageContainer);
	
	var chartIdRow = table.insertRow(6);
	var chartIdCell = chartIdRow.insertCell(0);
	this.chartIdMode = document.createElement("input");
	this.chartIdMode.setAttribute("type", "text");
	setClass(this.chartIdMode, "inactive");
	chartIdCell.appendChild(this.chartIdMode);
	
	setClass(this.chartMode, "inactive");
}


/**
 * Shows the help section for the set chart mode window.
 */
function showSetChartModeHelp() {
	showHelp("doc/index.html?page=files%2Fch03s04.html");
}


/**
 * Creates the header html code.
 */
ChartManager.prototype.createChartModeHeader = function() {
	var header = document.createElement("table");
	var headerTableBody = document.createElement("TBODY");
	header.appendChild(headerTableBody);
	setClass(header, "chartModeHeader");
	
	var row = document.createElement("tr");
	var textCell = document.createElement("td");
	headerTableBody.appendChild(row);
	row.appendChild(textCell);
	textCell.appendChild(document.createTextNode("Set the chart mode"));
	
	var helpCell = document.createElement("td");
	row.appendChild(helpCell);
	helpCell.style.color = "#5f6266";
	helpCell.style.fontSize = "12px";
	helpCell.style.cursor = "pointer";
	helpCell.setAttribute("align", "right");
	helpCell.appendChild(document.createTextNode("?"));
	helpCell.onmouseover = function() {
		this.style.color = "black";
		this.style.fontWeight  = "bold";
		Tip(helptext('setChartMode'), BGCOLOR, '#ffffe1', ABOVE, true, DELAY, 500, DELAY, 10000);
	}
	helpCell.onmouseout = function() {
		this.style.color = "#5f6266";
		this.style.fontWeight  = "normal";
	}
	helpCell.onclick = showSetChartModeHelp;
		
	var buttonCell = document.createElement("td");
	row.appendChild(buttonCell);
	buttonCell.setAttribute("align", "left");
	buttonCell.style.width = "10px";
	var closeButton = document.createElement("div");
	closeButton.setAttribute("title", "Close");
	setClass(closeButton, "closeChartMode");
	closeButton.onclick = hideChartMode;
	closeButton.onmouseover = function() {
		setClass(this, "closeChartModeMouseover");
	}
	closeButton.onmouseout = function() {
		setClass(this, "closeChartMode");
	}
	buttonCell.appendChild(closeButton);
	
	return header;
}


// implementation


/**
 * Initalizes the chart manager and sets everything up.
 */
ChartManager.prototype.init = function() {
	this.createSetChartModeSetup();
	this.inited = true;
}	