/**
* This class managesand updates a topn chart for a mytable. 
*/


/**
 * Create a instance of this class. A mychart is a subclass
 * of Chart.
 * @param chartCode The charts html code.
 * @param title The title of this chart.
 * @param window The window the chart is in.
 * @param tableId The id of the table this chart is for.
 * @return A instance of this class.
 * @throws Error if chartCode is undefined or null.
 * @throws Error if title is undefined, null or empty.
 * @throws Error if window is undefined or null.
 * @throws Error if tableId is undefined, null or empty.
 */
function createMyChart(chartCode, title, window, tableId) {
	assertExists(chartCode, "The charts html code is undefined or null");
	assertNotEmpty(title, "Title for chart is undefined, null or empty: " + title);
	assertExists(window, "The window the chart container is in is undefined or null");
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);

	var superInstance = createTopNChart(chartCode, title, window, tableId); 
	var subInstance = new MyChart(chartCode, tableId);
	subInstance = createSubClass(superInstance, subInstance);
	
	ChartManager.instance().removeChart(superInstance);
	ChartManager.instance().registerNewChart(subInstance);
	
	// let the event listener call event handler functions as a method of
	// the mychart object instead of the topn chart super class object 
	superInstance.eventListener.changeObject(subInstance);
	superInstance.eventListener.addEvent("tableUpdated", processUpdatedEventMyChart);
	
	return subInstance;
}


// construction


/**
 * Constructs a my chart.
 * @param image The image tag for this chart.
 * @param tableId The id of the table this chart is for.
 * @throws Error if tableId is undefined, null or empty.
 */
function MyChart(image, tableId) {
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);
	
	this.htmlCode = image;
	this.tableId = tableId;
	// from the field with this name in the table the sample values are generated
	this.dataFieldName = "";
	
	// the number of rows in the table this is used to resize the chart if needed
	this.rowCount = 0;
	
	// the height of each bar in the chart
	this.barHeight = 0;
	
	// if this is true the chart will be updated
	this.updating = false;
	
	// if this is true the table this chart is for has loaded data once.
	this.tableDataLoaded = false;
}	


// event handling


/**
 * Processes a updated event from the table.
 * @param event The updated event to process.
 */
function processUpdatedEventMyChart(event) {
	if (event.getData()["id"] != this.tableId) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	if (this.rowCount == 0) {
		this.rowCount = event.getData()["newRowCount"];	
	}
	if (this.barHeight == 0 || this.barHeight == TopNChart.DEFAULT_BAR_HEIGHT && this.tableDataLoaded) {
		this.barHeight = this.calculateBarHeight(this.rowCount, this);
	}
	this.rowCount = event.getData()["newRowCount"];	
	var chartHeightDiff = this.calculateChartHeightDiff(this.barHeight, this.rowCount, this);
	if (chartHeightDiff != 0) {
		this.resize(chartHeightDiff, this); 
	}
	
	if (this.updating) {
		this.updateChart();
	}
	else if (this.window.getActiveChart() == this) {
		this.startUpdate();
	}
	this.tableDataLoaded = true;
}


/**
 * Create a my chart url.
 * @return The new creatd topn url.
 */
MyChart.prototype.createUrl = function() {
	var table = TableManager.instance().getTable(this.tableId);
	if (!exists(table)) {
		return "";
	}	
	return this.createTopnChartUrl();	
}