/**
 * This class manages and updates a topn chart. 
 */


/**
 * The default height of the bars in the horizontal topn bar charts.
 */
TopNChart.DEFAULT_BAR_HEIGHT = 20;


/**
 * The default n used in the topn tables.
 */
TopNChart.DFAULT_TOPN = 5;


/**
 * The top offset from a charts top to the top of the first bar in a horizontal
 * bar chart in pixels.
 */
TopNChart.BAR_TOP_OFFSET = 13;


/**
 * The bottom offset from a charts bottom to the bottom of the lowest bar in a 
 * horizontal bar chart, in pixels. 
 */
TopNChart.BAR_BOTTOM_OFFSET = 42;


/**
 * The base url used to create topn chart urls. 
 */
TopNChart.TOPN_CHART_BASE_URL = "/?resource=charts&sid=&topn=5&chart=bar&width=&height=&baralignment=horizontal&barOutlineOff=true&barWidth=0.7&barLabelsOn=true&3dmodeon=true&antialias=true&valueLabelsOn=true&valueLinesOn=true&3ddepth=5&background=%23ffffff|%23cccccc&alternateChartBackground=%23e7e7e7&barShape=cylinder&sampleColors=%231e7096&range=&sampleValues=&sampleLabels=&rangeAxisLabel=&";
 
 
/**
 * The names of the data column for the topn charts.
 */
var dataColumnName = new Array();
dataColumnName["topNProtocolsTable"]  = "octets";
dataColumnName["myProtocolsTable"]  = "octets";
dataColumnName["topNHostsTable"]  = "octets";
dataColumnName["myHostsTable"]  = "octets";
dataColumnName["topNHostListenersTable"]  = "inOctets";
dataColumnName["topNHostTalkersTable"]  = "outOctets";
dataColumnName["topNConversationsTable"]  = "octets";
dataColumnName["myConversationsTable"]  = "octets";
dataColumnName["myProtocolsPerHostTable"]  = "inOctets";
dataColumnName["topNNetworksTable"]  = "octetCount";


/**
 * The range axis labels to use for some field names. 
 */
var rangeAxisLabels = new Array();
rangeAxisLabels["octets"]  = "Bytes";
rangeAxisLabels["packets"]  = "Packets";
rangeAxisLabels["hosts"]  = "Hosts";
rangeAxisLabels["conversations"]  = "Conversations";
rangeAxisLabels["protocols"]  = "Protocols";
rangeAxisLabels["outOctets"]  = "Bytes";
rangeAxisLabels["inOctets"]  = "Bytes";
rangeAxisLabels["inPackets"]  = "Packets";
rangeAxisLabels["outPackets"]  = "Packets";
rangeAxisLabels["octetCount"]  = "Bytes";
rangeAxisLabels["hostCount"]  = "Hosts";
rangeAxisLabels["packetCount"]  = "Packets"
rangeAxisLabels["conversationCount"]  = "Conversations";
rangeAxisLabels["protocolCount"]  = "Protocols";
rangeAxisLabels["cardCount"]  = "Cards";


/**
 * The fields of the label names for the topn charts.
 */ 
var labelNames = new Array();
labelNames["topNProtocolsTable"] = "protocolName";
labelNames["myProtocolsTable"] = "protocolName";
labelNames["topNHostsTable"] =  "hostName";
labelNames["myHostsTable"] =  "hostName";
labelNames["topNHostListenersTable"] = "hostName";
labelNames["topNHostTalkersTable"] = "hostName";
var conversationLabes = new Array();
conversationLabes[0] = "srcHostName";
conversationLabes[1] = "dstHostName";
labelNames["topNConversationsTable"] = conversationLabes;
labelNames["myConversationsTable"] = conversationLabes;
var protoclsPerHostLabels = new Array();
protoclsPerHostLabels[0] = "protocolName";
protoclsPerHostLabels[1] = "hostName";
labelNames["myProtocolsPerHostTable"] = protoclsPerHostLabels;
labelNames["topNNetworksTable"] = "probeName";


/**
 * Create a instance of this class. A topnchart is a subclass
 * of Chart.
 * @param chatCode The charts html code.
 * @param title The title of this chart.
 * @param window The window the chart is in.
 * @param tableId The id of the table this chart is for.
 * @return A instance of this class.
 * @throws Error if chartCode is undefined or null.
 * @throws Error if title is undefined, null or empty.
 * @throws Error if window is undefined or null.
 * @throws Error if tableId is undefined, null or empty.
 */
function createTopNChart(chartCode, title, window, tableId) {
	assertExists(chartCode, "The charts html code is undefined or null");
	assertNotEmpty(title, "Title for chart is undefined, null or empty: " + title);
	assertExists(window, "The window the image container is in is undefined or null");
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);

	var superInstance = new Chart(chartCode, Chart.TOPN_CHART, title, window);
	var subInstance = new TopNChart(chartCode, tableId);
	subInstance = createSubClass(superInstance, subInstance);
	
	ChartManager.instance().removeChart(superInstance);
	ChartManager.instance().registerNewChart(subInstance);
	
	return subInstance;
}


// construction


/**
 * Constructs a topn chart.
 * @param image The image tag for this chart.
 * @param tableId The id of the table this chart is for.
 * @throws Error if tableId is undefined, null or empty.
 */
function TopNChart(image, tableId) {
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);
	
	this.htmlCode = image;
	this.tableId = tableId;
	// from the field with this name in the table the sample values are generated
	this.dataFieldName = "";
	
	// the number of rows in the table this is used to resize the chart if needed
	this.rowCount = TopNChart.DFAULT_TOPN;
	
	// the height of each bar in the chart
	this.barHeight = TopNChart.DEFAULT_BAR_HEIGHT;
	
	// listen to events
	this.eventListener = new Listener(this);
	this.eventListener.addEvent("tableUpdated", processUpdatedEvent);
	this.eventListener.addEvent("tableSorted", processSortedEvent);
	this.eventListener.addEvent("tableIndexChanged", processIndexChangedEvent);
	this.eventListener.addEvent(WINDOW_HIDDEN_EVENT, processWindowHiddenChart);
	this.eventListener.addEvent(WINDOW_SHOWN_EVENT, processWindowShownChart);
	this.eventListener.addEvent("tabDeactivated", processTabDeactivatedEventChart);
	this.eventListener.addEvent("tabActivated", processTabActivatedEventChart);
	this.eventListener.addEvent("chartShown", processChartShownEvent);
	this.eventListener.addEvent("chartHidden", processChartHiddenEvent);
	this.eventListener.addEvent("chartClosed", processChartClosedEvent);
	this.eventListener.addEvent("probeNameChanged", processProbeNameChangedEvent);
	this.eventListener.addEvent("chartDetached", processChartDetachedEvent);
	this.eventListener.addEvent("chartAttached", processChartAttachedEvent);
	this.eventListener.addEvent("resized", processWindowResizedTopnChart);
	
	// if this is true the chart will be updated
	this.updating = false;
}	


// services


/**
 * Get the bar height of the chart.
 * @return The height of the bars in the chart. 
 */
TopNChart.prototype.getBarHeight = function() {
	return this.barHeight;
}


// event handling


/**
 * Processes a window resized event. 
 * @param event The event to process.
 */
function processWindowResizedTopnChart(event) {
	if (event.getData()["id"] != this.window.getId()) {
		// the chart only cares about events from its window, none of the others
		return;
	}
	
	this.barHeight = this.calculateBarHeight(this.rowCount, this);
}


/**
 * Processes a index chagned event.
 * @param event The event to process.
 */
function processIndexChangedEvent(event) {
	if (event.getData()["id"] != this.tableId) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	// change the data column for the table
	dataColumnName[this.tableId] = event.getData()["newIndex"];
}


/**
 * Processes a updated event from the table.
 * @param event The updated event to process.
 */
function processUpdatedEvent(event) {
	if (event.getData()["id"] != this.tableId) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	var newRowCount = event.getData()["newRowCount"];
	var chartHeightDiff = this.calculateChartHeightDiff(this.barHeight, newRowCount, this);
	if (chartHeightDiff != 0 && exists(newRowCount) && newRowCount != this.rowCount) {
		this.resize(chartHeightDiff, this);
		this.rowCount = newRowCount;
	}
	
	if (this.updating) {
		this.updateChart();
	}
	else if (this.window.getActiveChart() == this) {
		this.startUpdate();
	}
}


/**
 * Processes a sorted event from the table.
 * @param event The sorted event to process.
 */
function processSortedEvent(event) {
	if (event.getData() != this.tableId) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	this.updateChart();
}


// updating


/**
 * Returns true if the chart is beeing updated periodically, false 
 * if the chart is not being updated. 
 */
TopNChart.prototype.isUpdating = function() {
	return this.updating;
}


/**
 * Starts the updating of the chart. This is overridden to not start the 
 * updating sequence. A topn chart is updated when its table is udpated. 
 */
TopNChart.prototype.startUpdate = function() {
	if (!this.isUpdating() && ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		this.updating = true;	
		this.updateChart();
	}
}


/**
 * Stops the updating of the chart.
 */
TopNChart.prototype.cancelUpdate = function() {
	this.updating = false;	
	this.udpateCounter = 0;
}


// resizing of chart


/**
 * Depending on how much data we received from the server, the window
 * the table is in might need to be resized.
 * @param chartHeightDiff The height diff of the chart.
 * @param chart The chart to resize.
 */
TopNChart.prototype.resize = function(chartHeightDiff, chart) {
	if (chartHeightDiff == 0) {
		return;
	}
	
	chartHeightDiff = this.resizeChart(chartHeightDiff, chart);
	chart.fireChartResizedEvent(chartHeightDiff, 0);
}


/**
 * Resizes the chart.
 * @param chartHeightDiff The height to resize the window with.
 * @param chart The chart to resize. 
 * @param returns The height in pixels the chart was actually resized with.
 */
TopNChart.prototype.resizeChart = function(chartHeightDiff, chart) {
	if (chartHeightDiff == 0) {
		return 0;
	}
	var imageWidth = parseInt(chart.htmlCode.style.width);
	var imageHeight = parseInt(chart.htmlCode.style.height);
	var newHeight = imageHeight + chartHeightDiff;
	// set the new height of the chart
	chart.height = newHeight;
	
	// resize the chart
	chart.htmlCode.style.height = newHeight + "px";
	chart.updateChart();
	return chartHeightDiff;
}


/**
 * Calculates the height diff for the chart.
 * @param barHeight The barHeight to use. 
 * @param rowCount The number of rows received from the server. 
 * @param chart The chart to calculate the height diff for. 
 * @return The calculated height diff for the chart.
 */
TopNChart.prototype.calculateChartHeightDiff = function(barHeight, rowCount, chart) {
	var chartHeight = this.calculateChartHeight(barHeight, rowCount);
	var actualChartHeight = parseInt(chart.htmlCode.style.height);
	return chartHeight - actualChartHeight;
}


/**
 * Calculates the height of the chart.
 * @param barHeight The barHeight to use. 
 * @param rowCount The number of rows received from the server. 
 * @param oldRowCount The number of rows before inserting the new data, including the header.
 * @return The calculated height of the chart.
 */
TopNChart.prototype.calculateChartHeight = function(barHeight, rowCount) {
	// calculate the size of the chart in pixels
	var allBarsHeight = rowCount * barHeight;
	var chartHeight = TopNChart.BAR_TOP_OFFSET + TopNChart.BAR_BOTTOM_OFFSET;
	chartHeight = chartHeight + allBarsHeight;
	chartHeight = Math.round(chartHeight);
	return chartHeight;
}


/**
 * Calculates the bar height.
 * @param oldRowCount The number of rows before inserting the new data, including the header.
 * @param chart The chart to calculate the bar height for. 
 * @return The bar height.
 */
TopNChart.prototype.calculateBarHeight = function (oldRowCount, chart) {
	var barHeight = 0;
	if (oldRowCount > 0) {
		var oldChartHeight = parseInt(chart.htmlCode.style.height);
		oldChartHeight = oldChartHeight - TopNChart.BAR_TOP_OFFSET - TopNChart.BAR_BOTTOM_OFFSET;
		var barHeight = oldChartHeight / (oldRowCount);
		barHeight = Math.round(barHeight);
	}
	
	if (barHeight < 0) {
		barHeight = 0;
	}
	else if (barHeight == 0) {
		barHeight = TopNChart.DEFAULT_BAR_HEIGHT;
	}
	return barHeight
}


// implementation


/**
 * Create a topn chart url.
 * @return The new creatd topn url.
 */
TopNChart.prototype.createUrl = function() {
	var table = TableManager.instance().getTable(this.tableId);
	if (!exists(table) || table.getRowCount() == 0) {
		return "";
	}	
	return this.createTopnChartUrl();	
}


/**
 * Creates the a url for a topn chart.
 * @return The created url. 
 */
TopNChart.prototype.createTopnChartUrl = function() {
	var table = TableManager.instance().getTable(this.tableId);
	var newUrl = replaceParameterValue(TopNChart.TOPN_CHART_BASE_URL, "sid", getSid(), "&");
	var max = table.getFieldValues(dataColumnName[this.tableId], 1);
	max = isNumberString(max) ? parseInt(max) : 0;
	var range = this.calculateNewRange(max);
	newUrl = replaceParameterValue(newUrl, "range", range, "&");
	newUrl = replaceParameterValue(newUrl, "sampleValues", table.getFieldValues(dataColumnName[this.tableId], -1), "&");
	newUrl = replaceParameterValue(newUrl, "sampleLabels", table.getFieldValues(labelNames[this.tableId], -1), "&");
	newUrl = replaceParameterValue(newUrl, "rangeAxisLabel", rangeAxisLabels[dataColumnName[this.tableId]], "&");
	newUrl = replaceParameterValue(newUrl, "height", this.getHeight(), "&");
	newUrl = replaceParameterValue(newUrl, "width", this.getWidth(), "&");
	if (needTimestamp(newUrl)) {
		// in Opera when chart is taken from cache the onload is not fired, add
		// a timestamp to make sure the chart is not loaded from cache
		newUrl = this.updateChartTimestamp(newUrl);
	}
	newUrl += "servletMode=true&";
	return newUrl;
}


/**
 * Calculates the new chart range. 
 * @param max The max value.
 * @return The new range of the chart. 
 */
TopNChart.prototype.calculateNewRange = function(max) {
	// count the number of digits 
	var digitCount = 0;
	var temp = max;
	while (temp > 0) {
		temp = parseInt(temp / 10);
		digitCount++;
	}
	
	// calculate the new range
	var step = Math.pow(10, digitCount-1);
	var newRange = max;
	if (newRange % step != 0) {
		newRange = newRange - (newRange % step) + step;
	}
	return newRange;
}
