/**
 * The license manager is responsible for the license management on the client side. 
 */
 
// constants


/**
 * The url to use in the license tab when the probe is run in home edition.
 */
var HOME_EDITION_LICENSE_URL = "http://support.objectplanet.com/licenser/generate_probe_test_key.jsp?";


/**
 * The url to use in the license tab when a test key is used.
 */
var TEST_KEY_LICENSE_URL = "https://support.objectplanet.com/licenser/purchase.jsp?families=12";


/**
 * The url to use in the license tab when a valid license key is used.
 */
var LICENSED_LICENSE_URL = "http://www.objectplanet.com/probe/probe_licensed_version";


/**
 * The url used to load the license mode.
 */
var GET_LICENSE_URL  = "?resource=ProbeManager&mode=get&command=license&format=xml&sid=";
	

/**
 * The base url used to set a license key.
 */
var SET_LICENSE_URL = "?resource=ProbeManager&mode=set&command=license&format=xml&sid=";


/**
 * The url used to load the hash of the NICs mac address. The hash is used to create
 * a test key.
 */
var MAC_HASH_URL = "?resource=ProbeManager&mode=get&command=macHash&format=xml&sid=";
 
 
/**
 * Gets the single instance of the license manager. 
 */
function getLicenseManager() {
		
	if (!top.LicenseManager) {
		// create the license manager
		
		/**
		 * Creates the license manager instance
		 */ 
		top.LicenseManager = new Object();
		
		
		/**
		 * Create the event listener and listen to layout set events. 
		 */
		var eventListener = new Listener(this);
		eventListener.addEvent(LAYOUT_SET_EVENT, processLayoutSetEventLicenseManager);
		
		
		/** 
		 * When the probe is in home edition or a test key is present
		 * a window is shown where the user can get a test key. This window is shown at each login
		 * after the layout has been set, but not when the layout is after the first time
		 */
		var firstLayoutSet = false;
		
		
		/**
		 * Storing the license xml string loaded. 
		 */
		 var license;
		 
		 
		// servicess
		
		
		/**
		 * Loads the current license mode. 
		 */
		top.LicenseManager.loadLicense = function() {
			loadXML(GET_LICENSE_URL + getSid(), "licenseLoaded", top.LicenseManager);
		}
		
		
		/**
		 * Sets a license key.
		 * @param key The license key to set. 
		 * @throws Error if key is null, undefined or empty.
		 */
		top.LicenseManager.setLicense = function(key) {
			assertNotEmpty(key, "The key to set is undefined null or empty");
			var url = SET_LICENSE_URL + getSid() + "&key=" + key;
			loadXML(url, "licenseLoaded", top.LicenseManager);
		} 
		
		
		// implementation
		
		
		/**
		 * Callback function for hte loading of the license information. 
		 * Parse the loaded xml data. 
		 * @param xml The loaded xml data. 
		 */
		top.LicenseManager.licenseLoaded = function(xml) {
			license = xml;
			var keyVersion = getTagNodeValue("keyVersion", license);
			var testKey = getTagNodeValue("testKey", license);
			var requiredVersion = getTagNodeValue("requiredkeyVersion", license);
			var validLicense = getTagNodeValue("validLicense", license);
			top.LicenseManager.toggleLicenseTabContent(keyVersion, testKey, requiredVersion, validLicense);
			top.LicenseManager.toggleLicenseWindow(keyVersion, testKey, requiredVersion, validLicense);
			
			if (exists(top.tabSection.getTab("Setup")) && top.tabSection.getTab("Setup").isLoaded() && exists(top.frames["Setup"].setLicenseLoaded)) {
				top.frames["Setup"].setLicenseLoaded(xml);	
			}
		}
		
		
		/**
		 * A layout has been set, if this is the first time a layout is set the user has just logged in
		 * if the probe is run in Home Edition or with a test key, show the license window.
		 */
		top.LicenseManager.layoutSet = function() {
			if (firstLayoutSet == false && exists(license)) {
				var keyVersion = getTagNodeValue("keyVersion", license);
				var testKey = getTagNodeValue("testKey", license);
				var requiredVersion = getTagNodeValue("requiredkeyVersion", license);
				top.LicenseManager.toggleLicenseTabContent(keyVersion, testKey, requiredVersion);
				top.LicenseManager.toggleLicenseWindow(keyVersion, testKey, requiredVersion);
			}
			firstLayoutSet = true;
		}
		 
		
		/**
		 * Sets the content of the license tab according to a given layout.
		 * @param keyVersion The version of the current license key.
		 * @param testKey Whether the current license key is a test key.
		 * @param requiredVersion The required test key version.
		 */
		top.LicenseManager.toggleLicenseTabContent = function(keyVersion, testKey, requiredVersion, validLicense) {
			if ("-1" == keyVersion || ("-1" != keyVersion && parseInt(keyVersion) < parseInt(requiredVersion))) {
				// no key set, Home Edition
				// or a new version of the probe which requires a new key is installed,
				// permit the user to generate a test key
				top.LicenseManager.setLicenseTabUrl(HOME_EDITION_LICENSE_URL + "hostid=" + getHostId() + "&version=" + getVersion() + "&showHeader=false");
				top.tabSection.changeClass("tab createTestKey", "License");
			}
//			else if ("-1" != keyVersion && "true" == testKey) {
//				// test key
//				top.LicenseManager.setLicenseTabUrl(TEST_KEY_LICENSE_URL);
//				top.tabSection.changeClass("tab purchase", "License");
//			}
//			else if ("-1" != keyVersion && "false" == testKey && "false" == validLicense) {
//				// valid key has expired
//				top.LicenseManager.setLicenseTabUrl(TEST_KEY_LICENSE_URL);
//				top.tabSection.changeClass("tab purchase", "License");
//			}
			else if ("-1" != keyVersion && "false" == testKey) {
				// valid key
				top.LicenseManager.hideLicenseTab();
			}
		}
		
		
		/**
		 * Shows or hides the license window according to the given license. 
		 * @param keyVersion The version of the current license key.
		 * @param testKey Whether the current license key is a test key.
		 * @param requiredVersion The required test key version.
		 */
		top.LicenseManager.toggleLicenseWindow = function(keyVersion, testKey, requiredVersion, validLicense) {
			if ("-1" == keyVersion || ("-1" != keyVersion && parseInt(keyVersion) < parseInt(requiredVersion))) {
				// no key set, Home Edition
				// or a new version of the probe which requires a new key is installed,
				// permit the user to generate a test key
				top.LicenseManager.showHomeEditionWindow();
			}
//			else if ("-1" != keyVersion && "true" == testKey) {
//				// test key
//				top.LicenseManager.showTestKeyWindow();
//			}
//			else if ("-1" != keyVersion && "false" == testKey && "false" == validLicense) {
//				// valid key has expired
//				top.LicenseManager.showTestKeyWindow();
//			}
			else if ("-1" != keyVersion && "false" == testKey) {
				// a valid key
				top.LicenseManager.hideLicenseStatusWindow();	
			}
		}
		
		
		/**
		 * Hides the license tab.
		 */		
		top.LicenseManager.hideLicenseTab = function() {
			top.tabSection.changeClass("inactive", "License");
		}
		
		
		/**
		 * Sets a given url for the license tab.
		 * @param url The url to set. 
		 */ 
		top.LicenseManager.setLicenseTabUrl = function(url) {
			top.tabSection.getTab("License").setUrl(url);
		}
		
		
		/**
		 * Hides the license status window. 
		 */
		top.LicenseManager.hideLicenseStatusWindow = function() {
			var licenseStatusWindow = WindowManager.instance().getWindow("licenseStatusWindow");
			if (!exists(licenseStatusWindow)) {
				return;
			}
			WindowManager.instance().hideWindow(licenseStatusWindow.getId());
		}
		 
		 
		/**
		 * Shows the test key status window. 
		 */
		top.LicenseManager.showTestKeyWindow = function() {
			var licenseStatusWindow = WindowManager.instance().getWindow("licenseStatusWindow");
			if (!exists(licenseStatusWindow)) {
				return;
			}
			var homeEdition = getElementWithAttribute(licenseStatusWindow.getHtmlCode(), "homeEdition", "name", "table");
			setClass(homeEdition, "inactive");
			var testKey = getElementWithAttribute(licenseStatusWindow.getHtmlCode(), "testKey", "name", "table");
			setClass(testKey, "active");
			WindowManager.instance().activateWindow(licenseStatusWindow.getId());
			licenseStatusWindow.resized();
		}
		
		
		/**
		 * Shows the home edition license status window.
		 */
		top.LicenseManager.showHomeEditionWindow = function() {
			var licenseStatusWindow = WindowManager.instance().getWindow("licenseStatusWindow");
			if (!exists(licenseStatusWindow)) {
				return;
			}
			var testKey = getElementWithAttribute(licenseStatusWindow.getHtmlCode(), "testKey", "name", "table");
			setClass(testKey, "inactive");
			var homeEdition = getElementWithAttribute(licenseStatusWindow.getHtmlCode(), "homeEdition", "name", "table");
			setClass(homeEdition, "active");
			WindowManager.instance().activateWindow(licenseStatusWindow.getId());
			licenseStatusWindow.resized();
		}
	}
	
	return top.LicenseManager;
}



// event processing


/**
 * A layout has been set. If this is the first layout set and license mode is 
 * LICENSE_MODE_HOME_EDITION or LICENSE_MODE_TEST_KEY show the license status window.
 */
function processLayoutSetEventLicenseManager() {
	getLicenseManager().layoutSet();
}
