/**
 * The time of the log entry currently shown in the appplet.
 */
var currentTime = 0;


/**
 * Any previous error messages are stored here.
 */
var previousMessage = "";


/**
 * Sends a request to delete log entries.
 */
function deleteLogEntries() {
	if (confirm('Are you sure?')) {
		var form = document.getElementById('logDeleteForm');
		var session = document.getElementById('sid').value;
		var url = "?resource=DataManager&mode=set&command=delete&sid=" + session;
		var forwardIp = getParameterValue(location.href, "forwardIp", "&");
		url += isEmpty(forwardIp) ? "" : "&forwardIp=" + forwardIp + "&";
		url += "&age=" + form.age.value;
		loadXML(url, logManagerHandler);
	}
}


/**
 * Opens a new window with the specified url, width and height.
 */
function winOpen (url, width, height) {
	time = new Date();
    theName = Math.ceil(time);
    var options='';
    if (width != null) {
		options += 'width='+width;
    }
	if (height != null) {
		options += ',height='+height;
    }
	options += ',resizable=yes,scrollbars=yes';
	w = window.open (url, theName, options);
}


/**
 * Inserts data into the applet.
 */
function setTableData() {
	if (document.log_selection.interval.value == 0) {
		return;
	}
	// get the timestamp of the selected entry
	selected = document.log_selection.interval;
	value = selected.value;
	if (value > 0) {
		currentTime = value;
		top.frames["TrafficLog"].document.TrafficApplet.updateData(value + "", getSelectedSlave());
	} 
	// clear the data
	else {
		top.frames["TrafficLog"].document.TrafficApplet.removeLogData();
	}
}

/**
 * Inserts data into the applet, when an entry is selected from the log manager.
 */
function setSelectedTableData(value) {
	// set the new data url in the table
	document.log_selection.interval.value = value;
	if (value > 0) {
		top.frames["TrafficLog"].document.TrafficApplet.updateData(value + "", getSelectedSlave());
		currentTime = value;
	} 
	// clear the data
	else {
		top.frames["TrafficLog"].document.TrafficApplet.removeLogData();
	}
}


/**
 * Loads the timestamps of the stored data from the server. 
 */
function loadTimeStamps() {
	deleteTimeStamps();
	sendTimeStampsRequest();
	showLoading();	
}


/**
 * Removes the current timestamps. 
 */
function deleteTimeStamps() {
	var intervalSelect = getElementWithAttribute(document.body, "interval", "name", "select");
	var options = intervalSelect.options;
	for (var i = options.length - 1; i >= 0; i--) {
		intervalSelect.remove(options[i]);
		if (isSafari()) {
			// in safari remove does not work
			intervalSelect.options[i] = undefined;
		}
	}
	intervalSelect.appendChild(createOption(0, "(select log entry)"));
	intervalSelect.selectedIndex = 0;
}


/**
 * Send the request for new timestamps
 */
function sendTimeStampsRequest() {
	// send the request
	var sid = getElementWithAttribute(top.document.body, "sid", "name", "input").value;
	var url = "/?resource=datamanager&mode=getdata&command=timestamps&sid=" + sid + "&";
	if (!isEmpty(getSelectedSlave()) && !"master".equalsIgnoreCase(getSelectedSlave())) {
		url += "forwardIp=" + getSelectedSlave() + "&";
	}
	loadXML(url, parseTimeStamps);
}


/** 
 * Parses the timestamps sendt from the server.
 * @param xmlData the xml response from the server
 */
function parseTimeStamps(xmlData) {
	var timeStamps = xmlData.getElementsByTagName("timeStamp");
	
	// to prevent problems with IE when inserting options we have to hide 
	// the selector before adding options
	var intervalSelect = getElementWithAttribute(document.body, "interval", "name", "select");
	setClass(intervalSelect, "inactive");

	// add timestamps
	for (var i = 0; timeStamps != null && i < timeStamps.length; i++) {
		addTimeStamp(timeStamps[i]);
		var value = getXmlContent(timeStamps[i].getElementsByTagName("time")[0]);
		if (value == currentTime) {
			intervalSelect.selectedIndex = (i+1);
		}
	}
	
	// show the select again
	setClass(intervalSelect, "active");

	checkLogData();
	
	// hide the loading animation
	var loadingAnimationCell = getElementWithAttribute(document.body, "loadingAnimationCell", "name", "td");
	setClass(loadingAnimationCell, "inactive");
}


/**
 * Adds a timestamp the interval selector
 * @param timeStam The xml formatted timestamp to add.
 */
function addTimeStamp(timeStamp) {
	var intervalSelect = getElementWithAttribute(document.body, "interval", "name", "select");
	
	var value = getXmlContent(timeStamp.getElementsByTagName("time")[0]);;
	var lastReset = getXmlContent(timeStamp.getElementsByTagName("formattedLastReset")[0]);
	var time = getXmlContent(timeStamp.getElementsByTagName("formattedTime")[0]);
	
	// append the option at the end
	intervalSelect.appendChild(createOption(value, lastReset + " - " + time));
}


/**
 * Checks that the log data currently shown in the applet is one of 
 * choices in the log data selector, if not the data in the applet
 * is cleared.
 */
function checkLogData() {
	if (currentTime == 0) {
		// no log data is shown in the applet
		return;
	}
	
	var intervalSelect = getElementWithAttribute(document.body, "interval", "name", "select");
	var options = intervalSelect.options;
	for (var i = 0; i < options.length; i++) {
		if (currentTime == options[i].value) {
			// the log entry currently shown in the applet
			// is still valid, select it 
			intervalSelect.selectedIndex = i;
			return;
		}
	}

	// the log data currently in the applet has been deleted, clear the applet
	top.frames["TrafficLog"].document.TrafficApplet.removeLogData();
	currentTime = 0;
}


/**
 * Show the loading animation
 */
function showLoading() {
	var loadingAnimationCell = getElementWithAttribute(document.body, "loadingAnimationCell", "name", "td");
	setClass(loadingAnimationCell, "active");
}


/**
 * Returns the content of all the text nodes inside a xml node.
 * @param node The xml node to get the content from.
 */
function getXmlContent(node) {
	var content = "";
	for (var i = 0; i < node.childNodes.length; i++) {
		if (node.childNodes[i].nodeType == 3) {
			// text node
			content += node.childNodes[i].nodeValue; 
		}
	}
	return content;
}


/**
 * Create a option for a select.
 * @param value The value to set.
 * @param text The text to append to the option.
 * @returns The created option.
 */
function createOption(value, text) {
	var option = document.createElement("option");
	option.value = value;
	var textNode = document.createTextNode(text);
	option.appendChild(textNode);
	return option;
}



// functions used from the log manager page

/**
 * Loads the log entries.
 */
function loadLogEntries() {
	// send the request
	var sid = getElementWithAttribute(top.document.body, "sid", "name", "input").value;
	var url = "/?resource=datamanager&mode=getdata&command=timestamps&time=-1&sid=" + sid;
	var forwardIp = getParameterValue(location.href, "forwardIp", "&");
	url += isEmpty(forwardIp) ? "" : "&forwardIp=" + forwardIp + "&";
	loadXML(url, logManagerHandler);
}


/** 
 * Parses the timestamps sendt from the server.
 * @param xmlData the xml response from the server
 */
function logManagerHandler(xmlData) {
	var timeStamps = xmlData.getElementsByTagName("timeStamp");
	var logList = document.getElementById("logList");	
	
	var htmlTable = "<table id='logEntries' border=0 width='100%' cellpadding=2 cellspacing=0>";
	if (timeStamps.length == 0) {
		setMessageField(document.getElementById("logTable"), "id", "td", "No logs available until next reset.");
	}
	
	for (var i = 0; i < timeStamps.length; i++) {
		var timeStamp = timeStamps[i];
		var value = getXmlContent(timeStamp.getElementsByTagName("time")[0]);;
		var lastReset = getXmlContent(timeStamp.getElementsByTagName("formattedLastReset")[0]);
		var time = getXmlContent(timeStamp.getElementsByTagName("formattedTime")[0]);
		var link = "javascript:setTableData(" + value + ");";
		htmlTable += "<tr><td><a href='" + link + "'>" + lastReset + " - " + time + "</a></td>";
		htmlTable += "<td align='right'>" + age(value) + "</td>";
		htmlTable += "</tr>";
	}
	htmlTable += "</table>";
	logList.innerHTML = htmlTable;	
	opener.top.frames["LogSelector"].deleteTimeStamps();
	opener.top.frames["LogSelector"].parseTimeStamps(xmlData);
}


/**
 * Returns the age in days for the given time stamp.
 */
function age(timeStamp) {
	var DAY = 86400000;
	var age = (new Date().valueOf() - timeStamp) / DAY;
	var ageString = Math.round(age) + " ";
	if (ageString == 0 || ageString > 1) {
		ageString += "days";
	} 
	else if (ageString == 1) {
		ageString += "day";
	}
	return ageString;
}


/**
 * Shows the data manager.
 */
function showDataManager() {
	var url = "/?sid=" + getSid() + "&resource=DataManager&mode=getPage&page=setup&gui=html&";
	url += isEmpty(getSelectedSlave()) || "master".equalsIgnoreCase(getSelectedSlave()) ? "" : "forwardIp=" + getSelectedSlave() + "&";  
	winOpen(url, 500, 600);
}