/**
 * The url used to delete protocol entries.
 */
MyConversationsTable.DELETE_CONVERSATION_ENTRY_URL = MyTable.BASE_DELETE_ENTRY_URL + "command=myconversations&";


/**
 * The url used to add protocol entries.
 */
MyConversationsTable.ADD_CONVERSATION_ENTRY_URL = MyTable.BASE_ADD_ENTRY_URL + "command=myconversations&";


/**
 * Factory method to create new MyHostsTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createMyConversationsTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new MyConversationsTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}


/**
 * Subclasses table.js to add probe specifc behaviour.
 */
function MyConversationsTable() {
}


// services


/**
 * Adds a coversation to the myconversations table. This is called when the right click choice
 * Add to my conversations is clicked. 
 */
function addToMyConversations() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var srcAddress = getFieldFromRow("srcAddress", row).getValue();
		var dstAddress = getFieldFromRow("dstAddress", row).getValue();
		WindowManager.instance().activateWindow("myconversationwindow");
		var myConversationsTable = TableManager.instance().getTable("myConversationsTable");
		if (myConversationsTable.isPainted() == false) {
			myConversationsTable.paint();
		}
	
		var srcAddressField = getInput(myConversationsTable.getAddEntryContainerCode(), "srcAddress");		
		var dstAddressField = getInput(myConversationsTable.getAddEntryContainerCode(), "dstAddress");		
		srcAddressField.value = srcAddress;
		dstAddressField.value = dstAddress;
		myConversationsTable.addEntry();
	}
}
 
  
/**
 * Checks for valid input before a entry is added. 
 * @returns True if the input is valid, false otherwise. 
 */
MyConversationsTable.prototype.validInput = function() {	
	var srcAddress = getElementWithAttribute(this.getAddEntryTableCode(), "srcAddress", "name", "input").value;
	if (srcAddress == "[Enter IP]" || dstAddress == "[Enter IP]") {
		this.setMessage(undefined, "Please enter an IP address");
	}
	if (isEmpty(srcAddress) || !isValidIpAddress(srcAddress)) {
		this.setMessage(undefined, "Invalid source address: " + srcAddress);
		return false;
	}
	var dstAddress = getElementWithAttribute(this.getAddEntryTableCode(), "dstAddress", "name", "input").value;
	if (isEmpty(dstAddress) || !isValidIpAddress(dstAddress)) {
		this.setMessage(undefined, "Invalid destination address: " + dstAddress);
		return false;
	}
	
	return true;
}



/**
 * Checks if a entry to add already exists. This is to be overridden by subclases.
 * @returns True if the entry already exists false otherwise. 
 */
MyConversationsTable.prototype.exists = function() {
	var values = new Array();
	values[0] = getElementWithAttribute(this.getAddEntryTableCode(), "srcAddress", "name", "input").value;
	values[1] = getElementWithAttribute(this.getAddEntryTableCode(), "dstAddress", "name", "input").value;
	
	if (this.getRowIndexFieldValues(this.getIdFields(), values) > -1) {
		this.setMessage(undefined, "The entry already exists");
		return true;
	}
	return false;
}


/**
 * A entry has been added. Reset the add entry fields.
 */ 
MyConversationsTable.prototype.entryAdded = function() {
	this.clearCommentField();
}


/**
 * Returns the url to add a protocol entry.
 */
MyConversationsTable.prototype.createAddEntryUrl = function() {
	var srcHostField = getElementWithAttribute(this.getAddEntryTableCode(), "srcAddress", "name", "input");
	var srcHost = srcHostField.value;
	var dstHostField = getElementWithAttribute(this.getAddEntryTableCode(), "dstAddress", "name", "input");
	var dstHost = dstHostField.value;
	var commentField = getElementWithAttribute(this.getAddEntryTableCode(), "comment", "name", "input");
	var comment = ""
	if (isDefined(commentField) && !isNull(commentField) && commentField != "[Enter a comment]") {
		comment = commentField.value;
	}
	
	return MyConversationsTable.ADD_CONVERSATION_ENTRY_URL + "src=" + srcHost + "&dst=" + dstHost + "&comment=" + encodeURIComponent(comment) + "&";
}


/**
 * Creates the url needed to delete a protocol entry.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
MyConversationsTable.prototype.createDeleteEntryUrl = function(value) {
	var srcAddress = getParameterValue(value, "srcAddress", "&");
	var dstAddress = getParameterValue(value, "dstAddress", "&");
	return MyConversationsTable.DELETE_CONVERSATION_ENTRY_URL + "src=" + srcAddress + "&dst=" + dstAddress + "&";
}


/**
 * Creates the url to change the comment.
 * @param value The value for the id field of the entry to change the comment for.
 */ 
MyConversationsTable.prototype.createChangeCommentUrl = function(value) {
	var row = this.getTableCode().rows[this.getIdRowNumber(value)];
	var srcField = getHtmlFieldFromRow("srcAddress", row);
	var dstField = getHtmlFieldFromRow("dstAddress", row);
	var commentCell = getHtmlFieldFromRow("comment", row);
	var srcAddress = getTekstValue(srcField);
	var dstAddress = getTekstValue(dstField);
	var comment = getElementWithAttribute(commentCell, "comment", "name", "input").value;
	
	return MyConversationsTable.ADD_CONVERSATION_ENTRY_URL + "src=" + srcAddress + "&dst=" + dstAddress + "&comment=" + comment + "&";
}


/**
 * Paint the add entry row.
 * @param row The row to paint into.
 */
MyConversationsTable.prototype.paintAddEntryRow = function(row) {
	var cellCode;
	var thisObj = this;
	for (var i = (this.columnOrder.length -1); i >= 0; i--) {
		fieldName = this.columnOrder[i].getName();
		if (!this.isVisibleColumn(fieldName)) {
			continue;
		}

		if ("srcAddress".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var srcAddressField = this.createAddEntryInputfield("srcAddress", MyTable.INPUT_FIELD_SIZE, "[Enter IP]", MyTable.IP_INPUT_FIELDS_MAX_LENGTH); 
			srcAddressField.onfocus = function() {
				srcAddressField.value = "";
				tt_Hide();
			}
			srcAddressField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myconversationssrcaddress'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(srcAddressField);
		}
		else if ("dstAddress".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var dstAddressField = this.createAddEntryInputfield("dstAddress", MyTable.INPUT_FIELD_SIZE, "[Enter IP]", MyTable.IP_INPUT_FIELDS_MAX_LENGTH); 
			dstAddressField.onfocus = function() {
				dstAddressField.value = "";
				tt_Hide();
			}
			dstAddressField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myconversationsdstaddress'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(dstAddressField);
		}
		else if ("comment".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var commentField = this.createAddEntryInputfield("comment", MyTable.INPUT_FIELD_SIZE, null, MyTable.IP_INPUT_FIELDS_MAX_LENGTH);
			commentField.onfocus = function() {
				commentField.value = "";
				tt_Hide();
			}
			commentField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myconversationscomment'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(commentField);
		}
		else {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
		}
		this.setCellWidth(cellCode, fieldName);
		cellCode = null;	
	}
	
	var addButtonCell = row.insertCell(row.cells.length);
	addButtonCell.appendChild(this.createAddEntryLink());
}
