/**
 * The image used to add entries.
 */
MyTable.ADD_ENTRY_IMAGE_URL = "themes/black/images/icon_pluss_small.gif";


/**
 * The width of the delete button cell to the right in the table. 
 */
MyTable.DELETE_BUTTON_CELL_WIDTH = 15;


/**
 * The row height used for the myTables in safari. Because of problems
 * with cell borders and offsetHeight the row height can not be calculated in safari.
 */
MyTable.SAFARI_ROW_HEIGHT = 20;


/**
 * The size of the input fields, relative to the size of the fields they are in. 
 */
MyTable.INPUT_FIELD_SIZE = "98%";


/**
 * The max length to set for the port input fields.
 */
MyTable.PORT_INPUT_FIELDS_MAX_LENGTH = 14;


/**
 * The max length to set for the ip address input fields. 
 */
MyTable.IP_INPUT_FIELDS_MAX_LENGTH = 15;


/**
 * The base url used to add entries.
 */
var sid = getSid();
MyTable.BASE_ADD_ENTRY_URL = "/?resource=Statistics&mode=set&action=set&sid=" + sid + "&";


/**
 * The base url used to delete entries.
 */
MyTable.BASE_DELETE_ENTRY_URL = "/?resource=Statistics&mode=set&action=delete&sid=" + sid + "&";


/**
 * The name of a delete link for a entry in the my table. 
 */
MyTable.DELETE_LINK_NAME = "deleteLink";


/**
 * The prefixes for the protocols (ether = 1; ip = 1.2048....);
 */
var PROTOCOL_PREFIXES = new Array();
PROTOCOL_PREFIXES["ethernet"] = "1.";
PROTOCOL_PREFIXES["ip"] = "1.2048.";
PROTOCOL_PREFIXES["tcp"] = "1.2048.6.";
PROTOCOL_PREFIXES["udp"] = "1.2048.17.";


/**
 * Factory method to create new MyTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined or null.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createMyTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertExists(url, "Url to set for the table is undefined or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new MyTable();
	var superInstance = createProbeTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	// add events to be processed by myTables
	subInstance.eventListener.addEvent(WINDOW_COLLAPSED_EVENT, processColapsedEvent);
	subInstance.eventListener.addEvent(WINDOW_EXPANDED_EVENT, processExpandedEvent);

	// overwrite the repeater to call the load function on the MyTable object	
	if (isDefined(time)) {
		instance.repeater = new Repeater(load, instance, time);
	}

	return instance;
}


/**
 * Subclasses table.js to add mytable specifc behaviour.
 */
function MyTable() {
	// if this is set to true the add entry row has been painted
	this.addEntryPainted = false;
	
	// the add entry table.
	this.addEntryTable;
	
	// to make room for the table where the user can add entries to the table
	// we need to make the heigh of the data table container and the window larger,
	// if this is true the data table and window have been made larger
	this.heightIncreased = false;
	
	// to make room for the delete column all the table columns need to be made smaller,
	// if this is true the table columns have been made smaller
	this.cellWidthDecreased = false;
	
	// the id fields of a MyTable are the fields that identify each row, the
	// id field are typed in by the user to add new entries
	this.idFields = new Array();
	
	// when the user focuses on one of the comment fields in the data talbe, the comment
	// field is blocked, this is to prevent the users input data from being removed
	// and replaced with new data when the table is updated. This is the row index 
	// of the focused comment field.
	this.blockedCommentRowIndex = -1;
	
	// if a message is shown and there is no whitespace under the table the window
	// size is increased to make room for the message, if this is true the window
	// height has been increased
	this.messageHeightIncreased = false;
	
	// if this is set to true the window the table is in is colapsed, when the window
	// is colapsed browsers report the height of the table to be 0, so heights can not
	// be calculated while the window is colapsed
	this.windowColapsed = false;
}


// event processing

	
/**
 * Process a colapsed event. 
 * @param event The event to process. 
 */
function processColapsedEvent(event) {
	if (!this.windowName.equalsIgnoreCase(event.getData())) {
		// the event is not from the window this table is in
		return;
	}
	this.windowColapsed = true;
}


/**
 * Process a expanded event. 
 * @param event The event to process. 
 */
function processExpandedEvent(event) {
	if (!this.windowName.equalsIgnoreCase(event.getData())) {
		// the event is not from the window this table is in
		return;
	}
	this.windowColapsed = false;
}


// services


/**
 * Sets id fields for the table. The id fields are the fields that identify 
 * each row in the table.
 * @param Any number of field names.
 */
MyTable.prototype.setIdFields = function() {
	for (var i = 0; i < arguments.length; i++) {
		if (!isEmpty(arguments[i])) {
			this.setIdField(arguments[i]);
		}
	}
}


/**
 * Sets the id field to the table. The id field is the field that identifies
 * each row. 
 * @param fieldName The name of the field. 
 * @throws Error if fieldName is undefined, null or empty. 
 */
MyTable.prototype.setIdField = function(fieldName) {
	assertNotEmpty(fieldName, "FieldName is undefined, null or empty: " + fieldName);
	this.idFields[this.idFields.length] = fieldName;
}


/**
 * Returns the id field of this table, a empty string if no id field has been set. 
 */
MyTable.prototype.getIdFields = function() {
	return this.idFields;	
}


/**
 * Adds a entry to the table.
 */
MyTable.prototype.addEntry = function() {
	// clear old messages
	this.clearMessageField();
	if (!this.validInput()) {
		return;
	}
	if (this.exists()) {
		return;
	}

	var url = this.createAddEntryUrl();
	loadXML(url, "dataLoaded", this);
}


/**
 * Checks for valid input before a entry is added. This method is to be overriden by subclasses.
 * @returns True if the input is valid, false otherwise. 
 */
MyTable.prototype.validInput = function() {
	return "";
}


/**
 * Checks if a entry to add already exists. This is to be overridden by subclases.
 * @returns True if the entry already exists false otherwise. 
 */
MyTable.prototype.exists = function() {
	return false;
}


/**
 * Returns the url to add a entry. This method is to be overridden by subclasses.
 */
MyTable.prototype.createAddEntryUrl = function() {
	return "";
}


/**
 * Sets the data loaded, any messages are also set. 
 * @param xmlDoc The data returned from the server. 
 */
MyTable.prototype.dataLoaded = function(xmlDoc) {
	var message = xmlDoc.getElementsByTagName("message")[0];
	if (!isDefined(message) || isNull(message)) {
		this.restartUpdate();
		this.entryAdded();
		return;
	}
	
	var error = message.getElementsByTagName("error")[0];
	var success = message.getElementsByTagName("success")[0];
	if (isDefined(error) && !isNull(error)) {
		this.setErrorMessage(error);
	}
	else if (isDefined(success) && !isNull(success)) {
		this.setSuccessMessage(success);
	}
}


/**
 * A entry has been added. To be overridden by subclasses to implement
 * any functionality that need to bo done after a entry has been added. 
 */ 
MyTable.prototype.entryAdded = function() {

}


/**
 * Deletes a entry.
 * @param value The value for the id field of the row to delete. 
 */
MyTable.prototype.deleteEntry = function(value) {	
	var rowNumber = this.getIdRowNumber(value);
	if (rowNumber == -1) {
		return;
	}
	var deleteRow = this.rows[rowNumber];
	this.deleteRow(rowNumber);
	var deleteUrl = this.createDeleteEntryUrl(value);
	if (!isEmpty(deleteUrl)) {
		loadXML(deleteUrl);
	}
	
	// a row has been delete the scroll bar if any might have dissappeared
	this.adjustSize();
	// the data in the table has been changed fire a udpated event
	this.fireUpdatedEvent();

	// the add entry table should always be right under the data table, to do this
	// we need to set the height of the data table container for each row that is added
	if (isDefined(this.getDataTableContainer()) && !isNull(this.getDataTableContainer())) {
		this.setTableContainerHeight(this.calculateTableContainerHeight());
	}

	// after deleting a row the row classes might need to set again, because
	// of alternating row classes might be wrong after a row has been delete
	for (var i = 0; i < this.getTableCode().rows.length; i++) {
		this.addRowClasses(this.getTableCode().rows[i], i);
	}
	this.entryDeleted(deleteRow);
}


/**
 * Creates the url needed to delete a entry. This needs to be overridden by subclasses.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
MyTable.prototype.createDeleteEntryUrl = function() {
	return "";
}


/**
 * A entry has been deleted remove the chart for that entry if any. 
 * @param deletedRow The row that has been deleted. 
 */
MyTable.prototype.entryDeleted = function(deletedRow) {
	if (!isDefined(deletedRow) || isNull(deletedRow)) {
		return;
	}
	// go through each of the link fiels for this row and remove any existing charts
	for (var i = 0; i < this.linkFields.length; i++) {
		var linkField = getFieldFromRow(this.linkFields[i], deletedRow);
		if (!isDefined(linkField) || isNull(linkField)) {
			continue;
		}
		
		// create the chart id for the row and the field, and remove all the matching charts
		var linkId = createChartTitle(linkField, deletedRow, this.header, this.id);
		for (var j = 0; j < this.window.charts.length; j++) {
			if (this.window.charts[j].getTitle() == linkId) {
				this.window.closeChart(this.window.charts[j]);
			}	
		}
	}
}
 
 
/**
 * Changes the comment for a given field.
 * @param value The value for the id field of the entry to change the comment for.
 */ 
MyTable.prototype.changeComment = function(value) {
	var url = this.createChangeCommentUrl(value);
	loadXML(url, "dataLoaded", this);
}


/**
 * Creates a url to change the comment for a entry. This needs to be overridden by subclasses.
 * @param value The value for the id field of the entry to change the comment for.
 */
MyTable.prototype.createChangeCommentUrl = function(value) {
	return "";
}


/**
 * Clears the comment field in the add entry row.
 */
MyTable.prototype.clearCommentField = function() {
	var commentField = getElementWithAttribute(this.getAddEntryTableCode(), "comment", "name", "input");
	if (isDefined(commentField) && !isNull(commentField)) {
		commentField.value = "[Enter a comment]";
	}
}


/**
 * Sets a layout for the table based on a block of xml.  
 * @param xmlLayout The block of xml containing the layout to set. 
 */
MyTable.prototype.setXmlLayout = function(xmlLayout) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = xmlLayout;
	this.callSuper("setXmlLayout", argumentsToSuper);
}


// messages


/**
 * Set a error message in the tables message field.
 * @param eror The xml tag containing the error message.
 */
MyTable.prototype.setErrorMessage = function (error) {
	if (error.firstChild.nodeType != 3) {
		// the node is not a text node
		return;
	}

	var message = error.firstChild.nodeValue;
	this.setMessage(undefined, message);
}


/**
 * Set a success message in the tables message field.
 * @param success The xml tag containing the success message.
 */
MyTable.prototype.setSuccessMessage = function (success) {
	if (success.firstChild.nodeType != 3) {
		// the node is not a text node
		return;
	}

	var message = success.firstChild.nodeValue;
	this.setMessage(message,undefined);
}


/**
 * Set the tables mesage field.
 * @param successMessage The success message to set. 
 * @param errorMessage The error message to set. 
 */
MyTable.prototype.setMessage = function(sucessMessage, errorMessage) {
	if (isDefined(sucessMessage) && !isEmpty(sucessMessage)) {
		setClass(this.getMessageContainer(), "messageContainer ok");
		this.getMessageContainer().innerHTML = "<nobr>" + sucessMessage + "</nobr>";
	}
	else if (isDefined(errorMessage) && !isEmpty(errorMessage)) {
		setClass(this.getMessageContainer(), "messageContainer error");
		this.getMessageContainer().innerHTML = "<nobr>" + errorMessage + "</nobr>";
	}
	
	// increase the height to make room for the message container if needed
	if (this.rows.length >= ProbeTableManager.MAX_ROW_NO_SCROLLBAR) {
		this.messageHeightIncreased = true;
		this.increaseHeight(this.getMessageContainer().offsetHeight);
	}
}


/**
 * Clear the message field.
 */
MyTable.prototype.clearMessageField = function() {
	if (this.messageHeightIncreased) {
		this.messageHeightIncreased = false;
		this.increaseHeight(this.getMessageContainer().offsetHeight * -1); 
	}
	setClass(this.getMessageContainer(), "inactive");
}


// htmlCode


/**
 * Paints the table. This is overridden to paint the bottom row where
 * users can add entries to the table. 
 */
MyTable.prototype.paint = function() {
	this.callSuper("paint");
	
	if (!isNull(this.getAddEntryTableCode())) { 
		this.getAddEntryContainerCode().removeChild(this.getAddEntryTableCode());
	}
	
	this.paintAddEntryTable();
	
	if (isNull(this.getMessageContainer())) {
		var messageDiv = document.createElement("div");
		messageDiv.setAttribute("name", "messageContainer" + this.id);
		setClass(messageDiv, "inactive");
		this.getTableContainerCode().appendChild(messageDiv);
	}
	
	if (!this.heightIncreased) {
		// make room for the add entry table
		this.increaseHeight(exists(this.getAddEntryTableCode().offsetHeight) ? this.getAddEntryTableCode().offsetHeight : 0);
		this.heightIncreased = true;
	}	
	
	this.adjustSize();
}


/**
 * Updates the html table according to the data in memory.
 * This is overridden to make room for the delete column.
 */
MyTable.prototype.updateHtmlTable = function() {
	this.callSuper("updateHtmlTable");
	if (!this.cellWidthDecreased && this.calculateButtonColumnWidth() > 0) {
		// make room for the button column
		this.resizeAllColumns(this.calculateButtonColumnWidth() * (-1));
		this.cellWidthDecreased = true;
	}
	
	// the add entry table should always be right under the data table, to do this
	// we need to set the height of the data table container for each row that is added
	if (isDefined(this.getDataTableContainer()) && !isNull(this.getDataTableContainer())) {
		this.setTableContainerHeight(this.calculateTableContainerHeight());
	}
}


/**
 * Updates a html row of the table according to the data in the specified data row.
 * This is overridden to update the delete link as the order of the row changes,
 * this need to be done or the delete links my delete the wrong rows.
 * @param dataRow The array containing the data to insert into the html row.
 * @param htmlRow The html row that is to be update with the data in dataRow.
 */
MyTable.prototype.updateTableRow = function(dataRow, htmlRow) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = dataRow;
	argumentsToSuper[1] = htmlRow;
	this.callSuper("updateTableRow", argumentsToSuper);
	
	// update the rows delete link, so the link deletes the correct entry
	var deleteLink = getElementWithAttribute(htmlRow, MyTable.DELETE_LINK_NAME, "name", "a");
	if (isDefined(deleteLink) && !isNull(deleteLink)) {
		this.setDeleteLinkOnClick(deleteLink, dataRow);
	}
}


/**
 * Updates a html table cell of the table according to the data in the specified field.
 * This is overridden to insert the new data into input fields like the comment field.
 * @param dataField The field containing the data to update the cell with.
 * @param htmlCell The html cell that is to be updated with the data in dataField.
 * @param htmlRow The row the cell to update is in.
 */
MyTable.prototype.updateTableCell = function(dataField, htmlCell, htmlRow) {
	if (dataField == null) {
		var dataValue = "";
	}
	else {
		var dataValue = dataField.getValue();
	}
	
	// if the field has a input field, set hte new value for the input field
	var inputField = htmlCell.getElementsByTagName("input")[0];
	if (isDefined(inputField) && !isNull(inputField)) {
		this.updateCommentField(dataField, htmlCell, htmlRow);
		return;
	}
	
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = dataField;
	argumentsToSuper[1] = htmlCell;
	argumentsToSuper[2] = htmlRow;
	this.callSuper("updateTableCell", argumentsToSuper);
}


/**
 * Update the data table comment field.
 * @param dataField The field containing the data to update the cell with.
 * @param htmlCell The html cell that is to be updated with the data in dataField.
 * @param htmlRow The row the cell to update is in.
 */
MyTable.prototype.updateCommentField = function(dataField, htmlCell, htmlRow) {
	// if the input field is blocked do not update it
	var blocked = this.getRowIndex(dataField) == this.blockedCommentRowIndex;
	if (blocked) {
		// the comment field has been blocked
		return;
	}
	
	// update the text of the comment field
	var inputField = htmlCell.getElementsByTagName("input")[0];
	inputField.value = URLDecode(dataField.getValue());
	
	// since the order the rows are in can change on a update we need to change the
	// id of the comment field so that the onfocus, onblur and onchange change
	// the correct row
	var rowIndex = this.getRowIndex(dataField);
	inputField.setAttribute("id", this.getRowId(this.rows[rowIndex]));	
}


/**
 * Calculates the width of the button column.
 * @return The width of the delete cell. 
 */
MyTable.prototype.calculateButtonColumnWidth = function() {
	var deleteCell = getElementWithAttribute(this.getTableCode(), "buttonCell", "name", "td");
	if (!isDefined(deleteCell) || isNull(deleteCell)) {
		return 0;
	}

	return deleteCell.offsetWidth;
}


/**
 * Create a table cell for each entry in a array.
 * This is overridden to add the delete column to the data rows. 
 * @param data The array to create data from.
 * @param row The row to insert the table cells created.
 * @throws Error if data is null or not defined.
 * @throws Error if row is null or not defined.
 */
MyTable.prototype.createRowCode = function(data, row) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = data;
	argumentsToSuper[1] = row;
	this.callSuper("createRowCode", argumentsToSuper);
	
	// add the delete column if the row is defined and not the header row
	if (!isDefined(row) || row.length == 0) {
		return;
	}

	if (!this.cellWidthDecreased) {
		// make room for the button column
		this.resizeAllColumns(MyTable.DELETE_BUTTON_CELL_WIDTH * (-1));
		this.cellWidthDecreased = true;
	}
	this.addButtonColumn(data, row);

	// the add entry table should always be right under the data table, to do this
	// we need to set the height of the data table container for each row that is added
	if (isDefined(this.getDataTableContainer()) && !isNull(this.getDataTableContainer())) {
		this.setTableContainerHeight(this.calculateTableContainerHeight());
	}
}


/**
 * Inserts a field value into a given html node. 
 * This method is overridden to add KB, MB, and GB to byte type columns, and add 
 * input fields to the comment fields. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
MyTable.prototype.insertFieldValue = function(field, node, cell) {
	if (field != null && !this.isHeader(field) && "comment".equalsIgnoreCase(field.getName())) {
		// add the input field where the user can change the comment
		cell.style.paddingTop = "0px";
		cell.style.paddingBottom = "0px";
		var inputField = this.createInputField("comment", MyTable.INPUT_FIELD_SIZE, URLDecode(field.getValue()));
		setClass(inputField, "comment");
		
		var rowIndex = this.getRowIndex(field);
		inputField.setAttribute("id", this.getRowId(this.rows[rowIndex]));
		var thisObj = this;
		// when the user has focused a comment field the field is not updated
		// until the user moves focus away from the comment field, this is to prevent
		// the users changes to be overridden when the table is updated
		inputField.onfocus = function() {
			thisObj.blockCommentField(this.getAttribute("id"));
		}
		
		inputField.onkeyup = function(event) {
			if (!event) {
				// IE
				event = window.event;
			}
			if (event.keyCode == 13) { 
				thisObj.changeComment(this.getAttribute("id"));
			}
		}
		
		inputField.onblur = function() {
			thisObj.changeComment(this.getAttribute("id"));
			thisObj.unBlockCommentField(this.getAttribute("id"));
		}
		node.appendChild(inputField);
		return;
	}
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = field;
	argumentsToSuper[1] = node;
	argumentsToSuper[2] = cell;
	this.callSuper("insertFieldValue", argumentsToSuper);
}


/**
 * Blocks a comment field, when a comment field is blocked it is not udpated any more. 
 * This is to prevent the users changes to be overridden when the table is updated. 
 * @param value The value for the id field of the entry to change the comment for.
 */
MyTable.prototype.blockCommentField = function(value) {
	this.blockedCommentRowIndex = this.getIdRowNumber(value);
}


/**
 * Unblocks a comment field.
 * @param value The value for the id field of the entry to change the comment for.
 */
MyTable.prototype.unBlockCommentField = function(value) {
	this.blockedCommentRowIndex = -1;
}


/**
 * Adds the button cell to a given table row. 
 * @param data The array to create data from.
 * @param row The row to add the delete cell to.
 */
MyTable.prototype.addButtonColumn = function(data, row) {
	var cell = row.insertCell(row.cells.length);
	cell.setAttribute("name", "deleteCell");
	cell.style.padding = "2px";
	cell.setAttribute("valign", "middle");
	setClass(cell, "deleteCell");
	
	if (this.isHeader(data[0])) {
		cell.style.width = MyTable.DELETE_BUTTON_CELL_WIDTH + "px";
		return;
	}
	this.addDeleteLink(cell, data);
}


/**
 * Appends the delete link to a table cell.
 * @param cell The table cell to append the delete link to.
 * @param data The array to create data from.
 */
MyTable.prototype.addDeleteLink = function(cell, data) {
	var deleteLink = document.createElement("a");
	cell.appendChild(deleteLink);
	deleteLink.setAttribute("name", MyTable.DELETE_LINK_NAME);
	this.setDeleteLinkOnClick(deleteLink, data);
	
	var deleteImage = document.createElement("div");
	deleteImage.setAttribute("name", "deleteEntryImage");
	deleteImage.style.cursor = "pointer";
	deleteImage.onmouseover = showTableTooltip;
	setClass(deleteImage, "deleteImage");
	deleteLink.appendChild(deleteImage);
}


/**
 * Set the onclick value for a delete link. 
 * @param deleteLink The link to set the onclick value for. 
 * @param data The array to create data from.
 */
MyTable.prototype.setDeleteLinkOnClick = function(deleteLink , data) {
	// the id of the delete link is used to identify the row that is to be deleted
	// onclick of the link, update the id with the correct value
	deleteLink.setAttribute("id", this.getRowId(data));
	var thisObj = this;
	deleteLink.onclick = function() {
		thisObj.deleteEntry(this.getAttribute("id"));
		// hide tooltip when clicked
		tt_Hide();
	}
}


/**
 * Paints the add entry table.
 */
MyTable.prototype.paintAddEntryTable = function() {
	if (isNull(this.getAddEntryContainerCode())) {
		var addEntryContainer = this.createContainer();
		addEntryContainer.style.width = this.width + "px";
		addEntryContainer.setAttribute("id", "addEntryContainer" + this.id);
		addEntryContainer.style.overflow = "hidden";
		this.getTableContainerCode().appendChild(addEntryContainer);
		setClass(addEntryContainer, "addEntryContainer");
	}
	
	this.addEntryTable = document.createElement("table");
	this.addEntryTable.setAttribute("id", "addEntryTable" + this.id);
	this.addEntryTable.style.width = "100%";
	setClass(this.addEntryTable, this.className);
	this.getAddEntryContainerCode().appendChild(this.addEntryTable);
	
	var tBody = document.createElement("tbody");
	this.addEntryTable.appendChild(tBody);
	var row = tBody.insertRow(0);
	this.paintAddEntryRow(row);
}


/**
 * Paint the add entry row. This need to be overridden by subclasses to paint the row. 
 * @param row The row to paint into.
 */
MyTable.prototype.paintAddEntryRow = function(row) {
	// subclasses need to implement this method	
}


/**
 * Creates the add entry link used in the add entry table.
 */
MyTable.prototype.createAddEntryLink = function() {
	var addLink = document.createElement("a");
	var addImage = document.createElement("img");
	addImage.src = MyTable.ADD_ENTRY_IMAGE_URL;	
	addImage.style.cursor = "pointer";
	addImage.onmouseover = function() {
		Tip(helptext('addmytable'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
	}
	addLink.appendChild(addImage);

	var thisObj = this;
	addLink.onclick = function() {
		thisObj.addEntry();
	}
	return addLink;
}


/**
 * Creates the add entry input fields.
 * @param name The name to set for the input field. 
 * @param size The size of the input field. 
 * @param value If this is defined, this is set as the value for the input field. 
 * @param maxLength Optional parameter The max length to set for the input field.
 */
MyTable.prototype.createAddEntryInputfield = function (name, size, value, maxLength) {
	var inputField = this.createInputField(name, size, value);
	setClass(inputField, "addTableEntry");
	if (isDefined(maxLength) && !isNull(maxLength) && !isNegative(maxLength)) {
		inputField.setAttribute("maxLength", maxLength);
	}
		
	var thisObj = this;
	inputField.onkeyup = function(event) {
		if (!event) {
			// IE
			event = window.event;
		}
		if (event.keyCode == 13) { 
			thisObj.addEntry();
		}
		else {
			thisObj.clearMessageField();
		}
	}	
	
	inputField.onfocus = function() {
		thisObj.clearMessageField();
	}
	
	return inputField;	
}


/**
 * Creates a input field to insert into the table.
 * @param name The name to set for the input field. 
 * @param size The size of the input field. 
 * @param value If this is defined, this is set as the value for the input field. 
 */
MyTable.prototype.createInputField = function(name, size, value) {
	var inputField = document.createElement("input");
	inputField.setAttribute("type", "text");
	inputField.setAttribute("name", name);
	inputField.style.width = size;
	if (!isEmpty(value)) {
		inputField.value = value;
	}
	return inputField;	
}


/**
 * Makes the table container and the window larger to make room for the add entry row. 
 * @height The amount of pixels to increase the height with.
 */
MyTable.prototype.increaseHeight = function(height) {
	if (exists(this.window.container) && exists(this.window.setLayout) && this.window.setLayout == true) {
		// a layout has been set, the window has been resized to the correct height from the layout
		// but the container and the chart have been made to large because the add entry table did
		// not exist
		var height = this.getAddEntryTableCode().offsetHeight;
		var containerHeight = parseInt(this.window.container.style.height);
		var newHeight = containerHeight - height;
		this.window.container.style.height = newHeight + "px";
		this.window.activeChart.htmlCode.style.height = newHeight + "px";
		this.window.activeChart.height = newHeight;
		this.window.activeChart.updateChart();
	}
	
	var containerHeight = parseInt(this.getTableContainerCode().style.height);
	var newHeight = containerHeight + height;
	this.getTableContainerCode().style.height = newHeight + "px";
	
	if (exists(this.window.setLayout) && this.window.setLayout == true) {
		// a layout has been set, we do not need to increase the height of the window, because it already 
		// has the correct height from the layout
		this.window.setLayout = false;
		return;
	}
	
	// increase the size of the window
	var windowHeight = parseInt(this.window.getHtmlCode().style.height);
	var newWindowHeight = windowHeight + height;
	this.window.getHtmlCode().style.height = newWindowHeight + "px";
}



/**
 * Creates the data table. This is overriden to remove the heihg of the
 * container the data table is in. We remove this height to make the add entry
 * table be underneath the data table with no empty space.
 */
MyTable.prototype.createDataTable = function() {
	this.callSuper("createDataTable");
	
	this.getDataTableContainer().style.height = "0px";
}


/**
 * Set the container containing the data table to a given height. This needs to be done
 * so the add entry section can be right under the data table. 
 * @param height The height to set in pixels. 
 */
MyTable.prototype.setTableContainerHeight = function(height) {
	if (this.windowColapsed) {
		return;
	}
	this.getDataTableContainer().style.height = height + "px";
}


/**
 * Calculate the height to set for the data table container based on the amound
 * of rows in the table.
 */
MyTable.prototype.calculateTableContainerHeight = function() {
	var rowCount = this.getTableCode().rows.length;
	var rowHeight = this.getRowHeight();
	var height = 0;
	if (rowCount > ProbeTableManager.MAX_ROW_NO_SCROLLBAR) {
		height = ProbeTableManager.MAX_ROW_NO_SCROLLBAR * rowHeight;
	}
	else if (rowCount >= 0) {
		height = rowHeight * rowCount;
	}
	return height;
}


/**
 * Gets the row height. 
 * @return The height of the rows.
 */
MyTable.prototype.getRowHeight = function() {
	// because of trouble in safari with cell borders and offsetHeight the
	// rowHeight can not be calculated in safari
	if (isSafari()) {
		return MyTable.SAFARI_ROW_HEIGHT;
	}
	
	var rowCount = this.getTableCode().rows.length;
	var rowHeight = 0;
	if (rowCount > 0 ) {
		rowHeight = Math.ceil(this.getTableCode().offsetHeight / rowCount);
	}
	return rowHeight;
}


/**
 * Gets html code for the container containing the add entry table, null if not created yet. 
 */
MyTable.prototype.getAddEntryContainerCode = function() {
	if (!isDefined(this.addEntryContainer) || isNull(this.addEntryContainer)) {
		this.addEntryContainer = getElementWithAttribute(document.body, "addEntryContainer" + this.id, "id", "div");
	}
	return this.addEntryContainer;
}


/**
 * Gets the add entry table code, null if not created yet. 
 */
MyTable.prototype.getAddEntryTableCode = function() {
	if (!isDefined(this.addEntryTable) || isNull(this.addEntryTable)) {
		this.addEntryTable = getElementWithAttribute(document.body, "addEntryTable" + this.id, "id", "table");
	}
	return this.addEntryTable;
}


/**
 * Gets the message container, null if not created yet.
 */
MyTable.prototype.getMessageContainer = function() {
	if (!isDefined(this.messageContainer) || isNull(this.messageContainer)) {
		this.messageContainer = getElementWithAttribute(document.body, "messageContainer" + this.id, "name", "div");
	}
	return this.messageContainer;
}

 
/**
 * Gets the row number containing a given id value. The id value must be created as the 
 * getRowId function does (idFieldName=fieldValue&idFieldName2=fieldValue2&...);
 * @param value The value to get the rowNumber for. 
 * @return The row index for the row containing the values for the idFields in value, 
 * -1 if not found.
 */
MyTable.prototype.getIdRowNumber = function(value) {
	var length = this.rows.length;
	var idFields = this.getIdFields();
	for (var i = 0; i < length; i++) {
		// for each row we have to check if it contains the given value for the id fields
		for (var j = 0; j < idFields.length; j++) {
			var field = getFieldFromRow(idFields[j], this.rows[i]);
			if (field.getValue() != getParameterValue(value, idFields[j], "&")) {
				break;
			}
			if (j == idFields.length -1) {
				// the row contains all the values for the id field
				return i;
			}
		}
	}	
	return -1;
}


/**
 * Gets the id for a row. The id is based on the id fields set. 
 * @param row The row to get the id for.
 */
MyTable.prototype.getRowId = function(row) {
	var idFields = this.getIdFields();
	var id = "";
	for (var i = 0; i < this.getIdFields().length; i++) {
		var field = getFieldFromRow(idFields[i], row);
		if (isDefined(field) && !isNull(field)) {
			id += idFields[i] + "=" + field.getValue() + "&";
		}
	}
	return id;
}


// resizing


/**
 * Resizes the table containers. This is overriden to also resize the 
 * addEntryTableContainer.
 * @param deltaHeight The height to change the size of the table with.
 * @param deltaWidth The width to change the size of the table with.
 * @throws Error if deltaHeight is undefined.
 * @throws Error if deltaWidth is undefined.
 */
MyTable.prototype.resizeContainers = function(deltaHeight, deltaWidth) {
	assertDefined(deltaHeight, "DeltaHeight is undefined");
	assertDefined(deltaWidth, "DeltaWidth is undefined");
	
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = deltaHeight;
	argumentsToSuper[1] = deltaWidth;
	this.callSuper("resizeContainers", argumentsToSuper);
	
	this.resizeAddEntryContainer(deltaHeight, deltaWidth);	
}


/**
 * Resizes the add entry table container.
 * @param deltaHeight The height to change the size of the table with.
 * @param deltaWidth The width to change the size of the table with.
 */
MyTable.prototype.resizeAddEntryContainer = function (deltaHeight, deltaWidth) {
	if (isDefined(this.getAddEntryContainerCode()) && !isNull(this.getAddEntryContainerCode())) {
		resizeHtmlElement(this.getAddEntryContainerCode(), deltaHeight, deltaWidth);
	}
}


/**
 * Get the html code of all the cells in one column.
 * This is overriden to add the cells in the add entry table.
 * @param name The name of the column to get the html fields for.
 * @return A array containing all the table cells in the column with name, including the header cell. 
 * @throws Error if name is undefined null or empty.
 */
MyTable.prototype.getAllColumnHtmlCells = function(name) {
	assertDefined(name, "Name is undefined, null or empty: " + name);
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = name;
		
	var dataCells = this.callSuper("getAllColumnHtmlCells", argumentsToSuper);
	
	if (!isDefined(this.getAddEntryTableCode()) || isNull(this.getAddEntryTableCode())) {
		return dataCells;
	}
	var cell = getElementWithAttribute(this.getAddEntryTableCode(), name, "name", "td");
	if (isDefined(cell) && !isNull(cell)) {
		dataCells[dataCells.length] = cell;
	}
	return dataCells;
}


/**
 * Returns true if the scrollbars are visible, false otherwise.
 */
MyTable.prototype.isVerticalScrollBarVisible = function() {
	if (!exists(this.getTableCode())) {
		return false;
	}
	// in the mytables the height of the data table container is set depending
	// on how many rows are visible, so before we can check if the vertical
	// scrollbar is visible we need to update the height of the container
	if (isDefined(this.getDataTableContainer()) && !isNull(this.getDataTableContainer())) {
		this.setTableContainerHeight(this.calculateTableContainerHeight());
	}
	
	var tableHeight = this.getTableCode().offsetHeight;
	var tableContainerHeight = this.getDataTableContainer().offsetHeight;
	if (tableHeight > tableContainerHeight) {
		return true;
	}
	
	var headerHeight = this.getHeaderTableCode().offsetHeight;
	var tableCode = this.getTableCode();
	if (tableCode.rows.length == 0) {
		// no data rows are present
		return false;
	}
	
	var addEntryTableHeight = 0;
	if (!isNull(this.getAddEntryTableCode())) {
		addEntryTableHeight = this.getAddEntryContainerCode().offsetHeight;
	}
	
	// in safari we can not measure the height of a row with offsetHeight on the row itself
	// we have to do it on one of the rows cells
	var dataRowHeight = 0;
	if (isDefined(tableCode.rows[0].cells[0])) {
		dataRowHeight = tableCode.rows[0].cells[0].offsetHeight
	}
	var totalHeight = (dataRowHeight * tableCode.rows.length) + headerHeight + addEntryTableHeight;
	if (totalHeight > this.getTableContainerCode().offsetHeight) {
		return true;
	}
	return false;
}