/**
 * This file is responsible for managing windows.
 */


/**
 * The offset to add to a parent windows left position to get the left position
 * of detached windows.
 */
ProbeWindowManager.LEFT_DETACHED_OFFSET = 23;


/**
 * The offset to add to a parent windows top position to get the top position
 * of a detached window.
 */
ProbeWindowManager.TOP_DETACHED_OFFSET = 0;


/**
 * The id of the dummy window copied when a chart is detached. We copy a window
 * instead of creating a new window for performance reasons.
 */
ProbeWindowManager.DUMMY_WINDOW_ID = "dummyWindow";


/**
 * The functions used to init the windows.
 */
var initWindowFunction = {summarywindow: initSummaryWindow, topnhostswindow: initTopnHostsWindow,
	topnprotocolswindow: initTopnProtocolsWindow, topnhostlistenerswindow: initTopnHostListenersWindow,
	topnhosttalkerswindow: initTopnHostTalkersWindow, topnconversationwindow: initTopnConversationsWindow,
	myprotocolswindow: initMyProtocolsWindow, myhostswindow: initMyHostsWindow, 
	myconversationwindow: initMyConversationsWindow, myprotocolsperhostwindow: initMyProtoclsPerHostWindow,
	alarmDetailWindow: initAlarmDetailWindow, alldefinedalarmswindow: initAllDefinedAlarmWindow, 
	protocoldefinedalarmswindow: initProtocolDefinedAlarmWindow, hostsdefinedalarmswindow: initHostDefinedAlarmWindow,
	conversationsdefinedalarmswindow: initConversationsDefinedAlarmWindow, summarydefinedalarmswindow: initSummaryDefinedAlarmWindow,
	alltriggeredalarmswindow: initAllTriggeredAlarmWindow, summarytriggeredalarmswindow: initSummaryTriggeredAlarmWindow,
	protocoltriggeredalarmswindow: initProtocolTriggeredAlarmWindow, hoststriggeredalarmswindow: initHostTriggeredAlarmWindow,
	conversationstriggeredalarmswindow: initConversationTriggerdAlarmWindow, profilewindow: initProfileWindow,
	triggervalueswindow: initTriggerValuesWindow, alarmactionswindow: initAlarmActionsWindow,
	alarmactionslibrarywindow: initAlarmActionsLibraryWindow, licenseStatusWindow: initLicenseStatusWindow,
	protocolDetailWindow: initProtocolDetailWindow, nlhostDetailWindow: initNlHostDetailWindow,
	topnNetworksWindow: initTopnNetworksWindow, slaveSetup: initSlaveSetupWindow, slaveDetails: initSlaveDetailWindow,
	alltriggeredmasteralarmswindow: initAllTriggeredMasterAlarmWindow};


/**
 * Array containing the windows inited indexed by their id.
 */
var initedWindows = new Array();


// construction


/**
 * Creates the probe window manager.
 */
function createProbeWindowManager() {
	var instance = new ProbeWindowManager();
	WindowManager.instance().eventProcessor.changeObject(instance);
	WindowManager.instanceObject = createSubClass(WindowManager.instance(), instance);
	
	instance.eventProcessor.addEvent(STATUSBAR_BUTTON_REMOVED_EVENT, processButtonRemovedProbeWindowManager);
}


/**
 * Returns the instance of this class.
 */
ProbeWindowManager.instance = function() {
	return WindowManager.instanceObject;
}


/**
 * Constructs the probe window manager.
 */
function ProbeWindowManager() {
}


// services



/**
 * Gets a window. This is overridden to init the window if it not has
 * has been inited yet.
 * @param id The id of the window to get.
 * @return The window, null if a windwo with this name not exists.
 * @throws Error if the id is null, not defined or empty.
 */
ProbeWindowManager.prototype.getWindow = function(id) {
	assertNotEmpty(id, "Id is null, undefined or empty");
	
	for (i = 0; i < WindowManager.windows.length; i++) {
		if (id == WindowManager.windows[i].id) {
			return WindowManager.windows[i];
		}
	}
	
	// the window might not have been initied yet
	if (exists(initWindowFunction[id]) && !this.windowInited(id)) {
		return initWindowFunction[id]();
	}
	return null;
}


/**
 * Checks if a window has been initialized.
 * @param id The id of the window to check.
 * @return True if the window has been inited, false otherwise.
 * @throws Error if id is undefined, null or empty.
 */
ProbeWindowManager.prototype.windowInited = function(id) {
	assertNotEmpty(id, "The id to check for is undefined, null or empty");
	if (initedWindows[id]) {
		return true;
	}
	return false;
}


/**
 * Initalizes the default windwos.
 */
function initDefaultWindows() {
	if (currentTabName == MASTER_MAIN_TAB_NAME) {
		WindowManager.instance().activateWindow("topnNetworksWindow");
		WindowManager.instance().showWindow("alltriggeredmasteralarmswindow");
	}
	else {
		WindowManager.instance().activateWindow("summarywindow");
		WindowManager.instance().showWindow("topnprotocolswindow");
	}
}


// implementation


/**
 * Inits the status window.
 */
function initLicenseStatusWindow() {
	if (!exists(getElementWithAttribute(document.body, "licenseStatusWindow", "id", "table"))) {
		// license window does not exist, might be the setup tab
		return null;
	}
	initedWindows["licenseStatusWindow"] = true;
	var statusWindow = new Window("licenseStatusWindow")
	adjustWindowSize(statusWindow);
	return statusWindow;
}


/**
 * Inits the summary window.
 */
function initSummaryWindow() {
	initedWindows["summarywindow"] = true;
	var summaryWindow = createProbeWindow("summarywindow", "summaryTable");
	summaryWindow.setMinWidth(400);
	var throughputChartsContainer = getElementWithAttribute(summaryWindow.htmlCode, "throughputChartsContainer", "name", "div");
	var probeName = document.getElementsByName("probeName")[0].value;
	// the id for the chart needs to be unique, the uniqueInteger() woud return the same
	// number on the next login
	throughputChartsContainer.setAttribute("name", probeName + ", Bytes:summarywindow" + randomInteger(1000, 10000));
	probeThroughputContainer = createChartContainer(throughputChartsContainer, probeName + ", Bytes", summaryWindow, 'summaryTable');
	probeThroughputContainer.setField(5);
	probeThroughputContainer.showLoading();
	// need to reset the oncontextmenu due to some problems
	var gaugeChartContainer = getElementWithAttribute(throughputChartsContainer, "gaugeChartContainer", "name", "div");
	gaugeChartContainer.oncontextmenu = showSetChartMode;
	summaryWindow.addChart(probeThroughputContainer);
	adjustWindowSize(summaryWindow);
	return summaryWindow;
}


/**
 * Inits the master topn networks window.
 */
function initTopnNetworksWindow() {
	initedWindows["topnNetworksWindow"] = true;
	var topnNetworksWindow = createProbeWindow("topnNetworksWindow", "topNNetworksTable")
	topnNetworksWindow.setMinWidth(450);
	topnNetworksWindow.toggleAutomaticPositioning(true);	
	var topNNetworksChartCode = getElementWithAttribute(topnNetworksWindow.htmlCode, "topnnetworkschartimage", "name", "div");
	var topnNetworksChart = createLineChartContainer(topNNetworksChartCode, "Top n networks chart", topnNetworksWindow, "topNNetworksTable");
	topnNetworksChart.setField(5);
	topnNetworksWindow.addChart(topnNetworksChart);
	
	if (isIE()) {
		// add the index selects to the select tags that need to be replaced with text in IE, when window
		// inactivated or dragged
		topnNetworksWindow.addToSelectReplaceDiv("Bytes", getElementWithAttribute(topnNetworksWindow.getHtmlCode(), "indexSelecttopNNetworksTable", "id", "select"), "selectReplace");
	}
	adjustWindowSize(topnNetworksWindow);
	return topnNetworksWindow;
}


/**
 * Inits the topn hosts window.
 */
function initTopnHostsWindow() {
	initedWindows["topnhostswindow"] = true;
	var topNHostsWindow = createProbeWindow("topnhostswindow", "topNHostsTable");
	topNHostsWindow.setMinWidth(400);
	topNHostsWindow.toggleAutomaticPositioning(true);	
	var topNHostsChartCode = getElementWithAttribute(topNHostsWindow.htmlCode, "topnhostschartimage", "name", "div");
	var topnHostsChart = createTopNChart(topNHostsChartCode, "Top n hosts chart", topNHostsWindow, "topNHostsTable");
	topnHostsChart.showLoading();
	topNHostsWindow.addChart(topnHostsChart);
	
	if (isIE()) {
		// add the index selects to the select tags that need to be replaced with text in IE, when window
		// inactivated or dragged
		topNHostsWindow.addToSelectReplaceDiv("Bytes", getElementWithAttribute(topNHostsWindow.getHtmlCode(), "indexSelecttopNHostsTable", "id", "select"), "selectReplace");
	}
	adjustWindowSize(topNHostsWindow);
	return topNHostsWindow;
}


/**
 * Inits the topn protocols window.
 */
function initTopnProtocolsWindow() {
	initedWindows["topnprotocolswindow"] = true;
	var topNProtocolsWindow = createProbeWindow("topnprotocolswindow", "topNProtocolsTable");
	topNProtocolsWindow.setMinWidth(400);
	var topnProtocolsChartCode = getElementWithAttribute(topNProtocolsWindow.htmlCode, "topnprotocolschartimage", "name", "div");
	var topnChart = createTopNChart(topnProtocolsChartCode, "Top n protocols chart", topNProtocolsWindow, "topNProtocolsTable");
	topnChart.showLoading();
	topNProtocolsWindow.addChart(topnChart);
	
	if (isIE()) {
		// add the index selects to the select tags that need to be replaced with text in IE, when window
		// deactivated or dragged
		topNProtocolsWindow.addToSelectReplaceDiv("Bytes", getElementWithAttribute(topNProtocolsWindow.getHtmlCode(), "indexSelecttopNProtocolsTable", "id", "select"), "selectReplace");
	}
	adjustWindowSize(topNProtocolsWindow);
	return topNProtocolsWindow;
}


/**
 * Inits the nlhost detail window.
 */
function initNlHostDetailWindow() {
	initedWindows["nlhostDetailWindow"] = true;
	var nlhostDetailWindow = createEntryDetailWindow("nlhostDetailWindow", "nlhostDetailTable", "nlhostTopProtocolsTable", "nlhostTopConversationsTable");	
	nlhostDetailWindow.setMinWidth(400);
	nlhostDetailWindow.toggleAutomaticPositioning(true);
	adjustWindowSize(nlhostDetailWindow);
	return nlhostDetailWindow;
}


/**
 * Inits the protocol detail window.
 */
function initProtocolDetailWindow() {
	initedWindows["protocolDetailWindow"] = true;
	var protocolDetailWindow = createEntryDetailWindow("protocolDetailWindow", "protocolDetailTable", "protocolTopHostsTable", "protocolTopConversationsTable");	
	protocolDetailWindow.setMinWidth(400);
	protocolDetailWindow.toggleAutomaticPositioning(true);
	adjustWindowSize(protocolDetailWindow);
	return protocolDetailWindow;
}


/**
 * Inits the topn hosts listeners window.
 */
function initTopnHostListenersWindow() {
	initedWindows["topnhostlistenerswindow"] = true;
	var topNHostListenersWindow = createProbeWindow("topnhostlistenerswindow", "topNHostListenersTable");
	topNHostListenersWindow.setMinWidth(450);
	topNHostListenersWindow.toggleAutomaticPositioning(true);	
	var topNHostListenersChartCode = getElementWithAttribute(topNHostListenersWindow.htmlCode, "topnhostlistenerschartimage", "name", "div");
	var topnChart = createTopNChart(topNHostListenersChartCode, "Top n listeners chart", topNHostListenersWindow, "topNHostListenersTable");
	topnChart.showLoading();
	topNHostListenersWindow.addChart(topnChart);
	
	if (isIE()) {
		// add the index selects to the select tags that need to be replaced with text in IE, when window
		// inactivated or dragged
		topNHostListenersWindow.addToSelectReplaceDiv("Bytes", getElementWithAttribute(topNHostListenersWindow.getHtmlCode(), "indexSelecttopNHostListenersTable", "id", "select"), "selectReplace");
	}
	adjustWindowSize(topNHostListenersWindow);
	return topNHostListenersWindow;
}


/**
 * Inits the topn host talkers window.
 */
function initTopnHostTalkersWindow() {
	initedWindows["topnhosttalkerswindow"] = true;
	var topNHostTalkersWindow = createProbeWindow("topnhosttalkerswindow", "topNHostTalkersTable");
	topNHostTalkersWindow.setMinWidth(400);
	topNHostTalkersWindow.toggleAutomaticPositioning(true);		
	var topNHostTalkersChartCode = getElementWithAttribute(topNHostTalkersWindow.htmlCode, "topnhosttalkerschartimage", "name", "div");
	var topnChart = createTopNChart(topNHostTalkersChartCode, "Top n talkers chart", topNHostTalkersWindow, "topNHostTalkersTable");
	topnChart.showLoading();
	topNHostTalkersWindow.addChart(topnChart);
	
	if (isIE()) {
		topNHostTalkersWindow.addToSelectReplaceDiv("Bytes", getElementWithAttribute(topNHostTalkersWindow.getHtmlCode(), "indexSelecttopNHostTalkersTable", "id", "select"), "selectReplace");
	}
	adjustWindowSize(topNHostTalkersWindow);
	return topNHostTalkersWindow;	
}


/**
 * Inits the topn conversations window.
 */
function initTopnConversationsWindow() {
	initedWindows["topnconversationwindow"] = true;
	var topNConversationWindow = new createProbeWindow("topnconversationwindow", "topNConversationsTable");
	topNConversationWindow.setMinWidth(450);
	topNConversationWindow.toggleAutomaticPositioning(true);
	var topNConversationChartCode = getElementWithAttribute(topNConversationWindow.htmlCode, "topnconversationschartimage", "name", "div");
	var topnChart = createTopNChart(topNConversationChartCode, "Top n conversations chart", topNConversationWindow, "topNConversationsTable");
	topnChart.showLoading();
	topNConversationWindow.addChart(topnChart);
		
	if (isIE()) {
		// add the index selects to the select tags that need to be replaced with text in IE, when window
		// inactivated or dragged
		topNConversationWindow.addToSelectReplaceDiv("Bytes", getElementWithAttribute(topNConversationWindow.getHtmlCode(), "indexSelecttopNConversationsTable", "id", "select"), "selectReplace");
	}
	adjustWindowSize(topNConversationWindow);
	return topNConversationWindow;
}


/**
 * Inits the myprotocols window.
 */
function initMyProtocolsWindow() {
	initedWindows["myprotocolswindow"] = true;
	var myProtocolsWindow = createProbeWindow("myprotocolswindow", "myProtocolsTable");
	myProtocolsWindow.setMinWidth(400);
	myProtocolsWindow.toggleAutomaticPositioning(true);	
	var myProtocolsChartCode = getElementWithAttribute(myProtocolsWindow.htmlCode, "myprotocolschartimage", "name", "div");
	var myChart = createMyChart(myProtocolsChartCode, "My protocols chart", myProtocolsWindow, "myProtocolsTable");
	myChart.showLoading();
	myProtocolsWindow.addChart(myChart);
	adjustWindowSize(myProtocolsWindow);	
	return myProtocolsWindow;
}


/**
 * Inits the myhost window.
 */
function initMyHostsWindow() {
	initedWindows["myhostswindow"] = true;
	var myHostsWindow = createProbeWindow("myhostswindow", "myHostsTable");
	myHostsWindow.setMinWidth(400);
	myHostsWindow.toggleAutomaticPositioning(true);	
	var myHostsChartCode = getElementWithAttribute(myHostsWindow.htmlCode, "myhostschartimage", "name", "div");
	var myChart = createMyChart(myHostsChartCode, "My hosts chart", myHostsWindow, "myHostsTable");
	myChart.showLoading();
	myHostsWindow.addChart(myChart);
	adjustWindowSize(myHostsWindow);
	return myHostsWindow;
}


/**
 * Init the myconversatinos window.
 */
function initMyConversationsWindow() {
	initedWindows["myconversationwindow"] = true;
	var myConversationWindow = createProbeWindow("myconversationwindow", "myConversationsTable");
	myConversationWindow.setMinWidth(450);
	myConversationWindow.toggleAutomaticPositioning(true);		
	var myConversationChartCode = getElementWithAttribute(myConversationWindow.htmlCode, "myconversationschartimage", "name", "div");
	var myChart = createMyChart(myConversationChartCode, "My conversations chart", myConversationWindow, "myConversationsTable");
	myChart.showLoading();
	myConversationWindow.addChart(myChart);
	adjustWindowSize(myConversationWindow);
	return myConversationWindow;
}


/**
 * Init the myprotocolsperhost window.
 */
function initMyProtoclsPerHostWindow() {
	initedWindows["myprotocolsperhostwindow"] = true;
	var myProtocolsPerHostWindow = createProbeWindow("myprotocolsperhostwindow", "myProtocolsPerHostTable");
	myProtocolsPerHostWindow.setMinWidth(400);
	myProtocolsPerHostWindow.toggleAutomaticPositioning(true);		
	var myProtocolsPerHostChartCode = getElementWithAttribute(myProtocolsPerHostWindow.htmlCode, "myprotocolsperhostchartimage", "name", "div");
	var myChart = createMyChart(myProtocolsPerHostChartCode, "My protocols per host chart", myProtocolsPerHostWindow, "myProtocolsPerHostTable");
	myChart.showLoading();
	myProtocolsPerHostWindow.addChart(myChart);
	adjustWindowSize(myProtocolsPerHostWindow);
	return myProtocolsPerHostWindow;
}


/**
 * Inits the alarm detail window.
 */
function initAlarmDetailWindow() {
	initedWindows["alarmDetailWindow"] = true;
	var alarmDetailWindow = createAlarmDetailWindow("alarmDetailWindow");
	adjustWindowSize(alarmDetailWindow);
	
	// set the correct class for the table select and the default field chooser
	var tableChooser = getElementWithAttribute(alarmDetailWindow.getHtmlCode(), "tableChooser", "name", "select"); 
	setClass(tableChooser, isIE() ? "alarmTableChooserIE" : "alarmTableChooser");
	var defaultFieldChooser = getElementWithAttribute(alarmDetailWindow.getHtmlCode(), "defaultFieldChooser", "name", "select");
	setClass(defaultFieldChooser, isIE() ? "fieldChooserIE" : "fieldChooser");
	return alarmDetailWindow;
}


/**
 * Inits the all defined alarms window.
 */
function initAllDefinedAlarmWindow() {
	initedWindows["alldefinedalarmswindow"] = true;
	var allDefinedAlarmsWindow = createProbeWindow("alldefinedalarmswindow", "allDefinedAlarmsTable");
	allDefinedAlarmsWindow.setMinWidth(400);
	allDefinedAlarmsWindow.setMinHeight(100);
	allDefinedAlarmsWindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(allDefinedAlarmsWindow);
	TableManager.instance().getTable("allDefinedAlarmsTable");
	return allDefinedAlarmsWindow;
}


/**
 * Init the protocol defined alarm window.
 */
function initProtocolDefinedAlarmWindow() {
	initedWindows["protocoldefinedalarmswindow"] = true;
	var protocoldefinedalarmswindow = createProbeWindow("protocoldefinedalarmswindow", "protocolDefinedAlarmsTable");
	protocoldefinedalarmswindow.setMinWidth(400);
	protocoldefinedalarmswindow.setMinHeight(100);
	protocoldefinedalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(protocoldefinedalarmswindow);
	TableManager.instance().getTable("protocolDefinedAlarmsTable");
	return protocoldefinedalarmswindow;
}


/**
 * Init the hosts defind alarms window.
 */
function initHostDefinedAlarmWindow() {
	initedWindows["hostsdefinedalarmswindow"] = true;
	var hostsdefinedalarmswindow = createProbeWindow("hostsdefinedalarmswindow", "hostsDefinedAlarmsTable");
	hostsdefinedalarmswindow.setMinWidth(400);
	hostsdefinedalarmswindow.setMinHeight(100);
	hostsdefinedalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(hostsdefinedalarmswindow);
	TableManager.instance().getTable("hostsDefinedAlarmsTable");
	return hostsdefinedalarmswindow;
}


/**
 * Init the conversations defind alarms window.
 */
function initConversationsDefinedAlarmWindow() {
	initedWindows["conversationsdefinedalarmswindow"] = true;
	var conversationsdefinedalarmswindow = createProbeWindow("conversationsdefinedalarmswindow", "conversationsDefinedAlarmsTable");
	conversationsdefinedalarmswindow.setMinWidth(400);
	conversationsdefinedalarmswindow.setMinHeight(100);
	conversationsdefinedalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(conversationsdefinedalarmswindow);
	TableManager.instance().getTable("conversationsDefinedAlarmsTable");
	return conversationsdefinedalarmswindow;
}


/**
 * Init the summary defined alarm window.
 */
function initSummaryDefinedAlarmWindow() {
	initedWindows["summarydefinedalarmswindow"] = true;
	var summarydefinedalarmswindow = createProbeWindow("summarydefinedalarmswindow", "summaryDefinedAlarmsTable");
	summarydefinedalarmswindow.setMinWidth(400);
	summarydefinedalarmswindow.setMinHeight(100);
	summarydefinedalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(summarydefinedalarmswindow);
	TableManager.instance().getTable("summaryDefinedAlarmsTable");
	return summarydefinedalarmswindow;
}


/**
 * Inits the all triggered alarm window.
 */
function initAllTriggeredAlarmWindow() {
	initedWindows["alltriggeredalarmswindow"] = true;
	var alltriggeredalarmswindow = createProbeWindow("alltriggeredalarmswindow", "allTriggeredAlarmsTable");
	alltriggeredalarmswindow.setMinWidth(400);
	alltriggeredalarmswindow.setMinHeight(100);
	alltriggeredalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(alltriggeredalarmswindow);
	TableManager.instance().getTable("allTriggeredAlarmsTable");
	return alltriggeredalarmswindow;
}


/**
 * Inits the all master all triggered alarm window. 
 */
function initAllTriggeredMasterAlarmWindow() {
	initedWindows["alltriggeredmasteralarmswindow"] = true;
	var alltriggeredmasteralarmswindow = createProbeWindow("alltriggeredmasteralarmswindow", "allTriggeredMasterAlarmsTable");
	alltriggeredmasteralarmswindow.setMinWidth(400);
	alltriggeredmasteralarmswindow.setMinHeight(100);
	alltriggeredmasteralarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(alltriggeredmasteralarmswindow);
	TableManager.instance().getTable("allTriggeredMasterAlarmsTable");
	return alltriggeredmasteralarmswindow;
}


/**
 * Init the summary triggerd alarm window.
 */
function initSummaryTriggeredAlarmWindow() {
	initedWindows["summarytriggeredalarmswindow"] = true;
	var summarytriggeredalarmswindow = createProbeWindow("summarytriggeredalarmswindow", "summaryTriggeredAlarmsTable");
	summarytriggeredalarmswindow.setMinWidth(400);
	summarytriggeredalarmswindow.setMinHeight(100);
	summarytriggeredalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(summarytriggeredalarmswindow);
	TableManager.instance().getTable("summaryTriggeredAlarmsTable");
	return summarytriggeredalarmswindow;
}


/**
 * Init the protocol triggered alarm window.
 */
function initProtocolTriggeredAlarmWindow() {
	initedWindows["protocoltriggeredalarmswindow"] = true;
	var protocoltriggeredalarmswindow = createProbeWindow("protocoltriggeredalarmswindow", "protocolTriggeredAlarmsTable");
	protocoltriggeredalarmswindow.setMinWidth(400);
	protocoltriggeredalarmswindow.setMinHeight(100);
	protocoltriggeredalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(protocoltriggeredalarmswindow);
	TableManager.instance().getTable("protocolTriggeredAlarmsTable");
	return protocoltriggeredalarmswindow;
}


/**
 * Init the host triggered alarm window.
 */
function initHostTriggeredAlarmWindow() {
	initedWindows["hoststriggeredalarmswindow"] = true;
	var hoststriggeredalarmswindow = createProbeWindow("hoststriggeredalarmswindow", "hostsTriggeredAlarmsTable");
	hoststriggeredalarmswindow.setMinWidth(400);
	hoststriggeredalarmswindow.setMinHeight(100);
	hoststriggeredalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(hoststriggeredalarmswindow);
	TableManager.instance().getTable("hostsTriggeredAlarmsTable");
	return hoststriggeredalarmswindow;
}


/**
 * Init the conversations triggered alarm window.
 */
function initConversationTriggerdAlarmWindow() {
	initedWindows["conversationstriggeredalarmswindow"] = true;
	var conversationstriggeredalarmswindow = createProbeWindow("conversationstriggeredalarmswindow", "conversationsTriggeredAlarmsTable");
	conversationstriggeredalarmswindow.setMinWidth(400);
	conversationstriggeredalarmswindow.setMinHeight(100);
	conversationstriggeredalarmswindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(conversationstriggeredalarmswindow);
	TableManager.instance().getTable("conversationsTriggeredAlarmsTable");
	return conversationstriggeredalarmswindow;
}


/**
 * Inits the slave setup window. 
 */
function initSlaveSetupWindow() {
	initedWindows["slaveSetup"] = true;
	var slaveSetupWindow = new Window("slaveSetup");
	slaveSetupWindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(slaveSetupWindow);
	return slaveSetupWindow;
}


/**
 * Inits the slave detail window.
 */
function initSlaveDetailWindow() {
	initedWindows["slaveDetails"] = true;
	var slaveDetailsWindow = new Window("slaveDetails");
	slaveDetailsWindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(slaveDetailsWindow);
	return slaveDetailsWindow;
}

 
/**
 * Init the profile window.
 */
function initProfileWindow() {
	initedWindows["profilewindow"] = true;
	var profileWindow = createProfileWindow("profilewindow", "profilesTable");
	profileWindow.toggleAutomaticPositioning(true);	
	adjustWindowSize(profileWindow);
	return profileWindow;
}


/**
 * Init the alarm trigger value window.
 */
function initTriggerValuesWindow() {
	initedWindows["triggervalueswindow"] = true;
	var triggerValuesWindow = createProbeWindow("triggervalueswindow", "triggervaluestable");
	triggerValuesWindow.toggleAutomaticPositioning(true);	
	triggerValuesWindow.setMinWidth(200);
	triggerValuesWindow.setMinHeight(100);
	adjustWindowSize(triggerValuesWindow);
	return triggerValuesWindow;
}


/**
 * Init the alarm actions window. 
 */
function initAlarmActionsWindow() {
	initedWindows["alarmactionswindow"] = true;
	var alarmActionsWindow = createActionWindow("alarmactionswindow", "emailAlarmActionsTable");
	alarmActionsWindow.toggleAutomaticPositioning(true);	
	alarmActionsWindow.setMinWidth(200);
	alarmActionsWindow.setMinHeight(100);
	adjustWindowSize(alarmActionsWindow);
	
	// init the email action detail
	var emailSection = getElementWithAttribute(alarmActionsWindow.getHtmlCode(), "emailActionDetail", "name", "table");
	var emailActionDetail = new createEmailActionDetail(SET_EMAIL_ACTION_BASE_URL, EDIT_ACTION_BASE_URL, SET_LIBRARY_EMAIL_ACTION_BASE_URL, emailSection);
	alarmActionsWindow.setEmailDetailSection(emailActionDetail);
	alarmActionsWindow.setEmailTableId("emailAlarmActionsTable");
	// init the program action detail
	var programSection = getElementWithAttribute(alarmActionsWindow.getHtmlCode(), "programActionDetail", "name", "table");
	var programActionDetail = new createProgramActionDetail(SET_PROGRAM_ACTION_BASE_URL, EDIT_ACTION_BASE_URL, SET_LIBRARY_PROGRAM_ACTION_BASE_URL, programSection);
	alarmActionsWindow.setProgramDetailSection(programActionDetail);
	alarmActionsWindow.setProgramTableId("programAlarmActionsTable");
	return alarmActionsWindow;
}


/**
 * Init the alarm action library window.
 */
function initAlarmActionsLibraryWindow() {
	initedWindows["alarmactionslibrarywindow"] = true;
	var alarmActionsLibraryWindow = createActionWindow("alarmactionslibrarywindow", "emailAlarmActionLibraryTable");
	alarmActionsLibraryWindow.toggleAutomaticPositioning(true);	
	alarmActionsLibraryWindow.setMinWidth(200);
	alarmActionsLibraryWindow.setMinHeight(100);
	adjustWindowSize(alarmActionsLibraryWindow);
	
	// init the email action detail 
	var emailLibrarySection = getElementWithAttribute(alarmActionsLibraryWindow.getHtmlCode(), "emailActionDetail", "name", "table");
	var emailLibraryActionDetail = new createEmailActionDetail(SET_LIBRARY_EMAIL_ACTION_BASE_URL, EDIT_LIBRARY_EMAIL_ACTION_BASE_URL, null, emailLibrarySection);
	alarmActionsLibraryWindow.setEmailDetailSection(emailLibraryActionDetail);
	alarmActionsLibraryWindow.setEmailTableId("emailAlarmActionLibraryTable");
	// init the program action detail
	var programSection = getElementWithAttribute(alarmActionsLibraryWindow.getHtmlCode(), "programActionDetail", "name", "table");
	var programActionDetail = new createProgramActionDetail(SET_LIBRARY_PROGRAM_ACTION_BASE_URL, EDIT_LIBRARY_EMAIL_ACTION_BASE_URL, null, programSection);
	alarmActionsLibraryWindow.setProgramDetailSection(programActionDetail);
	alarmActionsLibraryWindow.setProgramTableId("programAlarmActionLibraryTable");
		
	return alarmActionsLibraryWindow;
}


/**
 * Adjusts the size of a given window. In Opera the border of the windows is drawn on the outside of
 * the window, while in FF and IE the border is drawn on the inside of the table. So if the browser
 * used is Opera we need to make the windows smaller. 
 * @param window The window to adjust the size for.
 */
function adjustWindowSize(window) {
	if (isOpera()) {
		window.getHtmlCode().style.width = (parseInt(window.getHtmlCode().style.width) - (Window.BORDER_WIDTH * 2)) + "px";
	}
}


/**
 * Processes a status bar button removed event. When a status bar button is removed the according window is hidden.
 * @param event The event to process. 
 */
function processButtonRemovedProbeWindowManager(event) {
	var window = this.getWindowStatusbarButton(event.getData());
	if (!exists(window)) {
		return;
	}
	// simulate a click on the close button of the window
	var closeButton = getElementWithAttribute(window.getHtmlCode(), "closeButton", "id", "div");
	closeButton = exists(closeButton) ? closeButton : getElementWithAttribute(window.getHtmlCode(), "closeChartButton", "id", "div");
	if (exists(closeButton) && getClassName(closeButton) != "inactive") {
		// if the close button is not shown the window is a topn chart window and should not be closed 
		closeButton.onclick();
	}
}


/**
 * Creates a statusbar button for a given window if it does not already exists.
 * This is overridden to remove the close option for topn chart windows in the status right click menu.
 * Instead of the close button we add the attach option.
 * @param window The window to create the button for. 
 * @return The button created. 
 */
ProbeWindowManager.prototype.createStatusbarButton = function(window)  {
	var button = this.getStatusbarButton(window.getId());
	if (exists(button)) {
		return button;
	}
	// button does not exist create it
	button = new StatusbarButton(window.getWindowTitle(), null, null);
	this.statusbar.addButton(button);
	this.windowStatusbarButtons[window.getId()] = button.getId();	
	this.setStatusButtonRightClickItems(button, window);
	return button;
}


/**
 * Set the right click items for a statusbar button.
 * @param button The button to set the right click items for. 
 * @param window The window the button is for. 
 */
ProbeWindowManager.prototype.setStatusButtonRightClickItems = function(button, window) {
	var closeButton = getElementWithAttribute(window.getHtmlCode(), "closeButton", "id", "div");
	var closeChartButton = exists(closeButton) ? closeButton : getElementWithAttribute(window.getHtmlCode(), "closeChartButton", "id", "div");
	var attachButton = getElementWithAttribute(window.getHtmlCode(), "attachButton", "id", "div");
	if (!exists(closeChartButton) && exists(attachButton)) {
		// this is a detached topn chart window, remove the close right click item and 
		// add instead a attach right click item
		button.removeRightClickItem(CLOSE_RIGHT_CLICK_MENU_ITEM_ID);
		button.removeRightClickItem(HELP_RIGHT_CLICK_MENU_ITEM_ID);
		// add the attach item
		button.addRightClickItem(new StatusButtonRightClickItem("attachItem", "Attach", attachRightClick));
		button.addRightClickItem(new StatusButtonRightClickItem(HELP_RIGHT_CLICK_MENU_ITEM_ID, "Help", helpClicked));
	}
	else if (exists(attachButton)) {
		// this is a detached chart window, add the attach option in the status right click menu
		button.removeRightClickItem(CLOSE_RIGHT_CLICK_MENU_ITEM_ID);
		button.removeRightClickItem(HELP_RIGHT_CLICK_MENU_ITEM_ID);
		button.addRightClickItem(new StatusButtonRightClickItem("attachItem", "Attach", attachRightClick));
		button.addRightClickItem(new StatusButtonRightClickItem(CLOSE_RIGHT_CLICK_MENU_ITEM_ID, "Close", closeClicked));
		button.addRightClickItem(new StatusButtonRightClickItem(HELP_RIGHT_CLICK_MENU_ITEM_ID, "Help", helpClicked));
	}	
}


/**
 * A attach right click item has been clicked, attach the window.
 */
function attachRightClick(event) {
	var button = Statusbar.instance().rightClickButton;
	var window = WindowManager.instance().getWindowStatusbarButton(button);
	window.attach(window.attachId);
	WindowManager.instance().activateWindow(window.attachId);
}


// html


/**
 * Creates a new chart window.
 * @param chart The chart to create a window for.
 * @param id The id to set for the window.
 * @param title The title to set for the window.
 * @param attachId The id of the window to set in the attach link.
 */
function createChartWindow(chart, id, title, attachId) {
	var clone = cloneDummyWindow(id, chart.htmlCode, title);
	setChartWindowColapseButton(clone, id);
	setChartWindowExpandButton(clone, id);
	setMinimizeButton(clone, id);
	
	var newWindow = createProbeWindow(id);
	newWindow.makeDragable();
	newWindow.makeResizable();
	
	// set the onmousedown to activate the window when it is clicked
	newWindow.htmlCode.onmousedown = function(event) {
		WindowManager.instance().activateWindow(id);
	}
	newWindow.setResizableElements(chart.htmlCode);
	newWindow.activeChart = chart;
	newWindow.charts[0] = chart;
	newWindow.chartCount = 1;

	// add gauge legend if needed
	if (Chart.CHART_CONTAINER == chart.getType()) {
		addGaugeLegend(newWindow, chart);
	}

	positionChartWindow(newWindow, attachId);
	addAttachButton(newWindow, attachId);
	removeButton(newWindow, "closeButton");
	if (chart.isClosable()) {
		addCloseChartButton(newWindow, attachId);
	}
	newWindow.attachId = attachId;
	newWindow.setMinWidth(300);
	newWindow.setMinHeight(100);
	
	WindowManager.instance().createStatusbarButton(newWindow);
	return newWindow;
}


/**
 * Sets the minize button of the window to minize this window. 
 * @param window The htmlcode of the window.
 * @param id The id of the window.
 */
function setMinimizeButton(window, id) {
	var minimizeContentButton = getElementWithAttribute(window, "minimizeContentButton", "id", "div");
	minimizeContentButton.onclick = function() {
		WindowManager.instance().minimizeWindow(id);
	}
}


/**
 * Sets the event handling functions of a chart windows expand button.
 * @param window The htmlcode of the window.
 * @param id The id of the window.
 */
function setChartWindowExpandButton(window, id) {
	var expandContentButton = getElementWithAttribute(window, "expandContentButton", "id", "div");
	expandContentButton.onmouseout = function() {
		onMouseOutExpand(this, id, 'expandButton WindowButton');
	}
	expandContentButton.onclick = function() {
		WindowManager.instance().getWindow(id).expandContent();
		WindowManager.instance().getWindow(id).showLegend();
	}
}


/**
 * Sets the event handling function of a chart windows colapse button.
 * @param window The htmlcode of the window.
 * @param id The id of the window.
 */
function setChartWindowColapseButton(window, id) {
	var colapseContentButton = getElementWithAttribute(window, "colapseContentButton", "id", "div");
	colapseContentButton.onmouseout = function() {
		onMouseOutCollapse(this, id, 'colapseButton WindowButton');
	}
	colapseContentButton.onclick = function() {
		WindowManager.instance().getWindow(id).colapseContent();
		WindowManager.instance().getWindow(id).hideLegend();
	}
}


/**
 * Clones the dummy window.
 * @param id The id to set for the cloned window.
 * @param content The content to insert into the cloned window.
 * @param title The title to set for the cloned window.
 * @return The html code of the cloned window.
 */
function cloneDummyWindow(id, content, title) {
	var dummyWindow = getElementWithAttribute(document.body, ProbeWindowManager.DUMMY_WINDOW_ID, "id", "table");
	var clone = dummyWindow.cloneNode(true);
	WindowManager.instance().getWindowRootElement().appendChild(clone);
	
	clone.setAttribute("id", id);
	// the clone window needs to be active because of we are using offsetHeight to measure
	// the size of the window
	setClass(clone, "inactiveBorderColor");
	setClass(content, "active");
	var windowContentCell = getElementWithAttribute(clone, "windowContentCell", "id", "td");
	windowContentCell.appendChild(content);
	if (isOpera()) {
		// In Opera the border of the windows is drawn on the outside of
		// the window, while in FF and IE the border is drawn on the inside of the table. 
		clone.style.height = (clone.offsetHeight - (Window.BORDER_WIDTH * 2)) + "px";
		clone.style.width = (clone.offsetWidth - (Window.BORDER_WIDTH * 2)) + "px";
	}
	else {
		clone.style.height = parseInt(content.style.height) + (Window.RESIZE_AREA_SIZE * 2) + (Window.BORDER_WIDTH * 2) + Window.WINDOW_HEADER_HEIGHT + "px";
		clone.style.width = parseInt(content.style.width) + (Window.RESIZE_AREA_SIZE * 2) + (Window.BORDER_WIDTH * 2) + "px";
	}
	
	// set the title
	var titleNode = getElementWithAttribute(clone, "windowTitle", "name", "td");
	var title = document.createElement("nobr").appendChild(document.createTextNode(title));
	if (!exists(titleNode.firstChild)) {
		titleNode.appendChild(title);
	}
	else {
		titleNode.replaceChild(title, titleNode.firstChild);
	}
	
	return clone;
}


/**
 * Removes a button from a window.
 * @param w The window to remove the button from.
 * @param id The id of the button to remove.
 */
function removeButton(w, id) {
	var windowButtons = getElementWithAttribute(w.htmlCode, "windowButtons", "name", "td");
	var button = getElementWithAttribute(windowButtons, id, "id", "div");
	if (isDefined(button) && !isNull(button)) {
		windowButtons.removeChild(button);
	}
}


/**
 * Adds a attach button to a window.
 * @param w The window to add the button to.
 * @param attachId The id of the window to attach to.
 */
function addAttachButton(w, attachId) {
	var windowButtons = getElementWithAttribute(w.htmlCode, "windowButtons", "name", "td");
	windowButtons.appendChild(createAttachButton(w.id, attachId));
}


/**
 * Adds a close chart button to a window.
 * @param w The window to add the button to.
 * @param parentId The id of the parent window.
 */
function addCloseChartButton(w, parentId) {
	var windowButtons = getElementWithAttribute(w.htmlCode, "windowButtons", "name", "td");
	windowButtons.insertBefore(createCloseChartButton(w.id, parentId), windowButtons.firstChild);
}	


/**
 * Create the close button.
 * @param id The id of the window.
 * @param parentId The id of the parent window.
 * @return The close button.
 */
function createCloseChartButton(id, parentId) {
	var closeButton = document.createElement("div");
	closeButton.onclick = function() {
		var windowToClose = WindowManager.instance().getWindow(id);
		var parentWindow = WindowManager.instance().getWindow(parentId);
		parentWindow.removeDetachedChart(windowToClose.activeChart);
		windowToClose.fireChartClosedEvent(windowToClose.activeChart);
		ChartManager.instance().removeChart(windowToClose.activeChart);
		windowToClose.closeWindow();
	}
	closeButton.setAttribute("id", "closeChartButton");
	setClass(closeButton, "closeButton WindowButton");
	return closeButton;
}


/**
 * Creates the attach button.
 * @param id The id of the window.
 * @param attachId The id of the window to attach to.
 * @return The created attach button.
 */
function createAttachButton(id, attachId) {
	var attachButton = document.createElement("div");
	attachButton.onclick = function() {
		WindowManager.instance().getWindow(id).attach(attachId);
		WindowManager.instance().activateWindow(attachId);
	}
	attachButton.setAttribute("title", "Attach");
	attachButton.setAttribute("id", "attachButton");
	setClass(attachButton, "attachButton WindowButton");
	attachButton.onmouseover = function() {
		setClass(attachButton, "attachButtonMouseover WindowButton");
	}
	attachButton.onmouseout = function() {
		setClass(attachButton, "attachButton WindowButton");
	}
	return attachButton;
}


/**
 * Position a detached window according to its parent windows position.
 * @param window The window to position.
 * @param parentId The id of the parentWindow.
 */
function positionChartWindow(window, parentId) {
	var parentWindow = WindowManager.instance().getWindow(parentId);
	
	var chartRow = getElementWithAttribute(parentWindow.htmlCode, "chartRow", "name", "tr");
	var chartSectionShown = false;
	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		parentWindow.showChartSection();
		chartSectionShown = true;
	}
	var desktop = WindowManager.instance().getDesktopHtmlCode();
	var top = calculateTopOffset(parentWindow.getContainer(), desktop) + ProbeWindowManager.TOP_DETACHED_OFFSET;
	top = WindowManager.instance().isYpositionInScreen(top + window.getHeight()) ? top : WindowManager.instance().calculateMaxYPos() - window.getHeight();
	var left = calculateLeftOffset(parentWindow.getContainer(), desktop) + ProbeWindowManager.LEFT_DETACHED_OFFSET;
	left = WindowManager.instance().isXPositionInScreen(left + window.getWidth()) ? left : WindowManager.instance().calculateMaxXPos() - window.getWidth();
	window.htmlCode.style.top = top + "px";
	window.htmlCode.style.left = left + "px";
	
	if (chartSectionShown) {
		parentWindow.hideChartSection();;
	}
}


/**
 * Adds a gauge legend to window.
 * @param window The window to add the gauge legend to.
 */ 
function addGaugeLegend(window, chart) {
	var contentTable = getElementWithAttribute(window.htmlCode, "window", "class", "table");
	var legendRow = contentTable.insertRow(contentTable.rows.length);
	legendRow.setAttribute("id", "gaugeLegendRow");
	setClass(legendRow, "active");
	var legendCell = legendRow.insertCell(0);
	legendCell.setAttribute("name", "gaugeLegend");
	legendCell.colSpan = 2;
	legendCell.setAttribute("align", "right");
	setClass(legendCell, "gaugeLegend");
	var legendChart = document.createElement("img");
	legendChart.setAttribute("border", "0");
	legendChart.style.height = "14px";
	legendChart.src = "/themes/black/images/gauge_legend.gif";
	legendChart.setAttribute("id", "gaugechartlegend");
	legendCell.appendChild(legendChart);
	
	// increase the height of the window according to the size of the legend
	var legendHeight = legendCell.offsetHeight;
	window.htmlCode.style.height = (parseInt(window.htmlCode.style.height) +  legendHeight) + "px";
}
