

/**
 * Factory method to create a profile from a block of xml data.
 * @param xml The block of xml data to create a profile from.
 * @return The profile created, undefined if no name was specefied in xml.
 * @throws Erorr if xml is undefined or null. 
 */
function createProfile(xml) {
	assertExists(xml, "Xml data to create profile from is undefined or null");
	
	var name = getTagNodeValue("name", xml);
	var description = getTagNodeValue("description", xml);
	
	if (!isEmpty(name) && exists(description) && containsXmlTag("description", xml)) {
		return new Profile(URLDecode(name), URLDecode(description));
	}
	return null;
}
 

/**
 * Constructs a profile.
 * @param name The name to set for the profile.
 * @param description The description to set for the profile.
 * @throws Error if name is undefined null or empty.
 * @throws Erorr if description is undefined or null.
 */
function Profile(name, description) {
	assertNotEmpty(name, "The name to set for the profile is undefined null or empty");
	assertExists(description, "Description to set for the profiles is undefined or null");
	
	this.name = name;
	this.description = description;
}


/**
 * Sets the name of a profile.
 * @param name The name to set. 
 * @throws Error if name is undefined, null or empty.
 */
Profile.prototype.setName = function(name) {
	assertNotEmpty(name, "The name to set is undefined, null or empty");
	this.name = name;
}


/**
 * Returns the name of the profile.
 */
Profile.prototype.getName = function() {
	return this.name;
}


/**
 * Sets the description.
 * @param description The description to set. 
 * @throws Error if description is undefined or null.
 */
Profile.prototype.setDescription = function(description) {
	assertExists(description, "Description to set is undefined null or empty");
	this.description = description;
}


/**
 * Returns the descirption of the profile.
 */
Profile.prototype.getDescription = function() {
	return this.description;
}