/**
 * This file provides convinience methods for entry searching.
 */
 
 
/**
 * The base url used to search for entries.
 */
var ENTRY_SEARCH_BASE_URL = "/?resource=Statistics&mode=get&command=getEntry&matchAnyWhere=false&protocolSortField=octets&nlHostSortField=octets&protocolFields=index,protocolName,protocolId,protocolDescription,packets,octets,hosts,conversations,alarmCount&nlHostFields=index,hostName,address,inPackets,inOctets,outPackets,outOctets,outMacNonUnicastPackets,protocols,conversations,octets,packets,alarmcount&";


/**
 * The base request used to request for text suggestions. 
 */
var SUGGEST_BASE_URL = "/?resource=Statistics&mode=get&command=suggestEntries&maxEntryCount=100&protocolSortField=octets&nlHostSortField=octets&protocolFields=octets,protocolId&nlHostFields=octets,address&";


/**
 * The input field where the user can enter entries to search for. 
 */
var searchInput;


/**
 * The actual value searched for. When the user enters a host name into the search field for example 
 * we use this field to store the hostnames ip address. This way we can search for a ip address instead
 * of a hostname, since a host name is not unique. 
 */
var searchValue;


/**
 * The right part of the search field, showing the clear button. 
 */
var rightImage; 


/**
 * The container used to hold the serach results. 
 */
var searchResults;


/**
 * The button in the search results footer allowing the user to add a result to the myprotocols table. 
 */
var addToMyProtocolsButton;


/**
 * The button in the search results to show the protocol detail window.  
 */
var protocolDetailButton;
 

/**
 *  The button in the search results footer allowing the user to add a result to the myhosts table. 
 */
var addToMyHostsButton;


/**
 * The button in the search results to show the nlhost summary window. 
 */
var nlHostDetailButton;


/**
 * The protocols returned in the last search response. 
 */
var protocols;

 
/**
 * The nlhosts returned in the last search response.
 */
var nlHosts;


/**
 * The text suggest component used. 
 */
var textSuggest;


/**
 * The last search string.
 */
var lastSearchString = "";
 
 
/**
 * Adds the search field to the probe top menu.
 */
function addSearchField() {
	createSearchField();
	setSearchFieldEventListeners();
	
	// create the text suggestion component
	var suggestOptions = new Object();
	suggestOptions.suggestionHighlightClassName = "#b7b7b7";
	suggestOptions.yPos = 35;
	var rightImageRightOffset = getScreenWidth() - (calculateLeftOffset(rightImage) + rightImage.offsetWidth);
	suggestOptions.xPos = getScreenWidth() - rightImageRightOffset - 302;
	suggestOptions.maxCount = 5;
	suggestOptions.showSuggestionsFunction = textSuggestionsShown;
	suggestOptions.selectedFunction = search;
	suggestOptions.valueField = searchValue;
	suggestOptions.matchClassName = "bold";
	textSuggest = new TextSuggest(searchInput, SUGGEST_BASE_URL + "sid=" + getSid() + "&", suggestOptions);
	
	// init and create the event listener, listens to tab shown and hidden events
	// the search field shall only be visible in the main tab
	var eventListener = new Listener(this);
	eventListener.addEvent("tabActivated", processTabActivatedSearch);
	eventListener.addEvent("tabDeactivated", processTabDeactivatedSearch);
	createSearchResultContainer();
}


/**
 * Positions the suggestions html tag.
 */
function positionSuggestTag() {
	var rightImageRightOffset = getScreenWidth() - (calculateLeftOffset(rightImage) + rightImage.offsetWidth);
	searchInput.suggest.htmlCode.style.left = (getScreenWidth() - rightImageRightOffset - 302) + "px";;
}


/**
 * The right part of the search field has been clicked, if the close button is shown clear the search field.
 */
function searchRightClicked() {
	if (this.src.indexOf("search_right_close.jpg")) {
		clearSearch();
	}
}


/**
 * Clears the search field, the search value is emptied, search results are hidden and
 * the clear search button in the right of the search field is hidden. 
 */
function clearSearch() {
	searchInput.value = "";
	searchValue.value = "";
	hideClearSearchButton();
	searchInput.focus();
	hideSearchResults();
	lastSearchString = "";
}


/**
 * Hides the close button in the search field. 
 */
function hideClearSearchButton() {
	var searchRight = getElementWithAttribute(document.body, "searchRight", "id", "img");
	searchRight.src = "/themes/black/images/search_right.jpg"; 
}


/**
 * Shows the close button in the search field.
 */
function showClearSearchButton() {
	var searchRight = getElementWithAttribute(document.body, "searchRight", "id", "img");
	searchRight.src = "/themes/black/images/search_right_close.jpg";
}


/**
 * Shows the search field in the top menu.
 */
function showSearchField() {
	var searchContainer = getElementWithAttribute(document.body, "searchContainer", "id", "div");
	setClass(searchContainer, "searchContainer");
}


/**
 * Hides the search field.
 */
function hideSearchField() {
	var searchContainer = getElementWithAttribute(document.body, "searchContainer", "id", "div");
	setClass(searchContainer, "inactive");
	hideSearchResults();
}



/**
 * Show the search result container. 
 */
function showSearchResults() {
	setClass(searchResults, "searchResultContainer");
}


/**
 * Hides the search results.
 */
function hideSearchResults() {
	setClass(searchResults, "inactive");
}


/**
 * Returns true if the search results are shown, false otherwise. 
 */
function searchResultsShown() {
	return "searchResultContainer" == getClassName(searchResults);
}


/**
 * Search for the set value. 
 */
function search() {
	if (isEmpty(searchInput.value)) {
		return;
	}
	if (lastSearchString == valueToSearchFor()) {
		// when enter is pressed this is called twice one as a response to
		// a suggestion select event from the text suggest component and once
		// as a response to the key listener in this file
		lastSearchString = "";
		return;
	}
	
	var url = createSearchEntryUrl();	
	var protocolLength = exists(protocols) ? protocols.length : -1;
	var nlHostsLength = exists(nlHosts) ? nlHosts.length : -1;
	if (!isEmpty(url)) {
		lastSearchString = valueToSearchFor();
		loadXML(url, parseSearchResponse); 
	}	
} 
 

/**
 * Creates a search entry url.
 * @param entry The entry to search for.
 * @return The search entry url, a empty string if no search entry is set. 
 */
function createSearchEntryUrl() {
	if (isEmpty(searchInput.value) && isEmpty(searchValue.value)) {
		return "";
	}
	return ENTRY_SEARCH_BASE_URL + "sid=" + getSid() + "&entry=" + encodeURIComponent(valueToSearchFor()) + "&";
}


/**
 * Gets the value to search for based on the set input.
 */
function valueToSearchFor() {
	// this value is set to ip address (10.0.2.33) or protocol ids (1.2048.6.80) instead
	// of markuspc and http by the text suggest component
	var searchString = searchValue.value;
	// use the search string if not set
	return isEmpty(searchString) ? searchInput.value : searchString;
}


/**
 * Parses the search response. 
 * @param response The response to parse. 
 */
function parseSearchResponse(response) {
	protocols = response.getElementsByTagName("protocol");
	nlHosts = response.getElementsByTagName("nlhost");
	if ((!exists(protocols) || protocols.length == 0) && (!exists(nlHosts) || nlHosts.length == 0)) {
		showNoSearchResults();
		return;
	}

	// clear any previous results
	searchResults.innerHTML = "";
	searchResults.appendChild(createResultHeader());
	
	// append the results	
	appendSearchResults(protocols, nlHosts, searchResults);
	// need to add this empty div to solve the firefox floating problem
	appendClearedDiv(searchResults);
	searchResults.appendChild(createResultFooter());
	
	if (exists(protocols) && protocols.length == 1) {
		setClass(addToMyProtocolsButton, "active");
		setClass(protocolDetailButton, "active");
	}
	else if (exists(nlHosts) && nlHosts.length == 1) {
		setClass(addToMyHostsButton, "active");
		setClass(nlHostDetailButton, "active");
	}
	positionSearchResults();
	showSearchResults();
}


/**
 * Creates the html code for the search field.
 */
function createSearchField() {
	var tabSection = getElementWithAttribute(document.body, "tabSection", "name", "div");
	var tabContainer = getElementWithAttribute(tabSection, "tabContainer", "name", "div");
	
	var searchContainer = document.createElement("div");
	setClass(searchContainer, "searchContainer");
	searchContainer.setAttribute("id", "searchContainer");
	tabContainer.appendChild(searchContainer);
	
	var leftImage = document.createElement("img");
	leftImage.src = "/themes/black/images/search_left.jpg";
	setClass(leftImage, "searchLeft");
	searchContainer.appendChild(leftImage);
	
	searchInput = document.createElement("input");
	searchInput.setAttribute("type", "text");
	searchInput.setAttribute("size", "20");
	searchInput.setAttribute("id", "searchField");
	setClass(searchInput, "search");
	searchContainer.appendChild(searchInput);
	
	searchValue = document.createElement("input");
	searchValue.setAttribute("type", "text");
	searchValue.setAttribute("id", "searchValueField");
	setClass(searchValue, "inactive");
	searchContainer.appendChild(searchValue);
	
	rightImage = document.createElement("img");
	rightImage.src = "/themes/black/images/search_right.jpg";
	setClass(rightImage, "searchRight");
	rightImage.setAttribute("id", "searchRight");
	searchContainer.appendChild(rightImage);
	rightImage.onclick = searchRightClicked;
}


/**
 * Shows the no search results found message. 
 */
function showNoSearchResults() {
	// clear any previous results
	searchResults.innerHTML = "";
	// create the message
	var messageContainer = document.createElement("div");
	messageContainer.appendChild(document.createTextNode("No search results found"));
	setClass(messageContainer, "searchResultHeader");
	searchResults.appendChild(messageContainer);
	positionSearchResults();
	showSearchResults();
}


/**
 * Appends a cleared div, need to add this because of the firefox floating problem with div tags.
 * @param element The element to append the cleared div to.
 */
function appendClearedDiv(element) {
	var clearDiv = document.createElement("div");
	setClass(clearDiv, "clear");
	element.appendChild(clearDiv);
}


/**
 * Appends the given search results.
 * @param protocols The protocol results to append. 
 * @param nlHosts The nlhost results to append.
 * @param element The element to append the results to. 
 */
function appendSearchResults(protocols, nlHosts, element) {
	var protocolResults = createProtocolsResultHtmlCode(protocols);
	for (var i = 0; exists(protocolResults) && i < protocolResults.length; i++) {
		element.appendChild(protocolResults[i]);
		appendClearedDiv(element);		
	}
	
	var nlHostsResults = createNlHostsResultHtmlCode(nlHosts);
	for (var i = 0; exists(nlHostsResults) && i < nlHostsResults.length; i++) {
		element.appendChild(nlHostsResults[i]);
		appendClearedDiv(element);
	}
}


/**
 * Creates the html code for the nlhost responses. 
 * @param nlhosts The nlhost results to create the code for. 
 * @return A array containing the html code for the protocol responses. 
 */
function createNlHostsResultHtmlCode(nlHosts) {
	var nlHostResults = new Array();
	for (var i = 0; exists(nlHosts) && i < nlHosts.length; i++) {
		nlHostResults.push(createNlHostResultHtmlCode(nlHosts[i]));
	}
	return nlHostResults;
}


/**
 * Creates the html code for a nlhost result.
 * @param nlHost xml containing the nlhost. 
 * @return The create html code. 
 */
function createNlHostResultHtmlCode(nlHost) {
	var container = document.createElement("div");
	setClass(container, "searchResult");
	container.style.width = "100%";
	container.style.display = "block";
	container.style.position = "relative";
	
	var alarmCountValue = getTagNodeValue("alarmCount", nlHost);
	var alarmCount = createSearchItemCode("alarm count: ", (isEmpty(alarmCountValue) ? 0 : alarmCountValue));
	container.appendChild(alarmCount);
	
	var hostName = createSearchItemCode("host name: ", URLDecode(getTagNodeValue("hostName", nlHost)));
	container.appendChild(hostName);
	
	var address = createSearchItemCode("address: ", getTagNodeValue("address", nlHost));
	container.appendChild(address);
	
	var bytes = createSearchItemCode("bytes: ", formatOctetValue(getTagNodeValue("octets", nlHost)));
	container.appendChild(bytes);
	
	var conversations = createSearchItemCode("conversations: ", getTagNodeValue("conversations", nlHost));
	container.appendChild(conversations);
	
	var protocols = createSearchItemCode("protocols: ", getTagNodeValue("protocols", nlHost));
	container.appendChild(protocols);
	
	return container;
}


/**
 * Creates the html code for the protocol responses. 
 * @param protocols The protocol results to create the code for.
 * @return A array containing the html code for the protocol responses.  
 */
function createProtocolsResultHtmlCode(protocols) {
	var protocolResults = new Array();
	for (var i = 0; exists(protocols) && i < protocols.length; i++) {
		protocolResults.push(createProtocolResultHtmlCode(protocols[i]))		
	}
	return protocolResults;
}


/**
 * Creates the html code for a protocol result.
 * @param protocol The xml containing the protocol.
 * @return The created html code. 
 */
function createProtocolResultHtmlCode(protocol) {
	var container = document.createElement("div");
	setClass(container, "searchResult");
	container.style.width = "100%";
	container.style.display = "block";
	container.style.position = "relative";
	
	var alarmCountValue = getTagNodeValue("alarmCount", protocol);
	var alarmCount = createSearchItemCode("alarm count: ", (isEmpty(alarmCountValue) ? 0 : alarmCountValue));
	container.appendChild(alarmCount);
	
	var name = createSearchItemCode("name: ", getTagNodeValue("protocolName", protocol));
	container.appendChild(name);
	
	var port = createSearchItemCode("port: ", getTagNodeValue("protocolId", protocol));
	container.appendChild(port);
	
	var bytes = createSearchItemCode("bytes: ", formatOctetValue(getTagNodeValue("octets", protocol)));
	container.appendChild(bytes);
	
	var hosts = createSearchItemCode("hosts: ", getTagNodeValue("hosts", protocol));
	container.appendChild(hosts);
	
	var conversations = createSearchItemCode("conversations: ", getTagNodeValue("conversations", protocol));
	container.appendChild(conversations);
	
	return container;
}


/**
 * Creates the html code for the a search result item for example port: 1.0248.6.80 where
 * port is the category and 1.0248.6.80 is the value. 
 * @param category The category this item is for.
 * @param value The value for the category.
 */
function createSearchItemCode(category, value) {
	var container = document.createElement("div");
	setClass(container, "searchResultItem");
	
	var nobr = document.createElement("nobr");
	container.appendChild(nobr);
	var bold = document.createElement("b");
	bold.appendChild(document.createTextNode(category));
	nobr.appendChild(bold);
	nobr.appendChild(document.createTextNode(value));
	
	return container;	
}


/**
 * Creates the html code of the footer in the serach results. 
 * @return The create html code. 
 */
function createResultFooter() {
	var footer = document.createElement("div");
	setClass(footer, "searchResultFooter");
	addToMyProtocolsButton = document.createElement("input");
	addToMyProtocolsButton.setAttribute("type", "button");
	addToMyProtocolsButton.value = "Add to myProtocolts";
	addToMyProtocolsButton.style.marginRight = "5px";
	addToMyProtocolsButton.setAttribute("id", "addToMyProtocolsSearch");
	addToMyProtocolsButton.onmousedown = addSearchResultToMyProtocols;
	setClass(addToMyProtocolsButton, "inactive");
	
	protocolDetailButton = document.createElement("input");
	protocolDetailButton.setAttribute("type", "button");
	protocolDetailButton.value = "Show details";
	protocolDetailButton.style.marginRight = "5px";
	protocolDetailButton.setAttribute("id", "protocolDetailButton");
	protocolDetailButton.onmousedown = showProtocolDetails;
	setClass(protocolDetailButton, "inactive"); 
	
	addToMyHostsButton = document.createElement("input");
	addToMyHostsButton.setAttribute("type", "button");
	addToMyHostsButton.value = "Add to myHosts";
	addToMyHostsButton.style.marginRight = "5px";
	addToMyHostsButton.setAttribute("id", "addToMyHostsSearch");
	addToMyHostsButton.onmousedown = addSearchResultToMyHosts;
	setClass(addToMyHostsButton, "inactive");
	
	nlHostDetailButton = document.createElement("input");
	nlHostDetailButton.setAttribute("type", "button");
	nlHostDetailButton.value = "Show details";
	nlHostDetailButton.style.marginRight = "5px";
	nlHostDetailButton.setAttribute("id", "nlHostDetailButton");
	nlHostDetailButton.onmousedown = showNlHostDetails;
	setClass(nlHostDetailButton, "inactive");
	
	footer.appendChild(addToMyProtocolsButton);
	footer.appendChild(protocolDetailButton);
	footer.appendChild(addToMyHostsButton);
	footer.appendChild(nlHostDetailButton);
	return footer;
}


/**
 * Creates the search results header.
 * @return The created html code.  
 */
function createResultHeader() {
	var header = document.createElement("div");
	setClass(header, "searchResultHeader");
	header.appendChild(document.createTextNode("Search result"));
	header.setAttribute("id", "searchResultHeader");
	return header;
}


/**
 * Positions the search restuls.
 */
function positionSearchResults() {
	var rightImageRightOffset = getScreenWidth() - (calculateLeftOffset(rightImage) + rightImage.offsetWidth);
	searchResults.style.left = (getScreenWidth() - rightImageRightOffset - 302) + "px";;
}


/**
 * Creates the container for the serach results. 
 */
function createSearchResultContainer() {
	searchResults = document.createElement("div");
	setClass(searchResults, "inactive");
	searchResults.style.position = "absolute";
	// set a high zIndex to make sure the suggestions are shown above any content in the page
	searchResults.style.zIndex = 999999;
	document.body.appendChild(searchResults);
	searchResults.style.left = "0px";
	searchResults.style.top = "35px";
	searchResults.setAttribute("id", "searchResults");
}


/**
 * This prints a gioven text in the main screens console.
 * @param text The text to print.
 */
function print(text) {
	window.frames["Main"].print(text);
}


/**
 * Sets the topn value and the filter vlaues in a table.
 * @param n The n to set for the table.
 * @param srcFilterValue The source filter value to set for the table. 
 * @param dstFilterValue The destination filter value to set for the table. 
 * @param protocolFilterValue The protocol filter value to set for the table.
 * @param table The table to reset.  
 */
function setTableValues(n, srcFilterValue, dstFilterValue, protocolFilterValue, table) {
	var mainWindow = window.frames["Main"];
	if (!isEmpty(n)) {
		table.setUrl(replaceParameterValue(table.getUrl(), "top", n, "&"));
		var topNInput = getElementWithAttribute(mainWindow.document.body, "topN" + table.id, "name", "input");
		if (exists(topNInput)) {
			topNInput.value = n;
		}
	}
	if (!isEmpty(srcFilterValue)) {
		table.setUrl(replaceParameterValue(table.getUrl(), "srcFilterValue", srcFilterValue, "&"));
	}
	if (!isEmpty(dstFilterValue)) {
		table.setUrl(replaceParameterValue(table.getUrl(), "dstFilterValue", dstFilterValue, "&"));
	}	
	if (!isEmpty(protocolFilterValue)) {
		table.setUrl(replaceParameterValue(table.getUrl(), "protocolFilterValue", protocolFilterValue, "&"));
	}
	table.restartUpdate();
}


/**
 * Text suggestions have been shown. 
 */
function textSuggestionsShown() {
	positionSuggestTag();
	hideSearchResults();
	// reset this so selecting the same option twice does not prevent us from 
	// sending the search request to the server
	lastSearchString = "";
}


// event handling


/**
 * A key down event from the input element. If the search field is empty hide the close button
 * else show the close button. 
 */
function searchFieldKeyUpHandler(event) {
	event = exists(event) ? event : window.event;
	var escapeKey = 27;
	var enterKey = 13;
	if (enterKey == event.keyCode && !textSuggest.suggestionsShown()) {
		search();
		// exit this function here sp search results are not hidden
		return;
	}	
	else if (escapeKey == event.keyCode) {
		clearSearch();
	}
	
	if (!isEmpty(searchInput.value)) {
		showClearSearchButton();	
	}
	else {
		hideClearSearchButton();
	}
	
	hideSearchResults();
}


/**
 * The search field has lost focus. 
 */
function searchFieldBlurHandler() {
	hideSearchResults();
	lastSearchString = "";
}


/**
 * Adds the current protocol search result to the myprotocols table.
 */
function addSearchResultToMyProtocols() {
	if (!exists(protocols) || protocols.length == 0) {
		return;
	}
	var mainWindow = window.frames["Main"];
	if (!exists(mainWindow) || !exists(mainWindow.isDoneLoading) || !mainWindow.isDoneLoading()) {
		// main window has not finished loading
		return;
	}

	mainWindow.WindowManager.instance().activateWindow("myprotocolswindow");
	var myProtocolsTable = mainWindow.TableManager.instance().getTable("myProtocolsTable");
	if (!myProtocolsTable.painted) {
		myProtocolsTable.paint();
	}
	myProtocolsTable.addProtocol(getTagNodeValue("protocolId", protocols[0]));
	hideSearchResults();
}


/**
 * Shows the nlhost detail window for the current nlhost search result. 
 */
function showNlHostDetails() {
	if (!exists(nlHosts) || nlHosts.length == 0) {
		return;
	}
	var mainWindow = window.frames["Main"];
	if (!exists(mainWindow) || !exists(mainWindow.isDoneLoading) || !mainWindow.isDoneLoading()) {
		// main window has not finished loading
		return;
	}
	var nlHostDetailWindow = mainWindow.WindowManager.instance().getWindow("nlhostDetailWindow");
	var address = getTagNodeValue("address", nlHosts[0]);
	setTableValues(null, address, null, null, mainWindow.TableManager.instance().getTable("nlhostDetailTable"));
	setTableValues("4", address, null, null, mainWindow.TableManager.instance().getTable("nlhostTopProtocolsTable"));
	setTableValues("4", address, null, null, mainWindow.TableManager.instance().getTable("nlhostTopConversationsTable"));
	
	removeAllCharts(nlHostDetailWindow);
	var chart = createNlHostChart(address, nlHostDetailWindow); 
 	nlHostDetailWindow.appendChart(chart);
 	nlHostDetailWindow.setSrcAddress(address);
 	mainWindow.WindowManager.instance().activateWindow("nlhostDetailWindow");
	hideSearchResults();
}


/**
 * Shows the protocol detail window for the current protocol search result. 
 */
function showProtocolDetails() {
	if (!exists(protocols) || protocols.length == 0) {
		return;
	}
	var mainWindow = window.frames["Main"];
	if (!exists(mainWindow) || !exists(mainWindow.isDoneLoading) || !mainWindow.isDoneLoading()) {
		// main window has not finished loading
		return;
	}
	
	var protocolDetailWindow = mainWindow.WindowManager.instance().getWindow("protocolDetailWindow");
	// prepare the tables
	var protocolId = getTagNodeValue("protocolId", protocols[0]);
	setTableValues(null, null, null, protocolId, mainWindow.TableManager.instance().getTable("protocolDetailTable"));
	setTableValues("4", null, null, protocolId, mainWindow.TableManager.instance().getTable("protocolTopHostsTable"));
	setTableValues("4", null, null, protocolId, mainWindow.TableManager.instance().getTable("protocolTopConversationsTable"));
	
	removeAllCharts(protocolDetailWindow);
	var chart = createProtocolChart(protocolId, protocolDetailWindow);
 	protocolDetailWindow.appendChart(chart);
 	protocolDetailWindow.setProtocol(protocolId);
 	mainWindow.WindowManager.instance().activateWindow("protocolDetailWindow");

	hideSearchResults();
}


/**
 * Creates the nlhost chart for a given host.
 * @param address The address of the nlhost to create the chart for. 
 * @param nlHostDetailWindow The nlhost detail window the chart is for. 
 * @return The chart for the given nlhost. 
 */
function createNlHostChart(address, nlHostDetailWindow){
	var mainWindow = window.frames["Main"];
	var container = mainWindow.document.createElement("div");
	container.setAttribute("name", address + ", Bytes" + uniqueInteger());
	setClass(container, "chartContainer");
	container.style.width = nlHostDetailWindow.container.style.width;
	container.style.height = nlHostDetailWindow.container.style.height;
	var chartCode = mainWindow.document.createElement("img");
	chartCode.setAttribute("id", address + ", Bytes" + uniqueInteger());
	setClass(chartCode, "chart");
	container.appendChild(chartCode);
	chartCode.src = "/?resource=Statistics.nlhost&sid=" + getSid() + "&mode=getdata&command=intervalchart&field=6&src=" + address + "&dst=0&protocol=0&chart=line&width=827&height=155&3dmodeon=true&antialias=true&3ddepth=5&valueLinesOn=true&stackedOn=true&background=%23ffffff|%23cccccc&sampleColors=%231e7096f0|%232492c7f0&alternateChartBackground=%23e7e7e7"
	var chart = mainWindow.createLineChart(container, URLDecode(getTagNodeValue("hostName", nlHosts[0])) + ", Bytes", nlHostDetailWindow, "nlhostDetailTable");
	chart.setSrcAddress(address);
	chart.setField(6);
	return chart;
}


/**
 * Creates the protocol chart for a given protocol.
 * @param protocolId The id of the protocol to create the chart for. 
 * @param protocolDetailWindow The protocol detail window the chart is for. 
 * @return The chart for the given protocol. 
 */
function createProtocolChart(protocolId, protocolDetailWindow) {
	var mainWindow = window.frames["Main"];
	var container = mainWindow.document.createElement("div");
	container.setAttribute("name", protocolId + ", Bytes" + uniqueInteger());
	setClass(container, "chartContainer");
	container.style.width = protocolDetailWindow.container.style.width;
	container.style.height = protocolDetailWindow.container.style.height;
	var chartCode = mainWindow.document.createElement("img");
	chartCode.setAttribute("id", protocolId + ", Bytes" + uniqueInteger());
	setClass(chartCode, "chart");
	container.appendChild(chartCode);
	chartCode.src = "/?resource=Statistics.protocol&sid=" + getSid() + "&mode=getdata&command=intervalchart&field=1&src=0&dst=0&protocol=" + protocolId + "&chart=line&width=827&height=155&3dmodeon=true&antialias=true&3ddepth=5&valueLinesOn=true&stackedOn=true&background=%23ffffff|%23cccccc&sampleColors=%231e7096f0|%232492c7f0&alternateChartBackground=%23e7e7e7"
	var chart = mainWindow.createLineChart(container, getTagNodeValue("protocolName", protocols[0]) + ", Bytes", protocolDetailWindow, "protocolDetailTable");
	chart.setProtocol(protocolId);
	chart.setField(1);
	return chart;
}


/**
 * Removes all charts from a windows. 
 * @param chartWindow The window to remvoe the charts from.
 */
function removeAllCharts(chartWindow) {
	chartWindow.resetCharts();
	var mainWindow = window.frames["Main"];
	var windowLayout = mainWindow.getWindowLayout(chartWindow.id);
	var charts = exists(windowLayout) ? windowLayout.getElementsByTagName("charts") : null;
	if (!exists(charts) || exists(charts) && charts.length == 0) {
		return;
	}
	windowLayout.removeChild(charts[0]);
}


/**
 * Adds the current nlhost search result to the myhosts table.
 */
function addSearchResultToMyHosts() {
	if (!exists(nlHosts) || nlHosts.length == 0) {
		return;
	}
	var mainWindow = window.frames["Main"];
	if (!exists(mainWindow) || !exists(mainWindow.isDoneLoading) || !mainWindow.isDoneLoading()) {
		// main window has not finished loading
		return;
	}
	mainWindow.WindowManager.instance().activateWindow("myhostswindow");
	var myHostsTable = mainWindow.TableManager.instance().getTable("myHostsTable");
	if (!myHostsTable.painted) {
		myHostsTable.paint();
	}
	myHostsTable.addHost(getTagNodeValue("address", nlHosts[0]));
	hideSearchResults();
}


/**
 * Processes a tab activated event. 
 * @param event The event to process. 
 */
function processTabActivatedSearch(event) {
	if ("Main".equalsIgnoreCase(event.getData())) {
		showSearchField();
	} 
}


/**
 * Processes a tab deactivated event.
 * @param event The event to process. 
 */
function processTabDeactivatedSearch(event) {
	if ("Main".equalsIgnoreCase(event.getData())) {
		hideSearchField();
	}
}


/**
 * Sets the event listeners needed on the search field. 
 */
function setSearchFieldEventListeners() {
	registerEventHandler("keyup", searchFieldKeyUpHandler, searchInput);
	registerEventHandler("blur", searchFieldBlurHandler);
}