/**
 * This file contains convenience methos for the email configuration.
 */
 

// constants


/**
 * The base url used to set the email configuration.
 */
var SET_EMAIL_BASE_URL = "/?resource=EmailSender&mode=set&command=emailConfig&sid=";


/**
 * The base url used to load the email configuration.
 */
var GET_EMAIL_BASE_URL = "/?resource=EmailSender&mode=get&command=emailConfig&sid=";



// services


/**
 * Sets the email config. 
 */
function setEmailConfig() {
	clearEmailMessage();
	var url = emailConfigUrl();
	if (exists(url)) {
		loadXML(url, emailConfigSet);		
	}
}


/**
 * Loads the email config.
 */
function loadEmailConfig() {
	var session = document.getElementById('sid').value;
	loadXML(GET_EMAIL_BASE_URL + session, setXmlEmailConfig);
}


// implementation


/**
 * Sets a xml email config into the email config window. 
 * @param xmlEmailConfig The xml email config to set. 
 * @throws Error if xmlEmailconfig is null or undefined. 
 */
function setXmlEmailConfig(xmlEmailConfig) {
	assertExists(xmlEmailConfig, "The xml email config to set is undefined or null");
	getInput(emailWindow().getHtmlCode(), "smtpServer").value = getTagNodeValue("smtpServer", xmlEmailConfig);
	getInput(emailWindow().getHtmlCode(), "smtpPort").value = getTagNodeValue("port", xmlEmailConfig);
	if (!isEmpty(getTagNodeValue("login", xmlEmailConfig))) {
		getInput(emailWindow().getHtmlCode(), "authentication").checked = true;
		authenticationChanged();
	}
	else {
		getInput(emailWindow().getHtmlCode(), "authentication").checked = false;
		authenticationChanged();
	}	
	getInput(emailWindow().getHtmlCode(), "login").value = getTagNodeValue("login", xmlEmailConfig);
	getInput(emailWindow().getHtmlCode(), "password").value = getTagNodeValue("password", xmlEmailConfig);
	getInput(emailWindow().getHtmlCode(), "fromAddress").value = getTagNodeValue("fromAddress", xmlEmailConfig);
	getInput(emailWindow().getHtmlCode(), "fromName").value = URLDecode(getTagNodeValue("fromName", xmlEmailConfig));
	setupLoaded("emailConfig");
}


/**
 * The authentication check box has been clicked, if it was checked enable
 * the login and password fields, disable them otherwise. 
 */
function authenticationChanged() {
	if (getInput(emailWindow().getHtmlCode(), "authentication").checked) {
		getInput(emailWindow().getHtmlCode(), "login").disabled = false;
		getInput(emailWindow().getHtmlCode(), "password").disabled = false;
	}
	else {
		getInput(emailWindow().getHtmlCode(), "login").disabled = true;
		getInput(emailWindow().getHtmlCode(), "password").disabled = true;
	}
}


/**
 * Create a url to set the email configuration based on the values set in the email config window.
 */
function emailConfigUrl() {
	try {
		var smtpServer = getInput(emailWindow().getHtmlCode(), "smtpServer").value;
		if (isEmpty(smtpServer)) {
			throw new Error("No SMTP server set");
		}
		var session = document.getElementById('sid').value;
		var url = SET_EMAIL_BASE_URL + session + "&smtpServer=" + smtpServer + "&";
		url += smtpPortUrl();
		if (getInput(emailWindow().getHtmlCode(), "authentication").checked) {
			url += loginUrl();
		}
		url += "fromAddress=" + getInput(emailWindow().getHtmlCode(), "fromAddress").value + "&";
		url += "fromName=" + encodeURIComponent(getInput(emailWindow().getHtmlCode(), "fromName").value);
		return url;
	}
	catch (e) {
		setEmailMessage(undefined, e.message);
		return null;
	}
}


/**
 * Returns the smtp port part of a set email config url.
 */
function smtpPortUrl() {
	var port = getInput(emailWindow().getHtmlCode(), "smtpPort").value;
	if (isEmpty(port)) {
		return "port=&";
	}
	if (!isValidPortIdString(port)) {
		throw new Error("Invalid port. Valid is 0-65535");
	}
	return "port=" + port + "&";
}


/**
 * Returns the part of a set email request for the login information.
 */
function loginUrl() {
	var login = getInput(emailWindow().getHtmlCode(), "login");
	var password = getInput(emailWindow().getHtmlCode(), "password");
	if (!isEmpty(login.value) && isEmpty(password.value)) {
		throw new Error("No password set");
	}
	else if (!isEmpty(password.value) && isEmpty(login.value)) {
		throw new Error("No login set");
	}
	return "login=" + login.value + "&password=" + password.value + "&";
}


/**
 * Processes the response of a set email config request. 
 * @param xmlData The xml response from the server. 
 */
function emailConfigSet(xmlData) {
	if  (containsSuccessMessage(xmlData) && exists(top.returnTab)) {
		createSuccessReturnLink(xmlData);
	}
	else if (containsSuccessMessage(xmlData)) {
		setEmailMessage(getTagNodeValue("success", xmlData), undefined);
	}
	else if (containsErrorMessage(xmlData)) {
		setEmailMessage(undefined, getTagNodeValue("error", xmlData));
	}
}


/**
 * Creates a link to activate another tab when the email config has been set.
 * @param xmlData The xml data containing the message to include in the link.
 */
function createSuccessReturnLink(xmlData) {
	var message = getTagNodeValue("success", xmlData);
	var messageContainer = getEmailMessageContainer()
	setClass(messageContainer, "messageContainer ok");
	var link = document.createElement("a");
	messageContainer.appendChild(link);
	link.appendChild(document.createTextNode(message + ". Click to return to add/edit alarm actions"));
	link.style.cursor = "pointer";
	setClass(link, "ok");
	link.onclick = function() {
		top.tabSection.activateTab(top.returnTab);
		if (exists(top.callFunction)) {
			top.callFunction();
		}
	};
}


/**
 * Sets a email config message. 
 * @param successMessage The success message to set. 
 * @param errorMessage The error message to set. 
 */
function setEmailMessage(successMessage, errorMessage) {
	if (isDefined(successMessage) && !isEmpty(successMessage)) {
		setClass(getEmailMessageContainer(), "messageContainer ok");
		getEmailMessageContainer().innerHTML = "<nobr>" + successMessage + "</nobr>";
	}
	else if (isDefined(errorMessage) && !isEmpty(errorMessage)) {
		setClass(getEmailMessageContainer(), "messageContainer error");
		getEmailMessageContainer().innerHTML = "<nobr>" + errorMessage + "</nobr>";
	}
}


/**
 * Clears the email message.
 */
function clearEmailMessage() {
	getEmailMessageContainer().innerHTML = "";
}
	
/**
 * Returns the html code of the container for email messages in the 
 * email window.
 */
function getEmailMessageContainer() {
	return getElementWithAttribute(emailWindow().getHtmlCode(), "emailMessage", "name", "div");
}


/**
 * Gets the email config window.
 */
function emailWindow() {
	return WindowManager.instance().getWindow("emailConfiguration");
}