/**
 * Loads the filters.
 */
function loadFilters() {
	loadMacFilters();
	loadIpFilters();
	loadProtocolFilters();
}


/**
 * Loads the ethernet filters.
 */
function loadMacFilters() {
	var session = document.getElementById('sid').value;
	loadXML("?resource=FilterManager&mode=get&command=mac&format=xml&sid=" + session, loadMacFiltersCallback);
}


/**
 * Sets the ethernet filters.
 */
function loadMacFiltersCallback(xml) {
	setFilters("excluded_macs", "included_macs", xml.getElementsByTagName("filter"));
}


/**
 * Loads the IP filters.
 */
function loadIpFilters() {
	var session = document.getElementById('sid').value;
	loadXML("?resource=FilterManager&mode=get&command=ip&format=xml&sid=" + session, loadIpFiltersCallback);
}


/**
 * Sets the IP filters.
 */
function loadIpFiltersCallback(xml) {
	setFilters("excluded_ips", "included_ips", xml.getElementsByTagName("filter"));
}


/**
 * Loads the protocol filters.
 */
function loadProtocolFilters() {
	var session = document.getElementById('sid').value;
	loadXML("?resource=FilterManager&mode=get&command=protocol&format=xml&sid=" + session, loadProtocolFiltersCallback);
}


/**
 * Sets the protocol filters.
 */
function loadProtocolFiltersCallback(xml) {
	setFilters("excluded_protocols", "included_protocols", xml.getElementsByTagName("filter"));
}


/**
 * Adds a mac filter.
 */
function addMac(included) {
	var form = document.getElementById('macForm');
	var start = form.start.value;
	var end = form.end.value;
	if (start == "" && end == "") {
		setWindowMessageField(document.getElementById("macForm"), "name", "td", "Please specify at least one ethernet address", undefined, "filterSetup");		
	}
	else if (start != "" && end != "" && !isValidMac(start) && !isValidMac(end)) {
		setWindowMessageField(document.getElementById("macForm"), "name", "td", "Invalid start and end address. Valid format is xx:xx:xx:xx:xx:xx", undefined, "filterSetup");
	}
	else if (start != "" && !isValidMac(start)) {
		setWindowMessageField(document.getElementById("macForm"), "name", "td", "Invalid start address. Valid format is xx:xx:xx:xx:xx:xx", undefined, "filterSetup");
	}
	else if (end != "" && !isValidMac(end)) {
		setWindowMessageField(document.getElementById("macForm"), "name", "td", "Invalid end address. Valid format is xx:xx:xx:xx:xx:xx", undefined, "filterSetup");
	}	
	else {
		var session = document.getElementById('sid').value;
		var url = "?resource=FilterManager&mode=set&command=addmac&format=xml&sid=" + session;
		url += "&start=" + form.start.value;
		url += "&end=" + form.end.value;
		url += "&included=" + included;
		loadXML(url, addMacCallback);
	}
}


/**
 * Reloads the new filters or gives the user feedback.
 */
function addMacCallback(xml) {
	loadMacFilters();
	if (xml.getElementsByTagName("error") != null && xml.getElementsByTagName("error").length > 0) {
		var error = xml.getElementsByTagName("message");
		var message = error[0].firstChild.nodeValue;
	}
	setWindowMessageField(document.getElementById("macForm"), "name", "td", message, "Successfully added MAC filter", "filterSetup");	
	
	setupLoaded("filterSetup");
}


/**
 * Adds an ip filter.
 */
function addIp(included) {
	var form = document.getElementById('ipForm');
	var start = form.start.value;
	var end = form.end.value;
	if (start == "" && end == "") {
		setWindowMessageField(form, "name", "td", "Please specify at least one IP address", undefined, "filterSetup");
	}
	else if (start != "" && end != "" && !isValidIp(start) && !isValidIp(end)) {
		setWindowMessageField(form, "name", "td", "Invalid start and end address. Valid format is xxx.xxx.xxx.xxx", undefined, "filterSetup");
	}
	else if (start != "" && !isValidIp(start)) {
		setWindowMessageField(form, "name", "td", "Invalid start address. Valid format is xxx.xxx.xxx.xxx", undefined, "filterSetup");
	}
	else if (end != "" && !isValidIp(end)) {
		setWindowMessageField(form, "name", "td", "Invalid end address. Valid format is xxx.xxx.xxx.xxx", undefined, "filterSetup");
	}	
	else {
		var session = document.getElementById('sid').value;
		var url = "?resource=FilterManager&mode=set&command=addip&format=xml&sid=" + session;
		url += "&start=" + form.start.value;
		url += "&end=" + form.end.value;
		url += "&included=" + included;
		loadXML(url, addIpCallback);
	}
}


/**
 * Reloads the new filters or gives the user feedback.
 */
function addIpCallback(xml) {
	loadIpFilters();
	if (xml.getElementsByTagName("error") != null && xml.getElementsByTagName("error").length > 0) {
		var error = xml.getElementsByTagName("message");
		var message = error[0].firstChild.nodeValue;
	}
	setWindowMessageField(document.getElementById("ipForm"), "name", "td", message, "Successfully added IP filter", "filterSetup");
	
	setupLoaded("filterSetup");
}


/**
 * Adds a protocol filter.
 */
function addProtocol(included) {
	var form = document.getElementById('protocolForm');
	var protocol = form.protocol.value;
	var start = form.start.value;
	var end = form.end.value;
	if (start == "" && end == "") {
		setWindowMessageField(form, "name", "td", "Please specify at least one port number", undefined, "filterSetup");
	}
	else if (start != "" && end != "" && !isValidPort(start) && !isValidPort(end)) {
		setWindowMessageField(form, "name", "td", "Invalid start and end port.  Valid is 0-65535", undefined, "filterSetup");	
	}
	else if (start != "" && !isValidPort(start)) {
		setWindowMessageField(form, "name", "td", "Invalid start port. Valid is 0-65535", undefined, "filterSetup");		
	}
	else if (end != "" && !isValidPort(end)) {
		setWindowMessageField(form, "name", "td", "Invalid end port. Valid is 0-65535", undefined, "filterSetup");			
	}	
	else {
		var session = document.getElementById('sid').value;
		var url = "?resource=FilterManager&mode=set&command=addprotocol&format=xml&sid=" + session;
		url += "&start=" + form.start.value;
		url += "&end=" + form.end.value;
		url += "&included=" + included;
		url += "&protocol=" + protocol;
		loadXML(url, addProtocolCallback);
	}
}


/**
 * Reloads the new filters or gives the user feedback.
 */
function addProtocolCallback(xml) {
	loadProtocolFilters();
	if (xml.getElementsByTagName("error") != null && xml.getElementsByTagName("error").length > 0) {
		var error = xml.getElementsByTagName("message");
		var message = error[0].firstChild.nodeValue;
	}
	setWindowMessageField(document.getElementById("protocolForm"), "name", "td", message, "Successfully added protocol filter", "filterSetup");				
	
	setupLoaded("filterSetup");
}


/**
 * Sends a request to delete a filter.
 * @param type The type of filter (ethernet, ip, protocol).
 * @param start The start address or port.
 * @param end The end address or port.
 * @param protocol The protocol if protocol type filter (ethernet, ip, tcp, udp).
 * @param included True if include rule, false if exclude rule.
 */
function deleteFilter(type, start, end, protocol, included) {
	var session = document.getElementById('sid').value;
	var message, form;
	var url = "?resource=FilterManager&mode=set&format=xml&sid=" + session + "&command=";
	if (type == "ethernet") {
		url += "deletemac";
		form = document.getElementById("macForm");
		message = "Successfully deleted MAC filter";
	}
	else if (type == "ip") {
		url += "deleteip";
		form = document.getElementById("ipForm");
		message = "Successfully deleted IP filter";
	}
	else if (type == "protocol") {
		url += "deleteprotocol";
		form = document.getElementById("protocolForm");
		message = "Successfully deleted protocol filter";
	}
	setWindowMessageField(form, "name", "td", null, message, "filterSetup");						
	setupLoaded("filterSetup");
	
	url += "&start=" + start + "&end=" + end + "&protocol=" + protocol + "&included=" + included;
	loadXML(url, loadFilters);
}
 

// implementation


/**
 * Sets the list of filter rules.
 * @param excludeId The id of the exclude div tag.
 * @param includeId The id of the include div tag.
 * @param filters The array of filters.
 */
function setFilters(excludeId, includeId, filters) {
	var includeContainer = document.getElementById(includeId);
	var excludeContainer = document.getElementById(excludeId);
	clearContainer(includeContainer);
	clearContainer(excludeContainer);
	
	for (var i = 0; filters != null && i < filters.length ; i++) {
		var start = filters[i].getElementsByTagName("start")[0].firstChild.nodeValue;
		var end = filters[i].getElementsByTagName("end")[0].firstChild.nodeValue;
		var protocol = "";
		if (filters[i].getElementsByTagName("protocol").length > 0) {
			protocol = filters[i].getElementsByTagName("protocol")[0].firstChild.nodeValue;
		}
		var included = filters[i].getElementsByTagName("included")[0].firstChild.nodeValue;
	
		// create the rule text
		var rule = protocol != "" ? (protocol + " port ") : "";
		rule += start + (end != start ? " - " + end : "");
		
		// create a element for the rule
		var filterRule = document.createElement("div");
	
		// add the delete link
		type = filters[i].getElementsByTagName("type")[0].firstChild.nodeValue;
		params = "'" + type + "','" + start + "','" + end + "','" + protocol + "'," + included;
		link = "<a title='Delete filter' href='#' onClick=\"deleteFilter(" + params + ");\"><img src='images/icon_delete.gif' alt='Delete filter' border=0/></a>";
		rule = link + "&nbsp;" + rule;
		filterRule.innerHTML = rule;
		
		// add the rule to the list		
		if (included == "true") {
			setClass(filterRule, "includedFilter");
			includeContainer.appendChild(filterRule);
			if (includeContainer.style.height.length > 0) {			
				includeContainer.style.height = (parseInt(includeContainer.style.height) + filterRule.offsetHeight) + "px";
			}
		}
		else {
			setClass(filterRule, "excludedFilter");
			excludeContainer.appendChild(filterRule);
			if (excludeContainer.style.height.length > 0) {
				excludeContainer.style.height = (parseInt(excludeContainer.style.height) + filterRule.offsetHeight) + "px";
			}
		}
	}
	
	setupLoaded("filterSetup");
}


/**
 * Clears a container for filter rules.
 * @param container The container to clear.
 */
function clearContainer(container) {
	var len = container.childNodes.length;
	for (var i = len - 1; i >= 0; i--) {
		container.removeChild(container.childNodes[i]);
	}
	
	container.style.height = "0px";
	container.innerHTML = container.innerHTML;
}


/**
 * Checks if the specified ip address is valid.
 */
function isValidIp(ip) {
	var parts = ip.split('.');
	if (parts == null || parts.length != 4) {
		return false;
	}
	try {
		for (var i = 0; i < 4; i++) {
			n = parseInt(parts[i]);
			if (n == NaN || n < 0 || n > 255) {
				return false;
			}
		}
	}
	catch (error) {
		return false;
	}
	return true;
}