/**
 * The current probe name loaded from the server.
 */
var probeName = "";


/**
 * Loads the probe configuration.
 */
function loadProbeConfig() {
	loadNetworkCard();
	loadSilentMode();
	getLicenseManager().loadLicense();
	loadProbeInfo();
}


// network card info


/**
 * Loads the data in the network card setup.
 */
function loadNetworkCardSettings() {
	loadNetworkCard();
	loadSilentMode();
}


/**
 * Loads the network card information.
 */
function loadNetworkCard() {
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=get&command=nic&format=xml&sid=" + session;
	loadXML(url, loadNetworkCardCallback);
}


/**
 * Sets the network card info content.
 */
function loadNetworkCardCallback(xml) {
	populateNetworkInterfaceSelector(xml);
	
	// set the operation mode
	var form = document.getElementById('networkCardForm');
	var mode = xml.getElementsByTagName("promiscuous")[0].firstChild.nodeValue;
	form.promiscuous[0].checked = (mode == "true");
	form.promiscuous[1].checked = (mode == "false");
	
	// disable the fields if the probe is running
	var running = xml.getElementsByTagName("running")[0].firstChild.nodeValue == "true";
	form.networkcard.disabled = running;
	form.promiscuous[0].disabled = running;
	form.promiscuous[1].disabled = running;
	
	// set the correct button label and function
	document.getElementById("network_card_button").setAttribute("value", running ? "Stop" : "Start");
	document.getElementById("network_card_button").onclick = (running ? stopProbe : startProbe);
	
	// display any message
	var error = xml.getElementsByTagName("error");
	if (error != null && error.length > 0) {
		setWindowMessageField(form, "name", "td", error[0].firstChild.nodeValue, undefined, "probeConfig");		
	}
	
	var success = xml.getElementsByTagName("success");
	if (success != null && success.length > 0) {
		setWindowMessageField(form, "name", "td", null, success[0].firstChild.nodeValue, "probeConfig");
	}
	setupLoaded("networkCard");
}


/**
 * Populate the network interface selector.
 */
function populateNetworkInterfaceSelector(xml) {
	// clear the network interface selection list
	var selection = document.getElementById('networkCardForm').networkcard;
	for (var i = selection.options.length; i >= 0; i--) {
		selection.remove(i);
	}
	
	// create the network interface selection list
	var currentCard = xml.getElementsByTagName("current")[0].firstChild.nodeValue;
	var cards = xml.getElementsByTagName("interface");
	var selectedIndex = 0;
	if (cards != null && cards.length > 0) {
		for (var i = 0; i < cards.length; i++) {
			var number = cards[i].getElementsByTagName("cardnumber")[0].firstChild.nodeValue;
			var description = cards[i].getElementsByTagName("description")[0].firstChild.nodeValue;
			if (currentCard == number) {
				selectedIndex = i;
			}
			// the second argument to add() defines the option in the selects option array
			// before which the new element should be added, by setting it to undefined, the 
			// new option is added to the end 
			selection.add(createOption(number, description), undefined);
		}
	}
	else {
		selection.appendChild(createOption("-1", "No network card found"));
	}
	selection.selectedIndex = selectedIndex;
}


/**
 * Creates a option.
 * @param value The value to set for the option.
 * @param text The text to set for the description.
 * @return The created option.
 */
function createOption(value, text) {
	var option = new Option(text, value);
	return option;
}


/**
 * Loads the silent mode state from the probe.
 */
function loadSilentMode() {
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=get&command=silent&format=xml&sid=" + session;
	loadXML(url, loadSilentModeCallback);
}


/**
 * Sets the silent mode state in the setup box.
 */
function loadSilentModeCallback(xml) {
	// set the checkbox
	var form = document.getElementById('networkCardForm');
	var mode = xml.getElementsByTagName("on")[0].firstChild.nodeValue;
	form.silentMode[0].checked = (mode == "true");
	form.silentMode[1].checked = (mode == "false");
	
	// display the success method
	var success = xml.getElementsByTagName("success");
	if (success != null && success.length > 0) {
		setWindowMessageField(form, "name", "td", null, success[0].firstChild.nodeValue, "probeConfig");
	}
	setupLoaded("networkCard");
}


/**
 * Turns silent mode on or off.
 */
function setSilentMode() {
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=set&command=silent&format=xml&sid=" + session;
	var form = document.getElementById('networkCardForm');
	var silent = form.silentMode[0].checked ? form.silentMode[0].value : form.silentMode[1].value;
	url += "&silent=" + silent;
	loadXML(url, loadSilentModeCallback);
}


/**
 * Stops the probe.
 */
function stopProbe() {
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=set&command=stop&format=xml&sid=" + session;
	loadXML(url, loadNetworkCardCallback);
}


/**
 * Starts the probe.
 */
function startProbe() {
	var form = document.getElementById('networkCardForm');
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=set&command=start&format=xml&sid=" + session;
	url += "&cardnumber=" + form.networkcard[form.networkcard.selectedIndex].value;
	url += "&promiscuous=" + (form.promiscuous[0].checked ? form.promiscuous[0].value : form.promiscuous[1].value);
	loadXML(url, loadNetworkCardCallback);
}


// probe name and description


/**
 * Loads the probe info (name, description, version).
 */
function loadProbeInfo() {
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=get&command=probeinfo&format=xml&sid=" + session;
	loadXML(url, loadProbeInfoCallback);
}



/**
 * Sets the probe info content.
 */
function loadProbeInfoCallback(xml) {
	var info = xml.getElementsByTagName("info");
	if (info != null && info.length > 0) {
		var form = document.getElementById('probeInfoForm');
		var name = URLDecode(info[0].getElementsByTagName("name")[0].firstChild.nodeValue);
		setProbeName(name);
		if (info[0].getElementsByTagName("description")[0].childNodes.length > 0) {
			form.description.value = URLDecode(info[0].getElementsByTagName("description")[0].firstChild.nodeValue);
		}
		document.getElementById("current_version").innerHTML = info[0].getElementsByTagName("version")[0].firstChild.nodeValue;
		var latest = info[0].getElementsByTagName("latestversion");
		if (latest != null && latest.length > 0 && latest[0].childNodes.length > 0) {
			document.getElementById("latest_version").innerHTML = latest[0].firstChild.nodeValue;
		}
	}
	setupLoaded("probeInfo");
}


/**
 * Sets the probe name. 
 * @param name The probe name to set. 
 */
function setProbeName(name) {
	var form = document.getElementById('probeInfoForm');
	form.name.value = name;
	if (name != probeName) {
		fireProbeNameChangedEvent(name, probeName)
	}
	probeName = name;
}



/**
 * Fires a event when the probe name has been changed.
 * @param probeName The probe name that has been set.
 * @param oldName The old probe name.  
 */
function fireProbeNameChangedEvent(newProbeName, oldName) {
	var data = new Array();
	data["newProbeName"] = newProbeName;
	data["oldProbeName"] = oldName;
	getEventServer().notifyListeners(new Event("probeNameChanged", data));
}


/**
 * Updates the probe info (name and description).
 */
function updateProbeInfo() {
	var form = document.getElementById('probeInfoForm');
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=set&command=probeinfo&format=xml&sid=" + session;
	url += "&name=" + encodeURIComponent(form.name.value);
	url += "&description=" + encodeURIComponent(form.description.value);
	loadXML(url, updateProbeInfoCallback);
}


/**
 * Response to the user.
 */
function updateProbeInfoCallback(xml) {
	var info = xml.getElementsByTagName("info");
	if (info != null && info.length > 0) {
		loadProbeInfoCallback(xml);
	}

	// set any messages
	var form = document.getElementById("probeInfoForm");
	var error = xml.getElementsByTagName("error");
	if (error != null && error.length > 0) {
		setWindowMessageField(form, "name", "td", error[0].firstChild.nodeValue, undefined,  "probeConfig");
		return;
	}
	
	var success = xml.getElementsByTagName("success");
	if (success != null && success.length > 0) {
		setWindowMessageField(form, "name", "td", null, success[0].firstChild.nodeValue, "probeConfig");
		//top.setProbeName(form.name.value);		
		return;
	}
}


// license info


/**
 * Sets the license info content.
 */
function setLicenseLoaded(xml) {
	var form = document.getElementById('licenseKeyForm');
	// fill the form
	var license = xml.getElementsByTagName("license");
	if (license != null && license.length > 0 && exists(form)) {
		form.licensekey.value = URLDecode(license[0].getElementsByTagName("key")[0].firstChild.nodeValue);
		form.currentkey.value = form.licensekey.value;
		document.getElementById("hostid").innerHTML = license[0].getElementsByTagName("hostid")[0].firstChild.nodeValue;
		document.getElementById("expires").innerHTML = license[0].getElementsByTagName("expires")[0].firstChild.nodeValue;
	}
	var masterLicense = getTagNodeValue("masterLicense", xml);
	var slaveRow = getElementWithAttribute(document.body, "slaveRow", "name", "tr");
	if (exists(masterLicense) && "true" == masterLicense) {
		setClass(slaveRow, "active");
		var slaveCount = getTagNodeValue("maxSlaveCount", xml)
		getElementWithAttribute(document.body, "slaveCount", "id", "div").innerHTML = (slaveCount == "-1" ? "unlimited" : slaveCount);
	}
	else {
		setClass(slaveRow, "inactive");
	}
	
	if ((getMasterMode() + "") != masterLicense) {
		top.location = top.location;
	}	
	
	var error = xml.getElementsByTagName("error");
	if (error != null && error.length > 0) {
		setWindowMessageField(form, "name", "td", error[0].firstChild.nodeValue, undefined, "probeConfig");
		return;
	}
	
	var success = xml.getElementsByTagName("success");
	if (success != null && success.length > 0) {
		setWindowMessageField(form, "name", "td", null, success[0].firstChild.nodeValue, "probeConfig");
		// the promiscuous mode option might have changed, reload 
		loadNetworkCard();
		return;
	}
	
	if (exists(setupLoaded)) {
		// the file containing the function may not have been loaded yet
		setupLoaded("licenseInfo");
	}
}


/**
 * Sets the license key.
 */
function setLicenseKey() {
	var form = document.getElementById('licenseKeyForm');
	if (!isEmpty(encodeURIComponent(form.licensekey.value))) {
		getLicenseManager().setLicense(encodeURIComponent(form.licensekey.value));
	}
	else {
		setWindowMessageField(form, "name", "td", "No license key set", undefined, "probeConfig");
	}
}
