/**
 * The setup tab section.
 */
var setupTabSection; 

/**
 * This field keeps track of the previous message, used to clear the value when a new error is set,
 * or when the user switches menu section. 
 */
var previousMessage;


/**
 * When the page is loaded a loading screen is shown until the statistics setup has loaded. 
 * This is used to indicate whether the loading screen has been removed or not. 
 */
var loadingScreenRemoved = false;


/**
 * The name of the directory where images are in.
 */
var IMAGES_DIRECTORY = "themes/black/images/";


/**
 * The name of the setup tab.
 */
var SETUP_TAB_NAME = "Setup";


/**
 * The different parts of the setup load independent. When the statistics settings and the reset
 * settings have loaded the loading animation can be removed. This array is indexed by the
 * id of the setup loaded and its value is set to true if it has loaded.
 */
var loaded = new Array();


/**
 * Some of the setup parts are in the same window, for example the setup and the statistics. This
 * is indexed by the name of the setup and its value is the id of the window the setup part is in.
 */
var setupWindowId = new Array();
setupWindowId["statisticsSetup"] = "statisticsSetup";
setupWindowId["resetSettings"] = "statisticsSetup";
setupWindowId["userSetup"] = "userSetup";
setupWindowId["filterSetup"] = "filterSetup";
setupWindowId["probeInfo"] = "probeConfig";
setupWindowId["licenseInfo"] = "probeConfig";
setupWindowId["networkCard"] = "probeConfig";
setupWindowId["userDetail"] = "userDetail";
setupWindowId["adminPassword"] = "adminPassword";
setupWindowId["slaveSetup"] = "slaveSetup";


/**
 * The event listener used to listen to window events. Need to listen to events
 * to update the top menu. In the top menu the activate window is selected. 
 */
var setupEventListener = new Listener();


/**
 * Initializes the setup screen.
 */
function init() {
	Statusbar.instance().init();
	Desktop.instance().setTopMenu(getElementWithAttribute(document.body, "topMenu", "id", "div"));
	Desktop.instance().setDesktop(getElementWithAttribute(document.body, "desktop", "id", "div"));
	Desktop.instance().setStatusbar(Statusbar.instance());
	WindowManager.instance().setStatusbar(Statusbar.instance());
	WindowManager.instance().setDesktop(Desktop.instance());
	
	createProbeWindowManager();
	initStatisticsSetup();
	loadUsers(); 
	loadFilters();
	loadProbeConfig();
	loadResources();
	loadEmailConfig();
	setFrameSize(SETUP_TAB_NAME);
	initSlaves();
	toggleMasterMode();
	
	setupEventListener.addEvent(WINDOW_HIDDEN_EVENT, processWindowHiddenEventSetup);
	setupEventListener.addEvent(WINDOW_ACTIVATED_EVENT, processWindowShownEventSetup);
}


/**
 * Some of the setup screens has done loading.
 * @para setupId The id of the setup that has been loaded.
 */
function setupLoaded(setupId) {
	loaded[setupId] = true;
	if (doneIniting() && !loadingScreenRemoved) {
		doneLoadingSetup();
		Desktop.instance().setDesktopHeight();
	}
	
	// the height of the setup window might have changed update id
	if (exists(setupWindowUpdated)) {
		// the file containing the function may not have been loaded yet
		setupWindowUpdated(setupWindowId[setupId]);
	}
}


/**
 * Done loading setup screens.
 */
function doneLoadingSetup() {
	initSetupWindows();
	initSlaveWindow();
	top.tabSection.getTab('Setup').doneLoading();
	loadingScreenRemoved = true;
	setFrameSize(SETUP_TAB_NAME);
	if(!exists(top.showWindow) || isEmpty(top.showWindow)) {
		return;
	}
	
	WindowManager.instance().hideWindow("statisticsSetup");
	var windowToShow = WindowManager.instance().getWindow(top.showWindow);
	if (exists(windowToShow)) {
		showSetupWindow(top.showWindow, true);
	}
	top.showWindow = "";
}	


/**
 * Shows the email config window.
 */
function showEmailWindow() {
	WindowManager.instance().hideAllWindows();
	WindowManager.instance().activateWindow("emailConfiguration");
}


/**
 * Shows the slave setup window. 
 */
function showSlaveSetupWindow() {
	WindowManager.instance().hideAllWindows();
	WindowManager.instance().activateWindow("slaveSetup");
}


/**
 * Returns true if the data neccessary to remove the loading screen has been loaded.
 */
function doneIniting() {
	if (loaded["statisticsSetup"] && loaded["resetSettings"]) {
		return true;
	}
	return false;
}


/**
 * This is called when this tab has been activated. Set the frame size
 * the size of the window might have changed, also reload the active set windows.
 */
function tabActivated() {
	hideLogSelector();
	setFrameSize(SETUP_TAB_NAME);	
	reloadShownWindows();
	Desktop.instance().setDesktopHeight();
	// when the setup tab is hidden we stop the window manager from listening to events to prevent 
	// windowmanager in the other tabs to send events to this window manager
	WindowManager.instance().eventProcessor.startListening();
	setupEventListener.startListening();
}


/**
 * This is called when this tab has been deactivated. 
 */
function tabDeactivated() {
	clearAllMessageFields();
	// when the setup tab is hidden we stop the window manager from listening to events to prevent 
	// windowmanager in the other tabs to send events to this window manager
	if (exists(WindowManager.instance())) {
		// the user might switch tabs before this tab is completely loaded
		WindowManager.instance().eventProcessor.stopListening();
	}
	setupEventListener.stopListening();	
}


/**
 * The size of the window has been changed. 
 */
function windowResized() {
	setFrameSize(SETUP_TAB_NAME);
	Desktop.instance().setDesktopHeight();
}


/**
 * Toggles the master mode. If the probe is run in master mode, show the slaves 
 * menu options.
 */
function toggleMasterMode() {
	var manageProbes = getElementWithAttribute(document.body, "manageProbes", "name", "img");
	var slaveLink = getElementWithAttribute(document.body, "probesLink", "name", "td");
	// might not exist in tests when run in master mode
	if (!getMasterMode() && exists(manageProbes) && exists(slaveLink)) {
		setClass(manageProbes, "inactive");
		setClass(slaveLink, "inactive");
	}
}


// top menu


/**
 * The name of the active link in the menu
 */
var activeLink = "statisticsSetupLink";



/**
 * Process a window hidden event. Deactivate the windows menu link.
 * @param event The event to process.  
 */
function processWindowHiddenEventSetup(event) {
	if (!exists(WindowManager.instance().getWindow(event.getData()))) {
		return;
	}
	deActivateMenuLink(event.getData());	
}


/**
 * Processes a window shown event. Activate the windows menu link.
 * @param event The event to process. 
 */
function processWindowShownEventSetup(event) {
	if (!exists(WindowManager.instance().getWindow(event.getData()))) {
		return;
	}
	activateMenuLink(event.getData());
}


/**
 * A window has been activated, activate its link in the menu.
 * @param name The name of the window.
 */
function activateMenuLink(name) {
	deActivateLink(activeLink);
	activateLink(getLinkName(name));
	activeLink = getLinkName(name);
}



/**
 * A window has been hidden or deactivated deactivate its link in the menu.
 * @param name The name of the window.
 */
function deActivateMenuLink(name) {
	deActivateLink(getLinkName(name));
	activeLink = "";
}


/**
 * Returns the name of the link for a window.
 * @param windowName The name of the window.
 */
function getLinkName(windowName) {
	if(isEmpty(windowName)) {
		return "";
	}
	
	if ("statisticsSetup" == windowName) {
		return "statisticsSetupLink";
	}
	else if ("probeConfig" == windowName) {
		return "probeConfigLink";
	}
	else if ("userSetup" == windowName) {
		return "userSetupLink";
	}
	else if ("slaveSetup" == windowName) {
		return "manageProbes";
	}
	else if ("filterSetup" == windowName) {
		return "filterSetupLink";
	}
	else if ("emailConfiguration" == windowName) {
		return "emailConfigLink";
	}
	else {
		return "";
	}
}


/**
 * Deactivates a link in the menu.
 * @param name The name of the link to deactivate.
 */
function deActivateLink(name) {
	if(isEmpty(name)) {
		return;
	}
	var link = getElementWithAttribute(document.body, name, "name", "a");
	setClass(link, "inActiveLink");
}


/**
 * Deactivates a link in the menu.
 * @param name The name of the link to deactivate.
 */
function activateLink(name) {
	if(isEmpty(name)) {
		return;
	}
	var link = getElementWithAttribute(document.body, name, "name", "a");
	setClass(link, "activeLink");
}