/**
 * The statistics tables.
 */
var tables = ["protocol","nlhost","nlmatrix","alhost","almatrix","host","matrix","network"];


/**
 * Init the statistics setup.
 */
function initStatisticsSetup() {
	document.getElementById('statisticsTable').innerHTML = createTable();
	loadStatisticsSetup();
}



/**
 * Loads the statistics setup.
 */
function loadStatisticsSetup() {
	loadResetSettings();
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=get&command=statistics&format=xml&sid=" + session;
	loadXML(url, loadStatisticsSetupCallback);
}


/**
 * Sets the statistics settings.
 */
function loadStatisticsSetupCallback(xml) {
	var form = document.getElementById('statisticsForm');
	var tables = xml.getElementsByTagName("table");
	for (var i = 0; tables != null && i < tables.length; i++) {
		var name = tables[i].getElementsByTagName("name")[0].firstChild.nodeValue;
		var on = tables[i].getElementsByTagName("on")[0].firstChild.nodeValue;
		var duration = tables[i].getElementsByTagName("duration")[0].firstChild.nodeValue;
		var time = tables[i].getElementsByTagName("time")[0].firstChild.nodeValue;
		document.getElementById(name + ".on").checked = (on == "true");
		document.getElementById(name + ".duration").value = duration;
		document.getElementById(name + ".time").value = time;
	}
	
	// display any message
	var error = xml.getElementsByTagName("error");
	if (error != null && error.length > 0) {
		setWindowMessageField(form, "name", "td", error[0].firstChild.nodeValue, undefined, "statisticsSetup");
	}
	var success = xml.getElementsByTagName("success");
	if (success != null && success.length > 0) {
		setWindowMessageField(form, "name", "td", null, success[0].firstChild.nodeValue, "statisticsSetup");
	}

	setupLoaded("statisticsSetup");
}


/**
 * Updates the statistics settings.
 */
function updateStatistics() {
	var form = document.getElementById('statisticsForm');
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=set&command=statistics&format=xml&sid=" + session;
	url += "&on=" + getOn();
	url += getInterval();
	loadXML(url, loadStatisticsSetupCallback);
}


/**
 * Returns a comma-separated string of the checked tables.<b>
 */
function getOn() {
	var on = "";
	for (var i = 0; i < tables.length; i++) {
		if (document.getElementById(tables[i] + ".on").checked == true) {
			on += tables[i] + ",";
		}
	}
	return on;
}


/**
 * Returns the interval settings.
 */
function getInterval() {
	var interval = "";
	for (var i = 0; i < tables.length; i++) {
		var id = tables[i];
		var duration = document.getElementById(id + '.duration').value;
		var time = document.getElementById(id + '.time').value;
		interval += "&" + id + ".duration=" + duration;
		interval += "&" + id + ".time=" + time;
	}
	return interval;
}


// implementation


/**
 * Creates the statistics html table.
 */
function createTable() {
	// the header
	var table = "<table class='defaultTable innerBox' width='550' cellpadding=4 id='statisticsTable'>";
	table += "<tr class='header'>";
	table += "<td valign='top'>Collected Statistics</td><td  colspan='2' align='right' valign='top'>Interval Duration</td><td align='right' valign='top'>Interval Time</td>";
	table += "</tr>";
	
	// the top row, adjusting all tables
	table += "<tr>";
	table += "<td align='right'></td>";
	table += "<td align='right'>Set all&nbsp;&nbsp;</td>";
	table += "<td align='right'><select id='all.duration' onChange='setAllDuration();'>" + createDurationOptions() + "</select>";
	table += "</td>";
	table += "<td align='right'>";
	table += "<select id='all.time' onChange='setAllTime();'>" + createTimeOptions('all') + "</select>";
	table += "</td>";
	table += "</tr>";
		
	// each row
	table += createRow('protocol', 'Protocols', "even");
	table += createRow('nlhost', 'Hosts', "even");
	table += createRow('nlmatrix', 'Conversations', "even");
	table += createRow('alhost', 'Protocols per host', "even");
	table += createRow('almatrix', 'Protocols per conversation', "even");
	table += createRow('host', 'Network cards', "even");
	table += createRow('matrix', 'Network card conversations', "even");
	table += createRow('network', 'Network overview', "even");

	// add the message field and update button
	table += "<tr class='even'><td name='messageField' colspan=3></td><td align='right'>";
	table += "<input class='button' type='button' value='Update'onClick='updateStatistics();'/>";
	table += "</td></tr>";
	
	// return the html table code
	table += "</table>"
	return table;
}


/**
 * Creates a row with the specified label.
 */
function createRow(id, name, classToSet) {
	var row = "<tr class='" + classToSet + "'>";
	row += "<td><input type='checkbox' name='" + id + ".on' id='" + id + ".on'>" + name + "</td>";
	row += "<td colspan=2 align='right'><select name='" + id + ".duration' id='" + id + ".duration' onChange=\"adjustTime('" + id + "'); resetAllDuration();\";>";
	row += createDurationOptions();
	row += "</select></td>";
	row += "<td align='right'><select name='" + id + ".time' id='" + id + ".time' onChange=\"adjustInterval('" + id + "'); resetAllTime();\";>";
	row += createTimeOptions(id);
	row += "</select></td>";
	row += "</tr>";
	return row;
}


/**
 * Creates the duration selection options.
 */
function createDurationOptions() {
	var options = "";
	options += "<option default value='-1'>Select a value</option>";
	options += "<option value='0'>Off</option>";
	options += "<option value='60'>1 hour</option>";
	options += "<option value='120'>2 hours</option>";
	options += "<option value='180'>3 hours</option>";
	options += "<option value='360'>6 hours</option>";
	options += "<option value='480'>8 hours</option>";
	options += "<option value='720'>12 hours</option>";
	options += "<option value='1440'>1 day</option>";
	options += "<option value='2880'>2 days</option>";
	return options;
}


/**
 * Creates the interval time selection options.
 */
function createTimeOptions(id) {
	var options = "";
	options += "<option value='-1'>Select a value</option>";
	if (id == "network") {
		options += "<option value='5'>5 seconds</option>";
	}
	options += "<option value='10'>10 seconds</option>";
	options += "<option value='15'>15 seconds</option>";
	options += "<option value='20'>20 seconds</option>";
	options += "<option value='30'>30 seconds</option>";
	options += "<option value='60'>1 minute</option>";
	options += "<option value='120'>2 minutes</option>";
	options += "<option value='300'>5 minutes</option>";
	options += "<option value='600'>10 minutes</option>";
	if (id != "network") {
		options += "<option value='900'>15 minutes</option>";
		options += "<option value='1800'>30 minutes</option>";
		options += "<option value='3600'>1 hour</option>";
		options += "<option value='7200'>2 hours</option>";
		options += "<option value='10800'>3 hours</option>";
	}
	return options;
}


/**
 * Makes sure the interval time is not larger than the duration.
 */
function adjustTime(id) {
	var duration = (document.getElementById(id + '.duration').value - 0) * 60;
	var time = document.getElementById(id + '.time').value - 0;
	if (duration < time && duration > 0) {
		document.getElementById(id + '.time').value = duration;
	}
}


/**
 * Makes sure the interval duration is not less than the time.
 */
function adjustInterval(id) {
	var duration = (document.getElementById(id + '.duration').value - 0) * 60;
	var time = document.getElementById(id + '.time').value - 0;
	var allDuration = getElementWithAttribute(document.body, "all.duration", "id", "select");
	if (time > duration && time > 0 && allDuration.value > -1) {
		document.getElementById(id + '.duration').value = (time / 60);
	}
}


/**
 * Sets the duration for all tables.
 */
function setAllDuration() {
	var duration = document.getElementById('all.duration').value;
	for (var i = 0; duration > -1 && i < tables.length; i++) {
		document.getElementById(tables[i] + '.duration').value = duration;
		adjustTime(tables[i]);
	}
}


/**
 * Sets the time for all tables.
 */
function setAllTime() {
	var time = document.getElementById('all.time').value;
	for (var i = 0; time > -1 && i < tables.length; i++) {
		var timeField = document.getElementById(tables[i] + '.time');
		// don't set for the network checkbox, it has some values different than the rest
		if (timeField.name.indexOf("network") < 0) {
			document.getElementById(tables[i] + '.time').value = time;
			adjustInterval(tables[i]);
		}
	}
}


/**
 * Sets all interval duration back to its inital value. 
 */
function resetAllDuration() {
	var allDuration = getElementWithAttribute(document.body, "all.duration", "id", "select");
	allDuration.selectedIndex = 0;
}


/**
 * Sets all interval time back to its inital value.
 */
function resetAllTime() {
	var allTime = getElementWithAttribute(document.body, "all.time", "id", "select");
	allTime.selectedIndex = 0;
}


// reset settings


/**
 * Loads the reset settings.
 */
function loadResetSettings() {
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=get&command=resetsettings&format=xml&sid=" + session;
	loadXML(url, loadResetSettingsCallback);
}


/**
 * Sets the reset settings.
 */
function loadResetSettingsCallback(xml) {
	var settings = xml.getElementsByTagName("resetsettings");
	if (settings != null && settings.length > 0) {
		var form = document.getElementById('resetForm');
		document.getElementById("lastreset").innerHTML = settings[0].getElementsByTagName("lastreset")[0].firstChild.nodeValue;
		document.getElementById("nextreset").innerHTML = settings[0].getElementsByTagName("nextreset")[0].firstChild.nodeValue;
		form.resetday.value = settings[0].getElementsByTagName("resetday")[0].firstChild.nodeValue;
		form.resethour.value = settings[0].getElementsByTagName("resethour")[0].firstChild.nodeValue;
		form.resetinterval.value = settings[0].getElementsByTagName("resetinterval")[0].firstChild.nodeValue;
	}
	setupLoaded("resetSettings");
}


/**
 * Updates the reset settings.
 */
function updateResetSettings() {
	var form = document.getElementById('resetForm');
	var session = document.getElementById('sid').value;
	var url = "?resource=ProbeManager&mode=set&command=resetsettings&format=xml&sid=" + session;
	url += "&resetday=" + form.resetday.value;
	url += "&resethour=" + form.resethour.value;
	url += "&resetinterval=" + form.resetinterval.value;
	loadXML(url, updateResetSettingsCallback);
}


/**
 * Response to the user after updating the reset settings.
 */
function updateResetSettingsCallback(xml) {
	var info = xml.getElementsByTagName("resetsettings");
	if (info != null && info.length > 0) {
		loadResetSettingsCallback(xml);
		setWindowMessageField(document.getElementById("resetForm"), "name", "td", null, "Reset settings updated", "statisticsSetup");		
	}
	else if (xml.getElementsByTagName("error") != null && xml.getElementsByTagName("error").length > 0) {
		var error = xml.getElementsByTagName("error");
		var message = error[0].firstChild.nodeValue;
		setWindowMessageField(document.getElementById("resetForm"), "name", "td", message, undefined, "statisticsSetup");
	}
	setupLoaded("resetSettings");
}


/**
 * Resets the probe.
 */
function resetProbe() {
	if (confirm('Are you sure?')) {
		var session = document.getElementById('sid').value;
		var url = "?resource=ProbeManager&mode=set&command=reset&format=xml&sid=" + session;
		loadXML(url, resetProbeCallback);
	}
}


/**
 * Lets the user know the probe was reset.<b> 
 */
function resetProbeCallback(xml) {
	var info = xml.getElementsByTagName("resetsettings");
	if (info != null && info.length > 0) {
		loadResetSettingsCallback(xml);
		setWindowMessageField(document.getElementById("resetForm"), "name", "td", null, "Successfully reset", "statisticsSetup");
	}
	else if (xml.getElementsByTagName("error") != null && xml.getElementsByTagName("error").length > 0) {
		var error = xml.getElementsByTagName("error");
		var message = error[0].firstChild.nodeValue;
		setWindowMessageField(document.getElementById("resetForm"), "name", "td", message, undefined, "statisticsSetup");
	}
	setupLoaded("resetSettings");
}


// old code to be refactored


function checkResetValue() {
	if (document.form1.day.value == '8') {
		hideFields();
	} 
	else {
		showFields();
	}
}

function showFields() {
	document.form1.hour.disabled = false;
	document.form1.interval.disabled = false;
}

function hideFields() {
	document.form1.hour.disabled = true;
	document.form1.interval.disabled = true;
}

var currentValue = 0;

function setCurrentValue(field) {
	currentValue = field.value;
}

function setValues(field) {
	var i;
	var statistics = document.stats;
	var old_field_value = field.value;
	var currentTimeIntervalField;
	var currentDurationField;

	var interval_time = 0;
	var interval_duration = 0;
	for (i = 0; i < statistics.length; i++) {
		var currentEntry = statistics.elements[i];
		if (currentEntry.name.indexOf(".time") > -1) {
			if (currentEntry.name.indexOf(field.name) == 0) {
				currentDurationField = statistics.elements[i-1];
			}
		}

		if (currentEntry.name.indexOf(".duration") > -1) {
			// interval time for this entry
			var currentIntervalTime = statistics.elements[i+1];
			// checkbox: turn on or off the count for this entry
			var currentCheckBox = statistics.elements[i-1];

			if (currentEntry.name.indexOf(field.name) == 0) {
				currentTimeIntervalField = currentIntervalTime;
			}

			interval_time = currentIntervalTime.value;

			if (currentEntry.value == 0) {
				currentIntervalTime.disabled = true;
			} else {
				if (currentEntry.name.indexOf(field.name) == 0) {				
					currentIntervalTime.disabled = false;
				}
			}
			interval_duration = currentEntry.value;
		}
	}

	// validate values for interval durations
	if (field.name.indexOf(".duration") > -1
		&& (field.name.indexOf("nlhost.") > -1 
		|| field.name.indexOf("nlmatrix.") > -1
		|| field.name.indexOf("alhost.") > -1 
		|| field.name.indexOf("almatrix.") > -1)) {
		switch (field.value) {
			// 3 hour duration, 5 minute interval is min
			case '180':
				setMinValue(currentTimeIntervalField, 300);
				break;
			// 6 hour duration, 10 minute interval is min
			case '360':
				setMinValue(currentTimeIntervalField, 600);
				break;
			// 8 hour duration, 10 minute interval is min
			case '480':
				setMinValue(currentTimeIntervalField, 600);
				break;
			// 12 hour duration, 15 minute interval is min
			case '720':
				setMinValue(currentTimeIntervalField, 900);
				break;
			// 1 day duration, 30 minute interval is min
			case '1440':
				setMinValue(currentTimeIntervalField, 1800);
				break;
			// 2 day duration, 1 hour interval is min
			case '2880':
				setMinValue(currentTimeIntervalField, 3600);
				break;
			default:
		}
	}
	
	if (field.name.indexOf(".duration") > -1 && field.name.indexOf("network") > -1) {
		switch(field.value) {
			// 30 minute duration, 10 second interval is min
			case '30':
			// 1 hour duration, 10 second interval is min
			case '60':
			// 2 hour duration, 10 second interval is min
			case '120':
				setMinValue(currentTimeIntervalField, 10);
				break;
			// 3 hour duration, 20 second interval is min
			case '180':
				setMinValue(currentTimeIntervalField, 20);
				break;
			// 6 hour duration, 30 second interval is min
			case '360':
				setMinValue(currentTimeIntervalField, 30);
				break;
			// 8 hour duration, 30 second interval is min
			case '480':
				setMinValue(currentTimeIntervalField, 30);
				break;
			// 12 hour duration, 1 minute interval is min
			case '720':
				setMinValue(currentTimeIntervalField, 60);
				break;
			// 24 hour duration, 2 minute interval is min
			case '1440':
				setMinValue(currentTimeIntervalField, 120);
				break;
			default:
		}
	}					

	// one and 3 hour duration, 60 minute interval
	if (field.name.indexOf("network") < 0 && field.name.indexOf(".duration") > -1 && (field.value == '60' || field.value == '180')) {
		setMaxValue(currentTimeIntervalField, 3600);
	}

	// two hour interval, 6 hour duration
	if (field.name.indexOf("network") < 0 && field.name.indexOf(".time") > -1 && field.value == '7200') {
		setMinValue(currentDurationField, 360);
	}

	// validate values for interval times

	if (field.name.indexOf(".time") > -1 
		&& (field.name.indexOf("nlhost.") > -1 
		|| field.name.indexOf("nlmatrix.") > -1
		|| field.name.indexOf("alhost.") > -1 
		|| field.name.indexOf("almatrix.") > -1)) {
		switch (field.value) {
			// one minute interval, 1 hour duration is max
			case '60':
				setMaxValue(currentDurationField, 60);
				break;
			// 2 minute interval, 1 hour duration is max
			case '120':
				setMaxValue(currentDurationField, 60);
				break;
			// 5 minute interval, 3 hour duration is max
			case '300':
				setMaxValue(currentDurationField, 180);
				break;
			// 10 minute interval, 8 hour duration is max
			case '600':
				setMaxValue(currentDurationField, 480);
				break;
			// 15 minute interval, 12 hour duration is max
			case '900':
				setMaxValue(currentDurationField, 720);
				break;
			// 30 minute interval, 24 hour duration is max
			case '1800':
				setMaxValue(currentDurationField, 1440);
				break;
			// 1 hour interval, 2 day duration is max
			case '3600':
				setMaxValue(currentDurationField, 2880);
				break;
			default:
		}
	}

	if (field.name.indexOf(".time") > -1 && field.name.indexOf("network") > -1) { 
		switch (field.value) {
			// 10 second interval, 2 hour duration is max
			case '10':
				setMaxValue(currentDurationField, 120);
				break;
			// 20 second interval, 3 hour duration is max
			case '20':
				setMaxValue(currentDurationField, 180);
				break;
			// 30 second interval, 8 hour duration is max
			case '30':
				setMaxValue(currentDurationField, 480);
				break;
			// 1 minute interval, 12 hour duration is max
			case '60':
				setMaxValue(currentDurationField, 720);
				break;
			// 2 minute interval, 24 hour duration is max
			case '120':
				setMaxValue(currentDurationField, 1440);
				break;
			default:
		}
	}
}

function setMinValue(field, value) {
	if (field.value < value) {
		field.value = value;
	}
}

function setMaxValue(field, value) {
	if (field.value > value) {
		field.value = value;
	}
}

function disableEnable() {
	var i;
	var statistics = document.stats;
	for (i = 0; i < statistics.length; i++) {
		// the current field found in the statistics formular
		var currentField = statistics.elements[i];
		if (currentField.name.indexOf(".on") > -1) {
			if (!currentField.checked) {
				// the interval duration for this entry
				var currentIntervalDuration = statistics.elements[i+1];
				// the interval time for this entry
				var currentIntervalTime = statistics.elements[i+2];
				currentIntervalDuration.disabled = true;
				currentIntervalTime.disabled = true;
			} else {
				// the interval duration for this entry
				var currentIntervalDuration = statistics.elements[i+1];
				// the interval time for this entry
				var currentIntervalTime = statistics.elements[i+2];
				currentIntervalDuration.disabled = false;
				currentIntervalTime.disabled = false;
			}
		}
		if (currentField.name.indexOf(".duration") > -1) {
			// the interval time for this entry
			var currentIntervalTime = statistics.elements[i+1];
			// checkbox: turns on or off the count for this entry
			var currentCheckBox = statistics.elements[i-1];
			if (currentField.value == 0) {
				currentIntervalTime.disabled = true;
			} else {
				currentIntervalTime.disabled = false;
			}
			if (currentCheckBox.checked == false) {
				currentIntervalTime.disabled = true;
			}
		}
	}
}

/*
function resetProbe() {
	if(confirm('Are you sure?')) {
		top.reset = true; 
		
		// refresh the interval selection drop down box in the log applet page
		//window.setTimeout(refreshIntervals, 0);
		return true
	}
	else {
	 	return false;
	}	
}*/


function refreshIntervals() {
	top.frames["LogSelector"].loadTimeStamps();
}