/**
 * Loads the resources.
 */
function loadResources() {
	var session = document.getElementById('sid').value;
	var url = "?resource=UserManager&mode=get&command=resources&format=xml&sid=" + session;
	loadXML(url, loadResourcesCallback);
}


/**
 * The resources used for super users only.
 */
var superUserResources = new Array();


/**
 * Sets the resource list for a user.
 */
function loadResourcesCallback(xml) {
	var resources = xml.getElementsByTagName("resource");
	var htmlTable = "<table width='100%'>";
	superUserResources = new Array();
	for (var i = 0; resources != null && i < resources.length; i++) {
		var resource = resources[i];
		var id = URLDecode(resource.getElementsByTagName("id")[0].firstChild.nodeValue);
		var name = URLDecode(resource.getElementsByTagName("name")[0].firstChild.nodeValue);
		var visible = resource.getElementsByTagName("visible")[0].firstChild.nodeValue;
		var superuser = resource.getElementsByTagName("superuser")[0].firstChild.nodeValue;
		if (superuser == "true") {
			superUserResources[superUserResources.length] = id;
		}
		if (visible == "true") {
			if (id.indexOf('.') >= 0) {
				name = " - " + name;
			}
			htmlTable += "<tr><td align='right' width='30%'>";
			htmlTable += "<input onclick='toggleStatisticsResources(this)' type='checkbox' checked name='" + id + "' id='" + id + "'></td>";
			htmlTable += "<td width='70%'>" + name + "</td></tr>";
		}
		
	}
	htmlTable += "</table>";
	document.getElementById('userDetails_resources').innerHTML = htmlTable;
}


/**
 * Sends a request to get the current users.
 */
function loadUsers() {
	var session = document.getElementById('sid').value;
	var url = "?resource=UserManager&mode=get&command=getall&format=xml&sid=" + session;
	loadXML(url, loadUsersCallback);
}


/**
 * Sets the users in the page.
 */
function loadUsersCallback(xml) {
	var users = xml.getElementsByTagName("user");
	var htmlTable = 
		"<table width='600' class='defaultTable setupBox' cellpadding='4' cellspacing='0'>" +
		"<tr class='header'>" +
		"<td valign='top'>Login</td><td valign='top'>User name</td><td valign='top'>User type</td><td valign='top'>Logged in</td>" +
		"<td></td>" +
		"</tr>"
	;
	for (var i = 0 ; users != null && i < users.length ; i++) {
		var user = users[i];
		var login = URLDecode(user.getElementsByTagName("login")[0].firstChild.nodeValue);
		var name = URLDecode(user.getElementsByTagName("name")[0].firstChild.nodeValue);
		var type = user.getElementsByTagName("type")[0].firstChild.nodeValue;
		var loggedIn = user.getElementsByTagName("loggedin")[0].firstChild.nodeValue;
		if (i % 2 == 0) {
			htmlTable += "<tr>";
		}
		else {
			htmlTable += "<tr>";
		}
		if (type != "administrator") {
				htmlTable += "<td><div id='" + login + "'><a href='#' onClick='getUser(\"" + login + "\");'>" + login + "</a></div></td><td>" + name + "</td><td>" + type + "</td><td>" + loggedIn + "</td>"
				htmlTable += "<td align='right'>";
				htmlTable += "<a title='Delete user' href='#' onClick='deleteUser(\"" + login + "\");'><img src='images/icon_delete.gif' border=0/></a></td>";
		}
		else {
			htmlTable += "<td><div id='" + login + "'><a href='#' onClick='showSetupWindow(\"adminPassword\", true);'>" + login + "</a></div></td><td>" + name + "</td><td>" + type + "</td><td>" + loggedIn + "</td>"
		}
		htmlTable += "</tr>";
	}
	
	htmlTable += "</table>";
	document.getElementById('users').innerHTML = htmlTable;

	setupLoaded("userSetup");
}


/**
 * Sends a request to add a user.
 */
function addUser() {
	var form = document.getElementById('userDetailForm');
	
	var login = form.login.value;
	var password = form.password.value;
	if (login && document.getElementById(login) != null) {
		setWindowMessageField(document.getElementById("userDetails"), "name", "td", "The user already exists", undefined, "userDetails");
		setupLoaded("userDetails");
	}
	else if (login.length == 0 || password.length == 0) {
		setWindowMessageField(document.getElementById("userDetails"), "name", "td", "The login or password can not be empty", undefined, "userDetails");
		setupLoaded("userDetails");
	}
	else if (password.length > 0 && !isValidPassword(password)) {
		setWindowMessageField(document.getElementById("userDetails"), "name", "td", "Invalid characters in the password", undefined, "userDetails");
		setupLoaded("userDetails");
	}
	else {
		var session = document.getElementById('sid').value;
		var url = "?resource=UserManager&mode=set&command=add&format=xml&sid=" + session;
		url += "&login=" + encodeURIComponent(form.login.value);
		url += "&name=" + encodeURIComponent(form.name.value);
		url += "&password=" + form.password.value;
		url += "&type=" + form.type.value;
		url += getAccessTag(form.elements);
		loadXML(url, addUserCallback);
	}
}


/**
 * Adds a user to the table.
 */
function addUserCallback(xml) {
	var users = xml.getElementsByTagName("user");
	if (users != null && users.length > 0) {
		loadUsers();
		showSetupWindow("userDetails", false);
		name = URLDecode(users[0].getElementsByTagName("name")[0].firstChild.nodeValue);
		setWindowMessageField(document.getElementById("userListMessages"), "name", "td", null, "User successfully added: " + name, "userSetup");
	}
	else if (xml.getElementsByTagName("error") != null) {
		var error = xml.getElementsByTagName("message");
		var message = error[0].firstChild.nodeValue;
		setWindowMessageField(document.getElementById("userListMessages"), "name", "td", message, undefined, "userSetup");
		setupLoaded("userDetails");
	}
}


/**
 * Sends a request to modify a user.
 */
function modifyUser() {
	var form = document.getElementById('userDetailForm');
	var password = form.password.value;
	if (password.length > 0 && !isValidPassword(password)) {
		setWindowMessageField(form, "name", "td", "Invalid characters in the password", undefined, "userDetails");
	}
	else {
		var session = document.getElementById('sid').value;
		var url = "?resource=UserManager&mode=set&command=modify&format=xml&sid=" + session;
		url += "&login=" + encodeURIComponent(form.login.value);
		url += "&name=" + encodeURIComponent(form.name.value);
		if (password.length > 0) {
			url += "&password=" + form.password.value;
		}
		url += "&type=" + form.type.value;
		url += getAccessTag(form.elements);
		loadXML(url, modifyUserCallback);
	}
}


/**
 * The modify user callback. Removes the modify box and reloads the user list.<b> 
 */
function modifyUserCallback(xml) {
	var users = xml.getElementsByTagName("user");
	if (users != null && users.length > 0) {
		showSetupWindow("userDetails", false);
		loadUsers();
		login = URLDecode(users[0].getElementsByTagName("login")[0].firstChild.nodeValue);
		setWindowMessageField(document.getElementById("userListMessages"), "name", "td", null, "User successfully modified: " + login, "userSetup");
	}
	else if (xml.getElementsByTagName("error") != null) {
		var error = xml.getElementsByTagName("message");
		var message = error[0].firstChild.nodeValue;
		setWindowMessageField(document.getElementById("userDetails"), "name", "td", message, "userDetails");
	}
}


/**
 * Sends a request to get user information.<b> 
 */
function getUser(login) {
	var session = document.getElementById('sid').value;
	var url = "?resource=UserManager&mode=get&command=get&format=xml&sid=" + session;
	url += "&login=" + encodeURIComponent(login) + "&access=true";
	loadXML(url, getUserCallback);
}


/**
 * Displays the specified user.
 */
function getUserCallback(xml) {
	showUserForm('Modify user', true);
	var users = xml.getElementsByTagName("user");
	if (users != null && users.length == 1) {
		setUserData(users[0]);
	}
}


/**
 * Sends a request to delete a user.
 */
function deleteUser(login) {
	if (confirm('Are you sure?')) {
		var session = document.getElementById('sid').value;
		var url = "?resource=UserManager&mode=set&command=delete&format=xml&sid=" + session;
		url += "&login=" + encodeURIComponent(login);
		loadXML(url, loadUsers);
		setWindowMessageField(document.getElementById("userListMessages"), "name", "td", null, login + " successfully deleted", "userSetup");
	}
}


/**
 * Sends a request to update the admin password.
 */
function setAdminPassword() {
	// make sure the info is correct
	var form = document.getElementById('adminPasswordForm');
	if (form.oldpassword.value.length == 0 || form.newpassword.value.length == 0 || form.newpassword2.value.length == 0) {
		setWindowMessageField(document.getElementById("adminPassword"), "name", "td", "The passwords can not be empty", undefined, "adminPassword");
	}
	else if (form.newpassword.value != form.newpassword2.value) {
		setWindowMessageField(document.getElementById("adminPassword"), "name", "td", "Invalid password", undefined, "adminPassword");
	}
	else if (form.newpassword.value.toLowerCase() == 'admin' || form.newpassword.value.toLowerCase() == 'administrator') {
		setWindowMessageField(document.getElementById("adminPassword"), "name", "td", "You can not set the password to 'admin' or 'administrator'", undefined, "adminPassword");
	}
	else if (!isValidPassword(form.newpassword.value)) {
		setWindowMessageField(document.getElementById("adminPassword"), "name", "td", "Invalid characters in the password", undefined, "adminPassword");
	}
	// send the request
	else {
		var session = document.getElementById('sid').value;
		var url = "?resource=UserManager&mode=set&command=setadminpassword&format=xml&sid=" + session;
		url += "&old=" + form.oldpassword.value;
		url += "&new=" + form.newpassword.value;
		loadXML(url, setAdminPasswordCallback);
	}
}


/**
 * Clears and removes the password box.
 */
function setAdminPasswordCallback(xml) {
	var users = xml.getElementsByTagName("user");
	if (users != null && users.length > 0) {
		// display a note the admin password was changed
		name = users[0].getElementsByTagName("name")[0].firstChild.nodeValue;
		setWindowMessageField(document.getElementById("userListMessages"), "name", "td", null, "Password changed for " + name, "userSetup");		

		// clear and hide the password form
		var form = document.getElementById("adminPasswordForm");
		form.oldpassword.value = '';
		form.newpassword.value = '';
		form.newpassword2.value = '';
		showSetupWindow("adminPassword", false);
	}
	else if (xml.getElementsByTagName("error") != null) {
		var error = xml.getElementsByTagName("message");
		var message = error[0].firstChild.nodeValue;
		setWindowMessageField(document.getElementById("adminPassword"), "name", "td", message, undefined, "adminPassword");
		setupLoaded("adminPassword");
	}
}


// utility functions


/**
 * Displays or hides the user detail form.
 */
function showUserForm(label, show) {
	showSetupWindow("userDetails", true);
	WindowManager.instance().getWindow("userDetails").setWindowTitle(label);
	WindowManager.instance().getStatusbarButton("userDetails").setText(label);
	document.getElementById("userDetails_button").setAttribute("value", label);
	// set the correct callback
	var form = document.getElementById('userDetailForm');
	clearUserForm(true);
	
	// add user
	if (label == "Add user") {
		document.getElementById("userDetails_button").onclick = addUser;
		form.login.disabled = false;
		
		// check all default resources
		for (var i = 0; i < form.elements.length; i++) {
			if (form.elements[i].type == "checkbox" && form.elements[i].disabled != true) {
				form.elements[i].checked = true;
			}
		}
	}
	// modify user
	else {
		document.getElementById("userDetails_button").onclick = modifyUser;
		form.login.disabled = true;
	}
	
	//TODO: if statistics is unchecked, uncheck all subresources
}


/**
 * Sets the specified xml data in the user form.
 */
function setUserData(user) {
	clearUserForm(false);
	// set the user data
	var form = document.getElementById('userDetailForm');
	form.login.value = URLDecode(user.getElementsByTagName("login")[0].firstChild.nodeValue);
	form.name.value = URLDecode(user.getElementsByTagName("name")[0].firstChild.nodeValue);
	var type = user.getElementsByTagName("type")[0].firstChild.nodeValue;
	if (type == "super user") {
		form.type.value = "super";
	}
	else {
		form.type.value = "normal";
	}
	// set the resource checkboxes
	if (form.type.value == "super") {
		setSuperUser();
	}
	else {
		setNormalUserResources(user);
	}
}


/**
 * Sets a normal users resources based on resources from specified xml data. 
 */
function setNormalUserResources(user) {
	// uncheck all resources
	var form = document.getElementById('userDetailForm');
	for (var i = 0; i < form.elements.length; i++) {
		if (form.elements[i].type == "checkbox") {
			form.elements[i].checked = false;
		}
	}
	
	// check the resources the user can access
	var resources = user.getElementsByTagName("resource");
	for (var i = 0; resources != null && i < resources.length; i++) {
		var resource = resources[i].firstChild.nodeValue;
		document.getElementById(resource).checked = true;
	}
	
	// if the statistics resource is not checked disable all subresources
	if (!form.statistics.checked) {
		for (var i = 0; i < form.elements.length; i++) {
			if (form.elements[i].type == "checkbox" && form.elements[i].name.indexOf("statistics.") > -1) {
				form.elements[i].disabled = true;
			}
		}
	}
}


/**
 * Returns true if the resource is a super-user accessible resource only.
 */
function isSuperUserResource(resource) {
	for (var i = 0; superUserResources != null && i < superUserResources.length; i++) {
		if (superUserResources[i] == resource) {
			return true;
		}
	}
	return false;
}


/**
 * Clears the user form.
 */
function clearUserForm(checked) {
	var form = document.getElementById('userDetailForm');
	form.login.value = '';
	form.name.value = '';
	form.password.value = '';
	form.type.value = 'normal';
	setSuperUser();
}


/**
 * Sets the fields to super user mode.<b> 
 */
function setSuperUser() {
	var form = document.getElementById('userDetailForm');
	var superUser = form.type.value;
	for (var i = 0; i < form.elements.length; i++) {
		if (form.elements[i].type == "checkbox") {
			form.elements[i].disabled = false;
			if (superUser == "super") {
				form.elements[i].checked = true;
				form.elements[i].disabled = true;
			}
			else if (isSuperUserResource(form.elements[i].name)) {
				form.elements[i].disabled = true;
				form.elements[i].checked = false;
			}
		}
	}
}


/**
 * Checks/unchecks all the statistics sub resources.
 */
function toggleStatisticsResources(field) {
	if (field.name == "statistics") {
		var form = document.getElementById('userDetailForm');
		for (var i = 0; i < form.elements.length; i++) {
			if (form.elements[i].type == "checkbox" && form.elements[i].name.indexOf("statistics.") > -1) {
				if (field.checked) {
					form.elements[i].checked = true;
					form.elements[i].disabled = false;
				}
				else {
					form.elements[i].checked = false;
					form.elements[i].disabled = true;
				}
			}
		}
	}
	else {
		if (field.name.indexOf("statistics.") > -1) {
			var form = document.getElementById('userTableForm');
			for (var i = 0; i < form.elements.length; i++) {
				if (form.elements[i].type == "checkbox" && field.checked && form.elements[i].name == "statistics") {
					form.elements[i].checked = true;
					return;
				}
				else if (form.elements[i].type == "checkbox" && form.elements[i].name == "statistics") {
					form.elements[i].checked = true;
					return;
				}
			}
		}
	}
} 


/**
 * Returns the access tag.
 */
function getAccessTag(fields) {
	var url = "&access=";
	for (var i = 0; i < fields.length; i++) {
		if (fields[i].type == "checkbox" && fields[i].checked == true) {
			url += fields[i].name + ",";
		}
	}
	return url;
}