/**
 * This class is responsible for loading, and managing the slaves. 
 */
 

/**
 * The url used to load the slaves. 
 */
var LOAD_SLAVES_BASE_URL = "?resource=SlaveProbeManager&mode=get&command=allSlaves&sid=";


/**
 * The url used to load a slave. 
 */
var LOAD_SLAVE_BASE_URL = "?resource=SlaveProbeManager&mode=get&command=slave&sid=";


/**
 * Get the single instance of this class.
 */
function getSlaveProbeManager() {

	if (!top.SlaveProbeManager) {
		// create the slave probe manager
	
		/**
		 * Creates the single instance of this class
		 */ 
		top.SlaveProbeManager = new Object();
		
		
		/**
		 * The function to call when the slaves are loaded and parsed. 
		 */
		top.SlaveProbeManager.loadSlavesFunction;
		
		
		/**
		 * The array containing the slaves. 
		 */
		top.SlaveProbeManager.slaves = new Array();
		
		
		/**
		 * The function to call when a slave has been loaded. 
		 */
		top.SlaveProbeManager.loadSlaveFunction;
		
		
		// services
		
		
		/**
		 * Loads the slaves. 
		 * @param functionToCall The function to call when the slaves are loaded. 
		 */
		top.SlaveProbeManager.loadSlaves = function(functionToCall) {
			this.loadSlavesFunction = functionToCall;
			if (exists(top.loadXML)) {
				// might not exist in tests
				top.loadXML(LOAD_SLAVES_BASE_URL + getSid(), "slavesLoaded", top.SlaveProbeManager);
			}
		}

	
		/**
		 * The slaves have been loaded, parse the response.
		 * @param response The response from the server. 
		 */
		top.SlaveProbeManager.slavesLoaded = function(response) {
			this.clear();
			if (!exists(response)) {
				return;
			}
			this.parseXmlSlaves(response);
			if (exists(this.loadSlavesFunction)) {
				this.loadSlavesFunction(this.slaves);
			}
		}
		
		
		/**
		 * Parses the xml slave data. 
		 * @param slavesXml The xml slaves to parse. 
		 */
		top.SlaveProbeManager.parseXmlSlaves = function(slavesXml) {
			var slaves = slavesXml.getElementsByTagName("slave");
			for (var i = 0; i < slaves.length; i++) {
				var slave = createSlaveProbeXml(slaves[i]);
				this.slaves[this.slaves.length] = slave;
			}
		}
		
		
		/**
		 * Gets a slave with a given ip address.
		 * @param ipAddress The ip addresss of the slave to get. 
		 * @throws Error if ipAddress is undefined, null or empty.
		 */
		top.SlaveProbeManager.getSlave = function(ipAddress) {
			assertNotEmpty(ipAddress, "The ip address to get the slave for is undefined, null or empty");
			for (var i = 0; i < this.slaves.length; i++) {
				if (this.slaves[i].getIpAddress().equalsIgnoreCase(ipAddress)) {
					return this.slaves[i];
				}
			}
		}
		
		
		/**
		 * Gets all the slaves currently loaded. 
		 * @return All the slaves in the manager as a array. 
		 */
		top.SlaveProbeManager.getSlaves = function() {
			return this.slaves;
		}
		
		
		/**
		 * Clears all the slaves. 
		 */ 
		top.SlaveProbeManager.clear = function() {
			this.slaves = new Array();
		}
		
		
		/**
		 * Gets the slave count.
		 * @return The number of slaves currently in the slave probe manager.
		 */
		top.SlaveProbeManager.getSlaveCount = function() {
			return this.slaves.length;
		}
		
		
		/**
		 * Delete a slave. 
		 * @param slave The slave to delete.
		 * @throws Error if slave is undefined or null.
		 */
		top.SlaveProbeManager.deleteSlave = function(slave) {
			assertExists(slave, "The slave to delete is undefined or null");
			for (var i = 0; i < this.slaves.length; i++) {
				if (this.slaves[i] == slave) {
					this.slaves[i] = null;
					this.slaves = removeNullEntries(this.slaves);
				}
			}
		}
		
		
		/**
		 * Updates a slaves data. 
		 * @param slave The slave to update. 
		 * @param functionToCall The function to call when the slave has been loaded. 
		 * @throws Error if slave is undefined null or empty.
		 */
		top.SlaveProbeManager.updateSlave = function(slave, functionToCall) {
			assertExists(slave, "The slave to update is undefined or null");
			this.loadSlaveFunction = functionToCall;
			var request = LOAD_SLAVE_BASE_URL + getSid() + "&ip=" + slave.getIpAddress();
			if (exists(top.loadXML)) {
				// might not exist in tests
				top.loadXML(request, "slaveLoaded", top.SlaveProbeManager);
			}
		}
		
		
		/**
		 * A slave has been loaded. 
		 * @param response The response from the server. 
		 */
		top.SlaveProbeManager.slaveLoaded = function(response) {
			if (!exists(response)) {
				return;
			}
			var ip = getTagNodeValue("ip", response);
			var slave = isEmpty(ip) ? null : this.getSlave(ip);
			if (!exists(slave)) {
				return;
			}
			
			// update the slaves values
			slave.setAttribute("name", getTagNodeValue("name", response)); 
			slave.setAttribute("description", getTagNodeValue("description", response));
			slave.setAttribute("port", getTagNodeValue("port", response));
			slave.setAttribute("login", getTagNodeValue("login", response));
			slave.setAttribute("password", getTagNodeValue("password", response));
			slave.setAttribute("status", getTagNodeValue("status", response));
			
			if (exists(this.loadSlaveFunction)) {
				this.loadSlaveFunction(slave);
			}
		}
	}
	
	return top.SlaveProbeManager;
}