/**
 * The class to set to hide the log selector frame.
 */
var LOG_SELECTOR_FRAME_INACTIVE_CLASS = "inactiveLogSelector";


/**
 * The name of the tab the summary and the my networks tab when run in master mode, are in. 
 */
var SUMMARY_TAB_NAME = "Main";


/**
 * The name of the tab the master main page is in.
 */
var MASTER_MAIN_TAB_NAME = "MasterMain";


/**
 * The name of the directory where images are in.
 */
var IMAGES_DIRECTORY = "themes/black/images/";


/**
 * Set to true when the page has loaded and everything has been initialized. 
 */
var loaded = false;


/**
 * When the page is loaded a loading screen is shown until the layout has been loaded.
 * This is used to indicate whether the loading screen has been removed or not. 
 */
var loadingScreenRemoved = false;


/**
 * The listener used for listening to updated events from the layoutmanager. When the layout is set 
 * the loading screen is removed. 
 */
var eventListener;


/**
 * The name of the tab the summary screen is in.
 */
var currentTabName = "";


// image preloading


/**
 * Start preloading of images.
 */
function startImagePreloading() {
	var preloadedImages = new Array();
	preloadedImages[0] = "/images/sort_down.gif";
	preloadedImages[1] = "/images/sort_up.gif";
	preloadedImages[2] = IMAGES_DIRECTORY + "gauge_legend.gif";
	preloadedImages[3] = IMAGES_DIRECTORY + "icon_delete_entry.gif";
	preloadedImages[4] = IMAGES_DIRECTORY + "icon_delete_entry_odd.gif";
	preloadedImages[5] = IMAGES_DIRECTORY + "icon_delete_entry_mouseover.gif";
	preloadedImages[6] = IMAGES_DIRECTORY + "icon_pluss_small.gif";
	preloadedImages[7] = IMAGES_DIRECTORY + "statusbar_button_left_mouseover.jpg";
	preloadedImages[8] = IMAGES_DIRECTORY + "statusbar_button_middle_mouseover.jpg";
	preloadedImages[9] = IMAGES_DIRECTORY + "statusbar_button_right_mouseover.jpg";
	preloadedImages[10] = IMAGES_DIRECTORY + "statusbar_button_left_active.jpg";
	preloadedImages[11] = IMAGES_DIRECTORY + "statusbar_button_middle_active.jpg";
	preloadedImages[12] = IMAGES_DIRECTORY + "statusbar_button_right_active.jpg";
	preloadedImages[13] = IMAGES_DIRECTORY + "slider_button.png";
	preloadedImages[14] = IMAGES_DIRECTORY + "slider_couse.jpg";
	preloadedImages[15] = IMAGES_DIRECTORY + "slider_button_inactive.png";
	
	var imagePreloader = new ImagePreloader(preloadedImages);
	imagePreloader.start();
}


// init summary page


/**
 * Inits the summary or the master main page when run in master mode.
 */
function init() {
	currentTabName = masterMainTabActive() ? MASTER_MAIN_TAB_NAME : SUMMARY_TAB_NAME;
	if (masterMainTabActive()) {
		showMasterMenu();
	}	 
	if (getMasterMode() && !masterMainTabActive()){
		initSlaveSelector();
	}
	
	Statusbar.instance().init();
	Desktop.instance().setTopMenu(getElementWithAttribute(document.body, "topMenu", "id", "div"));
	Desktop.instance().setDesktop(getElementWithAttribute(document.body, "desktop", "id", "div"));
	Desktop.instance().setStatusbar(Statusbar.instance());
	WindowManager.instance().setStatusbar(Statusbar.instance());
	WindowManager.instance().setDesktop(Desktop.instance());
	ChartManager.instance();
	
	createProbeWindowManager();
	createProbeTableManager();

	handleUnsupportedBrowsers(); 
	startImagePreloading(); 
	initSlaves();
	initSlaveWindow();
	eventListener = new Listener(this);
	eventListener.addEvent(LAYOUT_SET_EVENT, processLayoutSetEvent);
	// set a different class for the line chart container if IE is used,
	// a necessary hack to get around a percentage rounding problem 
	var gaugeChartContainer = getElementWithAttribute(document.body, "gaugeChartContainer", "name", "div");
	setClass(gaugeChartContainer, isIE() ? "ieGaugeChartContainer" : "gaugeChartContainer");

	loadInitialData();
}


/**
 * Shows the master main menu.
 */
function showMasterMenu() {
	var summaryMenu = getElementWithAttribute(document.body, "summaryMenuImage", "id", "img");
	summaryMenu.setAttribute("menu", "");
	summaryMenu.onclick = function() {
		WindowManager.instance().activateWindow('topnNetworksWindow');
	}
	summaryMenu.style.cursor = "pointer";
	summaryMenu.onmouseover = null;
	summaryMenu.onmouseout = null;

	var alarmsMenu = getElementWithAttribute(document.body, "alarmsMenuImage", "id", "img");
	alarmsMenu.setAttribute("menu", "");
	alarmsMenu.onclick = function() {
		WindowManager.instance().activateWindow('alltriggeredmasteralarmswindow');
	}
	alarmsMenu.style.cursor = "pointer";
	alarmsMenu.onmouseover = null;
	alarmsMenu.onmouseout = null;
	
	// hide unused menu choices
	var protocolMenuCell = getElementWithAttribute(document.body, "protocolMenuCell", "name", "td")
	setClass(protocolMenuCell, "inactive");
	var hostsMenuCell = getElementWithAttribute(document.body, "hostsMenuCell", "name", "td");
	setClass(hostsMenuCell, "inactive");
	var conversationsMenuCell = getElementWithAttribute(document.body, "conversationsMenuCell", "name", "td");
	setClass(conversationsMenuCell, "inactive");
	
	// show the slaves link in the menu
	var manageProbes = getElementWithAttribute(document.body, "manageProbes", "name", "img");
	setClass(manageProbes, "inActiveLink");
}


/**
 * Loads the initial data. 
 */
function loadInitialData() {
	getLicenseManager().loadLicense();
	if (getMasterMode() && !masterMainTabActive() && !isEmpty(top.showSlave)) {
		// drilldown into MyNetworks tab, show the given slave and window
		showSlaveWindow(top.showSlave, top.showWindow);		
	}	
	else if (getMasterMode() && !masterMainTabActive()) {
		// MyNetworks tab activated, select the local probe
		selectSlave(getServerAddress());
	}
	else {
		// load data for local probe
		loadLayout();	
		ProfileManager.loadProfiles();
	}
}


/**
 * This is called when this tab has been activated. Start updating
 * of tables.
 */
function tabActivated() {
	hideLogSelector();
	setFrameSize(currentTabName);
	Desktop.instance().setDesktopHeight();
	// when the summary tab is hidden we stop the window manager from listening to events to prevent 
	// windowmanager in the setup tab to send events to this window manager
	WindowManager.instance().eventProcessor.startListening();
	eventListener.startListening();
	ProfileManager.eventListener.startListening();
}


/**
 * This is called when this tab has been deactviated. Stop 
 * updating of the tables.
 */
function tabDeactivated() {
	// if the loading screen has not been removed remove it
	if (!loaded) {
		loadingScreenRemoved = true;
		doneLoading();
	}
	// when the summary tab is hidden we stop the window manager from listening to events to prevent 
	// windowmanager in the setup tab to send events to this window manager
	if (exists(WindowManager.instance())) {
		// the user might switch tabs before this tab is completely loaded
		WindowManager.instance().eventProcessor.stopListening();
	}
	if (exists(ProfileManager)) {
		// the user might switch tabs before this tab is completely loaded
		ProfileManager.eventListener.stopListening();
	}
	eventListener.stopListening();
}


/**
 * The size of the window has been changed. 
 */
function windowResized() {
	setFrameSize(currentTabName);
	Desktop.instance().setDesktopHeight();
}

 
/**
 * Shows the help for the status bar.
 */
function showStatusbarHelp() {
	showHelp("doc/index.html?page=files%2Fstatusbar.html");
}


// loading


/**
 * Returns true if this page has been loaded, false otherwise.  
 */
function isDoneLoading() {
	return loaded;
}


/**
 * The page has finished loading.
 */
function doneLoading() {
	// the user might switch tab before it is fully loaded
	if (exists(top.tabSection.getTab(currentTabName)) && !loaded) {
		top.tabSection.getTab(currentTabName).doneLoading();
	}
	loaded = true;
}


/**
 * Processes a layout set event.
 * @param event the event to process.
 */
function processLayoutSetEvent(event) {
	loadingScreenRemoved = true;
	if (top.tabSection) {
		doneLoading();
	}
}


/**
 * Activates the Main tab and shows a given window in it as response to a right click
 * on one of the entries in the top networks window. 
 * @param windowId The id of the window to show. 
 */
function showMainWindow(windowId) {
	if (!exists(lastTableObject) || !exists(lastTdNode) || isEmpty(windowId)) {
		// lastTdNode and lastTableObject are variables set by the right click menu
		// and refer to the last td html node and the last table right clicked
		return;
	}
	var row = lastTableObject.getRowFromTd(lastTdNode);
	var ipField = exists(row) ? getFieldFromRow("src", row) : null;
	if (!exists(ipField)) {
		return;
	}
	if (!exists(top.frames[SUMMARY_TAB_NAME])) {	
		// My networks tab has not been loaded yet
		top.tabSection.activateTab(SUMMARY_TAB_NAME);
		top.showSlave = ipField.getValue();
		top.showWindow = windowId;	
	}
	else {
		top.tabSection.activateTab(SUMMARY_TAB_NAME);
		top.frames[SUMMARY_TAB_NAME].showSlaveWindow(ipField.getValue(), windowId);
	}
}


/**
 * Returns true if the current active tab is the master main tab. 
 */
function masterMainTabActive() {
	if (exists(top.tabSection) && top.tabSection.getActiveTab().getName() == MASTER_MAIN_TAB_NAME) {
		return true;
	}
	return false;
}