/**
 * This class makes sure input the user types in a given input field is a
 * valid ip address. If the user types in a invalid ip address a error message
 * is shown in a given field. 
 */
 

/**
 * The message shown when the ip address is invalid.
 */
var INVALID_IP_ADDRESS_MESSAGE = "Invalid ip address. Valid format is xxx.xxx.xxx.xxx";


/**
 * The message shown when the ip address entered contains invalid numbers or characters.
 */
var INVALID_IP_NUMBERS_MESSAGE = "Invalid number in ip address. Valid is 0-255";


/**
 * The message shown when the ip address entered is invalid.
 */
var INVALID_IP_ADDRESS_MESSAGE = "Invalid ip address. Valid format is xxx.xxx.xxx.xxx";


// construction


/**
 * Factory method to create a ip field validator. 
 * @param ipField The input field where the user types the ip address.
 * @param messageField The field where to show messages in.
 * @return The ip field validator created. 
 * @throws Error if ipField is null or empty.
 * @throws Erorr if message is null or empty.
 */
function createIpFieldValidator(ipField, messageField) {
	assertExists(ipField, "The field to validate does not exists");
	assertExists(messageField, "The message field does not exist");
	
	var subInstance = new IpFieldValidator();
	var superInstance = new InputFieldValidator(ipField, messageField);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance.addListeners();
	return subInstance;
}


/**
 * Constructs a instance of this class. 
 */
function IpFieldValidator() {
	
}


// event handling


/**
 * Handles the keyup events generated when the user types in the ip address. 
 * @param event The onkeyup event to process. 
 */
IpFieldValidator.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;	
	if (!isEmpty(this.inputField.value) && !validIpNumbers(this.inputField.value)) {
		this.setErrorMessage(INVALID_IP_NUMBERS_MESSAGE);
		this.markError();
		return;
	}
	if (this.countIpNumbers() > 4) {
		this.markError();
		this.setErrorMessage(INVALID_IP_ADDRESS_MESSAGE);
		return;
	}
	
	this.markValid();
	this.clearMessage();
	if (exists(event) && event.keyCode == 13) { 
		this.enterPressed();
	}
}


/**
 * The ip field has lost focus, check the ip address is set correct.
 */
IpFieldValidator.prototype.onBlurHandler = function() {
	if (!isEmpty(this.inputField.value) && !validIpNumbers(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_IP_NUMBERS_MESSAGE);
	}
	else if (isEmpty(this.inputField.value) || !isValidIpAddress(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_IP_ADDRESS_MESSAGE);
	}
}


/**
 * The ip field has been focused, validate the input and set any error message if invalid input.
 */
IpFieldValidator.prototype.onFocusHandler = function() {
	if (!isEmpty(this.inputField.value) && !validIpNumbers(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_IP_NUMBERS_MESSAGE);
	}
	else if (!isEmpty(this.inputField.value) && !isValidIpAddress(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_IP_ADDRESS_MESSAGE);
	}
	else {
		this.clearMessage();
		this.markValid();
	}
}


// implementation


/**
 * Counts the number of numbers in the ip field (10.0 will return 2).
 * @return The count of numbers in the ip field (10.0.2 returns 3).
 */
IpFieldValidator.prototype.countIpNumbers = function() {
	if (isEmpty(this.inputField.value)) {
		return 0;
	}
	var tokenizer = new StringTokenizer(this.inputField.value, ".");
	return tokenizer.countTokens();
}