/**
 * This class validates port ids (1.2048.6.80) and sets a erorr message if the port field is not valid. 
 */



/**
 * The message shown when the port entered is invalid. 
 */
var INVALID_PORT_ID_MESSAGE = "Invalid port set";


/**
 * The message shown when a number in the port entered is not a valid port number. 
 */
var INVALID_PORT_ID_NUMBER_MESSAGE = "Invalid port number in port. Valid is 0-65535";


// construction


/**
 * Factory method to create a port id validator. 
 * @param portfield The port field to validate. 
 * @param messageField The message field where error messages are to be set. 
 * @throws Error if port field is null or empty. 
 * @throws Error if messageField is null or empty. 
 */
function createPortIdFieldValidator(portField, messageField) {
	assertExists(portField, "The port fiel to validate is null or undefined");
	assertExists(messageField, "The message field is null or undefined");
	
	var subInstance = new PortIdFieldValidator();
	var superInstance = new InputFieldValidator(portField, messageField);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance.addListeners();
	return subInstance; 
}


/**
 * Creates a instance of this class. 
 */
function PortIdFieldValidator() {

}


// event 


/**
 * Handles the keyup events generated when the user types in something in the input field.
 * @param event The onkeyup event to process. 
 */
PortIdFieldValidator.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;
	if (!isEmpty(this.inputField.value) && !validPortNumbers(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_PORT_ID_NUMBER_MESSAGE);
		return;
	}
	else {
		this.markValid();
		this.clearMessage();
	}
	if (exists(event) && event.keyCode == 13) { 
		enterPressed();
	}
}


/**
 * The input field has lost focus, validate the input.
 * This needs to be implemented by subclasses. 
 */
PortIdFieldValidator.prototype.onBlurHandler = function() {
	if (!isEmpty(this.inputField.value) && !validPortNumbers(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_PORT_ID_NUMBER_MESSAGE);
	}
	else if (!isValidPortIdString(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_PORT_ID_MESSAGE);
	}
}


/**
 * the port field has been focused, validate the input and set any error message if invalid input. 
 */
PortIdFieldValidator.prototype.onFocusHandler = function() {
	if (!isEmpty(this.inputField.value) && !validPortNumbers(this.inputField.value)) {
		this.markError();
		this.setErrorMessage(INVALID_PORT_ID_NUMBER_MESSAGE);
	}
	else {
		this.clearMessage();
		this.markValid();
	}
}