/**
 * This class makes sure input the user types in a given input field is a
 * valid port number (1.2048.6.80). If the user types in a invalid ip address a error message
 * is shown in a given field.
 */
 

/**
 * The message shown when the port entered is invalid. 
 */
var INVALID_PORT_MESSAGE = "Invalid port. Valid is 0-65535";


// construction


/**
 * Factory method to create a port field validator.  
 * @param portField The input field where the user types the port.
 * @param messageField The field where to show messages in.
 * @return The port field validator created. 
 * @throws Error if portField is null or empty.
 * @throws Erorr if message is null or empty.
 */
function createPortFieldValidator(portField, messageField) {
	assertExists(portField, "The port field does not exist");
	assertExists(messageField, "The message field does not exist");
	
	var subInstance = new PortFieldValidator();
	var superInstance = new InputFieldValidator(portField, messageField);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance.addListeners();
	return subInstance;	
}


/**
 * Constructs the instance of this class.
 */
function PortFieldValidator() {
	
}


// event handling


/**
 * Handles the keyup events generated when the user types in something in the input field.
 * @param event The onkeyup event to process. 
 */
PortFieldValidator.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;
	if (!isEmpty(this.inputField.value) && (!isNumberString(this.inputField.value) || !isValidPort(this.inputField.value))) {
		this.setErrorMessage(INVALID_PORT_MESSAGE);
		this.markError();
		return;
	}
	else {
		this.clearMessage();
		this.markValid();
	}
	if (exists(event) && event.keyCode == 13) { 
		this.enterPressed();
	}
}


/**
 * The input field has lost focus, validate the input.
 * This needs to be implemented by subclasses. 
 */
PortFieldValidator.prototype.onBlurHandler = function() {
	if (isEmpty(this.inputField.value)) {
		this.setErrorMessage(INVALID_PORT_MESSAGE);
		this.markError();
	}
}


/**
 * The input fiels has been focused, validate the input.
 */
PortFieldValidator.prototype.onFocusHandler = function() {
	if (!isEmpty(this.inputField.value) && (!isNumberString(this.inputField.value) || !isValidPort(this.inputField.value))) {
		this.setErrorMessage(INVALID_PORT_MESSAGE);
		this.markError();
	}
	else {
		this.clearMessage();
		this.markValid();
	}
}