/**
 * This class is a item that can be shown in the statusbar button right click menu.
 */
 

// construction


/**
 * Constructs a instance of this class.
 * @param id The id of this button to use. This must be unique for the button this item is for. 
 * @param text The text to show in the right click item. 
 * @param functionToCall The function to call when the item has been clicked.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if text is undefined, null or empty.
 * @throws Error if functionToCall is undefined or null.
 */
function StatusButtonRightClickItem(id, text, functionToCall) {
	assertNotEmpty(id, "The id to set for the right click item is undefined, null or empty");
	assertNotEmpty(text, "The text to set for the right clik item is undefined, null or empty");
	assertExists(functionToCall, "The function to call when the right click item is clicked is undefined or null");
	
	this.id = id;
	this.text = text;
	this.functionToCall = functionToCall;
}


// services


/**
 * Gets the id of this item.
 * return The id of this right click menu item.
 */
StatusButtonRightClickItem.prototype.getId = function() {
	return this.id;
}


/**
 * Get the text to show in the right click menu item.
 * @return The text shown in the right click item.
 */
StatusButtonRightClickItem.prototype.getText = function() {
	return this.text;
}


/**
 * Gets the function to call when this item is clicked. 
 * @return The function that is to be called when this item is clicked. 
 */
StatusButtonRightClickItem.prototype.getFunctionToCall = function() {
	return this.functionToCall;
}
	