/**
 * Constructs a field in the table.
 * @param name The name of the field.
 * @param value The value of the field.
 * @param id Optional parameter specifying a unique id of the field.
 * @throws Error if the name is undefined, null or empty.
 * @throws Error if the value is undefined or null. 
 */
function Field(name, value) {
	assertNotEmpty(name, "Name is undefined, null or empty: " + name);
	assertDefined(value, "Value is undefined or null: " + value);
	
	this.name = name;
	this.value = value;
	if (arguments.length == 3) {
		this.id = arguments[2];
	}
	
	this.properties = new Array();
}


// services


/**
 * Returns the name of a field.
 */
Field.prototype.getName = function() {
	return this.name;
}


/**
 * Returns the value of the field.
 */
Field.prototype.getValue = function() {
	return this.value;
}


/**
 * Returns the unique id of the field.
 */
Field.prototype.getId = function() {
	return this.id;
}


/**
 * Get the width of the field.
 * @return The width of this field.
 */
Field.prototype.getWidth = function() {
	return this.width;
}


/**
 * Sets the value of a field.
 * @param value The value to set. 
 * @throws Error if value is undefined or null. 
 */
Field.prototype.setValue = function(value) {
	assertExists(value, "Value to set for the field is undefined or null");
	this.value = value;
}


// properties


/**
 * Add a property of a field. If no handler for the propertys is set the property is ignored.
 * @param property The property to set (bold, italic...).'
 * @throws Error if property is undefined, null or empty. 
 */
Field.prototype.setProperty = function(property) {
	assertNotEmpty(property, "Property is undefined, null or empty");
	this.properties[this.properties.length] = property;
}