/** 
 * This class is responsible for managing tables.
 */
 
 
// construction


/*
 * The instance of this class.
 */
TableManager.instanceObject = new TableManager();


/**
 * Constructs a instance of this class.
 */
function TableManager() {
	TableManager.tables = new Array();
}


/**
 * Returns the instance of this class.
 */
TableManager.instance = function() {
	return TableManager.instanceObject;
}


// services


/**
 * Get a table.
 * @param id The id of the table to get.
 * @return The table, null if no table with this id exists.
 * @throws Error if id is undefined, empty or null.
 */
TableManager.prototype.getTable = function(id) {
	assertNotEmpty(id, "Id is undefined, empty, or null");

	var len = TableManager.tables.length;
	for (var i = 0; i < len; i++) {
		if (id == TableManager.tables[i].id) {
			return TableManager.tables[i];
		}
	}
	return null;
}


/**
 * Register a new table.
 * @param id The id of the table.
 * @param table The table object to register.
 * @throws Error if the table is not defined or null.
 */
TableManager.prototype.registerNewTable = function(table) {
	assertDefined(table, "Table is null or not defined");

	TableManager.tables[TableManager.tables.length] = table;
}


/**
 * Removes a table. 
 * @param table The table object to remove. 
 * @throws Error if the table to remove is undefined or  null.
 */
TableManager.prototype.removeTable = function(table) {
	assertDefined(table, "Table to remove is undefined or null: " + table);
	
	var length = TableManager.tables.length;
	for (var i = 0; i < length; i++) {
		if (table == TableManager.tables[i]) {
			TableManager.tables[i] = null;
			TableManager.tables = removeNullEntries(TableManager.tables);
		}
	}
}