/**
 * The name of the option to activate the mouseover of the tabs.
 * If this option is set to true and mouse over classes are not specified
 * the css class for active tab is used as mouse over class.
 */
var MOUSE_OVER_PROPERTY = "mouseOver";


/**
 * The css class to set for the left part of the tab on mouse over.
 */
var MOUSE_OVER_LEFT_CLASS_PROPERTY = "mouseOverLeftClass";


/**
 * The css class to set for the middle part of the tab on mouse over if the tab is not active.
 */
var MOUSE_OVER_MIDDLE_CLASS_PROPERTY = "mouseOverMiddleClass";


/**
 * The css class to set for the middle part of the tab on mouse over if the tab is active.
 */
var MOUSE_OVER_MIDDLE_CLASS_ACTIVE_PROPERTY = "mouseOverMiddleClassActive";


/**
 * The css class to set for the right part of the tab on mouse over.
 */
var MOUSE_OVER_RIGHT_CLASS_PROPERTY = "mouseOverRightClass";


/**
 * The css clas to set for the tab section, containing all the tabs. 
 */
var TAB_SECTION_CLASS = "tabSectionClass";


/**
 * The iframe for the content of a tab. If this propertys is not set a
 * iframe is created based on the tabs url.
 */
var TAB_FRAME = "tabFrame";


/**
 * Set this property to true if the bar under the tabs is to be shown.
 */
var TABS_BAR = "tabsBar";


/**
 * If this property is set the content of the tabs is not loaded into the iframe
 * before the tab is activated for the first time potentially reducing the first
 * loading time of the page. 
 */
var LOAD_ONCLICK = "LoadOnClick";


/** 
 * IF this option is set the tab section  will be appended to the element specified by this option.
 */
var TAB_SECTION_PARENT_ELEMENT = "tabSectionParent";


/**
 * The css class to use for the div tag containing all the tags. With this css class one can
 * for example specify the margin-left for the tags. This property needs to be given to the 
 * tabSection not to each tab.
 */
var TABCONTAINER_CLASS = "tabContainerClass";


/**
 * The css class to set for each of the divs containing the three parts of the tag. With this
 * class one can for example adjust the margin between the tabs. This proeprty must be given
 * to each tab.
 */
var TAB_CLASS = "tabClass";


/**
 * The class to set for a tabs iframe.
 */
var FRAME_CLASS = "frameClass";
 

/**
 * The class to set for the tabs iframe to hide it. This class is set for a tabs frame when the 
 * tab is deactivated. If this property is specified you must also 
 * set the ACTIVE_FRAME_CLASS property.
 */
var INACTIVE_FRAME_CLASS = "inactiveFrameClass";


/**
 * The class to set for a tabs frame when the tab has been activated, this must be provided
 * if the INACTIVE_FRAME_CLASS property is given.
 */
var ACTIVE_FRAME_CLASS = "activeFrameClass";


/**
 * Set this to off to turn of the scroll bars for a frame, set to on to 
 * activate them. default for this option is true.
 */
var FRAME_SCROLLBAR = "scrollbarof";


/**
 * The url for a loading image. If this is given the first time the tabs iframe is loaded
 * the loading image will be shown, until the javascript in the iframe notices the tabSection
 * it has finished loading an initing.
 */
var LOADING_IMAGE = "loadingImage";


/**
 * The class to set for the div tag containing the loading image. If this
 * is not set no class is set for the div tag.
 */
var LOADING_SCREEN_CLASS = "loadingScreenClass";


/**
 * The class to set for the loading image. If this is not set no 
 * class is set for the loading image. 
 */
var LOADING_IMAGE_CLASS = "loadingImageClass";

/**
 * The class to set for the icon in the tabs.
 */
var ICON_CLASS = "iconClass";


/**
 * The name of the directory where images are in.
 */
var IMAGES_DIRECTORY = "themes/black/images/";


/**
 * Set the get property function to be a method of both the tab and 
 * TabSection class
 */
TabSection.prototype.getProperty = GetProperty;
Tab.prototype.getProperty = GetProperty;


/**
 * Constructs the tab section.
 * @param tabs The tabs to add for the section, this is a array containing the tabs. Each tab is
 * a associative array containing the tabs id, the text, icon and url. 
 * @param Optional argument, properties A associative array containing the properties for the tab section.
 * @throws Error if tabs is undefined or null.
 */
function TabSection (tabs, properties) {
	assertDefined(tabs, "Tabs is undefined or null");
	
	this.tabs = new Array();
	for (var i = 0; i < tabs.length; i++) {
		var tab = tabs[i];
		this.tabs[i] = new Tab(tab["name"], tab["text"], tab["icon"], tab["inactive_icon"], tab["url"], this, tab["properties"]);
	}

	this.properties = properties;
	// the previous activated tab
	this.history;
	// the active tab
	this.activeTab;
}


// services


/**
 * Gets the active tab.
 * @return The active tab.
 */
TabSection.prototype.getActiveTab = function() {
	return this.activeTab;
}


// tabs


/**
 * Constructs a tab. 
 * @param name The name of the tab.
 * @param text The text to show in the tab.
 * @param icon The icon to show in the tab.
 * @param inactiveIcon The icon to show in the inactive tab (may have different background color for example).
 * @param url The url the tab is to load.
 * @param tabSection The tabSection the tab is in.
 * @param optional argument properties A associative array containing the properties for the tab.
 * @thrwos Error if the name is undefine, null or empty.
 * @throws Error if the text is unefined, null or empty.
 */
function Tab(name, text, icon, inactiveIcon, url, tabSection, properties) {
	assertDefined(name, "Name to set for the tab is undefined, null or empty: " + name);
	assertDefined(text, "Text is undefined, null or empty: " + text);

	this.name = name;
	this.properties = properties;
	this.tabSection = tabSection;
	if (url != undefined) {
		this.url = url;
	}
	else {
		this.url = this.getProperty(TAB_FRAME).src;
	}
	this.icon = icon;
	this.inactiveIcon = inactiveIcon;
	this.preloadIcons();
	
	this.text = text;

	// this is true when the mouse is over the tab
	this.mouseover = false;
	// if the LOAD_ONCLICK property is set this is used to indicate
	// whether the content has been loaded or not
	this.loaded = false;
	// if INACTIVE_FRAME_CLASS property is not provided the height of the
	// iframe is set to 0 on deactivatoin of the tab. This stores
	// the height before we set it to 0
	this.height;
}


/**
 * Preloads the icons if any. 
 */
Tab.prototype.preloadIcons = function() {
	var preloadedGraphics = new Array();
	if (isDefined(this.icon)) {
		preloadedGraphics[preloadedGraphics.length] = this.icon;
	}
	if (isDefined(this.inactiveIcon)) {
		preloadedGraphics[preloadedGraphics.length++] = this.inactiveIcon;
	}

	if(preloadedGraphics.length > 0) {
		var imagePreloader = new ImagePreloader(preloadedGraphics);
		imagePreloader.start();
	}
}


// tab services


/**
 * Checks wheter a tab has been loaded. 
 * @return True if the tab has been loaded, false otherwise. 
 */
Tab.prototype.isLoaded = function() {
	return this.loaded;
}


/**
 * Sets the url for the tab.
 * @param url The url to set. 
 * @throws Error if url is null, undefined or empty.
 */
Tab.prototype.setUrl = function(url) {
	assertNotEmpty(url, "The url to set is undefined, null or empty");
	this.url = url;
	if (this.loaded == true) {
		this.frame.src = url;
	}
}


/**
 * Returns the name of the tab.
 */
Tab.prototype.getName = function() {
	return this.name;
}


/**
 * Activates a tab.
 */
Tab.prototype.activate = function() {
	this.tabSection.setClass(this.name, "active_left", "active_bg", "active_right");

	if (isDefined(this.loadingFrame)) {
		document.body.removeChild(this.loadingFrame);
		this.loadingFrame = undefined;
		this.loading = false;
	}

	// load the content of the frame if the LOAD_ONCLICK property is set and this
	// is the first time the tab is activated
	if (this.getProperty(LOAD_ONCLICK) && this.loaded == false) {
		this.tabSection.appendIframe(this);
		this.tabSection.loadIframeContent(this);
	}
	
	if (this.name == "TrafficStatistics" || this.name == "TrafficLog") {
		// we do this to prevent firefox 2.0 from misplacing the applets
		this.tabSection.activateIFrame(this);
	}
	
	if (!this.loading || !this.getProperty(LOADING_IMAGE)) {
		this.tabSection.activateIFrame(this);
		this.tabSection.notifyActivatedTab();
	}
}


/**
 * Deactivates a tab.
 */
Tab.prototype.deactivate = function() {
	this.tabSection.setClass(this.name, "inactive_left", "inactive_bg", "inactive_right");
	if (this.loading) {
		setClass(this.loadingFrame, "inactive");
	}
	
	// deactivate the frame
	this.tabSection.deactivateIFrame(this);
}


/**
 * Returns a tab with a given name, undefined if no tabs are defined or
 * no tab with this name exists.
 * @param name The name to get the tab of.
 */
TabSection.prototype.getTab = function(name) {
	if (this.tabs == undefined) {
		return undefined;
	}
	
	for (var i = 0; i < this.tabs.length; i++) {
		if (name == this.tabs[i].name) {
			return this.tabs[i];
		}
	}
	
	return undefined;
}
 

// loading of a tab


/**
 * Shows a loading screen until the tab has been loaded and inited.
 * @param tab The tab to show the loading screen for.
 */
Tab.prototype.showLoading = function(tab) {
	this.loading = true;
	this.initLoadingFrame();
	this.tabSection.deactivateIFrame(this);
}


/**
 * Inits the loading frame.
 */
Tab.prototype.initLoadingFrame = function() {
	this.loadingFrame = document.createElement("div");
	this.setLoadingFrameClass();
	
	var loadingImage = document.createElement("img");
	loadingImage.setAttribute("name", "loadingImage");
	if (this.getProperty(LOADING_IMAGE_CLASS)) {
		setClass(loadingImage, this.getProperty(LOADING_IMAGE_CLASS));
	}
	loadingImage.src = this.getProperty(LOADING_IMAGE);
	this.loadingFrame.appendChild(loadingImage);
	this.loadingFrame.innerHTML += "<center><br><br><font size=2>Loading...</font></center>";
	document.body.appendChild(this.loadingFrame);
}


/**
 * Sets the class of the loading frame.
 */
Tab.prototype.setLoadingFrameClass = function() {
	if (this.getProperty(LOADING_SCREEN_CLASS)) {
		setClass(this.loadingFrame, this.getProperty(LOADING_SCREEN_CLASS));
	}
	else {
		setClass(this.loadingFrame, "active");
	}
}


/**
 * Called by the tab content if the tab is done loading and the loading image
 * should be removed.
 */
Tab.prototype.doneLoading = function() {
	this.loading = false;
	this.height = "";
	if (!this.tabSection.isActive(this.name)) {
		// the tab is not the active tab any more
		return;
	}

	if (isDefined(this.loadingFrame)) {
		document.body.removeChild(this.loadingFrame);
		this.loadingFrame = undefined;
	}
	this.tabSection.activateIFrame(this);
	this.tabSection.notifyActivatedTab();
}

 
// activation/deactivation


/** 
 * Returns true if a tab is the active one, false otherwise.
 * @param name The name of the tab to check.
 */
TabSection.prototype.isActive = function(name) {
	if (this.activeTab != undefined && name == this.activeTab.name) {
		return true;
	}
	return false;
}


/**
 * Activates a tab.
 * @param name The name of the tab to activate.
 */
TabSection.prototype.activateTab = function(name) {
	if (isDefined(this.activeTab) && this.activeTab.name == name) {
		return;
	}
	
	var activeTab = this.activeTab;
	this.activeTab = this.getTab(name);
	if (isDefined(activeTab)) {
		activeTab.deactivate();
	}
	this.activeTab.activate();
	
	// we notify the tab that it has been deactivated here so that the users
	// perception of the tab switching is faster than it realy is
	if (isDefined(activeTab)) {
		this.notifyDeactivatedTab(activeTab.name);
	}
}


/**
 * Activates a tabs frame.
 * @param tab The tab to activate the frame for.
 */
TabSection.prototype.activateIFrame = function(tab) {
	if (tab.getProperty(ACTIVE_FRAME_CLASS) != undefined) {
		tab.frame.setAttribute("class", tab.getProperty(ACTIVE_FRAME_CLASS));
		tab.frame.setAttribute("className", tab.getProperty(ACTIVE_FRAME_CLASS));
	}
	// the height has been set to 0 on hiding of the frame
	// set the height back to the original height
	tab.frame.style.height = tab.height;
}


/**
 * Notify the tab that was activated.
 */
TabSection.prototype.notifyActivatedTab = function() {
 	// for this to work the tabs iframe need to have the same name as the tab
 	var tabActivated;
 	try {
 		tabActivated = window.frames[this.activeTab.name].tabActivated;
 	}
 	catch (e) {
 		// FF may throw a exception here, when the content of the page is not from the 
 		// same domain
 	}
	if (this.activeTab != undefined && window.frames[this.activeTab.name] && exists(tabActivated)) {
		window.frames[this.activeTab.name].tabActivated(this.activeTab.name);
	}
	this.fireActivatedEvent(this.activeTab.getName());
}


/**
 * Notify the tab that was deactivated.
 * @param name The name of the tab deactivated.
 */
TabSection.prototype.notifyDeactivatedTab = function(name) {
 	// for this to work the tabs iframe need to have the same name as the tab
 	var tabDeactivated;
 	try {
 		tabDeactivated = window.frames[name].tabDeactivated;
 	}
 	catch (e) {
 		// FF may throw a exception here, when the content of the page is not from the 
 		// same domain
 	}
 	if (name != undefined && window.frames[name] && exists(tabDeactivated)) {
 		window.frames[name].tabDeactivated(name);
 	}
 	this.fireDeactivatedEvent(name);
}


/**
 * Deactivates a tabs frame.
 * @param tab The tab to deactivate the frame for.
 */
TabSection.prototype.deactivateIFrame = function(tab) {
	if (tab.getProperty(INACTIVE_FRAME_CLASS) != undefined) {
		tab.frame.setAttribute("class", tab.getProperty(INACTIVE_FRAME_CLASS));
		tab.frame.setAttribute("className", tab.getProperty(INACTIVE_FRAME_CLASS));
	}
	
	// set the height of the iframe to 0
	tab.height = tab.frame.height;
	if (tab.height != undefined && (!isEmpty(tab.height))) {
		tab.frame.height = "";
	}
	else {
		tab.height = tab.frame.style.height;
	}
	
	tab.frame.style.height = "0px";
}


// css classes



/**
 * Changes the class of a tab. 
 * @param className The class to set. 
 * @param tabName The name of the tab to set the class for. 
 * @throws Error if className is undefined, null or empty.
 * @throws Error if tabName is undefined, null or empty.
 */
TabSection.prototype.changeClass = function(className, tabName) {
	assertNotEmpty(className, "Class to set for the tab is empty");
	assertNotEmpty(tabName, "TabName is undefined, null or empty");
	var tab = this.getTab(tabName);
	if (!exists(tab)) {
		// no tab with this name found
		return;
	}
	tab.properties[TAB_CLASS] = className;
	var tabCode = getElementWithAttribute(this.tabSection, tabName, "name", "div");
	setClass(tabCode, className);
}


/**
 * Set classes for the tab.
 * @param name The name of the tab to set the class for.
 * @param left The css class to set for the left part of the tab.
 * @param middle The css class to set for the right part of the tab.
 * @param right The css class to set for the right part of the tab.
 * @throws Error if name is undefined, empty or null.
 * @throws Error if left is undefined, null or empty.
 * @throws Error if middle is undefined, null or empty.
 * @throws Error if right is undefined, null or empty.
 */
TabSection.prototype.setClass = function(name, left, middle, right) {
	assertNotEmpty(name, "Name is undefined, empty or null: " + name);
	assertNotEmpty(left, "Left is undefined, empty or null: " + left);
	assertNotEmpty(middle, "Middle is undefined, empty or null: " + middle);
	assertNotEmpty(right, "Right is undefined, empty or null: " + right);

	var tab = this.getTab(name);
	var tabCode = getElementWithAttribute(this.tabSection, name, "name", "div");
	this.setLeftClass(name, left);
	this.setMiddleClass(name, middle);
	this.setRightClass(name, right);
}


/**
 * Sets the class of the left part of the window.
 * @param name The name of the tab to set the class for.
 * @param left The css class to set for the left part of the tab.
 * @throws Error if name is undefined, empty or null.
 * @throws Error if left is undefined, null or empty. 
 */
TabSection.prototype.setLeftClass = function(name, left) {
	assertNotEmpty(name, "Name is undefined, empty or null: " + name);
	assertNotEmpty(left, "Left is undefined, empty or null: " + left);
	
	var tabCode = getElementWithAttribute(this.tabSection, name, "name", "div");
	var leftPart = getElementWithAttribute(tabCode, "leftTab", "name", "div");
	leftPart.setAttribute("class", left);
	leftPart.setAttribute("className", left);
}



/**
 * Sets the class of the middle part of the window.
 * @param name The name of the tab to set the class for.
 * @param middle The css class to set for the lemiddleft part of the tab.
 * @throws Error if name is undefined, empty or null.
 * @throws Error if middle is undefined, null or empty.
 */
TabSection.prototype.setMiddleClass = function(name, middle) {
	assertNotEmpty(name, "Name is undefined, empty or null: " + name);
	assertNotEmpty(middle, "Middle is undefined, empty or null: " + middle);
	
	var tabCode = getElementWithAttribute(this.tabSection, name, "name", "div");
	var middlePart = getElementWithAttribute(tabCode, "middleTab", "name", "div");
	middlePart.setAttribute("class", middle);
	middlePart.setAttribute("className", middle);
}


/**
 * Sets the class of the right part of the window.
 * @param name The name of the tab to set the class for.
 * @param right The css class to set for the right part of the tab.
 * @throws Error if name is undefined, empty or null.
 * @throws Error if right is undefined, null or empty.
 */
TabSection.prototype.setRightClass = function(name, right) {
	assertNotEmpty(name, "Name is undefined, empty or null: " + name);
	assertNotEmpty(right, "Right is undefined, empty or null: " + right);
	
	var tabCode = getElementWithAttribute(this.tabSection, name, "name", "div");
	var rightPart = getElementWithAttribute(tabCode, "rightTab", "name", "div");
	rightPart.setAttribute("class", right);
	rightPart.setAttribute("className", right);
}



/**
 * Returns the mouse over class for the left part of the tab.
 * @param tab The tab to get the left mouse over class.
 */
TabSection.prototype.getLeftMouseOverClass = function(tab) {
	var leftClass;
	if (tab.getProperty(MOUSE_OVER_LEFT_CLASS_PROPERTY)) {
		leftClass = tab.getProperty(MOUSE_OVER_LEFT_CLASS_PROPERTY);
	}
	else {
		leftClass = "active_left";
	}
	return leftClass;
}


/**
 * Returns the mouse over class for the middle part of the tab.
 * @param tab The tab to get the middle mouse over class.
 */
TabSection.prototype.getMiddleMouseOverClass = function(tab) {
	var middleClass;
	if(this.isActive(tab.name) && tab.getProperty(MOUSE_OVER_MIDDLE_CLASS_ACTIVE_PROPERTY)) {
		middleClass = tab.getProperty(MOUSE_OVER_MIDDLE_CLASS_ACTIVE_PROPERTY);
	}
	else if (!this.isActive(tab.name) && tab.getProperty(MOUSE_OVER_MIDDLE_CLASS_PROPERTY)) {
		middleClass = tab.getProperty(MOUSE_OVER_MIDDLE_CLASS_PROPERTY);
	}
	else {
		middleClass = "active_bg";
	}
	return middleClass;
}


/**
 * Gets the right class to set for the tab.
 * @param tab The tab to get the right class.
 */
TabSection.prototype.getRightClass = function(tab) {
	if (tab.mouseover) {
		return this.getRightMouseOverClass(tab);
	}
	else if (this.isActive(tab.name)) {
		return "active_right";
	}
	else {
		return "inactive_right";
	}
}

/**
 * Returns the mouse over class for the right part of the tab.
 * @param tab The tab to get the right mouse over class.
 */
TabSection.prototype.getRightMouseOverClass = function(tab) {
	var rightClass;
	if (tab.getProperty(MOUSE_OVER_RIGHT_CLASS_PROPERTY)) {
		rightClass = tab.getProperty(MOUSE_OVER_RIGHT_CLASS_PROPERTY);
	}
	else {
		rightClass = "active_right";
	}
	return rightClass;
}


/**
 * Returns the class to set for the tab container. The tab container contains 
 * all the tabs. A class to use for this can be specified by the TABCONTAINER_CLASS property.
 * If the property is not given a default class is used.
 */
TabSection.prototype.getTabContainerClass = function() {
	var containerClass = this.getProperty(TABCONTAINER_CLASS);
	if (containerClass == undefined) {
		containerClass = "tabs";
	}
	return containerClass;
}


/**
 * Returns the class to set for a tab. A class to set for the tab can be 
 * specified by the TAB_CLASS. If the propertys is not given a 
 * default class is used.
 * @param tab The tab to get the class for.
 */
TabSection.prototype.getTabClass = function(tab) {
	var tabClass = tab.getProperty(TAB_CLASS);
	if (tabClass == undefined) {
		tabClass = "tab";
	}
	return tabClass;
}


// event handling


/**
 * Handling the onmouse over of the tabs. if 
 * MOUSE_OVER_LEFT_CLASS_PROPERTY, MOUSE_OVER_MIDDLE_CLASS_PROPERTY, MOUSE_OVER_RIGHT_CLASS_PROPERTY
 * properties are set this classes are set for the tab, if not the class for
 * active is set for the tab.
 * @param name The name of the tab.
 */
TabSection.prototype.mouseOver = function(name) {
	var tab = this.getTab(name);	
	tab.mouseover = true;
	var leftClass = this.getLeftMouseOverClass(tab);
	var middleClass = this.getMiddleMouseOverClass(tab);
	var rightClass = this.getRightClass(tab);
	
	this.setClass(name, leftClass, middleClass, rightClass);
}


/**
 * Handling the mouse out of the tabs, the classes are set back to the normal classes.
 */
TabSection.prototype.mouseOut = function(name) {
	if (this.isActive(name)) {
		this.setClass(name, "active_left", "active_bg", "active_right");
	}
	else {
		this.setClass(name, "inactive_left", "inactive_bg", "inactive_right");
	}
	
	var tab = this.getTab(name);
	tab.mouseover = false;
}
 
 
// html code


/**
 * Paint the tab section.
 */
TabSection.prototype.paint = function() {
	// create the tab section
	this.tabSection = this.createTabSection();

	// create the tab container
	this.tabContainer = this.createTabContainer();
	this.tabSection.appendChild(this.tabContainer);
	
	// create code for all the tabs
	for (var i = 0; i < this.tabs.length; i++) {
		this.tabContainer.appendChild(this.createTab(this.tabs[i]));
	}
	
	// create the tabs bar
	if (this.getProperty(TABS_BAR)) {
		this.tabSection.appendChild(this.createTabsBar());
	}
	
	this.appendTabSection();
}


/**
 * Appends the tab section to the document or hte element specified
 * by TAB_SECTION_PARENT_ELEMENT. 
 */
TabSection.prototype.appendTabSection = function() {
	var parent = this.getProperty(TAB_SECTION_PARENT_ELEMENT);
	if (parent != undefined) {
		parent.appendChild(this.tabSection);
		return;
	}
	
	// append the tab section as the first child of the document
	if (document.body.firstChild) {
		document.body.insertBefore(this.tabSection, document.body.firstChild);
	}
	else {
		document.body.appendChild(this.tabSection);
	}
}

 
/**
 * Creates the html code for the tabs bar.
 * @returns the created code.
 */
TabSection.prototype.createTabsBar = function() {
	var tabsBar = document.createElement("div");
	tabsBar.setAttribute("name", "tabsBar");
	tabsBar.setAttribute("class", "tabs_bar");
	tabsBar.setAttribute("className", "tabs_bar");
	return tabsBar;
}


/**
 * Creates the html code for a tab.
 * @param tab The tab to create the code for.
 * @param container 
 */
TabSection.prototype.createTab = function(tab) {
	var tabCode = this.createTabCode(tab);
	
	if (tab.getProperty(TAB_FRAME) != undefined) {
		tab.frame = tab.getProperty(TAB_FRAME);
	}		
	else {
		this.createIframe(tab);
	}
	
	// to begin with all tabs are inactive
	this.deactivateIFrame(tab);
	
	return tabCode;
}


/**
 * Appends a tabs frame to the document.
 * @param tab The tab to append the frame for.
 */
TabSection.prototype.appendIframe = function(tab) {
	document.body.appendChild(tab.frame);	
}


/**
 * Create the iframe for a tab.
 * @param tab The tab to create the iframe for.
 */
TabSection.prototype.createIframe = function (tab) {
	tab.frame = document.createElement("iframe");
	tab.frame.setAttribute("name", tab.name);
	tab.frame.setAttribute("value", "test");
	tab.frame.name = tab.name;
	tab.frame.setAttribute("id", tab.name);
	
	this.addFrameScrollbar(tab);
	this.setFrameClass(tab);
	
	if (!tab.getProperty(LOAD_ONCLICK)) {
		this.appendIframe(tab);
		this.loadIframeContent(tab);
	}
	// this can be overridden in css but we need to set it here
	// to avoid problems with IE
	tab.frame.setAttribute("frameBorder", "0");
}


/**
 * Set the scroll bar for a frame.
 * @param tab The tab to set the scroll bars for.
 */
TabSection.prototype.addFrameScrollbar = function(tab) {
	if ("on" == tab.getProperty(FRAME_SCROLLBAR)) {
		tab.frame.setAttribute("scrolling", "yes");
	}
	else if ("off" == tab.getProperty(FRAME_SCROLLBAR)) {
		tab.frame.setAttribute("scrolling", "no");
	}
}


/**
 * Loads the content of the iframe.
 * @param tab The tab to load the content for.
 */
TabSection.prototype.loadIframeContent = function(tab) {
	if (tab.getProperty(LOADING_IMAGE)) {
		tab.showLoading();
	}
	tab.frame.setAttribute("src", tab.url);
	tab.loaded = true;
}


/**
 * Sets the class for a tabs frame.
 * @param tab The tab to set the frame properties for.
 */
TabSection.prototype.setFrameClass = function(tab) {
	if (tab.getProperty(FRAME_CLASS)) {
		tab.frame.setAttribute("class", tab.getProperty(FRAME_CLASS));
		tab.frame.setAttribute("className", tab.getProperty(FRAME_CLASS));
	}
}

 
/**
 * Creates the html code for a tab.
 * @param tab The tab to create the code for.
 */
TabSection.prototype.createTabCode = function(tab) {
	var tabDiv = document.createElement("div");
	var tabClass = this.getTabClass(tab);
	tabDiv.setAttribute("name", tab.name);
	tabDiv.setAttribute("class", tabClass);
	tabDiv.setAttribute("className", tabClass);

	tabDiv.appendChild(this.createLeft());
	tabDiv.appendChild(this.createMiddle(tab));
	tabDiv.appendChild(this.createRight(tab));
	
	var tabSection = this;
	if (tab.getProperty(MOUSE_OVER_PROPERTY)) {
		tabDiv.onmouseover = function() {
			tabSection.mouseOver(tab.name);
		}
		
		tabDiv.onmouseout = function() {
			tabSection.mouseOut(tab.name);
		}
	}
	
	tabDiv.onclick = function() {
		tabSection.activateTab(tab.name);
	}
	
	return tabDiv;
}


/**
 * Creates and returns the html code for hte left part of the tab.
 */
TabSection.prototype.createLeft = function() {
	var left = document.createElement("div");
	left.setAttribute("name", "leftTab");
	left.setAttribute("class", "inactive_left");
	left.setAttribute("className", "inactive_left");
	return left;
}



/**
 * Creates and returns the html code for hte right part of the tab.
 * @param tab The tab to create the code for.
 */
TabSection.prototype.createRight = function(tab) {
	var right = document.createElement("div");
	right.setAttribute("name", "rightTab");
	right.setAttribute("class", "inactive_right");
	right.setAttribute("className", "inactive_right");	
	
	return right;
}


/**
 * Creates and returns the html code for the middle part of the tab.
 * @param tab The tab to create the code for.
 */
TabSection.prototype.createMiddle = function(tab) {
	var middle = document.createElement("div");
	middle.setAttribute("align", "middle");
	middle.setAttribute("name", "middleTab");
	middle.setAttribute("class", "inactive_bg");
	middle.setAttribute("className", "inactive_bg");
	//middle.setAttribute("style", "margin-left: auto; margin-right: auto; text-align: center");

	// create the table and cells to keep the icon and text in
	var middleTable = document.createElement("table");
	middle.appendChild(middleTable);
	var tbody = document.createElement("tbody");
	middleTable.appendChild(tbody);
	
	// create a row for the icon if a icon class is specified
	if (isDefined(tab.getProperty(ICON_CLASS))) {
		var imageTr = document.createElement("tr");
		tbody.appendChild(imageTr);
		var imageTd = document.createElement("td");
		imageTr.appendChild(imageTd);		
		
		// create the div tag to keep the table in
		var middleDiv = document.createElement("div");
		setClass(middleDiv, tab.getProperty(ICON_CLASS));	
		imageTd.appendChild(middleDiv);
	}

	var textTr = document.createElement("tr");	
	tbody.appendChild(textTr);
	var textTd = document.createElement("td");
	textTr.appendChild(textTd);
	textTd.appendChild(document.createTextNode(tab.text))
	
	return middle;
}


/**
 * Returns the code for the tab section.
 * The class name set for the tab section is "tabSection" if TAB_SECTION_CLASS property is not set.
 */
TabSection.prototype.createTabSection = function() {
	var tabSection = document.createElement("div");
	tabSection.setAttribute("name", "tabSection");
	
	if (this.getProperty(TAB_SECTION_CLASS)) {
		tabSection.setAttribute("class", this.getProperty(TAB_SECTION_CLASS));
		tabSection.setAttribute("className", this.getProperty(TAB_SECTION_CLASS));
	}
	else {
		tabSection.setAttribute("class", "tabSection");
		tabSection.setAttribute("className", "tabSection");
	}
	return tabSection;
} 
 
 
/**
 * Create the container containing all the tabs.
 */
TabSection.prototype.createTabContainer = function() {
	var tabContainer = document.createElement("div");
	tabContainer.setAttribute("name", "tabContainer");
	
	var containerClass = this.getTabContainerClass();
	tabContainer.setAttribute("class", containerClass);
	tabContainer.setAttribute("className", containerClass);
	return tabContainer;
}
 

/**
 * Returns a property of the tab section, undefined if the property array is not defined
 * for the object the function is used on, or the property is not defined.
 * @param property The property to return.
 * @throws Error if the property is undefined, null or empty.
 */
function GetProperty(property) {
	assertNotEmpty(property, "Property to get is undefined, null or empty: " + property);
	
	if (this.properties == undefined) {
		return undefined;
	}
	
	for (prop in this.properties) {
		if (property == prop) {
			return this.properties[prop]
		}
	}
	
	return undefined;
}


// events


/**
 * Fire activated event.
 * @param tab The name of the tab that has been activated. 
 */
TabSection.prototype.fireActivatedEvent = function(tab) {
	if (!eventsAvailable()) {
		return;
	}
	
	getEventServer().notifyListeners(new Event("tabActivated", tab));
}


/**
 * Fires a deactivated event.
 * @param tab The name of the tab that has been deactivated. 
 */
TabSection.prototype.fireDeactivatedEvent = function(tab) {
	if (!eventsAvailable()) {
		return;
	}
	
	getEventServer().notifyListeners(new Event("tabDeactivated", tab));
}


/**
 * Returns true if a EventServer is available.
 */
function eventsAvailable() {
	try {
		if (isDefined(getEventServer)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	try {
		if (isDefined(Event)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	return true;
}
