
/**
 * Toggels the given sub menus on or off. If they are shown they are hidden
 * and if the currently are not shown they are hidden.
 * @param menuName The name of the tree menu to show a sub menu in.
 * @param The menu items to show. This can be any number of sub menu names.
 */
function toggleSubMenu(menuName) {
	var len = arguments.length;
	for (var i = 1; i < len; i++) {
		var subMenu = getSubMenu(menuName, arguments[i]);
		if ("inactive" == subMenu.getAttribute("className") || "inactive" == subMenu.getAttribute("class")) {
			activateSubMenu(subMenu);
	  	}
	  	else {
		  	deactivateSubMenu(subMenu);
	  	}
	}
}


/**
 * Hides all the sub menus in a tree menu.
 * @param menuName The name of the tree menu to hide sub menus in. 
 */
function hideSubMenus(menuName) {
	var menu = getTreeMenu(menuName);
	var menuItems = menu.getElementsByTagName("tr");
	
	var len = menuItems.length;
	for (var i = 0; i < len; i++) {
		if ("submenu" == menuItems[i].getAttribute("id")) {
			deactivateSubMenu(menuItems[i]);
		}
	}
}


/**
 * Deactivates a subMenu.
 * @param The tr tag containing the sub menu.
 */
function deactivateSubMenu(subMenu) {
	subMenu.setAttribute("className", "inactive");
  	subMenu.setAttribute("class", "inactive");
}


/**
 * Activates a subMenu.
 * @param The tr tag containing the sub menu.
 */
function activateSubMenu(subMenu) {
	subMenu.setAttribute("className", "active");
  	subMenu.setAttribute("class", "active");
}


/**
 * Get a tree menu with a given name.
 * @param name The name of the menu to get.
 * @return The tree menu if found, null otherwise.
 */
function getTreeMenu(name) {
	var tables = document.getElementsByTagName("table");
	var len = tables.length;
	for (var k = 0; k < len; k++) {
		if (name == tables[k].getAttribute("name")) {
			return tables[k];
		}
	}
	return null;
}


/**
 * Get a sub menu.
 * @param treeMenuName The name of the tree menu the sub menu is in
 * @param subMenuName The name of the sub menu to get.
 * @return The sub menu, null if not found in the tree menu.
 */
function getSubMenu(treeMenuName, subMenuName) {
	var treeMenu = getTreeMenu(treeMenuName);
	var menuItems = treeMenu.getElementsByTagName("tr");
	
	var len = menuItems.length;
	for (var j = 0; j < len; j++) {
		if (subMenuName == menuItems[j].getAttribute("name")) {
			return menuItems[j];
		}
	}
}