
/**
 * All requests get their forward ip parameter set to this ip address. 
 */
var forwardIp = null;


/**
 * Creates a new XML HTTP request object browser independent.<b> 
 */
function createXmlHttpRequest() {
	var xml = false;
	try {
		xml = new ActiveXObject("Msxml2.XMLHTTP");
	}
	catch (error) {
		try {
			xml = new ActiveXObject("Microsoft.XMLHTTP");
		}
		catch (error) {
			try {
				xml = new XMLHttpRequest();
			}
			catch (error) {
				xml = false;
			}
		}
	}
	return xml;
}

// create the browser-universal http request object
HttpRequest = 
function () {
	var xmlhttp = null;
	try {
		xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
	}
	catch(e) {
	    try {
	    	xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
	    }
	    catch (e_) {
	    }
	}
	if (!xmlhttp && typeof XMLHttpRequest != 'undefined') {
	    try {
	    	xmlhttp = new XMLHttpRequest();
	    }
	    catch (e) {
	    	xmlhttp = false;
	    }
	}
	return xmlhttp;
}

// create the dom parser for browsers not supporting it
var req = null;
if (typeof DOMParser == "undefined") {
	DOMParser = function () {}
   	DOMParser.prototype.parseFromString = function (str, contentType) {
    	if (typeof ActiveXObject != "undefined") {
        	var d = new ActiveXObject("MSXML.DomDocument");
         	d.loadXML(str);
         	return d;
      	}
      	else {
      		if (typeof XMLHttpRequest != "undefined") {
         		req = new XMLHttpRequest;
         		req.open("GET", "data:" + (contentType || "application/xml") +
                	";charset=utf-8," + encodeURIComponent(str), true);
         		if (req.overrideMimeType) {
		            req.overrideMimeType(contentType);
        		}
		      	req.send(null);
         		return req.responseXML;
      		}
   		}
   	}
}


/**
 * Loads the XML data from the specified URL and, when it is fully loaded,
 * passes that document and the URL to the specified handler function.
 * @param url The url from which to fetch the xml data.
 * @param handler The handler function to call with the xml data.
 * @param optional argument object, The object the to call a method on, if this argument 
 * is given the handler must be a string containing the methods name.
 * @param optional argument errorHandler if this is set this method will be called when a error 
 * occurs during loading of data. When object is set the function is called as a method of the object. 
 */
function loadXML(url, handler, object, errorHandler) {
	var request = new HttpRequest();
   	request.open("GET", setForwardIpParameter(url), true);
   	// prevents the browser from using the cached entry
   	request.setRequestHeader("Cache-Control", "no-cache");
	var len = arguments.length;
	if (len == 3) {
		var object = arguments[2];
	}
	if (request.overrideMimeType) {
		// if the server closes the connection without returning any response
		// firefox still tries to parse a xml document, since there is no response you get
		// a exception in the error console, this overrides the mime type of the response send 
		// by the server, need to parse the response as a xml document 
		request.overrideMimeType('text/plain');
	}
	
	
   	request.onreadystatechange = function() {
   			if (request.readyState == 4 && request.responseText) {
   				var response = (new DOMParser()).parseFromString(request.responseText, "text/xml");
				if (len == 3 && isDefined(handler) && object[handler]) {
					object[handler](response, url);
				}
				else if (isDefined(handler)) {
	   				handler(response, url);
	   			}
				delete request.onreadystatechange;
	   			request = null;
	   			response = null;
			 }
			 
			 // a error occured retrieving the data
			 else if (request.readyState == 4) {
				 // deallocate
			 	delete request.onreadystatechange;
	   			request = null;
	   			response = null;
			 	
			 	// notify about the error occured
			 	if (exists(object) && object[errorHandler]) {
					object[errorHandler]();
				}
				else if (exists(errorHandler)) {
	   				errorHandler();
	   			}
			 }
		};
	request.send(null);
}


/**
 * Sends a get request and forwards the response to a named method.
 * @param url The get request to send.
 * @param method The method to forward the result to.
 * @param optional argument object, The object the to call a method on, if this argument 
 * is given the method must be a string containing the methods name.
 */
function getResponse(url, method, object) {
	var request = new HttpRequest();
   	request.open("GET", setForwardIpParameter(url), true);
   	var len = arguments.length;
	if (len == 3) {
		var object = arguments[2];
	}
   	request.onreadystatechange =    
   		function () {
    		if (request.readyState == 4) {
    			var result = request.responseText;
    			if (len == 3 && isDefined(method) && object[method]) {
					object[method](result, url);
				}
				else {
					method(result);		    
				}
		    }
        };
  	request.send(null);
}


/**
 * Posts a request. 
 * @param url The url from which to fetch the xml data.
 * @param content The content to post. 
 * @param handler The handler function to call with the xml data.
 * @param object, The object the to call a method on, if this argument 
 * is given the handler must be a string containing the methods name.
 * @param errorHandler if this is set this method will be called when a error 
 * occurs during loading of data. When object is set the function is called as a method of the object. 
 */
function postRequest(url, content, handler, object, errorHandler) {
	var request = new HttpRequest();
   	request.open("POST", url, true);
   	// prevents the browser from using the cached entry
   	request.setRequestHeader("Cache-Control", "no-cache");
	var len = arguments.length;
	if (len == 4) {
		var object = arguments[3];
	}

   	request.onreadystatechange = 
   		function() {
   			if (request.readyState == 4 && request.responseText) {
   				var response = (new DOMParser()).parseFromString(request.responseText, "text/xml");
				if (len == 4 && isDefined(handler) && object[handler]) {
					object[handler](response, url);
				}
				else if (isDefined(handler)) {
	   				handler(response, url);
	   			}
				delete request.onreadystatechange;
	   			request = null;
	   			response = null;
			 }
			 
			 // a error occured retrieving the data
			 else if (request.readyState == 4) {
				 // deallocate
			 	delete request.onreadystatechange;
	   			request = null;
	   			response = null;
			 	
			 	// notify about the error occured
			 	if (isDefined(object) && object[errorHandler]) {
					object[errorHandler]();
				}
				else if (isDefined(errorHandler)) {
	   				errorHandler();
	   			}
			 }	
		};
	request.send(setForwardIpParameter(content));
}


/**
 * Set the forward ip parameter. All requests sendt through this file will get this
 * ip address set as their forward ip parameter. 
 * @param ip A string containing the ip to set.
 */
function setForwardIp(ip) {
	forwardIp = ip;
}


/**
 * Gets the forward ip address set. 
 * @return The forward ip address set, null if none set.  
 */
function getForwardIp() {
	return forwardIp;
}
 
 
// implementation


/**
 * Adds the forward ip parameter to the given request if any set. 
 * @param request The request to add the parameter to. 
 * @return The request with the forward ip parameter set if neccessary. 
 */
function setForwardIpParameter(request) {
	if (exists(forwardIp) && !isEmpty(forwardIp) && forwardIp != getServerAddress()) {
		return request + "&forwardIp=" + forwardIp + "&";
	}
	return request;
}