/**
 * This class can show and create a loading animation. 
 */


/**
 * The css class set for the loading animation. 
 */
LoadingAnimation.CLASS = "loadingScreen";


/**
 * The z index to use for the loading animation.
 */
LoadingAnimation.ZINDEX = 999999;


/**
 * The source url of the loading image used. 
 */
LoadingAnimation.LOADING_IMAGE_SRC = "images/loading_big_blue.gif";


// construction 


/**
 * Constructs a loading animation. 
 * @param parent The parent node, the animation is appended to this node.
 * @param width The width to set for this animation as a string (100%, 100px).
 * @param height The height to set for this animation as a string (100%, 100px);
 * @param text The text to show in the animation. 
 * @throws Error if parent is undefined, null or not a html node. 
 * @thorws Error if height is undefined, null or empty.
 * @throws Error if width is undefiend, null or empty. 
 */
function LoadingAnimation(parent, height, width, text) {
	assertNotEmpty(width, "The width to set is undefined null or empty");
	assertNotEmpty(height, "The height to set is undefined null or empty");
	assertExists(parent, "The parent node is undefined or null");
	if (!exists(parent.nodeType)) {
		throw new Error("Parent node is not a html node");
	}
	this.parent = parent;
	this.width = width;
	this.height = height;
	this.text = text;
	
		
	// the html code of this loading animation
	this.htmlCode = null;
	
	// the html class to set for the animation when it is created
	this.className = LoadingAnimation.CLASS;
	
	// the src of the loading image used
	this.loadingImageSrc = LoadingAnimation.LOADING_IMAGE_SRC;
}


// services


/**
 * Shows this loading animation. 
 */
LoadingAnimation.prototype.show = function() {
	if (!exists(this.htmlCode)) {
		this.createHtmlCode();
	}
	setClass(this.htmlCode, this.className);
}


/**
 * Hides the loading animation. 
 */
LoadingAnimation.prototype.hide = function() {
	if (exists(this.htmlCode)) {
		setClass(this.htmlCode, "inactive");
	}
}


/**
 * Returns the html code of this loading animation.
 */
LoadingAnimation.prototype.getHtmlCode = function(){
	if (!exists(this.htmlCode)) {
		this.createHtmlCode();
	}
	return this.htmlCode;
}


/**
 * Sets the class to use for the loading animation. 
 * @param className The class to set when the animation is created. If
 * this is not set the default class will be set.   
 * @throws Error if className is undefined null or empty. 
 */
LoadingAnimation.prototype.setClass = function(className) {
	assertNotEmpty(className, "The class name to set is undefined null or empty");
	this.className = className;
}


/**
 * Sets the url for the loading image to use, used to overide the default image source used. 
 * @param imageSrc The url to set as the source for the loading animation. 
 * @throws Error if imageSrc is undefined null or empty. 
 */
LoadingAnimation.prototype.setLoadingImageSrc = function(imageSrc) {
	assertNotEmpty(imageSrc, "The image source url to set is undefined null or empty");
	this.loadingImageSrc = imageSrc;
}


// implementation


/**
 * Creates the html code for this animation. 
 */
LoadingAnimation.prototype.createHtmlCode = function() {
	this.htmlCode = document.createElement("div");
	//setClass(this.htmlCode, this.className);
	this.htmlCode.style.zIndex = LoadingAnimation.ZINDEX;
	this.htmlCode.style.overflow = "hidden";
	this.htmlCode.style.width = this.width;
	this.htmlCode.style.height = this.height;
	
	var loadingTable = document.createElement("table");
	loadingTable.style.height = "100%";
	loadingTable.style.width = "100%";
	var tableBody = document.createElement("tbody");
	loadingTable.appendChild(tableBody);
	var row = loadingTable.insertRow(0);
	var outerCell = row.insertCell(0);
	outerCell.setAttribute("align", "center");
	
	var table = document.createElement("table");
	var body = document.createElement("tbody");
	table.appendChild(body);
	var topRow = body.insertRow(0);
	var cell = topRow.insertCell(0);
	var loadingImage = document.createElement("img");
	loadingImage.setAttribute("name", "loadingImage");
	loadingImage.src = this.loadingImageSrc;
	cell.appendChild(loadingImage);
	cell.setAttribute("align", "center");
	
	var bottomRow = body.insertRow(1);
	cell = bottomRow.insertCell(0);
	cell.setAttribute("align", "center");	
	cell.style.paddingTop = "20px";
	if (!isEmpty(this.text)) {
		cell.appendChild(document.createTextNode(this.text));
	}
	
	outerCell.appendChild(table);
	this.htmlCode.appendChild(loadingTable);
	this.parent.appendChild(this.htmlCode);	
}