/**
 * Compares this string to another ignoring case considerations.
 * @param compare The string to compare this string to.
 * @param return true if the strings are equals, false otherwise.
 */
function equalsIgnoreCase(compare) {
	if(!isDefined(compare)) {
		return false;
	}	
	
	var strToSearch = this.toLowerCase();
	compare = compare.toLowerCase();
	if (strToSearch == compare) {
		return true;	
	}
	return false;
}


// Add the equalsIgnoreCase function to the javascrip string objects
String.prototype.equalsIgnoreCase = equalsIgnoreCase;


/**
 * Gets a value for a parameter in a request.
 * @param request A string containing the request.
 * @param parameter The parameter to get the value for.
 * @param delimiter The character between key and value pairs in the request.
 * @return The value of the parameter, a empty string if not found.
 */
function getParameterValue(request, parameter, delimiter) {
	var value = "";
	var start = 0;
	
	var end = 0;
	while (true) {
		if (start >= request.length) {
			break;
		}
		
		end = request.indexOf(delimiter, start);
		if (end == -1) {
			end = request.length;
		}
		var keyValuePair = request.substring(start, end);
		if (keyValuePair.indexOf("/?") >= 0) {
			// the first key value pair might be somehow special
			var parameterStart = request.indexOf("/?") + "/?".length;
			keyValuePair = keyValuePair.substring(parameterStart, keyValuePair.length);
		}
		var valueStart = keyValuePair.indexOf("=");
		if (keyValuePair.indexOf(parameter) == 0 && valueStart == parameter.length) {
			// the parameter has been found get the value
			value = keyValuePair.substring(valueStart + "=".length, keyValuePair.length);
			break;				
		}
		
		if (end == request.length) {
			break;
		}
		start = end + delimiter.length;
	}
	return value;
}	


/**
 * Replaces a value for parameter in a request. 
 * @param request The string containing the request.
 * @param parameter The name of the parameter to replace the value of.
 * @param value The value to insert for the parameter.
 * @param delimiter The character between key and value pairs.
 * @returns The string with the value for the parameter replaced or the original
 * string if paramter is not found.
 */
function replaceParameterValue(request, parameter, value, delimiter) {
	var startIndex = request.indexOf(delimiter + parameter + "=");
	if (startIndex == -1) {
		return request;
	}

	var endIndex = request.indexOf(delimiter, startIndex + 1);
	var endPart = "";
	if (endIndex > -1) {
		endPart = request.substring(endIndex, request.length);
	}
	return request.substring(0, startIndex) + delimiter + parameter + "=" + (value + "") + endPart;
}


/**
 * Returns true if a request has a parameter, false otherwise.
 * @param request The request to check.
 * @param parameter The parameter to check for.
 * @param delimiter The delimiter to separate the parameters.
 */
function hasParameter(request, parameter, delimiter) {
	if (!isDefined(request)) {
		return false;
	}
	
	var startIndex = request.indexOf(delimiter + parameter + "=");
	if (startIndex > -1) {
		return true;
	}
	return false;
}


/**
 * Replaces a part of a string.
 * @param string The string to replace a part in.
 * @param replaceString The part of the string to replace.
 * @param replaceWith The string to replace with.
 * @return The string with replaceString replaced with replaceWith.
 * @throws Error if string is undefined, null or empty.
 * @throws Error if replace is undefined, null or empty.
 * @throws Error if replaceWith is undefined, null or empty.
 */
function replace(string, replaceString, replaceWith) {
	assertDefined(string, "String is undefined, null or empty");
	assertDefined(replaceString, "ReplaceString is undefined, null or empty");
	assertDefined(replace, "ReplaceWith is undefined, null or empty");

	var startIndex = string.indexOf(replaceString);
	if (startIndex == -1) {
		return string;
	}
	var oldString = string;
	string = oldString.substring(0, startIndex);
	string += replaceWith;
	string += oldString.substring(startIndex + replaceString.length);
	return string;
}



/**
 * Trim all the tab (/t) in a string.
 * @param string The string to trim tabs in. 
 * @return The string without any tabs.
 * @throws Error if string is undefined, null or empty.
 */
function trimTabs(string) {
	assertNotEmpty(string, "The string to trim tabs in is undefined, null or empty");
	var newString = "";
	var index = 0;
	while (true) {
		if (index == string.length) {
			break;
		}
		if (string.charAt(index) != "\t") {
			newString += string.charAt(index);
		}
		index++;
	} 
	return newString;
}


/**
 * Trims all newlines (/n) in a string.
 * @param string The string to trim all newlines in.
 * @return The string without any new lines. 
 * @throws Error if string is undefined, null or empty.
 */
function trimNewLines(string) {
	assertNotEmpty(string, "The string to trim tabs in is undefined, null or empty");
	var newString = "";
	var index = 0;
	while (true) {
		if (index == string.length) {
			break;
		}
		if (string.charAt(index) != "\n") {
			newString += string.charAt(index);
		}
		index++;
	} 
	return newString;
}

