/**
 * This class parses a delimited string into tokens. 
 * If two delimiters without space is in the string, an empty token is counted
 * between those delimiters.
 * String tokens = ","; 	// will count as two empty tokens.
 * String tokens = ",,";	// will count as three empty tokens.
 * String tokens = "";		// will count as zero tokens.
 * String tokens = "one";	// will count as one token.
 */

 
// construction

 
/**
 * Constructs the tokenizer. 
 * @param string The string to parse. 
 * @param delimiter The delimiter to user. 
 * @param throws Error if string is undefined, null or empty.
 * @param throws Error if delimiter is undefined null or empty.
 */
function StringTokenizer(string, delimiter) {
	assertNotEmpty(string, "The string to use in the tokenizer is undefined, null or empty: " + string);
	assertNotEmpty(delimiter, "The delimiter to user in the tokenizer is undefined, null or empty: " + delimiter);
	
	this.string = string;
	this.delimiter = delimiter;
	this.currentIndex = 0;
}


// services


/**
 * Returns true if the tokenizer contains more tokens. 
 */
StringTokenizer.prototype.hasMoreTokens = function() {
	var tokenEndIndex = this.string.indexOf(this.delimiter, this.currentIndex);
	if (tokenEndIndex == -1 && this.currentIndex > this.string.length) {
		return false;
	}
	return true;
}


/**
 * Gets the next token. 
 * @return the next token from this tokenizer. 
 * @throws Error if there are no more tokens.
 */
StringTokenizer.prototype.nextToken = function() {
	var tokenEndIndex = this.string.indexOf(this.delimiter, this.currentIndex);
	if (tokenEndIndex == -1 && this.currentIndex > this.string.length) {
		// there are no more tokens
		throw new Error("No such element");
	}
	else if (tokenEndIndex == -1) {
		// the last token has no delimiter
		tokenEndIndex = this.string.length;
	}
	
	var token = this.string.substring(this.currentIndex, tokenEndIndex);
	this.currentIndex = tokenEndIndex + this.delimiter.length;
	return token;
}


/**
 * Count the number of tokens in the tokenizer. 
 * @return the number of tokens left in the tokenizer. 
 */
StringTokenizer.prototype.countTokens = function() {
	var start = this.currentIndex;
	var count = 0;
	while (true) {
		var tokenEndIndex = this.string.indexOf(this.delimiter, start);
		if (tokenEndIndex == -1 && start > this.string.length) {
			// there are no more tokens
			break;
		}
		else if (tokenEndIndex == -1) {
			// the last token has no delimiter
			tokenEndIndex = this.string.length;
		}
		
		var token = this.string.substring(start, tokenEndIndex);
		start = tokenEndIndex + this.delimiter.length;
		count ++;
	}
	
	return count;
}