/**
 * Create a subclass.
 * @param superInstance A instance of the super class.
 * @param subInstance A instance of the sub class.
 * @return subInstance as a subclass of superInstance.
 * @throws Error if superInstance is undefined or null.
 * @throws Error if subInstance is undefined or null.
 */
function createSubClass(superInstance, subInstance) {
	assertDefined(superInstance, "Instance of super class is undefined or null");
	assertDefined(subInstance, "Instance of sub class is undefined or null");
	
	// save the old properties of the subclass
	var subProperties = new Array();
	for (property in subInstance) {
		subProperties[property] = subInstance[property]
	}
	
	// add all the properties from the super class to the sub class
	for (property in superInstance) {
		subInstance[property] = superInstance[property];
	}
	
	// add all the original properties of the subInstance.prototype
	// back to subInstance, Properties from superInstance that have the same
	// name as properties in sub are added as "super" + property name
	for (property in subProperties) {
		if (containsElement(subInstance, property)) {
			// the sub class overrides this property, prefix it with 'super' in sub class
			var superProperty = subInstance[property];
			subInstance["super" + property] = superProperty;
		}
		subInstance[property] = subProperties[property];
	}
	
	subInstance['callSuper'] = callSuper;
	return subInstance;
}



/**
 * Call the super function.
 * @param functionName The name of the function to call the super method for.
 * @param argumentsToSuper optional argument A array of arguments to pass to the super method. 
 * @return Whatever the super method returns.
 */
function callSuper(functionName, argumentsToSuper) {
	if (!this["super" + functionName]) {
		return;
	}
	
	if (isDefined(argumentsToSuper) && !isNull(argumentsToSuper)) {
		return this["super" + functionName].apply(this, argumentsToSuper);
	}
	else {
		return this["super" + functionName]();
	}
}