/**
 * This class subclasses the resize helper to resize windows. 
 */

 
// construction


/**
 * Factory method to construct a window resize helper.
 * @param window The window to drag. 
 * @throws Error if window is undefined or null.
 */
function createWindowResizeHelper(window) {
	assertExists(window, "Window is undefined or null");
	
	var superInstance = new ResizeHelper(window.getHtmlCode()); 
	var subInstance = new WindowResizeHelper(window);
	subInstance = createSubClass(superInstance, subInstance);
	return subInstance;	
}



/**
 * Constructs a window resize helper.  
 * @param window The window to be resized.
 * @throws Error if window is undefined or null.
 */
function WindowResizeHelper(window) {
	assertExists(window, "Window is undefined or null");
	
	
	/**
	 * The window to be resized.
	*/ 
	this.window = window;
	
	
	/**
	 * The function used to listen to mouse up events. 
	 */
	this.mouseUpListener = this.createMouseUpListener();
}


// resizing


/**
 * This is the handler that captures the final mouse up event
 * at the end of a resize.
 */
WindowResizeHelper.prototype.upHandler = function(event) {
	var resizedX = this.resizedX;
	var resizedY = this.resizedY;

	var argumentsToSuper = new Array();
	argumentsToSuper[0] = event;
	this.callSuper("upHandler", argumentsToSuper);
	
	if(this.resized) {
		this.window.resizeElements(resizedY, resizedX);
		this.window.resized();
	}
}


// implementation



/**
 * Creates the function to use to listen to mouse up events. This is overridden to use this object.
 * @return The function to set as mouse up listener. 
 */
WindowResizeHelper.prototype.createMouseUpListener = function() {
	var thisObject = this;
	return function(e) { thisObject.upHandler(e)};
}



/**
 * Create the resize helper element.
 */
WindowResizeHelper.prototype.createResizeHelper = function() {
	var resizer = document.createElement("div");
	setClass(resizer, "resizeHelper");
	WindowManager.instance().getWindowRootElement().appendChild(resizer);
	this.resizeHelper = resizer;
}